/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/
#include "condor_common.h"
#include "condor_query.h"
#include "condor_attributes.h"
#include "condor_collector.h"
#include "condor_config.h"
#include "condor_network.h"
#include "condor_io.h"
#include "condor_adtypes.h"
#include "condor_debug.h"
#include "get_daemon_addr.h"
#include "internet.h"
#include "daemon.h"


#define XDR_ASSERT(x) {if (!(x)) return Q_COMMUNICATION_ERROR;}

char *new_strdup (const char *);

// The order and number of the elements of the following arrays *are*
// important.  (They follow the structure of the enumerations supplied
// in the header file condor_query.h)
const char *ScheddStringKeywords [] = 
{
	ATTR_NAME 
};

const char *ScheddIntegerKeywords [] = 
{
	ATTR_NUM_USERS,
	ATTR_IDLE_JOBS,
	ATTR_RUNNING_JOBS
};

const char *ScheddFloatKeywords [] = 
{
	""		// add null string to avoid compiler error
};

const char *StartdStringKeywords [] = 
{
	ATTR_NAME,
	ATTR_MACHINE,
	ATTR_ARCH,
	ATTR_OPSYS
};

const char *StartdIntegerKeywords [] = 
{
	ATTR_MEMORY,
	ATTR_DISK
};

const char *StartdFloatKeywords [] =
{
	""		// add null string to avoid compiler error
};

// normal ctor
CondorQuery::
CondorQuery (AdTypes qType)
{
	queryType = qType;
	switch (qType)
	{
	  case STARTD_AD:
		query.setNumStringCats (STARTD_STRING_THRESHOLD);
		query.setNumIntegerCats(STARTD_INT_THRESHOLD);
		query.setNumFloatCats  (STARTD_FLOAT_THRESHOLD);
		query.setIntegerKwList ((char **)StartdIntegerKeywords);
		query.setStringKwList  ((char **)StartdStringKeywords);
		query.setFloatKwList   ((char **)StartdFloatKeywords);
		command = QUERY_STARTD_ADS;
		break;

	  case STARTD_PVT_AD:
		query.setNumStringCats (STARTD_STRING_THRESHOLD);
		query.setNumIntegerCats(STARTD_INT_THRESHOLD);
		query.setNumFloatCats  (STARTD_FLOAT_THRESHOLD);
		query.setIntegerKwList ((char **)StartdIntegerKeywords);
		query.setStringKwList  ((char **)StartdStringKeywords);
		query.setFloatKwList   ((char **)StartdFloatKeywords);
		command = QUERY_STARTD_PVT_ADS;
		break;

	  case SCHEDD_AD:
		query.setNumStringCats (SCHEDD_STRING_THRESHOLD);
		query.setNumIntegerCats(SCHEDD_INT_THRESHOLD);
		query.setNumFloatCats  (SCHEDD_FLOAT_THRESHOLD);
		query.setIntegerKwList ((char **)ScheddIntegerKeywords);
		query.setStringKwList  ((char **)ScheddStringKeywords);
		query.setFloatKwList   ((char **)ScheddFloatKeywords);
		command = QUERY_SCHEDD_ADS;
		break;

	  case SUBMITTOR_AD:
		query.setNumStringCats (SCHEDD_STRING_THRESHOLD);
		query.setNumIntegerCats(SCHEDD_INT_THRESHOLD);
		query.setNumFloatCats  (SCHEDD_FLOAT_THRESHOLD);
		query.setIntegerKwList ((char **)ScheddIntegerKeywords);
		query.setStringKwList  ((char **)ScheddStringKeywords);
		query.setFloatKwList   ((char **)ScheddFloatKeywords);
		command = QUERY_SUBMITTOR_ADS;
		break;

	  case LICENSE_AD:
		query.setNumStringCats (0);
		query.setNumIntegerCats(0);
		query.setNumFloatCats  (0);
		command = QUERY_LICENSE_ADS;
		break;

	  case MASTER_AD:
		query.setNumStringCats (0);
		query.setNumIntegerCats(0);
		query.setNumFloatCats  (0);
		command = QUERY_MASTER_ADS;
		break;

	  case CKPT_SRVR_AD:
		query.setNumStringCats (0);
		query.setNumIntegerCats(0);
		query.setNumFloatCats  (0);
		command = QUERY_CKPT_SRVR_ADS;
		break;

	  case COLLECTOR_AD:
		query.setNumStringCats (0);
		query.setNumIntegerCats(0);
		query.setNumFloatCats  (0);
		command = QUERY_COLLECTOR_ADS;
		break;

          case STORAGE_AD:
                query.setNumStringCats (0);
                query.setNumIntegerCats(0);
                query.setNumFloatCats  (0);
                command = QUERY_STORAGE_ADS;
                break;

          case ANY_AD:
                query.setNumStringCats (0);
                query.setNumIntegerCats(0);
                query.setNumFloatCats  (0);
                command = QUERY_ANY_ADS;
                break;

	  default:
		command = -1;
		queryType = (AdTypes) -1;
	}
}


// copy ctor; makes deep copy
CondorQuery::
CondorQuery (const CondorQuery &from)
{
}


// dtor
CondorQuery::
~CondorQuery ()
{	
}


// clear particular string category
QueryResult CondorQuery::
clearStringConstraints (const int i)
{
	return (QueryResult) query.clearString (i);
}


// clear particular integer category
QueryResult CondorQuery::
clearIntegerConstraints (const int i)
{
	return (QueryResult) query.clearInteger (i);
}

// clear particular float category
QueryResult CondorQuery::
clearFloatConstraints (const int i)
{
	return (QueryResult) query.clearFloat (i);
}


void CondorQuery::
clearORCustomConstraints (void)
{
	query.clearCustomOR ();
}


void CondorQuery::
clearANDCustomConstraints (void)
{
	query.clearCustomAND ();
}


// add a string constraint
QueryResult CondorQuery::
addConstraint (const int cat, const char *value)
{
	return (QueryResult) query.addString (cat, (char *) value);
}


// add an integer constraint
QueryResult CondorQuery::
addConstraint (const int cat, const int value)
{
	return (QueryResult) query.addInteger (cat, value);
}


// add a float constraint
QueryResult CondorQuery::
addConstraint (const int cat, const float value)
{
	return (QueryResult) query.addFloat (cat, value);
}


// add a custom constraint
QueryResult CondorQuery::
addANDConstraint (const char *value)
{
	return (QueryResult) query.addCustomAND ((char *) value);
}


// add a custom constraint
QueryResult CondorQuery::
addORConstraint (const char *value)
{
	return (QueryResult) query.addCustomOR ((char *) value);
}


// fetch all ads from the collector that satisfy the constraints
QueryResult CondorQuery::
fetchAds (ClassAdList &adList, const char *poolName)
{
	const char  *pool;
	Sock*    sock; 
	int			more;
	QueryResult result;
	ClassAd     queryAd, *ad;

	if( is_valid_sinful((char *)poolName) ) {
			// We already have a sinful string, use that.
		pool = poolName;
	} else {
			// Assume we have a hostname, and try to find a collector
			// with it. This will return the correct addr for the
			// local pool's collector if poolName is NULL.
		pool = get_collector_addr( poolName );
	}
	if( ! pool ) {
			// We were passed a bogus poolName, abort gracefully
		return Q_NO_COLLECTOR_HOST;
	}

	// make the query ad
	result = (QueryResult) query.makeQuery (queryAd);
	if (result != Q_OK) return result;

	// fix types
	queryAd.InsertAttr( ATTR_MY_TYPE, (string)QUERY_ADTYPE ); // NAC
	switch (queryType) {
	  case STARTD_AD:
	  case STARTD_PVT_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)STARTD_ADTYPE ); // NAC
		break;

	  case SCHEDD_AD:
	  case SUBMITTOR_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)SCHEDD_ADTYPE ); // NAC
		break;

	  case LICENSE_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)LICENSE_ADTYPE ); // NAC
		break;

	  case MASTER_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)MASTER_ADTYPE ); // NAC
		break;

	  case CKPT_SRVR_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)CKPT_SRVR_ADTYPE ); // NAC
		break;

	  case COLLECTOR_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)COLLECTOR_ADTYPE ); // NAC
		break;

	  case STORAGE_AD:
		queryAd.InsertAttr (ATTR_TARGET_TYPE, STORAGE_ADTYPE);
		break;

	  case ANY_AD:
		queryAd.InsertAttr ( ATTR_TARGET_TYPE, ANY_ADTYPE);
		break;

	  default:
		return Q_INVALID_QUERY;
	}

	// contact collector
	Daemon my_collector(DT_COLLECTOR, pool, NULL);
	sock = my_collector.startCommand(command, Stream::reli_sock, 0);
	if (!sock) {
        return Q_COMMUNICATION_ERROR;
    }

	sock->encode();
	if (!putOldClassAd((Stream *)sock, queryAd) ||		// NAC / ZKM
		!sock->end_of_message()) {						// NAC / ZKM

		delete sock;
		return Q_COMMUNICATION_ERROR;
	}

	// get result
	sock->decode ();
	more = 1;
	while (more)
	{
		if (!sock->code (more)) {
			sock->end_of_message();
			delete sock;
			return Q_COMMUNICATION_ERROR;
		}
		if (more) {
			ad = new ClassAd;
			if ( !getOldClassAd( (Stream *)sock, *ad) ) {	// NAC / ZKM
				sock->end_of_message();
				delete ad;
				delete sock;
				return Q_COMMUNICATION_ERROR;
			}
			adList.Insert (ad);
		}
	}
	sock->end_of_message();

	// finalize
	sock->close();
	delete sock;
	
	return (Q_OK);
}


QueryResult CondorQuery::
getQueryAd (ClassAd &queryAd)
{
	QueryResult	result;

	result = (QueryResult) query.makeQuery (queryAd);
	if (result != Q_OK) return result;

	// fix types
	queryAd.InsertAttr( ATTR_MY_TYPE, (string)QUERY_ADTYPE ); // NAC
	switch (queryType) {
	  case STARTD_AD:
	  case STARTD_PVT_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)STARTD_ADTYPE ); // NAC
		break;

	  case SCHEDD_AD:
	  case SUBMITTOR_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)SCHEDD_ADTYPE ); // NAC
		break;

	  case LICENSE_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)LICENSE_ADTYPE ); // NAC
		break;

	  case MASTER_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)MASTER_ADTYPE ); // NAC
		break;

	  case CKPT_SRVR_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)CKPT_SRVR_ADTYPE ); // NAC
		break;

	  case COLLECTOR_AD:
		queryAd.InsertAttr( ATTR_TARGET_TYPE, (string)COLLECTOR_ADTYPE ); // NAC
		break;

	  default:
		return Q_INVALID_QUERY;
	}

	return Q_OK;
}

	
QueryResult CondorQuery::
filterAds (ClassAdList &in, ClassAdList &out)
{
	ClassAd queryAd, *candidate;
	QueryResult	result;
	MatchClassAd mad;  // NAC
	bool match;

	// make the query ad
	result = (QueryResult) query.makeQuery (queryAd);
	if (result != Q_OK) return result;

	mad.ReplaceLeftAd(&queryAd);  // NAC

	in.Open();
	while( (candidate = (ClassAd *) in.Next()) )
    {
        // if a match occurs
			//if ((*candidate) >= (queryAd)) out.Insert (candidate);
		mad.ReplaceRightAd( candidate );	   						// NAC
		if( !mad.EvaluateAttrBool( "rightMatchesLeft",  match ) ) { // NAC
                // there was a problem with the match				// NAC
			return Q_INVALID_QUERY;									// NAC
		}															// NAC
		else if ( match ){											// NAC
			out.Insert( candidate );			   					// NAC
		}															// NAC
    }
    in.Close ();
    
	return Q_OK;
}


char *getStrQueryResult(QueryResult q)
{
	switch (q)
	{
    	case Q_OK:					return "ok";
    	case Q_INVALID_CATEGORY:	return "invalid category";
    	case Q_MEMORY_ERROR:		return "memory error";
    	case Q_PARSE_ERROR:			return "parse error";
	    case Q_COMMUNICATION_ERROR:	return "communication error";
	    case Q_INVALID_QUERY:		return "invalid query";
	    case Q_NO_COLLECTOR_HOST:	return "can't find collector";
		default:
			return "unknown error";
	}
}
