/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "analysis.h"
#include "list.h"
#include "simplelist.h"
#include "extArray.h"

ClassAdAnalyzer::
ClassAdAnalyzer( )
{
	jobReq = NULL;
	jobAd = NULL;
}

ClassAdAnalyzer::
~ClassAdAnalyzer( )
{
	if( jobReq ) {
		delete jobReq;
	}
}

bool ClassAdAnalyzer::
AnalyzeJobReqToBuffer( ClassAd *request, ClassAdList &offers, string &buffer )
{
	ResourceGroup rg;

		// create a ResourceGroup object for offer ClassAds
	if( !MakeResourceGroup( offers, rg ) ) {
		buffer += "Unable to process machine ClassAds";
		buffer += "\n";
		return true;
	}

	return AnalyzeJobReqToBuffer( request, rg, buffer );
}

#if defined( COLLECTIONS )
bool ClassAdAnalyzer::
AnalyzeJobReqToBuffer( ClassAd *request, ClassAdCollectionServer &offers,
						 string &buffer )
{
	ResourceGroup rg;

		// create a ResourceGroup object for offer ClassAds
	if( !MakeResourceGroup( offers, rg ) ) {
		buffer += "Unable to process machine ClassAds";
		buffer += "\n";
		return true;
	}

	return AnalyzeJobReqToBuffer( request, rg, buffer );
}
#endif

bool ClassAdAnalyzer::
AnalyzeJobAttrsToBuffer( ClassAd *request, ClassAdList &offers,
						 string &buffer )
{
	ResourceGroup rg;

		// create a ResourceGroup object for offer ClassAds
	if( !MakeResourceGroup( offers, rg ) ) {
		buffer += "Unable to process machine ClassAds";
		buffer += "\n";
		return true;
	}

	return AnalyzeJobAttrsToBuffer( request, rg, buffer );
}


bool ClassAdAnalyzer::
AnalyzeJobReqToBuffer( ClassAd *request, ResourceGroup &offers, string &buffer)
{
	if( !request ) {
			// request is NULL;
		return false;
	}

	PrettyPrint pp;

	ExprTree *reqExpr, *flatReqExpr, *prunedReqExpr;
	reqExpr = flatReqExpr = prunedReqExpr = NULL;
	Value val;

	jobAd = request->Copy( );

	jobReq = new MultiProfile( );
	Profile *profile = NULL;
	Condition *condition = NULL;

		// Look up Requirements expression in request ClassAd
	if( !( reqExpr = request->Lookup( ATTR_REQUIREMENTS ) ) ) {
		buffer += "Job ClassAd is missing ";
		buffer += ATTR_REQUIREMENTS;
		buffer += " expression.";
		buffer += "\n";
		return true;
	}

		// Print out requirements expression to buffer

		// Format req expression for 80 column screen.
	string temp_buffer;
	pp.Unparse( temp_buffer, reqExpr );
	string::iterator t, lastAnd, lineStart;
	t = lastAnd = lineStart = temp_buffer.begin( );
	while( t != temp_buffer.end( ) ) {
		if( ( *t == '&' ) && ( *( t+1 ) == '&' ) ) {
			lastAnd = t + 2;
		}
		if( t >= lineStart + 80 ) {
			if( lastAnd != lineStart ) {
				temp_buffer.replace( lastAnd, lastAnd + 1, 1, '\n' );
				lineStart = lastAnd + 1;
				lastAnd = lineStart;
			}
		}
		t++;
	}
		// Print formatted req expression
	buffer += "\n";
	buffer += "The ";
	buffer += ATTR_REQUIREMENTS; 
	buffer += " expression for your job is:";
	buffer += "\n";
	buffer += "\n";
	buffer += temp_buffer;
	buffer += "\n";
	buffer += "\n";

		// Try to flatten Requirements expression
	if( !( request->FlattenAndInline( reqExpr, val, flatReqExpr ) ) ) {
		return true;
	}

		// Check if Requirements expression flattened to a literal value
	if( !flatReqExpr ) {
		buffer += "Job ClassAd "; 
		buffer += ATTR_REQUIREMENTS; 
		buffer += " expression evaluates to ";
		pp.Unparse( buffer, val );
		buffer += "\n";
		buffer += "\n";
		return true;
	}

		// Get rid of dangling boolean literals created by Flatten
	if( !( PruneDisjunction( flatReqExpr, prunedReqExpr ) ) ) {
		return true;
	}

		// Convert Requirements expression to a MultiProfile object
	if( !( BoolExpr::ExprToMultiProfile( prunedReqExpr, jobReq ) ) ) {
		return true;
	}

		// Determine which Conditions should be removed from Requierments
		// expression
	if( !( SuggestCondition( jobReq, offers ) ) ) {
		return true;
	}

		// Determine which Conditions conflict with one another
	if( !( FindConflicts( jobReq, offers ) ) ) {
		return true;
	}

		// req->explain.match and req->explain.numberOfMatches contain
		// information about the offers that satisfy the requests requirements 

		// The following should probably make use of ClassAdPrintMask

		// Get information from data structures and print to the buffer
	string cond_s, value_s;
	char formatted[2048];
	char cond[1024];
	char info[64];
	char suggest[64];
	char value[64];
	char tempBuff[64];
	int p = 1;
	jobReq->Rewind( );
	while( jobReq->NextProfile( profile ) ) {

			// If we have more than one profile we need to number them.
		int numProfs;
		jobReq->GetNumberOfProfiles( numProfs );
		if( numProfs > 1 ) {
			buffer += "Profile ";
			sprintf( tempBuff, "%i", p );
			buffer += tempBuff;
			if( profile->explain.match ) {
				buffer += " matched ";
				sprintf( tempBuff, "%i", profile->explain.numberOfMatches );
				buffer += tempBuff;
			} else {
				buffer += " rejected all";
			}
			if( profile->explain.numberOfMatches == 1 ) {
				buffer += " machine";
			} else {
				buffer += " machines";
			}
			buffer += "\n";
		}

			// Sort list of conditions by number of matches
		List< Condition > sortedCondList;
		profile->Rewind( );
		Condition *sortedCond;
		SimpleList< int > mapList;
		int index = 0;
		int junk;
		while( profile->NextCondition( condition ) ) {			
			if( sortedCondList.IsEmpty( ) ) {
				sortedCondList.Append( condition );
				mapList.Append( index );
			} else {
				sortedCondList.Rewind( );
				mapList.Rewind( );
				while( sortedCondList.Next( sortedCond ) ) {
					mapList.Next( junk );
					if( condition->explain.numberOfMatches <
						sortedCond->explain.numberOfMatches ) {
						sortedCondList.Insert( condition );
						mapList.Insert( index );
						break;
					}
					if( sortedCondList.AtEnd( ) ) {
						sortedCondList.Append( condition );
						mapList.Append( index );
					}
				}
			}
			index++;
		}
		sortedCondList.Rewind( );
		mapList.Rewind( );

			// create map from original Condition order to sorted order
		int numConds = 0;
		profile->GetNumberOfConditions( numConds );
		int condMap[numConds];
		int i = 0;
		while( mapList.Next( index ) ) {
			condMap[index] = i;
			i++;
		}

			// print header for condition list
		sprintf( formatted, "    %-34s%-20s%s\n", "Condition",
				 "Machines Matched", "Suggestion" );
		buffer += formatted;
		sprintf( formatted, "    %-34s%-20s%s\n", "---------",
				 "----------------", "----------" );
		buffer += formatted;
		i = 1;

			// print each condition, number of matches, and suggestion
		while( sortedCondList.Next( condition ) ) {
			cond_s = "";
			value_s = "";
			condition->ToString( cond_s );
			strncpy( cond, cond_s.c_str( ), 1024 );

			sprintf( info, "%i", condition->explain.numberOfMatches );

			switch( condition->explain.suggestion ) {
			case ConditionExplain::REMOVE: {
				sprintf( suggest, "REMOVE" );
				break;
			}
			case ConditionExplain::MODIFY: {
				pp.Unparse( value_s, condition->explain.newValue );
				strncpy( value, value_s.c_str( ), 64 );
				sprintf( suggest, "MODIFY TO %s", value );
				break;
			}
			default: {
				sprintf( suggest, " " );
			}
			}

   			if( strlen( cond ) < 46 ) {
				sprintf( formatted, "%-4i%-34s%-20s%s\n", i, cond, info,
						 suggest );
			} else {
				sprintf( formatted, "%-4i%s\n%38s%-20s%s\n", i, cond, "", info,
						 suggest );
			}

			buffer += formatted;
			i++;
		}

			// print out conflicts
		IndexSet sortedIS;
		IndexSet *rawIS;
		profile->explain.conflicts->Rewind( );
		if( !profile->explain.conflicts->IsEmpty( ) ) {
			buffer += "\n";
			buffer += "Conflicts:\n";
			buffer += "\n";
			while( profile->explain.conflicts->Next( rawIS ) ) {
				sortedIS.Init( numConds );
				IndexSet::Translate( *rawIS, condMap, numConds, numConds,
									 sortedIS );
				buffer += "  conditions: ";
				bool firstNum = true;
				for( int i = 0; i < numConds; i++ ) {
					if( sortedIS.HasIndex( i ) ) {
						if( !firstNum ) {
							buffer += ", ";
						}
						else {
							firstNum = false;
						}
						sprintf( tempBuff, "%i", i+1 );
						buffer += tempBuff;
					}
				}
				buffer += "\n";
			}
		}	

		p++;
	}
	return true;
}

bool ClassAdAnalyzer::
AnalyzeJobAttrsToBuffer( ClassAd *request, ResourceGroup &offers,
						 string &buffer )
						                                        
{
	if( !request ) {
			// request is NULL;
		return false;
	}

	PrettyPrint pp;

	ClassAdExplain adExplain;
	char formatted[2048];
	char attr[64];
	char suggest[64];

	if( !( AnalyzeAttributes( request, offers, adExplain ) ) ) {
		cerr << "error in AnalyzeAttributes" << endl << endl;
	}

		// get information from ClassAdExplain

		// print list of undefined job attributes
	if( !adExplain.undefAttrs.IsEmpty( ) ) {
		buffer += "\n";
		buffer += "The following attributes are missing from the job ClassAd:";
		buffer += "\n";
		buffer += "\n";
		string undefAttr = "";
		adExplain.undefAttrs.Rewind( );
		while( adExplain.undefAttrs.Next( undefAttr ) ) {
			buffer += undefAttr;
			buffer += "\n";
		}
	}

		// print ideal ranges for attributes
	if( !adExplain.attrExplains.IsEmpty( ) ) {
		string value_s = "";
		string suggest_s = "";
		string tempBuff = "";
		int numModAttrs = 0;

		tempBuff += "\nThe following attributes should be added or modified:";
		tempBuff += "\n";
		tempBuff += "\n";

			// print header for attribute list
		sprintf( formatted, "%-24s%s\n", "Attribute", "Suggestion" );
		tempBuff += formatted;
		sprintf( formatted, "%-24s%s\n", "---------", "----------" );
		tempBuff += formatted;

			// print each attribute and suggestion
		AttributeExplain *attrExplain = NULL;
		adExplain.attrExplains.Rewind( );
		while( adExplain.attrExplains.Next( attrExplain ) ) {
			switch( attrExplain->suggestion ) {
			case AttributeExplain::MODIFY: {
				numModAttrs++;
				strncpy( attr, attrExplain->attribute.c_str( ), 64 );
				if( attrExplain->isInterval ) {
					double lower = 0;
					double upper = 0;
					GetLowDoubleValue( attrExplain->intervalValue, lower );
					GetHighDoubleValue( attrExplain->intervalValue, upper );
					suggest_s = "use a value ";
					if( lower > -( FLT_MAX ) ) { // lower bound exists
						if( attrExplain->intervalValue->openLower ) {
							suggest_s += "> ";
						}
						else {
							suggest_s += ">= ";
						}
						pp.Unparse( value_s,attrExplain->intervalValue->lower);
						suggest_s += value_s;
						value_s = "";
						if( upper < FLT_MAX ) {
							suggest_s += " and ";
						}
					}
					if( upper < FLT_MAX ) { // upper bound exists
						if( attrExplain->intervalValue->openUpper ) {
							suggest_s += "< ";
						}
						else {
							suggest_s += "<= ";
						}
						pp.Unparse( value_s,attrExplain->intervalValue->upper);
						suggest_s += value_s;
						value_s = "";
					}
				}
				else {  // attrExplain has a discrete value
					suggest_s = "change to ";
					pp.Unparse( value_s, attrExplain->discreteValue );
					suggest_s += value_s;
					value_s = "";
				}
				strncpy( suggest, suggest_s.c_str( ), 64 ); 
				sprintf( formatted, "%-24s%s\n", attr, suggest );
				tempBuff += formatted;
			}
			default: { }
			}
		}
		if( numModAttrs > 0 ) {
			buffer += tempBuff;
		}
	}
	return true;
}

bool ClassAdAnalyzer::
AnalyzeExprToBuffer( ClassAd *mainAd, ClassAd *contextAd, string &attr,
					 string &buffer )
{
	PrettyPrint pp;
	Value val;
	string tempBuff_s = "";
	ResourceGroup rg;
	List<ClassAd> contextList;
	MultiProfile *mp = new MultiProfile;
	Profile *profile = NULL;
	Condition *condition = NULL;
	ExprTree *expr = NULL;
	ExprTree *flatExpr = NULL;
	ExprTree *prunedExpr = NULL;
	char tempBuff[64];
	char formatted[2048];
	string cond_s = "";
	char cond[1024];
	string value_s = "";
	char value[64];

	contextList.Append( contextAd );

	if( !( rg.Init( contextList ) ) ) {
		cerr << "problem adding job ad to ResourceGroup\n";
	}
	if( !( expr = mainAd->Lookup( attr ) ) ) {
		cerr << "error looking up " << attr << " expression\n";
		return false;
	}
	
	if( !( mainAd->FlattenAndInline( expr, val, flatExpr ) ) ) {
		cerr << "error flattening machine ad\n";
		return false;
	}

	if( !flatExpr ) {
		buffer += attr;
		buffer += " expresion flattens to ";
		pp.Unparse( buffer, val );
		buffer += "\n";
		return true;
	}

	if( !PruneDisjunction( flatExpr, prunedExpr ) ) {
		cerr << "error pruning expression:\n";
		pp.Unparse( tempBuff_s, flatExpr );
		cerr << tempBuff_s << "\n";
		return false;
	}

	if( !( BoolExpr::ExprToMultiProfile( prunedExpr, mp ) ) ) {
		cerr << "error in ExprToMultiProfile\n";
		return false;
	}		
	
		// Do analysis
	if( !SuggestCondition( mp, rg ) ) {
		cerr << "error in SuggestCondition\n";
	}
	
		// Print results
	buffer += "\n";
	buffer += "=====================\n";
	buffer += "RESULTS OF ANALYSIS :\n";
	buffer += "=====================\n";
	buffer += "\n";

	buffer += attr;
	buffer += " expression ";
	if( mp->explain.match ) {
		buffer += "is true\n";
	} else {
		buffer += "is not true\n";
	}

	int p = 1;
	int numProfiles;
	mp->Rewind( );
	while( mp->NextProfile( profile ) ) {
		mp->GetNumberOfProfiles( numProfiles );
		if( numProfiles > 1 ) {
			buffer += "  Profile ";
			sprintf( tempBuff, "%i", p );
			buffer += tempBuff;
			if( profile->explain.match ) {
				buffer += " is true\n";
			} else {
				buffer += " is false\n";
			}
		}
		profile->Rewind( );
		while( profile->NextCondition( condition ) ) {
			condition->ToString( cond_s );
			strncpy( cond, cond_s.c_str( ), 1024 );
			cond_s = "";

			if( condition->explain.match ) {
				value_s = "is true";
			} else {
				value_s = "is false";
			}
			strncpy( value, value_s.c_str( ), 64 );
			value_s = "";

			sprintf( formatted, "    %-25s%s\n", cond, value );
			buffer += formatted;
		}
		p++;
	}		
	buffer += "=====================\n";
	buffer += "\n";
	return true;
}

ExprTree *ClassAdAnalyzer::
AddExplicitConditionals( ExprTree *expr )
{
	if( expr == NULL ) {
		return NULL;
	}
	ExprTree *currentExpr = expr;
	ExprTree::NodeKind nKind = expr->GetKind( );
	switch( nKind ) {
	case ExprTree::ATTRREF_NODE: {
		ExprTree *fnExpr = NULL;
		vector< ExprTree * > params( 1 );
		params[0] = expr->Copy( );
		ExprTree *condExpr = NULL;
		ExprTree *parenExpr = NULL;
		ExprTree *condExpr2 = NULL;
		ExprTree *parenExpr2 = NULL;
		Value val0, val1;
		val0.SetIntegerValue( 0 );
		val1.SetIntegerValue( 1 );
		fnExpr = FunctionCall::MakeFunctionCall( "IsBoolean", params );
		condExpr = Operation::MakeOperation( Operation::TERNARY_OP,
											 expr->Copy( ), 
											 Literal::MakeLiteral( val1 ),
											 Literal::MakeLiteral( val0 ) );
		parenExpr = Operation::MakeOperation( Operation::PARENTHESES_OP,
											  condExpr, NULL, NULL );
		condExpr2 = Operation::MakeOperation( Operation::TERNARY_OP,
											  fnExpr, parenExpr, 
											  expr->Copy( ) );
		parenExpr2 = Operation::MakeOperation( Operation::PARENTHESES_OP,
										 condExpr2, NULL, NULL );
		return parenExpr2;
	}
	case ExprTree::FN_CALL_NODE:
	case ExprTree::CLASSAD_NODE:
	case ExprTree::EXPR_LIST_NODE: {
		return NULL;
	}
	case ExprTree::LITERAL_NODE: {
		Value val;
		( ( Literal *)expr )->GetValue( val );
		bool b;
		if( val.IsBooleanValue( b ) ) {
			if( b ) {
				val.SetIntegerValue( 1 );
			}
			else {
				val.SetIntegerValue( 0 );
			}
			return Literal::MakeLiteral( val );
		}
		else {
			return NULL;
		}
	}
	case ExprTree::OP_NODE: {
		Operation::OpKind oKind;
		ExprTree * expr1 = NULL;
		ExprTree * expr2 = NULL;
		ExprTree * expr3 = NULL;
		( ( Operation * )expr )->GetComponents( oKind, expr1, expr2, expr3 );
		while( oKind == Operation::PARENTHESES_OP ) {
			currentExpr = expr1;
			( ( Operation * )expr1 )->GetComponents(oKind,expr1,expr2,expr3);
		}
		if( ( Operation::__COMPARISON_START__ <= oKind &&
			  oKind <= Operation::__COMPARISON_END__ ) ||
			( Operation::__LOGIC_START__ <= oKind &&
			  oKind <= Operation::__LOGIC_END__ ) ) {
			Value val0, val1;
			val0.SetIntegerValue( 0 );
			val1.SetIntegerValue( 1 );
			ExprTree *tern = NULL;
			tern = Operation::MakeOperation( Operation::TERNARY_OP,
											 expr->Copy( ),
											 Literal::MakeLiteral( val1 ),
											 Literal::MakeLiteral( val0 ) );
			return Operation::MakeOperation( Operation::PARENTHESES_OP,
											 tern, NULL, NULL );
		}
		else if( Operation::__ARITHMETIC_START__ <= oKind &&
				 oKind <= Operation::__ARITHMETIC_END__ ) {
			ExprTree *newExpr1 = AddExplicitConditionals( expr1 );
			if( oKind == Operation::UNARY_PLUS_OP || 
				oKind == Operation::UNARY_MINUS_OP ) {
				if( newExpr1 != NULL ) {
					return Operation::MakeOperation(oKind,newExpr1,NULL,NULL);
				}
				else {
					return NULL;
				}
			}
			else {
				ExprTree *newExpr2 = AddExplicitConditionals( expr2 );
				if( newExpr1 != NULL || newExpr2 != NULL ) {
					if( newExpr1 == NULL ) {
						newExpr1 = expr1->Copy( );
					}
					if( newExpr2 == NULL ) {
						newExpr2 = expr2->Copy( );
					}
					return Operation::MakeOperation( oKind, newExpr1, newExpr2,
													 NULL );
				}
				else {
					return NULL;
				}
			}
		}
		else if( oKind == Operation::TERNARY_OP ) {
			ExprTree *newExpr2 = AddExplicitConditionals( expr2 );
			ExprTree *newExpr3 = AddExplicitConditionals( expr3 );
			if( newExpr2 != NULL || newExpr3 != NULL ) {
				if( newExpr2 == NULL ) {
					newExpr2 = expr2->Copy( );
				}
				if( newExpr3 == NULL ) {
					newExpr3 = expr3->Copy( );
				}
				return Operation::MakeOperation( oKind, expr1->Copy( ), 
												 newExpr2, newExpr3 );
			}
			else {
				return NULL;
			}
		}
		return NULL;
	}
	default: {
		return NULL;
	}
	}
		
	return NULL;
}


ClassAd *ClassAdAnalyzer::
AddExplicitTargets( ClassAd *ad )
{
	string attr = "";
	set< string, CaseIgnLTStr > definedAttrs;
	
	for( ClassAd::iterator a = ad->begin( ); a != ad->end( ); a++ ) {
		definedAttrs.insert( a->first );
	}
	
	ClassAd *newAd = new ClassAd( );
	for( ClassAd::iterator a = ad->begin( ); a != ad->end( ); a++ ) {
		newAd->Insert( a->first,AddExplicitTargets( a->second, definedAttrs ));
	}
	return newAd;
}

// private methods

bool ClassAdAnalyzer::
BuildBoolTable( MultiProfile *mp, ResourceGroup &rg, BoolTable &result )
{
	BoolValue bval;
	Profile *profile;
	ClassAd *ad;
	List<ClassAd> contexts;
	int numProfs = 0;
	int numContexts = 0;

	if( !mp->GetNumberOfProfiles( numProfs ) ) {
		cerr << "BuildBoolTable: error calling GetNumberOfProfiles" << endl;
	}

	if( !rg.GetNumberOfClassAds( numContexts ) ) {
		cerr << "BuildBoolTable: error calling GetNumberOfClassAds" << endl;
	}

	if( !rg.GetClassAds( contexts ) ) {
		cerr << "BuildBoolTable: error calling GetClassAds" << endl;
	}

	if( !result.Init( numContexts, numProfs ) ) {
		cerr << "BuildBoolTable: error calling BoolTable::Init" << endl;
	}

	contexts.Rewind( );
	int col = 0;
	while( contexts.Next( ad ) ) {
		int row = 0;
		mp->Rewind( );
		while( mp->NextProfile( profile ) ) {
			profile->EvalInContext( mad, ad, bval );
			result.SetValue( col, row, bval );
			row++;
	    }
		col++;
	}

	return true;
}


bool ClassAdAnalyzer::
BuildBoolTable( Profile *p, ResourceGroup &rg, BoolTable &result ) {

	BoolValue bval;
	Condition *condition;
	ClassAd *ad;
	int numConds = 0;
	int numContexts = 0;

	p->GetNumberOfConditions( numConds );

	rg.GetNumberOfClassAds( numContexts );
	List<ClassAd> contexts;
	rg.GetClassAds( contexts );

	result.Init( numContexts, numConds );

	contexts.Rewind( );
	int col = 0;
	while( contexts.Next( ad ) ) {
		int row = 0;

		p->Rewind( );

		while( p->NextCondition( condition ) ) {
			condition->EvalInContext( mad, ad, bval );
			result.SetValue( col, row, bval );
			row++;
	    }
		col++;
	}

	return true;
}

bool ClassAdAnalyzer::
MakeResourceGroup( ClassAdList &caList, ResourceGroup &rg )
{
	List<ClassAd> newList;
	ClassAd *ad;
	caList.Rewind( );
	ad = caList.Next( );
	while( ad ) {
		newList.Append( ad->Copy( ) );
		ad = caList.Next( );
	}
	if( !rg.Init( newList ) ) {
		return false;
	}
	return true;
}


#if defined( COLLECTIONS )
bool ClassAdAnalyzer::
MakeResourceGroup( ClassAdCollectionServer &server, ResourceGroup &rg )
{
	List<ClassAd> newList;
	string q_key;
	ClassAd *tmp;
	LocalCollectionQuery a;
	a.Bind(&server);
	a.Query("root",NULL);
	a.ToFirst();
	bool ret=a.Current(q_key);
	if (ret==true){
		do{
			tmp=server.GetClassAd(q_key);
			newList.Append( tmp->Copy( ) );
		}while(a.Next(q_key)==true);
	}
	if( !rg.Init( newList ) ) {
		return false;
	}
	return true;
}
#endif

bool ClassAdAnalyzer::
SuggestCondition( MultiProfile *mp, ResourceGroup &rg )
{
	if( mp == NULL ) {
		cerr << "SuggestCondition: tried to pass null MultiProfile"
			 << endl;
		return false;
	}

	BoolTable bt;
	if( !BuildBoolTable( mp, rg, bt ) ) {
		return false;
	}

		// Get info from BoolTable for MultiProfileExplain
	int mpMatches = 0;
	int numCols = 0;
	bt.GetNumColumns( numCols );
	int currentColTotalTrue;
	IndexSet matchedClassAds;
	matchedClassAds.Init( numCols );
	for( int i = 0; i < numCols; i++ ) {
		bt.ColumnTotalTrue( i, currentColTotalTrue );
		if( currentColTotalTrue > 0 ) {
			mpMatches++;
			matchedClassAds.AddIndex( i );
		}
	}
	if( mpMatches > 0 ) {
		if( !( mp->explain.Init( true, mpMatches, matchedClassAds, 
								 numCols ) ) ) {
			return false;
		}
	}
	else {
		if( !( mp->explain.Init( false, 0, matchedClassAds, numCols ) ) ) {
			return false;
		}
	}

		// call SuggestConditionRemove on all profiles
	Profile *currentProfile;
	mp->Rewind( );
	while( mp->NextProfile( currentProfile ) ) {
		if( !SuggestConditionModify( currentProfile, rg ) ) {
			cerr << "error in SuggestConditionModify" << endl;
			return false;
		}
//		if( !SuggestConditionRemove( currentProfile, rg ) ) {
//			return false;
//		}
	}

	return true;
}


bool ClassAdAnalyzer::
SuggestConditionRemove( Profile *p, ResourceGroup &rg )
{
	List<AnnotatedBoolVector> abvList;
	BoolTable bt;
	Condition *condition;
	BoolValue bval;
	int profileMatches = 0;
	int numRows = 0;
	int numColumns = 0;
	int currentColTotalTrue = 0;
	int numberOfMatches = 0;
	int row = 0;
	bool match = false;

	AnnotatedBoolVector *bestABV = NULL;
	string buffer;

	if( !BuildBoolTable( p, rg, bt ) ) {
		return false;
	}

  	if( !bt.GenerateMaxTrueABVList( abvList ) ) {
  		return false;
  	}
	
		// Get info from BoolTable for Profile
	bt.GetNumRows( numRows );
	bt.GetNumColumns( numColumns );
	for( int i = 0; i < numColumns; i++ ) {
		bt.ColumnTotalTrue( i, currentColTotalTrue );
		if( currentColTotalTrue == numRows ) {
			profileMatches++;
		}
	}

	if( profileMatches > 0 ) {
		if( !( p->explain.Init( true, profileMatches ) ) ) {
			return false;
		}
	}
	else {
		if( !( p->explain.Init( false, 0 ) ) ) {
			return false;
		}
	}

		// Get info from BoolTable for ConditionExplains

	p->Rewind( );
	row = 0;
	while( p->NextCondition( condition ) ) {
		bt.RowTotalTrue( row, numberOfMatches );
		if( numberOfMatches == 0 ) {
			match = false;
		}
		else {
			match = true;
		}
		if( !( condition->explain.Init( match, numberOfMatches ) ) ) {
			return false;
		}
		row++;
	}
	
		// find first ABV with max frequency & max total true	
		// set up ConditionExplains using ABV
	if( !AnnotatedBoolVector::MostFreqABV( abvList, bestABV ) ) {
		cerr << "Analysis::SuggestConditionRemove(): error - bad ABV" << endl;
		return false;
	}
	else {
		int i = 0;
		p->Rewind( );
		while( p->NextCondition( condition ) ) {
			bestABV->GetValue( i, bval );
			if( bval == TRUE_VALUE ) {
				condition->explain.suggestion = ConditionExplain::KEEP;
			}
			else{
				condition->explain.suggestion = ConditionExplain::REMOVE;
			}
			i++;
		}
	}

	return true;
}

bool ClassAdAnalyzer::
SuggestConditionModify( Profile *p, ResourceGroup &rg )
{
	List<AnnotatedBoolVector> abvList;
	BoolTable bt;
	ValueTable vt;
	Condition *condition;
	int profileMatches = 0;
	int numConds = 0;
	int numContexts = 0;
	int currentColTotalTrue = 0;
	int numberOfMatches = 0;
	bool match = false;

		// debug
  	PrettyPrint pp;
  	string buffer = "";

	if( !BuildBoolTable( p, rg, bt ) ) {
		return false;
	}

		// debug
//  	bt.ToString( buffer );
//  	cout << "bt = " << buffer << endl;
//  	buffer = "";

		// Get info from BoolTable for Profile
	bt.GetNumRows( numConds );
	bt.GetNumColumns( numContexts );
	for( int i = 0; i < numContexts; i++ ) {
		bt.ColumnTotalTrue( i, currentColTotalTrue );
		if( currentColTotalTrue == numConds ) {
			profileMatches++;
		}
	}

	if( profileMatches > 0 ) {
		if( !( p->explain.Init( true, profileMatches ) ) ) {
			return false;
		}
	}
	else {
		if( !( p->explain.Init( false, 0 ) ) ) {
			return false;
		}
	}

		// Get info from BoolTable for ConditionExplains
		// set up array of operators and get list of attrs;
	ExtArray< string > attrs;
	ExtArray< ValueRange * > vrs;
	string attr = "";
	int vr4Cond[numConds];
	int attrNum = 0;
	int condNum = 0;
	int seenAttrNum = 0;
	Operation::OpKind ops[numConds];
//	Operation::OpKind op1, op2;
	Value val;
	p->Rewind( );
	while( p->NextCondition( condition ) ) {

		if( !condition->HasMultipleAttrs( ) ) {

				// Attribute
			condition->GetAttr( attr );
			string currAttr;
			bool seenAttr = false;
			seenAttrNum = 0;
			for( int i = 0; i < attrs.getsize( ); i++ ) {
				currAttr = attrs[i];
				if( EqualsIgnoreCase( attr, currAttr ) ) {
					seenAttr = true;
					seenAttrNum = i;
					break;
				}
			}

			if( !seenAttr ) {
				seenAttrNum = attrNum;
				attrs.resize( attrNum + 1 );
				attrs[attrNum] = attr;
				vrs.resize( attrNum + 1 );
				vrs[attrNum] = NULL;
				attrNum++;
			}

				// Operator(s) and Value(s)
			if( condition->IsComplex( ) ) {
				ops[condNum] = Operation::__NO_OP__;
				vr4Cond[condNum] = -1;
			}
			else {
				condition->GetOp( ops[condNum] );
				if( vrs[seenAttrNum] == NULL ) {
					vrs[seenAttrNum] = new ValueRange( );
				}
				AddConstraint( vrs[seenAttrNum], condition );
				vr4Cond[condNum] = seenAttrNum;
			}

		}
		else {
			ops[condNum] = Operation::__NO_OP__;
			vr4Cond[condNum] = -1;
		}

			// get info from BoolTable
		bt.RowTotalTrue( condNum, numberOfMatches );
		if( numberOfMatches == 0 ) {
			match = false;
		}
		else {
			match = true;
		}
		if( !( condition->explain.Init( match, numberOfMatches ) ) ) {
			return false;
		}
		condNum++;
	}

	int numAttrs = attrs.getsize( );
	Value *nearestValues[numAttrs];
	for( int i = 0; i < numAttrs; i++ ) {
		nearestValues[i] = NULL;
	}

		// debug
//  	cout << "attrs:" << endl;
//  	cout << "------" << endl;
//  	for( int i = 0; i < numAttrs; i++ ) {
//  		cout << attrs[i] << endl;
//  	}
//  	cout << endl;
//  	cout << "vrs:" << endl;
//  	cout << "----" << endl;
//  	string vrString = "";
//  	for( int i = 0; i < numAttrs; i++ ) {
//  		if( vrs[i] == NULL ) {
//  			cout << "NULL" << endl;
//  		}
//  		else {
//  			vrs[i]->ToString( vrString );
//  			cout << vrString << endl;
//  			vrString = "";
//  		}
//  	}
//  	cout << endl;
//  	cout << "vr4Cond:[";
//  	for( int i = 0; i < numConds; i++ ) {
//  		if( i > 0 ) {
//  			cout << ",";
//  		}
//  		cout << vr4Cond[i];
//  	}
//  	cout << "]" << endl;
//  	cout << endl;

	List<ClassAd> contexts;
	rg.GetClassAds( contexts );
	ClassAd *context = NULL;

		// debug
//  	cout << "numContexts = " << numContexts << endl;
//  	cout << "numAttrs = " << numContexts << endl;
//  	cout << endl;


		// build ValueTable
	vt.Init( numContexts, numAttrs );
	for( int row = 0; row < numAttrs; row++ ) {
		attr = attrs[row];
		p->Rewind( );
		string currAttr;
		while( p->NextCondition( condition ) ) {
			if( !condition->HasMultipleAttrs( ) ) {
				condition->GetAttr( currAttr );
				if( EqualsIgnoreCase( currAttr, attr ) ) {
					Operation::OpKind op;
					Value val;
					condition->GetOp( op );
					condition->GetVal( val );
					vt.SetOp( row, op );
					if( condition->IsComplex( ) ) {
						condition->GetOp2( op );
						condition->GetVal2( val );
						vt.SetOp( row, op );
					}
				}
			}
		}

		contexts.Rewind( );
		for( int col = 0; col < numContexts; col++ ) {
			contexts.Next( context );
			Value val;
			context->EvaluateAttr( attr, val );
			vt.SetValue( col, row, val );
		}
	}

		// DEBUG
//  	vt.ToString( buffer );
//  	cout << "vt = " << buffer << endl;
//  	buffer = "";
	double distances[numContexts];
	for( int i = 0; i < numContexts; i++ ) {
		distances[i] = 0;
	}

		// Calculate distances
	Value::ValueType type;
	Value currPt, currUpper, currLower;
	BoolValue currBVal, tempBVal;
	val.SetUndefinedValue( );
	Value *tempVals[numAttrs];
	for( int i = 0; i < numAttrs; i++ ) {
		tempVals[i] = NULL;
	}
	int closestCtx = -1;
	double currDist = 0, sumDist = 0, minSumDist = (double)numAttrs + 1; 
	for( int col = 0; col < numContexts; col++ ) {
		for( int row = 0; row < numAttrs; row++ ) {
			currBVal = TRUE_VALUE;
			for( int i = 0; i < numConds; i++ ) {
				if( vr4Cond[i] == row ) {
					bt.GetValue( col, i, tempBVal );
					And( currBVal, tempBVal, currBVal );
				}
			}
			if( currBVal == TRUE_VALUE ) {
				currDist = 0;
			}
			else if( currBVal == UNDEFINED_VALUE ) {
				currDist = 1;
			}
			else {
				vt.GetValue( col, row, currPt );
				type = currPt.GetType( );

					// debug
//  				if( col == 0 ) {
//  					cout << "row " << row << " type = " << type << endl;
//  				}

				if( type == Value::BOOLEAN_VALUE || 
					type == Value::STRING_VALUE ) {
					currDist = 1;
					tempVals[row] = new Value( );
					tempVals[row]->CopyFrom( currPt );
				}
				else {
					vt.GetUpperBound( row, currUpper );
					vt.GetLowerBound( row, currLower );
					vrs[row]->GetDistance( currPt, currLower, currUpper,
										   currDist, val );
					tempVals[row] = new Value( );

						// debug
//  					if( col == 0 ) {
//  						pp.Unparse( buffer, val );
//  						cout << "val for row " << row << " = " << buffer 
//  							 << endl;
//  					}

					tempVals[row]->CopyFrom( val );
				}
			}
			sumDist += currDist;
		}

			// experimental
		distances[col] = sumDist;

		if( sumDist < minSumDist ) {
			minSumDist = sumDist;
			closestCtx = col;
			for( int i = 0; i < numAttrs; i++ ) {
				if( tempVals[i] ) {				
					nearestValues[i] = tempVals[i];
				}
			}
		}
		sumDist = 0;
	}

		// experimental
	List<ClassAd> machineAds;
	rg.GetClassAds( machineAds );
	machineAds.Rewind( );
	ClassAd *machAd = NULL;
	int jobRanks[numContexts];
	for( int i = 0; i < numContexts; i++ ) {
		jobRanks[i] = 0;
	}
	int adNum = 0;


	mad.ReplaceLeftAd( jobAd );
	while( machineAds.Next( machAd ) ) {
		mad.RemoveRightAd( );
		mad.ReplaceRightAd( machAd );
		if( !mad.EvaluateAttrInt( "leftRankValue", jobRanks[adNum] ) ) {
			jobRanks[adNum] = -1;
		}
		adNum++;
	}
	mad.RemoveLeftAd( );
	mad.RemoveRightAd( );

// 	cout << endl;
// 	cout << "numContexts = " << numContexts << endl;
// 	printf( "%-10s%-10s\n", "DISTANCE", "RANK" );
// 	printf( "%-10s%-10s\n", "--------", "----" );
// 	for( int i=0; i < numContexts; i++ ) {
// 		printf( "%-10.5f%-10i\n", distances[i], jobRanks[i] );
// 	}
// 	cout << endl;
		
		// debug
 	cout << "minSumDist = " << minSumDist << endl;
 	cout << "closestCtx = " << closestCtx << endl;
 	cout << endl;
 	cout << "context = " << endl;
 	Value currVal;	
 	for( int row = 0; row < numAttrs; row ++ ) {
 		vt.GetValue( closestCtx, row, currVal );
 		pp.Unparse( buffer, currVal );
 		cout << buffer << endl;
 		buffer = "";
 	}
 	cout << endl;
 	cout << "nearestValues = " << endl;
 	for( int i = 0; i < numAttrs; i++ ) {
 		if( nearestValues[i] ) {
 			pp.Unparse( buffer, *nearestValues[i] );
 			cout << buffer << endl;
 			buffer = "";
 		} 
 		else {
 			cout << "NULL" << endl;
 		}
 	}

	Value condVal;
	condNum = 0;
  	p->Rewind( );
  	while( p->NextCondition( condition ) ) {
		bt.GetValue( closestCtx, condNum, currBVal );
		if( currBVal == TRUE_VALUE ) {
 			condition->explain.suggestion = ConditionExplain::KEEP;
  		}
  		else if( currBVal == UNDEFINED_VALUE ) {
  			condition->explain.suggestion = ConditionExplain::REMOVE;
		}
		else {
				// debug
 			condition->ToString( buffer );
 			cout << "condition " << condNum << ":" << buffer << endl;
 			buffer = "";
 			cout << "currBVal = FALSE" << endl;

			attrNum = vr4Cond[condNum];

				// debug
   			cout << "attrNum = " << attrNum << endl;

			if( attrNum < 0 ) {
				condition->explain.suggestion = ConditionExplain::REMOVE;
			}
			else {
				vt.GetValue( closestCtx, attrNum, val );
			
					// debug
				pp.Unparse( buffer, val );
				cout << "vt[" << closestCtx << "," << attrNum << "] = " << buffer
 				 << endl;
				buffer = "";

				if( nearestValues[attrNum] ) {

						// debug
					pp.Unparse( buffer, *nearestValues[attrNum] );
					cout << "nearestValues[" << attrNum << "] = " << buffer
						 << endl; 
					buffer = "";

					condition->GetVal( condVal );
					condition->explain.suggestion = ConditionExplain::MODIFY;
					if( EqualValue( condVal, *nearestValues[attrNum] ) ) {
						if( ops[condNum] == Operation::LESS_THAN_OP ) {
							IncrementValue( val );
						}
						else if ( ops[condNum] == Operation::GREATER_THAN_OP ) {
							DecrementValue( val );
						}
					}
					condition->explain.newValue.CopyFrom( val );
				}
				else {
					condition->explain.suggestion = ConditionExplain::REMOVE;
				}
			}
  		}
		
			// debug
   		cout << endl;
		
		condNum++;
 	}

	return true;
}


bool ClassAdAnalyzer::
FindConflicts( MultiProfile *mp, ResourceGroup &rg )
{
	Profile *currentProfile = NULL;
	mp->Rewind( );
	while( mp->NextProfile( currentProfile ) ) {
		if( !FindConflicts( currentProfile, rg ) ) {
			return false;
		}
	}
	return true;
}

bool ClassAdAnalyzer::
FindConflicts( Profile *p, ResourceGroup &rg )
{
	BoolTable bt;
	List< BoolVector > bvList;
	BoolVector *currBV = NULL;
	IndexSet *currIS = NULL;
	BoolValue bval;
	int numConds = 0;

	if( !p->GetNumberOfConditions( numConds ) ) {
		return false;
	}

	if( !BuildBoolTable( p, rg, bt ) ) {
		return false;
	}

	if( !bt.GenerateMinimalFalseBVList( bvList ) ) {
		return false;
	}

//  		// debug
//  	cout << "BoolVectors:" << endl;

		// Get info from BoolTable for Profile
	int card;
	bvList.Rewind( );
	while( bvList.Next( currBV ) ) {
		if( currBV == NULL ) {
			return false;
		}

//  			// debug
//  		string buffer;
//  		currBV->ToString( buffer );
//  		cout << buffer << endl;

		currIS = new IndexSet( );
		currIS->Init( numConds );
		for( int i = 0; i < numConds; i++ ) {
			currBV->GetValue( i, bval );
			if( bval == TRUE_VALUE ) {
				currIS->AddIndex( i );
			}
		}
		currIS->GetCardinality( card );
		if( card > 1 ) {
			p->explain.conflicts->Append( currIS );
		}
	}

	return true;
}

bool ClassAdAnalyzer::
AnalyzeAttributes( ClassAd *ad, ResourceGroup &rg, ClassAdExplain &caExplain )
{
	ExprTree *reqExpr, *flatReqExpr, *prunedReqExpr;
	Value val;

  	string buffer;
  	PrettyPrint pp;

	List<ClassAd> offerList;
	List<MultiProfile> reqList;
	ClassAd *offer;
	MultiProfile *currReq;
	List<AnnotatedBoolVector> abvList;
	BoolTable bt;
	ValueRangeTable vrt;

//  		// debug
//  	pp.Unparse( buffer, ad );
//  	cout << "request ad:" << endl
//  		 << "-----------" << endl
//  		 << buffer << endl;
//  	cout << endl;
//  	buffer = "";
	
		/////////////////////////////////////
		// STEP 1 - CREATE LIST OF OFFER   //
		// REQUIREMENTS FROM RESOURCEGROUP //                   
		/////////////////////////////////////

		// get list of ClassAds from ResourceGroup
	if( !( rg.GetClassAds( offerList ) ) ) {
		cerr << "CA::AA: error with GetClassAds" << endl << endl;
		return false;
	}

  	int numOffers = offerList.Number( );

//  		// debug
//  	cout << "numOffers = " << numOffers << endl;


		// Only do this if SuggestCondition has been run successfully
	if( jobReq ) {
		if( jobReq->explain.numberOfClassAds == numOffers ) { 
			offerList.Rewind( );
			for( int i = 0; i < numOffers; i++ ) {
				offerList.Next( );
				if( !jobReq->explain.matchedClassAds.HasIndex( i ) ) {
					offerList.DeleteCurrent( );
				}
			}
			if( offerList.Number( ) == 0 ) {
				return true;
			}
		}
//  		else {
//  				// debug
//  			cout << "jobReq->explain.numberOfClassAds = " 
//  				 << jobReq->explain.numberOfClassAds << endl;
//  		}
	}	
				
		// create MultiProfile object for each offer Requirements expression;
	offerList.Rewind( );
	while( offerList.Next( offer ) ) {

//  			// debug
//  		cout << endl;
//  		pp.Unparse( buffer, offer );
//  		cout << "offer = " << buffer << endl;
//  		buffer = "";

		currReq = new MultiProfile( );

		if( !( reqExpr = offer->Lookup( ATTR_REQUIREMENTS ) ) ) {
			cerr << "error looking up requirements" << endl << endl;
			return false;
		}
		
  			// debug
//  		pp.Unparse( buffer, reqExpr );
//  		cout << "reqExpr = " << buffer << endl;
//  		buffer = "";

		if( !( offer->FlattenAndInline( reqExpr, val, flatReqExpr ) ) ) {
			cerr << "error flattening request" << endl << endl;
			return false;
		}

//		pp.Unparse( buffer, flatReqExpr );
//		cout << "flatReqExpr = " << buffer << endl;
//		buffer = "";

		if( flatReqExpr ) {
				// we have a non-literal boolean expression 

/*
			if( flatReqExpr->GetKind( ) == ExprTree::ATTRREF_NODE ) {
					// This probably means the REQUIREMENTS expression is an
					// attributereference that eval's to undefined.  So we need
					// to look it up inthe ClassAd to see if it really is
					// undefined or contains an 'is' or'isnt' operator.
					// We assume there is only one level of indirection here
					// and theattribute reference is a simple one (i.e. a 
					// local attribute)
				ExprTree *junk;
				string attr;
				bool abs;
				( ( AttributeReference * )flatReqExpr )->GetComponents( junk,
																		attr,
																		abs );

//				cout << "Attribute is " << attr << endl;
																		
				if( !( reqExpr = offer->Lookup( attr ) ) ) {
					cerr << "error looking up " + attr + "expression" << endl;
					return false;
				}
				if( !( offer->Flatten( reqExpr, val, flatReqExpr ) ) ) {
					cerr << "error flattening request" << endl << endl;
					return false;
				}
				if( !flatReqExpr ) {
						// we have a literal boolean expression

//  						// debug
//  					pp.Unparse( buffer, val );
//  					cout << "val = " << buffer << endl;
//  					buffer = "";

					currReq = new MultiProfile( );
					if( !( BoolExpr::ValToMultiProfile( val, currReq ) ) ) {
						cerr << "error in ValToMultiProfile" << endl << endl;
						return false;
					}
					reqList.Append( currReq );
				}
					// we now should have a proper non-literal flattend
					// expression.
			}
*/	
			

//  				// debug
//  			pp.Unparse( buffer, flatReqExpr );
//  			cout << "flatReqEpxr = " << buffer << endl;
//  			buffer = "";

			if( !( PruneDisjunction( flatReqExpr, prunedReqExpr ) ) ) {
				cerr << "error pruning expression:" << endl;
				pp.Unparse( buffer, flatReqExpr );
				cerr << buffer << endl << endl;
				buffer = "";
				return false;
			}
			
//  				// debug
//  			pp.Unparse( buffer, prunedReqExpr );
//  			cout << "prunedReqEpxr = " << buffer << endl;
//  			buffer = "";

			currReq = new MultiProfile( );
			if( !( BoolExpr::ExprToMultiProfile( prunedReqExpr, currReq ) ) ) {
				cerr << "error in ExprToMultiProfile" << endl << endl;
				return false;
			}

				// Add MultiProfile to the list
			reqList.Append( currReq );
//			delete prunedReqExpr;
		}
		else {
				// we have a literal boolean expression

//  				// debug
//  			pp.Unparse( buffer, val );
//  			cout << "val = " << buffer << endl;
//  			buffer = "";

			currReq = new MultiProfile( );
			if( !( BoolExpr::ValToMultiProfile( val, currReq ) ) ) {
				cerr << "error in ValToMultiProfile" << endl << endl;
				return false;
			}
			reqList.Append( currReq );
		}			
		delete reqExpr;
		delete flatReqExpr;
	}

		////////////////////////////////////
		// STEP 2 - BUILD DATA STRUCTURES //
		////////////////////////////////////

	int numReqs = reqList.Number( );

//  		// debug
//  	cout << "numReqs = " << numReqs << endl;
//  	cout << "reqList = " << endl;
//  	reqList.Rewind( );
//  	while( reqList.Next( currReq ) ) {
//  		currReq->ToString( buffer );
//  		cout << buffer << endl;
//  		buffer = "";
//  	}
//  	cout << endl;

	int firstContext[numReqs]; 
	List< string > jobAttrs, undefAttrs, refdAttrs;
	ClassAd::iterator itr;

		// build list of attributes
	for( itr = ad->begin( ); itr != ad->end( ); itr++ ) {
		jobAttrs.Append( new string( itr->first ) );
	}

	int numRefdAttrs = 0;

//		// debug
//	cout << "numRefdAttrs = " << numRefdAttrs << endl;

	int arrayCount = 0;
	int reqNo = 0;
	List< ExtArray< BoolValue > > boolValueArrayList;
	List< ExtArray< ValueRange * > > intervalArrayList;
	ExtArray< BoolValue > literalValues;
	ExtArray< BoolValue > *tempBools;
	ExtArray< ValueRange * > *tempIntervals;
	Profile *currProfile;
	Condition *currCondition;

		// iterate through machine requirements statements
	reqList.Rewind( );
	while( reqList.Next( currReq ) ) {
		firstContext[reqNo] = arrayCount;
		literalValues.resize( arrayCount + 1);
		literalValues[arrayCount] = TRUE_VALUE;
			// None of the attribute values contribute to this expression
		if( currReq->IsLiteral( ) ) {
			BoolValue bVal = FALSE_VALUE;
			currReq->GetLiteralValue( bVal );
			literalValues[arrayCount] = bVal;
			if( numRefdAttrs == 0 ) {
				tempBools = NULL;
				tempIntervals = NULL;
			}
			else {
				tempBools = new ExtArray< BoolValue>( numRefdAttrs );
				tempIntervals = new ExtArray< ValueRange * >(numRefdAttrs);
				for( int i = 0; i < numRefdAttrs; i++ ) {
					( *tempBools )[i] = bVal;
					( *tempIntervals )[i] = NULL;
				}
			}
			boolValueArrayList.Append( tempBools );
			intervalArrayList.Append( tempIntervals );
			arrayCount++;
			reqNo++;
			continue;
		}
			

		    // iterate through Profiles in a machine requirements statement
		currReq->Rewind( );
		while( currReq->NextProfile( currProfile ) ) {

//    				//debug
//    			currProfile->ToString( buffer );
//    			cout << "Profile " << arrayCount << ": " << buffer << endl;
//    			buffer = "";
			
			if( numRefdAttrs == 0 ) {
				tempBools = NULL;
				tempIntervals = NULL;
			}
			else {
				tempBools = new ExtArray< BoolValue>( numRefdAttrs );
				tempIntervals = new ExtArray< ValueRange * >(numRefdAttrs);
				for( int i = 0; i < numRefdAttrs; i++ ) {
					( *tempBools )[i] = TRUE_VALUE;
//					( *tempIntervals )[i] = new ValueRange( );
					( *tempIntervals )[i] = NULL;
				}
			}

				// iterate through Conditions in a profile
			currProfile->Rewind( );
			while( currProfile->NextCondition( currCondition ) ) {
//    					// debug
//    				currCondition->ToString( buffer );
//    				cout << "currCondition = " << buffer << endl;
//    				buffer = "";

				if( currCondition->IsComplex( ) && 
					currCondition->HasMultipleAttrs( ) ) {
						// We need a better way to deal with complex Conditions
						// LOOK INTO THIS
					continue;
				}

				string currAttr;
				if( !currCondition->GetAttr( currAttr ) ) {
					cerr << "AA error: couldn't get attribute" << endl;
					exit(1);
				}

					// Try to find attribute in job
				bool attrInJob = false;
				string jobAttr;
				jobAttrs.Rewind( );
				while( jobAttrs.Next( jobAttr ) ) {
					if( EqualsIgnoreCase( currAttr, jobAttr ) ) {
						attrInJob = true;				
						break;
					}
				}

					// See if attribute has been previously referenced
				bool attrInRefdAttrs = false;
				string refdAttr;
				int attrNum = 0;
				refdAttrs.Rewind( );
				while( refdAttrs.Next( refdAttr ) ) { 
					if(EqualsIgnoreCase( currAttr, refdAttr ) ) {
						attrInRefdAttrs = true;
						break;
					}
					attrNum++;
				}

				BoolValue conditionValue;
				if( !currCondition->EvalInContext( mad, ad, conditionValue) ) {
					cerr << "AA error: BoolExpr::EvalInContext failed" << endl;
					exit(1);
				}

					// attribute has been previously encountered.
				if( attrInRefdAttrs ) {
					BoolValue newValue;
					BoolValue oldValue = ( *tempBools )[attrNum];
					And( oldValue, conditionValue, newValue );
					( *tempBools )[attrNum] = newValue;

//  						// debug
//						char bval_char;
//  					cout << "attrNum: " << attrNum << " ";
//  					GetChar( oldValue, bval_char );
//  					cout << "old: " << bval_char << " ";
//  					GetChar( conditionValue, bval_char );
//  					cout << "cond: " << bval_char << " ";
//  					GetChar( newValue, bval_char );
//  					cout << "new: " << bval_char << endl;
					


//  						// debug
//  					cout << "Calling AddConstraint (1)" << endl;

					if( ( ( *tempIntervals )[attrNum] == NULL ) ) {
						( *tempIntervals )[attrNum] = new ValueRange( );
					}
					AddConstraint( ( *tempIntervals )[attrNum],
								   currCondition );
//  						// debug
//  					cout << "Called AddConstraint (1)" << endl;
					
				}
					// attribute is undefined and has thus far not been in any
					// machine requirements.
				else {
					refdAttrs.Append( new string( currAttr ) );
					numRefdAttrs++;				   
					if( tempBools == NULL ) {
						tempBools = new ExtArray< BoolValue>( numRefdAttrs );
						tempIntervals = new ExtArray< ValueRange * >( numRefdAttrs );
						for( int i = 0; i < numRefdAttrs; i++ ) {
							( *tempBools )[i] = TRUE_VALUE;
							( *tempIntervals )[i] = NULL;
						}
					}
					else {
						tempBools->resize( numRefdAttrs );
						tempIntervals->resize( numRefdAttrs );
						(*tempIntervals)[numRefdAttrs - 1] = NULL;
					}
					( *tempBools )[numRefdAttrs - 1] = conditionValue;
//  						// debug
//  					cout << "Calling AddConstraint (2)" << endl;

					if( ( ( *tempIntervals )[attrNum] == NULL ) ) {
						( *tempIntervals )[attrNum] = new ValueRange( );
					}
					AddConstraint( ( *tempIntervals )[attrNum], 
								   currCondition );
//  						// debug
//  					cout << "Called AddConstraint (2)" << endl;
				}
					// attribute is not defined in job
				if( !attrInJob ) {
					if( undefAttrs.IsEmpty( ) ) {
						undefAttrs.Append( new string( currAttr ) );
					}
					else {
						string undefAttr = "";
						bool foundAttr = false;
						undefAttrs.Rewind( );
						while( undefAttrs.Next( undefAttr ) ) { 
							if( EqualsIgnoreCase( currAttr, undefAttr ) ) {
								foundAttr = true;
								break;
							}
						}
						if( !foundAttr ) {
							undefAttrs.Append( new string( currAttr ) );
						}
					}
				}
			}
			boolValueArrayList.Append( tempBools );
			intervalArrayList.Append( tempIntervals );
			arrayCount++;
		}
		reqNo++;
	}

//  		// debug
//  	cout << "Attributes:" << endl
//  		 << "-----------" << endl;
//  	refdAttrs.Rewind( );
//  	while( refdAttrs.Next( buffer ) ) {
//  		cout << buffer << endl;
//  		buffer = "";
//  	}
//  	cout << endl;

//  		// debug
//  	cout << "Undefined Attributes:" << endl
//  		 << "---------------------" << endl;
//  	undefAttrs.Rewind( );
//  	while( undefAttrs.Next( buffer ) ) {
//  		cout << buffer << endl;
//  		buffer = "";
//  	}
//  	cout << endl;


		// Convert machine map
	int numContexts = boolValueArrayList.Number();

//  		// debug
//  	cout << "numContexts = " << numContexts << endl;

//    		// debug
//   	cout << "machineForContext = [";

	int machineForContext[numContexts];
 	int m = 0;
  	for( int i = 0; i < numContexts; i++ ) {
  	    if( ( m+1 ) < numReqs) {
  			if( i >= firstContext[m+1] ) {
  				m++;
  			}
  	    }
  	    machineForContext[i] = m;

//  			// debug
//    		cout << m << " ";

  	}

//  		// debug
//    	cout << "]" << endl;

		// Create BoolTable and ValueRangeTable
	bt.Init( numContexts, numRefdAttrs );
	vrt.Init( numContexts, numRefdAttrs );
	ExtArray< BoolValue > *currBVArray;
	ExtArray< ValueRange * > *currVRLArray;
	boolValueArrayList.Rewind( );
	intervalArrayList.Rewind( );
	for( int ctxtNum = 0; ctxtNum < numContexts; ctxtNum++ ) {
		boolValueArrayList.Next( currBVArray );
		intervalArrayList.Next( currVRLArray );
		int arraySize = 0;
		if( currBVArray != NULL ) {
			arraySize = currBVArray->getsize( );
		}
		for( int attrNum = 0; attrNum < numRefdAttrs; attrNum++ ) {
			if( attrNum >= arraySize ) {
				bt.SetValue( ctxtNum, attrNum, literalValues[ctxtNum] );
				vrt.SetValueRange( ctxtNum, attrNum, NULL ); 
			}
			else {
				bt.SetValue( ctxtNum, attrNum, (*currBVArray)[ attrNum ] );
				vrt.SetValueRange( ctxtNum, attrNum,
									  ( *currVRLArray )[ attrNum ] );
			}
		}
	}

//    		// debug
//    	bt.ToString( buffer );
//  	cout << endl;
//    	cout << "BoolTable:" << endl;
//    	cout << buffer << endl;
//    	buffer = "";

//  		// debug
//  	vrt.ToString( buffer );
//    	cout << "ValueRangeTable:" << endl;
//    	cout << buffer << endl;
//    	buffer = "";

//  		// debug
//  	ValueRange *vr = NULL;
//  	for( int i = 0; i < numRefdAttrs; i++ ) {
//  		for( int j = 0; j < numContexts; j++ ) {
//  			vrt.GetValueRange( j, i, vr );
//  			if( vr == NULL ) {
//  				buffer = "NULL";
//  			}
//  			else {
//  				vr->ToString( buffer );
//  			}
//  			cout << "vrt[" << j << "," << i << "] = " << buffer << endl;
//  			buffer = "";
//  		}
//  	}
//  	cout << endl;

  	if( !bt.GenerateMaxTrueABVList( abvList ) ) {
		cout << "CA::AA: error in GenerateMaxTrueABVList" << endl;
  		return false;
  	}

//  		// debug
//  	AnnotatedBoolVector *currABV;
//  	cout << "AbvList:" << endl;
//  	abvList.Rewind( );
//  	while( abvList.Next( currABV ) ) {
//  		currABV->ToString( buffer );
//  		cout << buffer;
//  		buffer = "";
//  	}
//  	cout << endl << endl;

	///////////////////////////////////////
	// STEP 3 - GENERATE HYPERRECTANGLES //
	///////////////////////////////////////
	// find values corresponding to each ABV

	AnnotatedBoolVector *currentABV = NULL;
	List< ExtArray< HyperRect * > > allHyperRectangles;
	abvList.Rewind( );
	while( abvList.Next( currentABV ) ) {

//  			// debug
//  		currentABV->ToString( buffer );
//  		cout << buffer << endl;
//  		buffer = "";

	    ExtArray< ValueRange * > vrForAttr;
		vrForAttr.resize( numRefdAttrs );
		for( int i = 0; i < numRefdAttrs; i++ ) {
			vrForAttr[i] = NULL;
		}

	    // go through each attribute/boolean value (only look at non-true)
	    for( int i = 0; i < numRefdAttrs; i++ ) {
			ValueRange *mergedVR = new ValueRange( );
			
			BoolValue bval;
			currentABV->GetValue( i, bval ); 

			if( bval != TRUE_VALUE ) {
				ValueRange *vr = NULL;
				int context;
				bool firstContext = true;

					// - go through all contexts corresponding to current ABV
					// - merge the ValueRanges into one multiply indexed 
					//   ValueRange
				for( int j = 0; j < numContexts; j++ ) {
					bool hasContext = false;
					currentABV->HasContext( j, hasContext );
					if( hasContext ) {
						vrt.GetValueRange( j, i, vr );

//  							// debug
//  						vr->ToString( buffer );
//  						cout << "vrt[" << j << "," << i << "] = " << buffer
//  							 << endl;
//  						buffer = "";

						context = j;
						if( firstContext ) {
							if( vr != NULL ) {

//  								// debug
//  								vr->ToString( buffer );
//  								cout << "vr = " << buffer << endl;
//  								buffer = "";
//  								cout << "context = " << context << endl;
//  								cout << "numContexts = " << numContexts
//  									 << endl;

								mergedVR->Init( vr, context, numContexts );
								
//  								// debug
//  								mergedVR->ToString( buffer );
//  								cout << "mergedVR = " << buffer << endl;
//  								cout << endl;
//  								buffer = "";

								firstContext = false;
							}
						}
						else if( vr != NULL ) {							
							mergedVR->Union( vr, context );
						}

//							// debug
//						mergedVR.ToString( buffer );
//						cout << buffer;
//						buffer = "";

					}
				}
				vrForAttr[i] = mergedVR;
		    }
			else {
				vrForAttr[i] = NULL;
			}

//  				// debug
//  			if( vrForAttr[i] == NULL ) {
//  				buffer = "NULL";
//  			}
//  			else {
//  				vrForAttr[i]->ToString( buffer );
//  			}
//  			cout << "ValueRange " << i << ":  " << buffer << endl;
//  			buffer = "";
		}

//  			// debug
//  		cout << "numRefdAttrs = " << numRefdAttrs << endl;
//  		cout << "numContexts = " << numContexts << endl;
//  		cout << endl;

				// sticks all HyperRectangles together
				// may want to keep them partitioned by BoolVector

		ValueRange::BuildHyperRects( vrForAttr, numRefdAttrs,
									numContexts, allHyperRectangles );
	}
	
  	ExtArray< HyperRect * > *hrs = NULL;
  	HyperRect *currHR = NULL;

//  		// debug
//  	cout << "allHyperRectangles:" << endl;
//  	cout << "-------------------" << endl;
//  	AnnotatedBoolVector *currABV = NULL;
//  	abvList.Rewind( );
//  	allHyperRectangles.Rewind( );
//  	while( allHyperRectangles.Next( hrs ) ) {
//  		abvList.Next( currABV );
//  		currABV->ToString( buffer );
//  		cout << "currABV: " << buffer << endl;
//  		buffer = "";
//  		if( hrs == NULL ) {
//  			cout << "List is NULL" << endl;
//  		}
//  		else {
//  			for( int i = 0; i < hrs->getsize( ); i++ ) {
//  				currHR = ( *hrs )[i];
//  				if( currHR == NULL ) {
//  					buffer = "NULL";
//  				}
//  				else {
//  					currHR->ToString( buffer );
//  				}
//  				cout << buffer << endl;
//  				buffer = "";
//  			}
//  			cout << endl;
//  		}
//  	}

	// find HyperRect with most contexts (machines)
	int maxNumContexts = -1;
	int currNumContexts = 0;
	currentABV = NULL;
	hrs = NULL;
	currHR = NULL;
	AnnotatedBoolVector *bestABV = NULL;
	HyperRect *bestHR = NULL;
	IndexSet hasContext;
	IndexSet hasMachine;
	hasContext.Init( numContexts );
	hasMachine.Init( numReqs );
	abvList.Rewind( );
	allHyperRectangles.Rewind( );
	while( allHyperRectangles.Next( hrs ) ) {
		abvList.Next( currentABV );
		for( int i = 0; i < hrs->getsize( ); i++ ) {
			currHR = ( *hrs )[i];
			currHR->GetIndexSet( hasContext );
			IndexSet::Translate( hasContext, machineForContext, numContexts,
								 numReqs, hasMachine );
			hasMachine.GetCardinality( currNumContexts );
			if( currNumContexts > maxNumContexts ) {
				maxNumContexts = currNumContexts;
				bestABV = currentABV;
				bestHR = currHR;
			}
		}
	}

//  		// debug
//  	cout << "Best HyperRectangle:" << endl;
//  	cout << "--------------------" << endl;
//  	if( bestABV == NULL ) {
//  		buffer = "NULL";
//  	}
//  	else {
//  		bestABV->ToString( buffer );
//  	}
//  	cout << "abv: " << buffer << endl;
//  	buffer = "";
//  	if( bestHR == NULL ) {
//  		buffer = "NULL";
//  	}
//  	else {
//  		bestHR->ToString( buffer );
//  	}
//  	cout << buffer << endl;
//  	cout << endl;
//  	buffer = "";

	///////////////////////////////////////
	// STEP 4 - SET UP ATTRIBUTE EXPLAIN //
	///////////////////////////////////////

	List< AttributeExplain> attrExplains;
	AttributeExplain *currAttrExplain;
	string refdAttr = "";
	Interval *ival = NULL;
	refdAttrs.Rewind( );
	int i = 0;
	while( refdAttrs.Next( refdAttr ) ) { 
		bestHR->GetInterval( i, ival );
		currAttrExplain = new AttributeExplain( );
		if( ival == NULL ) {
			currAttrExplain->Init( refdAttr );
		}
		else {
			switch( GetValueType( ival ) ) {
			case Value::BOOLEAN_VALUE:
			case Value::STRING_VALUE: {
				currAttrExplain->Init( refdAttr, ival->lower );
				break;
			}
			case Value::INTEGER_VALUE:
			case Value::REAL_VALUE:
			case Value::RELATIVE_TIME_VALUE:
			case Value::ABSOLUTE_TIME_VALUE: {
				double low = 0;
				double high = 0;
				GetLowDoubleValue( ival, low );
				GetHighDoubleValue( ival, high );
				if( low == high && !ival->openLower && !ival->openUpper ) {
					currAttrExplain->Init( refdAttr, ival->lower );
				}
				else {
					currAttrExplain->Init( refdAttr, ival );
				}
				break;
			}
			default: {
				currAttrExplain->Init( refdAttr, ival->lower );
				break;
			}
			}
		}
		attrExplains.Append( currAttrExplain );
		i++;
	}

	if( !caExplain.Init( undefAttrs, attrExplains ) ) {
		cout << "error with ClassAdExplain::Init" << endl;
		return false;
	}

//  		// debug
//  	ClassAdParser parser;
//  	ClassAd *explainAd;
//  	caExplain.ToString( buffer );
//  	if( !( explainAd = parser.ParseClassAd( buffer ) ) ) {
//  		cout << "parse error" << endl;
//  		return true;
//  	}
//  	buffer = "";
//  	pp.Unparse( buffer, explainAd );
//  	cout << "explainAd:" << endl;
//  	cout << "----------" << endl;
//  	cout << buffer << endl;
//  	buffer = "";

	return true;
}

bool ClassAdAnalyzer::
EqualsIgnoreCase( const string &s1, const string &s2 )
{
	return( strcasecmp( s1.c_str( ), s2.c_str( ) ) == 0 );
}

bool ClassAdAnalyzer::
DefinedLiteralValue( Value &val )
{
	return ( val.IsStringValue( ) ||
			 val.IsBooleanValue( ) ||
			 val.IsNumber( ) ||
			 val.IsAbsoluteTimeValue( ) ||
			 val.IsRelativeTimeValue( ) );
}

bool ClassAdAnalyzer::
AddConstraint( ValueRange *&vr, Condition *condition )
{
	PrettyPrint pp;
	string buffer;
	if( condition == NULL ) {
		cerr << "Error: passed NULL Condition pointer to AddConstraint"
			 << endl;
		return false;
	}

	if( vr == NULL ) {
		cerr << "Error: passed NULL ValueRange pointer to AddConstraint"
			 << endl;
		return false;
	}

	if( condition->IsComplex( ) && condition->HasMultipleAttrs( ) ) {
		cerr << "AddConstraint: can't process complex Condition:" << endl;
		condition->ToString( buffer );
		cerr << buffer << endl;
		return false;
	}

	Operation::OpKind op;
	Value val;
	bool undef = false;

	if( condition->IsComplex( ) && !condition->HasMultipleAttrs( ) ) {
		Operation::OpKind op1, op2;
		condition->GetOp( op1 );
		condition->GetOp2( op2 );
		Value val1, val2;
		condition->GetVal( val1 );
		condition->GetVal2( val2 );

		undef = false;
		if( val1.IsUndefinedValue( ) &&
			DefinedLiteralValue( val2 ) ) {
			undef = true;
			val.CopyFrom( val2 );
			op = op2;
		}
		else if( val2.IsUndefinedValue( ) &&
			DefinedLiteralValue( val1 ) ) {
			undef = true;
			val.CopyFrom( val1 );
			op = op1;
		}
		else {
//			Value::ValueType type1 = val1.GetType( );
//			Value::ValueType type2 = val2.GetType( );
//			if( !SameType( type1, type2 ) ||
//				!DefinedLiteralValue( val1 ) ||
//				val1.IsStringValue ||
//				val1.IsBooleanValue ) {
			cerr << "AddConstraint: can't process complex Condition"<<endl;
			if( DefinedLiteralValue( val1 ) ) {
				cerr << "val1 is DLV" << endl;
			}
			else {
				pp.Unparse( buffer, val1 );
				cerr << "val1 is " << buffer << endl;
				buffer = "";
			}
			if( val2.IsUndefinedValue( ) ) {
				cerr << "val2 is undef" << endl;
			}
			condition->ToString( buffer );
			cerr << buffer << endl;
			return false;
//			}
		}
	}

	if( !condition->IsComplex( ) ) {
		condition->GetOp( op );
		condition->GetVal( val );
	}

	Value::ValueType type = val.GetType( );
	if( op == Operation::NOT_EQUAL_OP || op == Operation::ISNT_OP ) {
			// op is '!=' or 'isnt' so we need multiple intervals
		switch( type ) {
		case Value::BOOLEAN_VALUE: {
			bool b;
			if( val.IsBooleanValue( b ) ) {
				Interval *i = new Interval( );
				i->lower.SetBooleanValue( !b );
				if( vr->IsInitialized( ) ) {
					if( op == Operation::ISNT_OP ) {
						vr->Intersect( i, true );
					}
					else {
						vr->Intersect( i, undef );
					}
				}
				else {
					if( op == Operation::ISNT_OP ) {
						vr->Init( i, true );
					}
					else {
						vr->Init( i, undef );
					}
				}
				return true;
			}
			else {
				cerr << "AddConstraint: error: boolean value expected"
					 << endl;
				return false;
			}
		}
		case Value::STRING_VALUE: {
			Interval *i = new Interval( );
			i->lower.CopyFrom( val );
			if( vr->IsInitialized( ) ) {
				if( op == Operation::ISNT_OP ) {
					vr->Intersect( i, true, true );
				}
				else {
					vr->Intersect( i, undef, true );
				}
			}
			else {
				if( op == Operation::ISNT_OP ) {
					vr->Init( i, true, true );
				}
				else {
					vr->Init( i, undef, true );
				}
			}
			return true;
		}
		case Value::INTEGER_VALUE:
		case Value::REAL_VALUE:
		case Value::RELATIVE_TIME_VALUE:
		case Value::ABSOLUTE_TIME_VALUE: {
			Interval *i1 = new Interval( );
			Interval *i2 = new Interval( );
			i1->lower.SetRealValue( -( FLT_MAX ) );
			i1->upper.CopyFrom( val );
			i1->openLower = false;
			i1->openUpper = false;
			i2->lower.CopyFrom( val );
			i2->upper.SetRealValue( FLT_MAX );
			i2->openLower = false;
			i2->openUpper = false;
			if( vr->IsInitialized( ) ) {
				if( op == Operation::ISNT_OP ) {
					vr->Intersect( i1, i2, true );
				}
				else {
					vr->Intersect( i1, i2, undef );
				}
			}
			else {
				if( op == Operation::ISNT_OP ) {
					vr->Init( i1, i2, true );
				}
				else {
					vr->Init( i1, i2, undef );
				}
			}
		}
		return true;
		default: {
			cerr << "AddConstraint: Condition value not literal" << endl;
			return false;
		}
		}
	}
	else {
			// op is not '!=' or 'isnt' so we only need one interval
		Interval *i = new Interval( );
		switch( type ) {
		case Value::UNDEFINED_VALUE:
			if( op != Operation::IS_OP ) {
				vr->EmptyOut( );
				return true;
			}
			if( vr->IsInitialized( ) ) {
				vr->IntersectUndef( );
			}
			else {
				vr->InitUndef( );
			}
			return true;
		case Value::BOOLEAN_VALUE:
		case Value::STRING_VALUE: {
			if( op != Operation::EQUAL_OP && op != Operation::IS_OP ) {
					// should be type error
				vr->EmptyOut( );
				return true;
			}
			i->lower.CopyFrom( val );
			if( vr->IsInitialized( ) ) {
				vr->Intersect( i, undef );
			}
			else {
				vr->Init( i, undef );
			}
			return true;
		}
		case Value::INTEGER_VALUE:
		case Value::REAL_VALUE:
		case Value::RELATIVE_TIME_VALUE:
		case Value::ABSOLUTE_TIME_VALUE: {
			switch( op ) {
			case Operation::LESS_THAN_OP: 
				{
					i->lower.SetRealValue( -( FLT_MAX ) );
					i->upper.CopyFrom( val );
					i->openLower = true;
					i->openUpper = true;
					if( vr->IsInitialized( ) ) {
						vr->Intersect( i, undef );
					}
					else {
						vr->Init( i, undef );
					}
					return true;
				}
			case Operation::LESS_OR_EQUAL_OP:
				{
					i->lower.SetRealValue( -( FLT_MAX ) );
					i->upper.CopyFrom( val );
					i->openLower = true;
					i->openUpper = false;
					if( vr->IsInitialized( ) ) {
						vr->Intersect( i, undef );
					}
					else {
						vr->Init( i, undef );
					}
					return true;
				}
			case Operation::EQUAL_OP:
			case Operation::IS_OP:
				{
					i->lower.CopyFrom( val );
					i->upper.CopyFrom( val );
					i->openLower = false;
					i->openUpper = false;
					if( vr->IsInitialized( ) ) {
						vr->Intersect( i, undef );
					}
					else {
						vr->Init( i, undef );
					}
					return true;
				}
			case Operation::GREATER_OR_EQUAL_OP:
				{
					i->lower.CopyFrom( val );
					i->upper.SetRealValue( FLT_MAX );
					i->openLower = false;
					i->openUpper = true;
					if( vr->IsInitialized( ) ) {
						vr->Intersect( i, undef );
					}
					else {
						vr->Init( i, undef );
					}
					return true;
				}
			case Operation::GREATER_THAN_OP:
				{
					i->lower.CopyFrom( val );
					i->upper.SetRealValue( FLT_MAX );
					i->openLower = true;
					i->openUpper = true;
					if( vr->IsInitialized( ) ) {
						vr->Intersect( i, undef );
					}
					else {
						vr->Init( i, undef );
					}
					return true;
				}
			default:
				{
						// should be type error
					if( vr->IsInitialized( ) ) {						
						vr->EmptyOut( );	
					}
					return true;
				}
			}
		}
		default:
			{
					// should be type error
				if( vr->IsInitialized( ) ) {						
					vr->EmptyOut( );
				}
				return true;
			}
		}
	}
	
	return true;
}

bool ClassAdAnalyzer::
PruneDisjunction( ExprTree *expr, ExprTree *&result )
{
	if( !expr ) {
		cerr << "PD error: null expr" << endl; 
		return false;
	}

	ExprTree::NodeKind kind;
	Operation::OpKind op;
	ExprTree *left, *right, *junk;
	ExprTree *newLeft = NULL;
	ExprTree *newRight = NULL;
	Value val;
	bool boolValue;

	ExprTree *currentTree = expr;

	kind = currentTree->GetKind( );
	if( kind != ExprTree::OP_NODE ) {
		return PruneAtom( currentTree, result );
	}

	( ( Operation * )currentTree )->GetComponents( op, left, right, junk );
	
	if( op == Operation::PARENTHESES_OP ) {
		if( !PruneDisjunction( left, result ) ) {
			return false;
		}
		else if( !( result=Operation::MakeOperation( Operation::PARENTHESES_OP,
													 result, NULL, NULL ) ) ) {
			cerr << "PD error: can't make Operation" << endl;
			return false;
		}
		else {
			return true;
		}
	}

	if( op != Operation::LOGICAL_OR_OP ) {
		return PruneConjunction( currentTree, result );
	}

	kind = left->GetKind( );
	if( kind == ExprTree::LITERAL_NODE ) {
		( ( Literal * )left )->GetValue( val );
		if( val.IsBooleanValue( boolValue ) && boolValue == false ) {
			return PruneDisjunction( right, result );
		}
	}

	if( !PruneDisjunction( left, newLeft ) ||
		!PruneConjunction( right, newRight ) ||
		!newLeft || !newRight ||
		!( result = Operation::MakeOperation( Operation::LOGICAL_OR_OP,
											  newLeft, newRight, NULL ) ) ) {
		cerr << "PD error: can't make Operation" << endl;
		return false;
	}
	return true;
}

bool ClassAdAnalyzer::
PruneConjunction( ExprTree *expr, ExprTree *&result ) {
	if( !expr ) {
		cerr << "PC error: null expr" << endl; 
		return false;
	}

	ExprTree::NodeKind kind;
	Operation::OpKind op;
	ExprTree *left, *right, *junk;
	ExprTree *newRight = NULL;
	ExprTree *newLeft = NULL;
	Value val;
	bool boolValue;

	ExprTree *currentTree = expr;

	kind = currentTree->GetKind( );
	if( kind != ExprTree::OP_NODE ) {		
		return PruneAtom( currentTree, result );
	}
		
	( ( Operation * )currentTree )->GetComponents( op, left, right, junk );

	if( op == Operation::PARENTHESES_OP ) {
		if( !PruneConjunction( left, result ) ) {
			return false;
		}
		else if( !( result=Operation::MakeOperation( Operation::PARENTHESES_OP,
													 result, NULL, NULL ) ) ) {
			cerr << "PC error: can't make Operation" << endl;
			return false;
		}
		else {
			return true;
		}
	}

	if( op != Operation::LOGICAL_AND_OP && op != Operation::LOGICAL_OR_OP ) {
		return PruneAtom( currentTree, result );
	}

	if( op == Operation::LOGICAL_OR_OP ) {
		return PruneDisjunction( currentTree, result );
	}

	kind = left->GetKind( );
	if( kind == ExprTree::LITERAL_NODE ) {
		( ( Literal * )left )->GetValue( val );
		if( val.IsBooleanValue( boolValue ) && boolValue == true ) {
			return PruneConjunction( right, result );
		}
	}

	if( !PruneConjunction( left, newLeft ) ||
		!PruneDisjunction( right, newRight ) ||
		!newLeft || !newRight  ||
		!( result = Operation::MakeOperation( Operation::LOGICAL_AND_OP,
											  newLeft, newRight, NULL ) ) ) {
		cerr << "PC error: can't Make Operation" << endl;
		return false;
	}
	return true;
}

bool ClassAdAnalyzer::
PruneAtom( ExprTree *expr, ExprTree *&result )
{
	if( !expr ) {
		cerr << "PA error: null expr" << endl; 
		return false;
	}

	ExprTree::NodeKind kind;
	Operation::OpKind op;
	ExprTree *left, *right, *junk;
	Value val;
	bool boolValue;
	string attr;

	kind = expr->GetKind( );

	if( kind != ExprTree::OP_NODE ) {
		result = expr->Copy( );
		return true;
	}

	( ( Operation * )expr )->GetComponents( op, left, right, junk );

	if( op == Operation::PARENTHESES_OP ) {
		if( !PruneAtom( left, result ) ) {
			cerr << "PA error: problem with expression in parens" << endl;
			return false;
		}
		else if( !( result=Operation::MakeOperation(Operation::PARENTHESES_OP, 
													result, NULL, NULL ) ) ) {
			cerr << "PA error: can't make Operation" << endl;
			return false;
		}
		else {
			return true;
		}
	}

	if( op == Operation::LOGICAL_OR_OP &&
		left->GetKind( ) == ExprTree::LITERAL_NODE ) {
		( ( Literal *)left )->GetValue( val );
		if( val.IsBooleanValue( boolValue ) && !boolValue ) {
			return PruneAtom( right, result );
		}
	}

	if( !(op >= Operation::__COMPARISON_START__ && 
		  op <= Operation::__COMPARISON_END__) ) {
		cerr << "PA error: operator not comparison: " << (int)op << endl;
			// error: operator not comparison
		return false;
	}

	if( left == NULL || right == NULL ) {
		cerr << "PA error: NULL ptr in expr" << endl;
			// error: NULL ptr in expr
		return false;
	}

	if( !( result = Operation::MakeOperation( op, left->Copy( ), right->Copy(),
											  NULL ) ) ) {
		cerr << "PA error: can't make Operation" << endl;
		return false;
	}

	return true;
}

ExprTree *ClassAdAnalyzer::
AddExplicitTargets( ExprTree *tree, set< string, CaseIgnLTStr > &definedAttrs )
{
	if( tree == NULL ) {
		return NULL;
	}
	ExprTree::NodeKind nKind = tree->GetKind( );
	switch( nKind ) {
	case ExprTree::ATTRREF_NODE: {
		ExprTree *expr = NULL;
		string attr = "";
		bool abs = false;
		( ( AttributeReference * )tree )->GetComponents(expr,attr,abs);
		if( abs || expr != NULL ) {
			return tree->Copy( );
		}
		else {
			if( definedAttrs.find( attr ) == definedAttrs.end( ) ) {
					// attribute is not defined, so insert "target"
				AttributeReference *target = NULL;
				target = AttributeReference::MakeAttributeReference(NULL,
																	"target");
				return AttributeReference::MakeAttributeReference(target,attr);
			}
			else {
				return tree->Copy( );
			}
		}
	}
	case ExprTree::OP_NODE: {
		Operation::OpKind oKind;
		ExprTree * expr1 = NULL;
		ExprTree * expr2 = NULL;
		ExprTree * expr3 = NULL;
		ExprTree * newExpr1 = NULL;
		ExprTree * newExpr2 = NULL;
		ExprTree * newExpr3 = NULL;
		( ( Operation * )tree )->GetComponents( oKind, expr1, expr2, expr3 );
		if( expr1 != NULL ) {
			newExpr1 = AddExplicitTargets( expr1, definedAttrs );
		}
		if( expr2 != NULL ) {
			newExpr2 = AddExplicitTargets( expr2, definedAttrs );
		}
		if( expr3 != NULL ) {
			newExpr3 = AddExplicitTargets( expr3, definedAttrs );
		}
		return Operation::MakeOperation( oKind, newExpr1, newExpr2, newExpr3 );
	}
	default: {
 			// old ClassAds have no function calls, nested ClassAds or lists
			// literals have no attrrefs in them
		return tree->Copy( );
	}
	}
}

bool ClassAdAnalyzer::
InDNF( ExprTree * tree )
{
	if( tree == NULL ) {
		cerr << "InDNF: tried to pass null pointer" << endl;
		return false;
	}

	if( tree->GetKind( ) != ExprTree::OP_NODE ) {
		return false;
	}
	
	Operation::OpKind op;
	ExprTree *arg1;
	ExprTree *arg2;
	ExprTree *junk;

	( ( Operation *)tree )->GetComponents( op, arg1, arg2, junk );

		// all comparisons are atomic formulas
	if( op >= Operation::__COMPARISON_START__ &&
		op <= Operation::__COMPARISON_END__ ) {
		return true;
	}

	if( op == Operation::PARENTHESES_OP ) {
		return InDNF( arg1 );
	}

		// operators must be comparison or logical
	if( op < Operation::__LOGIC_START__ ||
		op > Operation::__LOGIC_END__ ) {
		return false;
	}

		// FINISH THIS CODE

	return true;
}

bool ClassAdAnalyzer::
IsAtomicBooleanFormula( Operation *tree )
{
	if( tree == NULL ) {
		cerr << "IsAtomicBooleanFormula: tried to pass null pointer" << endl;
		return false;
	}

		// FINISH THIS CODE

	return true;
}

bool ClassAdAnalyzer::
PropagateNegation( ExprTree *tree, ExprTree *&result )
{
	if( tree == NULL ) {
		cerr << "PropagateNegation: tried to pass null pointer" << endl;
		return false;
	}

		// FINISH THIS CODE

	return true;
}

bool ClassAdAnalyzer::
ToDNF( ExprTree *tree, ExprTree *&result )
{
	if( tree == NULL ) {
		cerr << "ToDNF: tried to pass null pointer" << endl;
		return false;
	}
		// FINISH THIS CODE

	return true;
}
