/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "boolExpr.h"
#include "list.h"
#include "Stack.h"

BoolExpr::
BoolExpr( )
{
	initialized = false;
	myTree = NULL;
}

BoolExpr::
~BoolExpr( )
{
	if( myTree ) {
		delete myTree;
	}
}

bool BoolExpr::
ExprToMultiProfile( ExprTree *expr, MultiProfile *&mp )
{
	if( expr == NULL ) {
		cerr << "error: input ExprTree is null" << endl;
			//  error: input ExprTree is null
		return false;
	}

	if( !mp->Init( expr ) ) {
		cerr << "error: problem with MultiProfile::Init" << endl;
		return false;
	}

	ExprTree::NodeKind kind;
	Operation::OpKind op;
	ExprTree *left, *right, *junk;
	Profile *currentProfile = new Profile;
	Stack<Profile> profStack;
	Value val;

	ExprTree *currentTree = expr;
	bool atLeftMostProfile = false;
	while( !atLeftMostProfile ) {

		kind = currentTree->GetKind( );
			
		if( kind == ExprTree::ATTRREF_NODE ) {
			atLeftMostProfile = true;
			continue;
		}

		if( kind != ExprTree::OP_NODE ) {
			cerr << "error: bad form" << endl;
				// error: bad form
			return false;
		}

		( ( Operation * )currentTree )->GetComponents( op, left, right, junk );

		while( op == Operation::PARENTHESES_OP ) {
			if( left->GetKind( ) == ExprTree::ATTRREF_NODE ) {
				atLeftMostProfile = true;
				break;
			}
			( ( Operation * )left )->GetComponents( op, left, right, junk );
		}

		if( atLeftMostProfile == true ) {
			continue;
		}

			// check if we have reached the leftmost profile
		if( op != Operation::LOGICAL_OR_OP ) {
			atLeftMostProfile = true;
			continue;
		}

			// Peel of the rightmost profile
		if( !ExprToProfile( right, currentProfile ) ) {
			cerr << "error: problem with ExprToProfile" << endl;
				// error: problem with ExprToProfile
			return false;
		}
		else {
			profStack.Push( currentProfile );
			currentTree = left;
		}
		currentProfile = new Profile;
	}

		// process the leftmost profile
	if( !ExprToProfile( currentTree, currentProfile ) ) {
		cerr << "error: problem with ExprToProfile" << endl;
			// error: problem with ExprToProfile
		return false;
	}

		// pull the profiles off the stack and append them to the MultiProfile
	mp->AppendProfile( currentProfile );

	while( !profStack.IsEmpty( ) ) {
		mp->AppendProfile( profStack.Pop( ) );
	}

	mp->isLiteral = false;

	return true;
}

bool BoolExpr::
ValToMultiProfile( Value &val, MultiProfile *&mp )
{
	if( !mp->InitVal( val ) ) {
		cerr << "error: problem with MultiProfile::Init" << endl;
		return false;
	}	
	return true;
}

bool BoolExpr::
ExprToProfile( ExprTree *expr, Profile *&p )
{
	if( expr == NULL ) {
		cerr << "error: input ExprTree is null" << endl;
			//  error: input ExprTree is null
		return false;
	}

		// create the Profile object
	if( !p->Init( expr ) ) {
		cerr << "error: problem with Profile::Init" << endl;
		return false;
	}
	
	ExprTree::NodeKind kind;
	Operation::OpKind op;
	ExprTree *left, *right, *junk;
	Condition *currentCondition = new Condition;
	Stack<Condition> condStack;
	Value val;
	
	ExprTree *currentTree = expr;
	bool atLeftMostCondition = false;

	while( !atLeftMostCondition ) {

		kind = currentTree->GetKind( );

		if( kind == ExprTree::ATTRREF_NODE ) {
			atLeftMostCondition = true;
			continue;
		}

		if( kind != ExprTree::OP_NODE ) {
			cerr << "error: bad form" << endl;
				// error: bad form
			return false;
		}

		( ( Operation * )currentTree )->GetComponents( op, left, right, junk );

		while( op == Operation::PARENTHESES_OP ) {
			if( left->GetKind( ) == ExprTree::ATTRREF_NODE ) {
				atLeftMostCondition = true;
				break;
			}
			( ( Operation * )left )->GetComponents( op, left, right, junk );
		}

		if( atLeftMostCondition == true ) {
			continue;
		}

			// check if we have reached the leftmost condition
		if( op != Operation::LOGICAL_AND_OP ) {
			atLeftMostCondition = true;
			continue;
		}

			// Peel of the rightMost condition
		if( !ExprToCondition( right, currentCondition ) ) {
			cerr << "error: found NULL ptr in expr" << endl;
				// error: found NULL ptr in expr
			return false;
		}
		else {
			condStack.Push( currentCondition );
			currentTree = left;
		}

		currentCondition = new Condition;
	}

		// process the leftmost condition
	if( !ExprToCondition( currentTree, currentCondition ) ) {
		cerr << "error: found NULL ptr in expr" << endl;
			// error: found NULL ptr in expr
		return false;
	}

		// pull the conditions off the stack and append them to the list
	p->AppendCondition( currentCondition );
	while( !condStack.IsEmpty( ) ) {
		p->AppendCondition( condStack.Pop( ) );
	}

	return true;
}

bool BoolExpr::
ExprToCondition( ExprTree *expr, Condition *&c )
{
	if( expr == NULL ) {
		cerr << "error: ExprToCondition given NULL ptr" << endl;
			// error: ExprToCondition given NULL ptr
		return false;
	}

	ExprTree::NodeKind kind; 
	Operation::OpKind op = Operation::__NO_OP__;
	ExprTree *left = NULL;
	ExprTree *right = NULL;
	ExprTree *base = NULL;
	ExprTree *junk = NULL;
	string attr;
	bool junkBool;
	Value val;


	kind = expr->GetKind( );

	if( kind == ExprTree::ATTRREF_NODE ) {
		( ( AttributeReference * )expr )->GetComponents( base, attr, junkBool);
		if( !c->Init( attr, expr->Copy( ), true ) ) {
  				cerr << "error: problem with Condition::Init" << endl;
  				return false;
		}
		return true;
	}

	if( kind != ExprTree::OP_NODE ) {
		cerr << "error: no operator/attribute found" << endl;
			// error: no operator/attribute  found;
		return false;
	}

	( ( Operation * )expr )->GetComponents( op, left, right, junk );

	while( op == Operation::PARENTHESES_OP ) {
		if( left->GetKind( ) == ExprTree::ATTRREF_NODE ) {
			((AttributeReference *)left)->GetComponents( base, attr, junkBool);
			if( !c->Init( attr, expr->Copy( ), true ) ) {
  				cerr << "error: problem with Condition::Init" << endl;
  				return false;
			}
			return true;
		}	
		( ( Operation * )left )->GetComponents( op, left, right, junk );
	}


		// Special case: expression is an OR of two conditions with the same
		// attribute. This is bit of a hack, but it is very precise.
	if( op == Operation::LOGICAL_OR_OP ) {
		Operation::OpKind leftOp, rightOp;
		ExprTree *left1 = NULL;
		ExprTree *right1 = NULL;
		ExprTree *left2 = NULL;
		ExprTree *right2 = NULL;
		string leftAttr, rightAttr;

		if( left != NULL && 
			right != NULL &&
			left->GetKind( ) == Operation::OP_NODE &&
			right->GetKind( ) == Operation::OP_NODE ) {
			( ( Operation * )left )->GetComponents( leftOp, left1, left2,
													junk );
			( ( Operation * )right )->GetComponents( rightOp, right1, right2,
													 junk );
			if( leftOp == Operation::PARENTHESES_OP && 
				rightOp == Operation::PARENTHESES_OP &&
				left1 != NULL &&
				right1 != NULL &&
				left1->GetKind( ) == Operation::OP_NODE &&
				right1->GetKind( ) == Operation::OP_NODE ) {
				( ( Operation * )left1 )->GetComponents( leftOp, left1, left2,
														 junk );
				( ( Operation * )right1 )->GetComponents( rightOp, right1,
														  right2, junk );
			}
		    if( left1 != NULL && 
				right1 != NULL &&
				leftOp >= Operation::__COMPARISON_START__ && 
				leftOp <= Operation::__COMPARISON_END__ &&
				rightOp >= Operation::__COMPARISON_START__ && 
				rightOp <= Operation::__COMPARISON_END__ &&
				left1->GetKind( ) == ExprTree::ATTRREF_NODE &&
				right1->GetKind( ) == ExprTree::ATTRREF_NODE &&
				left2->GetKind( ) == ExprTree::LITERAL_NODE &&
				right2->GetKind( ) == ExprTree::LITERAL_NODE ) {
				( ( AttributeReference * )left1 )->GetComponents( junk,
																  leftAttr,
																  junkBool ); 
				( ( AttributeReference * )right1 )->GetComponents( junk,
																   rightAttr, 
																   junkBool );
				if( strcasecmp( leftAttr.c_str( ),
								rightAttr.c_str( ) ) == 0 ) {
					Value leftVal, rightVal;
					( ( Literal * )left2 )->GetValue( leftVal );
					( ( Literal * )right2 )->GetValue( rightVal );
					if( !c->InitComplex( leftAttr, leftOp, leftVal,
										 rightOp, rightVal, expr ) ) {
						cerr << "error: problem with Condition:InitComplex"
							 << endl;
						return false;
					}
					return true;
				}
			}
		}
	}



	if( op >= Operation::__LOGIC_START__ &&
		op <= Operation::__LOGIC_END__ ) {
		if( !c->InitComplex( expr ) ) {
			cerr << "error: problem with Condition:InitComplex" << endl;
			return false;
		}
		return true;
	}

	if( !( op >= Operation::__COMPARISON_START__ && 
		   op <= Operation::__COMPARISON_END__) ) {
		cerr << "error: operator not comparison: " << (int)op << endl;
			// error: operator not comparison
		return false;
	}

	if( left == NULL || right == NULL ) {
		cerr << "error: NULL ptr in expr" << endl;
			// error: NULL ptr in expr
		return false;
	}

	kind = left->GetKind( );

	if( kind == ExprTree::ATTRREF_NODE ) {
		kind = right->GetKind( );
		if( kind == ExprTree::LITERAL_NODE ) {
				// Now we know expr is in Condition form: ATTR OP VALUE
			( ( AttributeReference * )left )->GetComponents( base, attr, 
															 junkBool );
			( ( Literal * )right )->GetValue( val ); 
 			if (!c->Init( attr, op, val, expr->Copy( ),
						  Condition::ATTR_POS_LEFT ) ) {
  				cerr << "error: problem with Condition::Init" << endl;
  				return false;
  			}
			return true;
		}
		if( !c->InitComplex( expr ) ) {
			cerr << "error: problem with Condition:InitComplex" << endl;
			return false;
		}
		return true;
	}

	if( kind == ExprTree::LITERAL_NODE ) {
		kind = right->GetKind( );
		if( kind == ExprTree::ATTRREF_NODE ) {
				// Now we know expr is in reverse Condition form: VALUE OP ATTR
			( ( AttributeReference * )right )->GetComponents( base, attr,
															  junkBool );
			( ( Literal * )left )->GetValue( val );
  			if( !c->Init( attr, op, val, expr->Copy( ),
						  Condition::ATTR_POS_RIGHT ) ) {
  				cerr << "error: problem with Condition::Init" << endl;
  				return false;
  			}
			return true;
		}
		if( !c->InitComplex( expr ) ) {
			cerr << "error: problem with Condition:InitComplex" << endl;
			return false;
		}
		return true;
	}

	if( !c->InitComplex( expr ) ) {
		cerr << "error: problem with Condition:InitComplex" << endl;
		return false;
	}
	return true;
}

bool BoolExpr::
EvalInContext( MatchClassAd &mad, ClassAd *context, BoolValue &result )
{
	if( !initialized || !context ) {
		return false;
	}
	ClassAd *emptyAd = new ClassAd( );
	Value val;
	bool b;
	mad.ReplaceLeftAd( emptyAd );
	mad.ReplaceRightAd( context );
	myTree->SetParentScope( emptyAd );
	if( !emptyAd->EvaluateExpr( myTree, val ) ) {
		mad.RemoveLeftAd( );
		mad.RemoveRightAd( );
		myTree->SetParentScope( NULL );
		return false;
	}
	if( val.IsBooleanValue( b ) ) {
		if( b ) {
			result = TRUE_VALUE;
		} else {
			result = FALSE_VALUE;
		}
	} else if( val.IsUndefinedValue( ) ) {
		result = UNDEFINED_VALUE;
	} else if( val.IsErrorValue( ) ) {
		result = ERROR_VALUE;
	} else {
		mad.RemoveLeftAd( );
		mad.RemoveRightAd( );
		myTree->SetParentScope( NULL );
		return false;
	}
	mad.RemoveLeftAd( );
	mad.RemoveRightAd( );
	myTree->SetParentScope( NULL );
	return true;
}

bool BoolExpr::
ToString( string& buffer )
{
	if( !initialized ) {
		return false;
	}
	PrettyPrint pp;
	pp.Unparse( buffer, myTree );
	return true;
}

bool BoolExpr::
Init( ExprTree * tree )
{
	if( !tree ) {
		return false;
	}
	if( myTree ) {
		delete myTree;
	}
	myTree = tree->Copy( );
	initialized = true;
	return true;
}

