/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/


#ifndef __BOOL_VALUE_H__
#define __BOOL_VALUE_H__

#include "list.h"
#include <string>
using namespace std;

/// List of possible values in the ClassAd 4-value logic
enum BoolValue
{
		/// True
	TRUE_VALUE,
		/// False
	FALSE_VALUE,
		/// Undefined
	UNDEFINED_VALUE,
		/// Error
    ERROR_VALUE
};

bool And( BoolValue, BoolValue, BoolValue &result );
bool Or( BoolValue, BoolValue, BoolValue &result );
bool Not( BoolValue, BoolValue, BoolValue &result );
bool GetChar( BoolValue, char &result );

class BoolVector
{
 public:
	BoolVector( );
	virtual ~BoolVector( );
	bool Init( int _length );
	bool Init( BoolVector *bv );
	bool SetValue( int index, BoolValue bval );
	bool GetValue( int index, BoolValue &result );
	bool GetNumValues( int &result );
	bool GetTotalTrue( int &result );
	bool TrueEquals( BoolVector &, bool &result );
	bool IsTrueSubsetOf( BoolVector &, bool &result );
	bool ToString( string &buffer );
 protected:
	bool initialized;
	BoolValue* boolvector;
	int length;
	int totalTrue;
};

class AnnotatedBoolVector : public BoolVector
{
 public:
	AnnotatedBoolVector( );
	~AnnotatedBoolVector( );
	bool Init( int length, int numContexts, int _frequency );
	bool SetContext( int index, bool b );
	bool GetFrequency( int &result );
	bool HasContext( int index, bool &result );
	bool ToString( string &buffer ); 
	static bool MostFreqABV( List<AnnotatedBoolVector> &,
							 AnnotatedBoolVector *&result);
 private:
	int frequency;
	bool* contexts;
	int numContexts;
};

class BoolTable
{
 public:
	BoolTable( );
	~BoolTable( );
	bool Init( int numCols, int numRows );
	bool SetValue( int col, int row, BoolValue bval );
	bool GetValue( int col, int row, BoolValue &result );
	bool GetNumRows( int &result );
	bool GetNumColumns(int &result );
	bool RowTotalTrue( int row, int &result );
	bool ColumnTotalTrue( int col, int &result );
	bool AndOfRow( int row, BoolValue &result );
	bool AndOfColumn( int col, BoolValue &result );
	bool OrOfRow( int row, BoolValue &result );
	bool OrOfColumn( int col, BoolValue &result );
	bool GenerateABVList( List<AnnotatedBoolVector> &result);
	bool GenerateMaxTrueABVList( List<AnnotatedBoolVector> &result );
	bool GenerateMaximalTrueBVList( List< BoolVector > &result );
	bool GenerateMinimalFalseBVList( List< BoolVector > &result );
	bool ToString( string &buffer );
 private:
	bool initialized;
	int numCols;
	int numRows;
	int* colTotalTrue;
	int* rowTotalTrue;
	BoolValue **table;
	bool CommonTrue( int col1, int col2, bool &result );
};

#endif	// __BOOL_VALUE_H__


