/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef __CONDITION_H__
#define __CONDITION_H__

#include "boolExpr.h"

/** A BoolExpr in Conjunctive Form, which consistes of an attribute, a
	comparison operator and a literal value. Currently the only way to
	initialize a Conditions is with the ExprToMultiProfile method in the
	BoolExpr class.
	@see BoolExpr
*/
class Condition : public BoolExpr
{
	friend class BoolExpr;
 public:
		//
	enum AttrPos {
			/** indicatss attr op value form */ ATTR_POS_LEFT,
			/** indicates value op attr form */ ATTR_POS_RIGHT,
	};

		/** Default Constructor */
	Condition( );

		/** Destructor */
	~Condition( );

		/** Gets the attribute part of the Condition.
			@param result The attribute in string form.
			@return true on success, false on failure.
		*/
	bool GetAttr( string &result );

		/** Gets the operator part of the Condition. (or the operator part of
			the first simple condition in a complex single attribute
			Condition.)
			@param result The operator in the form of an Operation::Opkind
			@return true on success, false on failure.
		*/
	bool GetOp( Operation::OpKind &result );

		/** Gets the operator part of the second simple condition in a complex
			single attribute Condition.
			@param result The operator in the form of an Operation::Opkind
			@return true on success, false on failure - or if Condition is
			not a complex single attribute Condition. 
		*/
	bool GetOp2( Operation::OpKind &result );

		/** Gets the literal value part of the Condition. (or the operator part
			of the first simple condition in a complex single attribute
			Condition.)
			@param result The literal as a Value.
			@return true on success, false on failure.
		*/
	bool GetVal( Value &result );

		/** Gets the literal value part of the second simple condition in a
			complex single attributeCondition.
			@param result The literal as a Value.
			@return true on success, false on failure.
		*/
	bool GetVal2( Value &result );

		/** Gets the type of the literal value part of the Condition.  It is
			assumed that in a valid context the attribute will evaluate to a
			literal value of the same type.
			@param result The type as a Value::Value.
			@return true on success, false on failure.
		*/
	bool GetType( Value::ValueType &result );

		/** Gets the position of the attribute (attr op value) or (value op 
			attr).
			@param result The attribute position as an AttrPos.
			@return true on success, false on failure.
		*/
	bool GetAttrPos( AttrPos &result );

		/** Determines whether the Condition is a complex Condition
			@return true if Condition is complex, false if it is normal
		*/
	bool IsComplex( );

		/** Determines whether the Condition is a complex Condition with
			multiple attributes;
			@return true if Condition is complex and has multiple attributes,
			false if it is normal or has only one attribute.
		*/
	bool HasMultipleAttrs( );

		/** A repository for annotations for the Profile.
			@see ConditionExplain
		*/
	ConditionExplain explain;
 private:
	string attr;
	Operation::OpKind op;
	Value val;
	Operation::OpKind op2;
	Value val2;
	AttrPos pos;
	bool isComplex;
	bool multiAttr;
	bool Init( const string &, ExprTree *, bool );
	bool Init( const string &, Operation::OpKind, const Value &,
			   ExprTree *, AttrPos );
	bool InitComplex( ExprTree * );
	bool InitComplex( const string &_attr,
					  Operation::OpKind _op1, const Value& _val1,
					  Operation::OpKind _op2, const Value& _val2,
					  ExprTree * );
};

#endif //__CONDITION_H__
