/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef __INTERVAL_H__
#define __INTERVAL_H__

#define EXPERIMENTAL  // necessary for using rectangle.h

#include "condor_classad.h"
#include "../condor_classad.V6/rectangle.h"
#include "list.h"
#include "extArray.h"

// Interval is currently defined in rectangle.h

/*
#ifndef RECTANGLE_H
struct Interval
{
	Interval() { key = -1; openLower = openUpper = false; }
	int		key;
	Value 	lower, upper;
	bool	openLower, openUpper;
};
#endif
*/

bool Copy( Interval *src, Interval *dest );

bool GetLowValue ( Interval *, Value &result );
bool GetHighValue ( Interval *, Value &result );
bool GetLowDoubleValue ( Interval *, double & );
bool GetHighDoubleValue ( Interval *, double & );

bool Overlaps( Interval *, Interval * );
bool Precedes( Interval *, Interval * );
bool Consecutive( Interval *, Interval * );
bool StartsBefore( Interval *, Interval * );
bool EndsAfter( Interval *, Interval * );

Value::ValueType GetValueType( Interval * );
bool IntervalToString( Interval *, string & );

bool Numeric( Value::ValueType );
bool SameType( Value::ValueType vt1, Value::ValueType vt2 );
bool GetDoubleValue ( Value &, double & );
bool EqualValue( Value &, Value & );
bool IncrementValue( Value & );
bool DecrementValue( Value & );


class HyperRect;

class IndexSet
{
 public:
	IndexSet( );
	~IndexSet( );
	bool Init( int _size );
	bool Init( IndexSet & );
	bool AddIndex( int );
	bool RemoveIndex( int );
	bool RemoveAllIndeces( );
	bool AddAllIndeces( );
	bool GetCardinality( int & );
	bool Equals( IndexSet & );
	bool IsEmpty( );
	bool HasIndex( int );
	bool ToString( string &buffer );
	bool Union( IndexSet & );
	bool Intersect( IndexSet & );

	static bool Translate( IndexSet &, int *map, int mapSize, int newSize, 
						   IndexSet &result );
	static bool Union( IndexSet &, IndexSet &, IndexSet &result );
	static bool Intersect( IndexSet &, IndexSet &, IndexSet &result );

 private:
	bool initialized;
	int size;
	int cardinality;
	bool *inSet;
};

struct MultiIndexedInterval
{
	MultiIndexedInterval( ) { ival = NULL; }
	Interval *ival;
	IndexSet iSet;
};

class ValueRange
{
 public:
	ValueRange( );
	~ValueRange( );
	bool Init( Interval *, bool undef=false, bool notString=false );
	bool Init2( Interval *, Interval *, bool undef=false );
	bool InitUndef( );
	bool Init( ValueRange *, int index, int numIndeces );
	bool Intersect( Interval *, bool undef=false, bool notString=false );
	bool Intersect2( Interval *, Interval *, bool undef=false );
	bool IntersectUndef( );
	bool Union( ValueRange *, int index );
	bool IsEmpty( );
	bool EmptyOut( );
	bool GetDistance( Value &pt, Value &min, Value &max, double &result,
					  Value &nearestVal );

	static bool BuildHyperRects( ExtArray< ValueRange * > &, int dimensions,
								 int numContexts, 
								 List< ExtArray< HyperRect * > > & );
   
	bool ToString( string & );
	bool IsInitialized( );
 private:
	bool initialized;
	Value::ValueType type;
	bool multiIndexed;
	List< MultiIndexedInterval > miiList;
	int numIndeces;
	List< Interval > iList;
	bool anyOtherString;
	IndexSet anyOtherStringIS;
	bool undefined;
	IndexSet undefinedIS;
};


class ValueRangeTable
{
 public:
	ValueRangeTable( );
	~ValueRangeTable( );
	bool Init( int numCols, int numRows );
	bool SetValueRange( int col, int row, ValueRange * );
	bool GetValueRange( int col, int row, ValueRange *& );
	bool GetNumRows( int & );
	bool GetNumColumns( int & );
	bool ToString( string &buffer );
 private:
	bool initialized;
	int numCols;
	int numRows;
	ValueRange ***table;
};

class ValueTable
{
 public:
	ValueTable( );
	~ValueTable( );
	bool Init( int numCols, int numRows );
	bool SetOp( int row, Operation::OpKind );
	bool SetValue( int col, int row, Value & );
	bool GetValue( int col, int row, Value & );
	bool GetNumRows( int & );
	bool GetNumColumns( int & );
	bool GetUpperBound( int row, Value & );
	bool GetLowerBound( int row, Value & );
	bool ToString( string &buffer );
 private:
	bool initialized;
	int numCols;
	int numRows;
	bool inequality;
	Value ***table;
	Interval **bounds;
	static bool IsInequality( Operation::OpKind );
	static bool OpToString( string &buffer, Operation::OpKind );
};

class HyperRect
{
 public:
	HyperRect( );
	~HyperRect( );
	bool Init( int dimensions, int numContexts );
	bool Init( int dimensions, int numContexts, Interval **& );
	bool GetInterval( int dim, Interval *& );
	bool SetIndexSet( IndexSet & );
	bool GetIndexSet( IndexSet & );
	bool FillIndexSet( );
	bool GetDimensions( int & );
	bool GetNumContexts( int & );
	bool ToString( string &buffer );
    HyperRect& operator=(const HyperRect & copy) { return *this; }
 private:
	bool initialized;
	int dimensions;
	int numContexts;
	IndexSet iSet;
	Interval **ivals;
};


#endif

