/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef __MULTIPROFILE_H__
#define __MULTIPROFILE_H__

#include "boolExpr.h"
#include "list.h"

/** A BoolExpr in Disjunctive Profile Form, which is a disjunction of Profiles.
	Currently the only way to initialize a MultiProfile is with the
	ExprToMultiProfile method in the BoolExpr class.
	@see BoolExpr
*/
class MultiProfile : public BoolExpr
{
	friend class BoolExpr;
 public:

		/** Default Constructor */
	MultiProfile( );

		/** Destructor */
	~MultiProfile( );

		/** Determines if the MultiProfile represents a literal value
			@return true if MulitProfile is a literal value, false otherwise.
		*/
	bool IsLiteral( );

		/** Gets the BoolValue if the MultiProfile represents a literal value
			@param result the BoolValue represented by the MultiProfile
			@return true on success, false on failure or if MultiProfile is not
			a literal value.
		*/
	bool GetLiteralValue( BoolValue &result );

		/** Gets the number of profiles contained in the MultiProfile
			@param result The number of profiles.
			@return true on success, false on failure.
		*/
	bool GetNumberOfProfiles( int &result );
		
		/** Rewinds the list of Profiles in the MultiProfile
			@return true on success, false on failure
		*/
	bool Rewind( );

		/** Gets the next Profile in the MultiProfile
			@param result the next profile in the MultiProfile
			@return true on success, false on failure
		*/
	bool NextProfile( Profile *&result );

		/** Generate a string representation of the MultiProfile.
			@param buffer A string to print the result to.
			@return true on success, false on failure.
		*/ 
	bool ToString( string &buffer );


		// unsupported methods
	bool AddProfile( Profile & );
	bool RemoveProfile( Profile & );
	bool RemoveAllProfiles( );

		/** A repository for annotations for the MultiProfile.
			@see MultiProfileExplain
		*/
	MultiProfileExplain explain;

    MultiProfile & operator=(const MultiProfile& copy) { return *this;}
 protected:
	bool InitVal( Value & );
 private:
	bool isLiteral;
	BoolValue literalValue;
	bool AppendProfile( Profile * );
	List<Profile> profiles;
};	

#endif // __MULTIPROFILE_H__
