/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef __PROFILE_H__
#define __PROFILE_H__

#include "boolExpr.h"
#include "list.h"

/** A BoolExpr in Profile Form, which is a conjunction of Conditions. Currently
	the only way to initialize a Profile is with the ExprToProfile method in
	the BoolExpr class.
	@see BoolExpr
*/
class Profile : public BoolExpr
{
	friend class BoolExpr;
 public:

		/** Default Constructor */
	Profile( );

		/** Destructor */
	~Profile( );

		/** Gets the number of conditions contained in the Profile
			@param result The number of profiles.
			@return true on success, false on failure.
		*/
   	bool GetNumberOfConditions( int &result );

		/** Rewinds the list of Conditions in the Profile
			@return true on success, false on failure
		*/
	bool Rewind( );

		/** Gets the next Condition in the Profile.
			@param result the next profile in the Profile
			@return true on success, false on failure
		*/
	bool NextCondition( Condition *&result );

		// unsupported methods
	bool AddCondition( Condition & );
	bool RemoveCondition( Condition & );
	bool RemoveAllConditions( );

		/** A repository for annotations for the Profile.
			@see ProfileExplain
		*/
    Profile& operator=(const Profile& copy) { return *this; } 
	ProfileExplain explain;
 private:
	bool AppendCondition( Condition * );
	List<Condition> conditions;
};

#endif // __PROFILE_H__


