/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef __RESOURCEGROUP_H__
#define __RESOURCEGROUP_H__

#include "condor_classad.h"
#include "list.h"

/** A wrapper for a list of ClassAds.  In the future this may be modified to
	handle ClassAdCollections.
*/
class ResourceGroup
{
 public:

		/** Default Constructor */
	ResourceGroup( );

		/** Destructor */
	~ResourceGroup( );

		/** Initialized the ResourceGroup to contain the given list of
			ClassAds. Only a shallow copy of the list is made.
			@param result the List of ClassAds for the ResourceGroup to
			contain.
			@return true on success, false on failure.
		*/
	bool Init( List<ClassAd>& result );

		/** Populates a list with the classads contained by the ResourceGroup
			Only a shallow copy of the list is made.
			@param result the destination List for the ClassAds.
			@return true on success, false on failure.
		*/
	bool GetClassAds( List<ClassAd>& result );

		/** Gets the number of classads in the ResourceGroup.
			@param result Assigned the number of ClassAds.
			@return true on success, false on failure.
		*/
	bool GetNumberOfClassAds( int& result );

		/** Generates a string representation of the ResourceGroup.  It prints
			the classads using PrettyPrint seperated by newlines.
			@param the buffer to print the ClassAds to.
			@return true on success, false on failure.
		*/
	bool ToString( string& buffer );
 private:
	bool initialized;
	List<ClassAd> classads;
};

#endif // __RESOURCEGROUP_H__
