#include "condor_common.h"
#include "condor_classad.h"

int main( int argc, char *argv[] )
{
	ClassAd ad;

		// test insert methods

	cout << "INSERT TEST:" << endl;
	cout << "------------" << endl;

	ad.Insert( "attr_int = 12" );
	ad.Insert( "attr_real = 3.14159" );
	ad.Insert( "attr_bool = false" );
	ad.Insert( "attr_string = \"foo\"" );
   
	ad.fPrint( stdout );
	cout << endl;
	ad.InsertOrUpdate( "attr_string = \"bar\"" );
	ad.fPrint( stdout );
	cout << endl;

		// test lookup methods

	cout << "LOOKUP TEST:" << endl;
	cout << "------------" << endl;

	char strVal[64];
	char strVal2[64];
	char *strVal3 = NULL;
	int intVal = 0;
	float doublefloatVal = 0;
	float intfloatVal = 0;
	int boolVal;

	ad.LookupString( "attr_string", strVal );
	ad.LookupString( "attr_string", strVal2, 64 );
	ad.LookupString( "attr_string", &strVal3 ); 
	ad.LookupInteger( "attr_int", intVal );
	ad.LookupFloat( "attr_int", intfloatVal );
	ad.LookupFloat( "attr_real", doublefloatVal );
	ad.LookupBool( "attr_bool", boolVal );

	cout << "attr_string is " << strVal << endl;
	cout << "attr_string(2) is " << strVal2 << endl;
	cout << "attr_string(3) is " << strVal3 << endl;
	cout << "attr_int is " << intVal << endl;
	cout << "attr_int(float) is " << intfloatVal << endl;
	cout << "attr_real is " << doublefloatVal << endl;
	cout << "attr_bool is " << boolVal << endl;
	cout << endl;

		// test eval methods

	cout << "EVAL TEST:" << endl;
	cout << "----------" << endl;
	
	ClassAd ad2;
	ad2.Insert( "string_expr = attr_string" );
	ad2.Insert( "int_expr = attr_int * 12" );
	ad2.Insert( "real_expr = attr_int * attr_real" );
	ad2.Insert( "bool_expr = attr_int < 15 || attr_bool" );

	ad2.EvalString( "string_expr", &ad, strVal );
	ad2.EvalInteger( "int_expr", &ad, intVal );
	ad2.EvalFloat( "real_expr", &ad, doublefloatVal );
	ad2.EvalBool( "bool_expr", &ad, boolVal );

	ad2.fPrint( stdout );
	cout << endl;
	cout << "string_expr evals to " << strVal << endl;
	cout << "int_expr evals to " << intVal << endl;
	cout << "real_expr evals to " << doublefloatVal << endl;
	cout << "bool_expr evals to " << boolVal << endl;
	cout << endl;

		// test type methods

	cout << "TYPE TEST:" << endl;
	cout << "----------" << endl;
	
	ad.SetMyTypeName( "boy" );
	ad.SetTargetTypeName( "girl" );

	ad.fPrint( stdout );
	cout << endl;
	cout << "My Type is " << ad.GetMyTypeName( ) << endl;
	cout << "Target Type is " << ad.GetTargetTypeName( ) << endl;
	cout << endl;

		// test read from file
	if( argc == 2 ) {
		FILE *fp = fopen( argv[1], "r" );
		if( !fp ) {
			cerr << "error: could not open file: " << argv[1] << endl;
			exit( 1 );
		}
		int eof, error, empty;

		eof = 0;
		error = 0;
		empty = 0;

		ClassAd *ad3 = new ClassAd( fp, "***", eof, error, empty); 
		if(error) {
			fprintf(stderr,"couldn't parse ClassAd in %s\n", argv[1]);
			exit( 1 );
		}
		if(empty) {
			fprintf(stderr,"%s is empty\n", argv[1]);
			exit( 1 );
		}

		eof = 0;
		error = 0;
		empty = 0;

		ClassAd *ad4 = new ClassAd( fp, "***", eof, error, empty); 
		if(error) {
			fprintf(stderr,"couldn't parse ClassAd in %s\n", argv[1]);
			exit( 1 );
		}
		if(empty) {
			fprintf(stderr,"%s is empty\n", argv[1]);
			exit( 1 );
		}

		cout << "FILE READ TEST:" << endl;
		cout << "---------------" << endl;
		ad3->fPrint( stdout );
		cout << endl;	
		ad4->fPrint( stdout );
		cout << endl;	
		
		fclose( fp );
	}
}

