#include "condor_common.h"
#include "condor_classad.h"

int main( int argc, char *argv[] )
{
	string adString = "";

	adString += "[";
	adString += "Requirements = ( Arch == \"INTEL\" ) && LocalReq && ";
	adString += "( OpSys == \"WINNT\" );";
	adString += "LocalReq = ( Foo == LocalVal + 3 ) && ( Bar == true );";
	adString += "LocalVal = 5";
	adString += "]";

	ClassAd* ad;
	ClassAdParser parser;

	if( !( ad = parser.ParseClassAd( adString ) ) ) {
		cerr << "error parsing ClassAd" << endl;
		exit(1);
	}

	PrettyPrint pp;
	string buffer;

	pp.Unparse( buffer, ad );

	cout << "ad =" << endl;
	cout << buffer << endl;
	cout << endl;
	buffer = "";

	ExprTree *reqExpr = NULL;
	ExprTree *flatReqExpr = NULL;
	ExprTree *flatInlinedReqExpr = NULL;
	Value val;
   
	if( !( reqExpr = ad->Lookup( "Requirements" ) ) ) {
		cerr << "error looking up Requirements expression" << endl;
		exit(1);
	}

	if( !( ad->Flatten( reqExpr, val, flatReqExpr ) ) ) {
		cerr << "error flattening Requirements expression" << endl;
	}

	if( !flatReqExpr ) {
		pp.Unparse( buffer, val );
		cout << "Requirements flattened to " << buffer << endl;
		return 0;
	}

	pp.Unparse( buffer, flatReqExpr );
	cout << "Requirements flattened to:" << endl;
	cout << buffer << endl;
	cout << endl;
	buffer = "";


	if( !( ad->FlattenAndInline( reqExpr, val, flatInlinedReqExpr ) ) ) {
		cerr << "error flattening (Inline) Requirements expression" << endl;
	}

	if( !flatInlinedReqExpr ) {
		pp.Unparse( buffer, val );
		cout << "Requirements flattened (Inline) to " << buffer << endl;
		return 0;
	}

	pp.Unparse( buffer, flatInlinedReqExpr );
	cout << "Requirements flattened (Inline) to:" << endl;
	cout << buffer << endl;
	cout << endl;
	buffer = "";
}

