#include <fcntl.h>
#include "constants.h"
#include "globus_ftp_client.h"
#include "reqex_logger.h"

static globus_mutex_t lock;
static globus_cond_t cond;
static globus_bool_t done;
static globus_bool_t error = GLOBUS_FALSE;
#define SIZE 512

FILE *infile;
int outfile;

char hostname[MAXSTR], filename[MAXSTR], logfilename[MAXSTR];
char dirname [MAXSTR], request_id[MAXSTR];

extern void release_disk_space(float newspace);

//----------------------- Transfer IN --------------------------------------
static void done_incb(
	void *					user_arg,
	globus_ftp_client_handle_t *		handle,
	globus_object_t *			err)
{
    char * tmpstr;

    if(err)
    {
	tmpstr = globus_object_printable_to_string(err);
	printf("%s\n", tmpstr); 
        error = GLOBUS_TRUE;
	globus_libc_free(tmpstr);
    }
    else {
      fclose(infile);
      cpp_write_log_basic(request_id,filename, logfilename, "transfered");
    }
    globus_mutex_lock(&lock);
    done = GLOBUS_TRUE;
    globus_cond_signal(&cond);
    globus_mutex_unlock(&lock);
       
}

static void data_incb(
    void *					user_arg,
    globus_ftp_client_handle_t *		handle,
    globus_object_t *				err,
    globus_byte_t *				buffer,
    globus_size_t				length,
    globus_off_t				offset,
    globus_bool_t				eof)
{
  
  fwrite(buffer, 1, length, infile);
  if(!eof){
    globus_ftp_client_register_read(handle,
				    buffer,
				    SIZE,
				    data_incb,
				    0);
    
  }
}

void transfer_IN(char *hostname, char *dirname, char *filename)
{
    globus_ftp_client_handle_t			handle;
    globus_ftp_client_attr_t			attr;
    globus_byte_t				buffer[SIZE];
    globus_size_t				buffer_length = sizeof(buffer);
    globus_result_t				result;
    char *					src;
    char *					dst;
    globus_ftp_client_plugin_t **		plugins;


    char infilename[MAXSTR];
    char targetfilename[MAXSTR];

    snprintf(infilename,MAXSTR,"%s/%s",SHARED_NEST_SPACE, filename);
    infile = fopen(infilename, "w");
    src = (char*) malloc (MAXSTR * sizeof(char));
    sprintf(src, "gsiftp://%s%s/%s",hostname,dirname, filename);

    printf("URL : %s\n",src);

    globus_module_activate(GLOBUS_FTP_CLIENT_MODULE);

    globus_mutex_init(&lock, GLOBUS_NULL);
    globus_cond_init(&cond, GLOBUS_NULL);

    globus_ftp_client_attr_init(&attr);
    globus_ftp_client_handle_init(&handle);

    plugins = GLOBUS_NULL;

    done = GLOBUS_FALSE;
    result = globus_ftp_client_get(&handle,
				   src,
				   &attr,
				   plugins,
				   done_incb,
				   0);
    
    if(result != GLOBUS_SUCCESS)
    {
      fprintf(stderr, globus_object_printable_to_string(globus_error_get(result)));
      error = GLOBUS_TRUE;
      done = GLOBUS_TRUE;
    }
    else
    {
      globus_ftp_client_register_read(
				      &handle,
				      buffer,
				      buffer_length,
				      data_incb,
				      0);
    }
    
    globus_mutex_lock(&lock);
    while(!done)
    {
	globus_cond_wait(&cond, &lock);
    }
    globus_mutex_unlock(&lock);

    globus_ftp_client_handle_destroy(&handle);

    globus_module_deactivate_all();

    //   return error;
}
//----------------------- Transfer OUT --------------------------------------

static void done_outcb(
	void *					user_arg,
	globus_ftp_client_handle_t *		handle,
	globus_object_t *			err)
{
    char * tmpstr;

    if(err) tmpstr = " an";
    else    tmpstr = "out";

    if(err) { printf("done with%s error\n", tmpstr); 
              error = GLOBUS_TRUE; }
    else{
      cpp_write_log_basic(request_id,filename, logfilename, "stage_out_completed");
    }

    globus_mutex_lock(&lock);
    done = GLOBUS_TRUE;
    globus_cond_signal(&cond);
    globus_mutex_unlock(&lock);
       
}

static void data_outcb(
    void *					user_arg,
    globus_ftp_client_handle_t *		handle,
    globus_object_t *				err,
    globus_byte_t *				buffer,
    globus_size_t				length,
    globus_off_t				offset,
    globus_bool_t				eof)
{
    if(!eof)
    {
	int rc;

    	rc = read(outfile, buffer, SIZE);
	globus_ftp_client_register_write(
	    handle,
	    buffer,
	    rc,
	    offset,
	    rc == 0,
	    data_outcb,
	    0);
    }
}

transfer_OUT(char *hostname, char *dirname, char *filename)
{
    globus_ftp_client_handle_t			handle;
    globus_ftp_client_attr_t			attr;
    globus_byte_t				buffer[SIZE];
    globus_size_t				buffer_length = SIZE;
    globus_result_t				result;
    globus_ftp_client_plugin_t **		plugins;
    char *					src;
    char *					dst;

    char outfilename[MAXSTR];
    
    snprintf(outfilename,MAXSTR,"%s/%s",SHARED_NEST_SPACE, filename);
    outfile = open(outfilename, O_RDONLY);
    dst = (char*) malloc (MAXSTR* sizeof(char));
    sprintf(dst, "gsiftp://%s%s/%s",hostname,dirname, filename);

    printf("URL : %s\n",dst);

    globus_module_activate(GLOBUS_FTP_CLIENT_MODULE);

    globus_mutex_init(&lock, GLOBUS_NULL);
    globus_cond_init(&cond, GLOBUS_NULL);

    globus_ftp_client_attr_init(&attr);

    globus_ftp_client_handle_init(&handle);

    plugins = GLOBUS_NULL;

    done = GLOBUS_FALSE;
    result = globus_ftp_client_put(&handle,
				   dst,
				   &attr,
				   plugins,
				   done_outcb,
				   0);
    if(result != GLOBUS_SUCCESS)
    {
	globus_object_t * err;
	err = globus_error_get(result);
	fprintf(stderr, "%s", globus_object_printable_to_string(err));
	done = GLOBUS_TRUE;
    }
    else
    {
	int rc;

	rc = read(outfile, buffer, buffer_length);
	globus_ftp_client_register_write(
	    &handle,
	    buffer,
	    rc,
	    0,
	    rc == 0,
	    data_outcb,
	    0);
    }
    globus_mutex_lock(&lock);
    while(!done)
    {
	globus_cond_wait(&cond, &lock);
    }
    globus_mutex_unlock(&lock);

    globus_ftp_client_handle_destroy(&handle);
    globus_module_deactivate_all();
    
    //    return error;
}


//----------------------- MAIN BODY --------------------------------------
main(int argc, char *argv[])
{

  printf("%s TRANSFER\n",argv[1]);
  
  strcpy(hostname, argv[2]);
  strcpy(dirname, argv[3]);
  strcpy(filename, argv[4]);
  strcpy(logfilename, argv[5]);
  strcpy(request_id, argv[6]);

  if (!strcmp(argv[1], "IN"))
      transfer_IN(hostname, dirname, filename);
  else if (!strcmp(argv[1], "OUT"))
      transfer_OUT(hostname, dirname, filename);

}

