#include "reqex_classad_reader.h"

extern int num_files;

//----------------------------------------
ClassAd_Reader::ClassAd_Reader(char *fname)
{
  currentAd = NULL;

  adfile = fopen(fname,"r");
  //  printf("classad:open:%d\n",num_files++);
  if (adfile == NULL) adfile = fopen(fname,"w");
  if (adfile == NULL) {
    dprintf(D_ALWAYS,"cannot open file:%s\n",fname);
  exit(1);
  }

}
//----------------------------------------  
ClassAd_Reader::~ClassAd_Reader()
{
  if (currentAd != NULL) delete currentAd;
  fclose(adfile);
  //  printf("classad:close:%d\n",num_files--);
}
//----------------------------------------  
void
ClassAd_Reader::reset()
{
  fseek(adfile,0,SEEK_SET);
}

//----------------------------------------
int
ClassAd_Reader::readAd()
{
  int i;
  char c;
  string adstr="";
  ClassAdParser parser;
  int leftparan = 0;

  i =0;
  while (1){
    c = fgetc(adfile);
    if (c == ']'){ 
      leftparan --; 
      if (leftparan == 0) break;
    }
    if (c == '[') leftparan ++; 

    if (c == EOF) return 0;
    adstr += c;
    i++;
  }
  adstr += c;

  //  dprintf(D_ALWAYS,"ClassAd:%s\n",adstr.c_str());
  if (currentAd != NULL) delete currentAd; 
  currentAd = parser.ParseClassAd(adstr);

  return 1;
}
//----------------------------------------
int
ClassAd_Reader::getValue(char *attr, char *attrval)
{
  string adbuffer = "";
  Value val;
  ClassAdUnParser unparser;
  ExprTree *attrexpr = NULL;

  if (!(attrexpr = currentAd->Lookup(attr)) ){
    printf("%s not found!\n", attr);
    return 0;
  }

  unparser.Unparse(adbuffer,attrexpr);
  strcpy(attrval,adbuffer.c_str());

  //printf("VALUE => %s\n",adbuffer.c_str());

  if (attrexpr != NULL) delete attrexpr;
  return 1;
}
/*----------------------------------------
 * Select filename
 * where attr = attrval
 *----------------------------------------*/
int 
ClassAd_Reader::queryName(char *fname, char *attr,char *attrval )
{
  char tempval[MAXSTR];
  string adbuffer = "";  
  ExprTree *attrexpr = NULL;
  ClassAdUnParser unparser;
  Value val;

  while (1){
    if (!readAd()){
      if (attrexpr != NULL) delete attrexpr;
      return 0;
    }
    getValue(attr,tempval);

    if (!strcmp(attrval, tempval)){
      attrexpr = currentAd->Lookup("FileName") ;
      unparser.Unparse(adbuffer,attrexpr);
      strcpy(fname,adbuffer.c_str());

      if (attrexpr != NULL) delete attrexpr;
      return 1;
    }
  }// while 

}
/*----------------------------------------
 * return next filename
 *----------------------------------------*/
ClassAd*
ClassAd_Reader::returncurrentAd()
{
  return currentAd;
}

/*----------------------------------------
 * Lookfor filename = fname
 * where attr = attrval
 *----------------------------------------*/
int 
ClassAd_Reader::lookFor(const string fname, char *attr,char *attrval )
{
  char tempval[MAXSTR];
  string adbuffer = "";  
  ExprTree *attrexpr = NULL;
  ClassAdUnParser unparser;
  Value val;
  
  reset();
  //  printf("looking for:%s\n",fname.c_str());
  while (1){
    if (!readAd()) { 
      if (attrexpr != NULL) delete attrexpr;
      return 0;
    }

    getValue(attr,tempval);
    
    if (!strcmp(attrval, tempval)){
      attrexpr = currentAd->Lookup("FileName") ;
      unparser.Unparse(adbuffer,attrexpr);

      //  printf("Matched:%s\n",adbuffer.c_str());
      if (fname == adbuffer){
        if (attrexpr != NULL) delete attrexpr;
	return 1;
      }
      adbuffer="";
    }
  }// while 

  if (attrexpr != NULL) delete attrexpr;
  return 0;
}

/*----------------------------------------
 * return value of qattr to qval for first match 
 * where attr = attrval
 *----------------------------------------*/
int 
ClassAd_Reader::returnFirst(const string qattr,char *qval, char *attr,char *attrval )
{
  char tempval[MAXSTR];
  string adbuffer = "";  
  ExprTree *attrexpr = NULL;
  ClassAdUnParser unparser;
  Value val;
  
  reset();

  while (1){
    if (!readAd()) {
      if (attrexpr != NULL) delete attrexpr;
      return 0;
    }

    getValue(attr,tempval);
    
    if (!strcmp(attrval, tempval)){
      attrexpr = currentAd->Lookup(qattr) ;
      unparser.Unparse(adbuffer,attrexpr);
      strcpy(qval,adbuffer.c_str());

      if (attrexpr != NULL) delete attrexpr;
      return 1;
    }
  }// while 

}

int get_subAdval(ClassAd *parentAd, char* subAdname, char *attr, char* attrval)
{
  string adbuffer = "";
  Value val;
  ClassAdUnParser unparser;
  ClassAdParser parser;
  ExprTree *attrexpr = NULL;
  char temp[MAXSTR];
  int len;
  
  //parse parent classAd -----------
  if (!(attrexpr = parentAd->Lookup(subAdname)) ){
    printf("%s not found!\n", subAdname);
    return 0;
  }

  unparser.Unparse(adbuffer,attrexpr);
  //printf("SUB_AD VALUE => %s\n",adbuffer.c_str());
  //  if (attrexpr != NULL) delete attrexpr;  
  

  //parse sub classAd ---------
  ClassAd *subAd = NULL;

  subAd = parser.ParseClassAd(adbuffer);
  adbuffer = "";
  if (!(attrexpr = subAd->Lookup(attr)) ){
    printf("%s not found!\n", attr);
    return 0;
  }

  unparser.Unparse(adbuffer,attrexpr);
  //  printf("ATTR VALUE => %s\n",adbuffer.c_str());

  strcpy(temp,adbuffer.c_str());

  if (temp[0] == '"'){ // contains quotes, so remove them
    len = strlen(temp);
    for (int i=1;i<len;i++)
      attrval[i-1] = temp[i];
    attrval[len-2] = '\0';
  }
  else
    strcpy(attrval,temp);

  //if (attrexpr != NULL) delete attrexpr;
  if (subAd != NULL) delete subAd;

  return 1;
}






