#include "constants.h"
#include "reqex_client_interface.h"

/* ============================================================================
 * establish connection to the ReqEx server
 * ==========================================================================*/
int reqex_open_connection(int & reqex_conn, char * hostname)
{
  struct sockaddr_in serv_addr; 
  struct hostent* server;
  char * hostIP;
  
  if ( (server = gethostbyname(hostname)) == NULL ){
    fprintf(stderr,"Cannot get address for host %s\n",hostname);
    exit(1);	
  }
  
  if ( (reqex_conn = socket(AF_INET,SOCK_STREAM,0)) < 0 ){
    printf("client:cannot open socket!\n");
    exit(1);
  }

  bzero(&serv_addr, sizeof(serv_addr)); 
  serv_addr.sin_family      = AF_INET;		
  memcpy(&(serv_addr.sin_addr.s_addr), server->h_addr, server->h_length);  
  serv_addr.sin_port        = htons(SERV_TCP_PORT);

  hostIP = (char *)malloc(16*sizeof(char));
  hostIP = (char *)inet_ntoa(serv_addr.sin_addr);
  printf("====================================================\n");
  printf("Trying %s...\n",hostIP);
  
  if ( connect(reqex_conn, (struct sockaddr*) &serv_addr, sizeof(serv_addr)) != 0){
    printf("client:cannot connect to %s!\n", hostname);
    exit(1);
  }
  
  printf("connected to %s..\n",hostname);

  free(hostIP);
  
  return REQEX_SUCCESS;
}

/* ============================================================================
 * close connection to the ReqEx server
 * ==========================================================================*/
int reqex_close_connection(int reqex_conn)
{
  if ( close(reqex_conn) != 0){
    printf("error in closeing the connection!\n");
    exit(1);
  }

  printf("connection closed.\n");
  return REQEX_SUCCESS;
}

/* ============================================================================
 * send a given request through the socket reqex_conn
 * ==========================================================================*/

int reqex_send_request(int reqex_conn, char* request_str)
{
  
  if ( send (reqex_conn, request_str, strlen(request_str),0) == -1){
    printf("client: send error!\n");
    exit(1);
  }

  return REQEX_SUCCESS;

}

/* ============================================================================
 * sample call to client library
 * ==========================================================================*/
/*
main(int argc, char* argv[])
{
 
  int reqex_conn;
  char request_str[MAX_TCP_BUF];

  if (argc < 2){
    fprintf(stderr,"Usage:\nclient <host_ip>\n");
    exit(1);
  }		

  strcpy(request_str," [
        request_id = 1;
        stage_in   = [
                        filename = tkx001.x;
                        source_dir = \"/tmp\";
                        filesize = 19003;
                        source = vulture.cs.wisc.edu;
                     ];

        job        = [
                        jobname = reqex_condor_job;
                     ];

        stage_out  = [
                        filename = tkx001.x.out;
                        dest_dir = \"/tmp\";
                        filesize = 19003;
                        destination = vulture.cs.wisc.edu;
                     ];
        duration = 4;
]");

  reqex_open_connection(reqex_conn, argv[1]);
  reqex_send_request(reqex_conn, request_str);
  reqex_close_connection(reqex_conn);


}
*/






