/* ============================================================================
 * ReqEx_daemon_main.C - Request Executer Daemon
 * Reliable File Transfer through SRB
 * by Tevfik Kosar <kosart@cs.wisc.edu>
 * University of Wisconsin - Madison
 * March 2000
 * ==========================================================================*/

#include "condor_common.h"
#include "condor_debug.h"
#include "../condor_daemon_core.V6/condor_daemon_core.h"
#include "basename.h"
#include "reqex_server.h"

// --------------
char* mySubSystem = "DATA_MANAGER";		// used by Daemon Core
// --------------

// command line arguments are read into these variables
extern char inputfilename[MAXSTR]; // $(WORK_DIR)/argv[1]

/* ==========================================================================
 * Parse command line arguments
 * ==========================================================================*/
void parse_arguments(int argc, char **argv)
{

  //check number of arguments
  if (argc != 2){
    dprintf(D_ALWAYS,
    "==============================================================\n");
    dprintf(D_ALWAYS,"USAGE: %s <files-to-transfer> \n",
	    argv[0]);
    dprintf(D_ALWAYS,
    "==============================================================\n");
    DC_Exit( 1 );
  }

  //dump the configuration
  dprintf(D_ALWAYS,
	  "==============================================================\n");
  dprintf(D_ALWAYS,
	  "              CONDOR_DATA_MANAGER CONFIGURATION:                 \n");
  dprintf(D_ALWAYS,
	   "==============================================================\n");
  dprintf(D_ALWAYS,
	  "WORK_DIR        : %s\n",WORK_DIR);
  
  //read the mode
  //mode = 1 -> transfer files directly to NEST
  //mode = 2 -> transfer files first to local disk then to NEST

  //read the inputfilename
  dprintf(D_ALWAYS,"req. input file : $(WORK_DIR)/%s\n",argv[1]);
  dprintf(D_ALWAYS,"req. log file   : $(WORK_DIR)/%s.log\n",argv[1]);
  sprintf(inputfilename,"%s/%s",WORK_DIR,argv[1]);
  
  dprintf(D_ALWAYS,
	  "==============================================================\n");


}


/* ============================================================================
 * initializations for the reqex_daemon
 * ==========================================================================*/
int main_init(int argc, char **argv)
{
  
  parse_arguments(argc, argv);


  //register timer (start_time,period,....)
  daemonCore->Register_Timer(0,(TimerHandler)initializations,"initializations");
  daemonCore->Register_Timer(1,2,(TimerHandler)call_main,"call_main");
  daemonCore->Register_Timer(5,2,(TimerHandler)check_for_transfered_files,"check_for_transfered_files");
  daemonCore->Register_Timer(5,2,(TimerHandler)check_for_condor_logfiles,"check_for_condor_logfiles");
  daemonCore->Register_Timer(5,2,(TimerHandler)check_for_completed_jobs,"check_for_completed_jobs");
  daemonCore->Register_Timer(5,2,(TimerHandler)check_for_completed_stage_outs,"check_for_completed_stage_outs");

  //daemonCore->Register_Timer(1,1,(TimerHandler)dummy1,"dummy1");

  //register reaper
  daemonCore->Register_Reaper("srb_transfer_reaper",(ReaperHandler)srb_transfer_reaper, "reaper for utime");
  daemonCore->Register_Reaper("send_condor_job_reaper",(ReaperHandler)send_condor_job_reaper, "reaper for utime");
  daemonCore->Register_Reaper("transfer_to_srb_reaper",(ReaperHandler)transfer_to_srb_reaper, "reaper for utime");
  daemonCore->Register_Reaper("transfer_to_NEST_reaper",(ReaperHandler)transfer_to_NEST_reaper, "reaper for utime");

  
  return TRUE;
}

/* ============================================================================
 * reconfigure reqex_daemon
 * ==========================================================================*/
int main_config()
{
  dprintf(D_ALWAYS,"RECONFIGURING ......\n");
  return TRUE;
}

/* ============================================================================
 * shutdown reqex_daemon fast
 * ==========================================================================*/
int main_shutdown_fast()
{
  dprintf(D_ALWAYS,"SHUTDOWN FAST ....\n");
  DC_Exit(0);
  return TRUE;	// to satisfy c++
}

/* ============================================================================
 * shutdown reqex_daemon gracefully
 * ==========================================================================*/
int main_shutdown_graceful()
{
    dprintf(D_ALWAYS,"SHUTDOWN GRACEFULLY ....\n");
    DC_Exit(0);
    return TRUE;	// to satisfy c++
}





