/* ============================================================================
 * ReqEx_Logger.C - Request Executer Logging Utility
 * Writes ids of transfered files to the logfile
 * by Tevfik Kosar <kosart@cs.wisc.edu>
 * University of Wisconsin - Madison
 * April 2000
 * ==========================================================================*/

#include "reqex_server.h"
#include "condor_classad.h"
#include "reqex_nest_interface.h"
#include "reqex_logger.h"

/* ##############################
 * REQEX LOGGER C INTERFACE 
 * ############################## */

/* ============================================================================
 *
 * ==========================================================================*/

void write_log_basic(char *request_id, char *targetfilename, char *logfilename, char *status)
{
  cpp_write_log_basic(request_id, targetfilename, logfilename, status);
}


/* ============================================================================
 * write fid to the logfile 
 * ==========================================================================*/
void cpp_write_log(char *targetfilename, char *logfilename, long filesize, char *location, char *status)
{


  // -------- ClassAd stuff ------------------------------

  //create ClassAd
  ClassAd *classad = new ClassAd;
  ClassAdParser parser;
  ExprTree *expr = NULL;

  //insert attributes to ClassAd
  classad->Insert("TimeStamp",Literal::MakeAbsTime());
  classad->InsertAttr("FileSize",(double)filesize,Value::NO_FACTOR);

  if ( !parser.ParseExpression(targetfilename, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("FileName",expr);

  if ( !parser.ParseExpression(status, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("Status",expr);

  if ( !parser.ParseExpression(location, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("Location",expr);

  //parse the ClassAd
  PrettyPrint unparser;
  string adbuffer = "";
  unparser.Unparse(adbuffer, classad);

  //destroy the ClassAd
  delete expr;
  delete classad;
  // -------- End of ClassAd stuff ------------------------------

  FILE *flog, *fclassad;

  //write the classad to log_classad file
  if ((fclassad = fopen(logfilename,"a+")) == NULL){
    dprintf(D_ALWAYS,
                "cannot open classad_logfile :%s...\n",logfilename);
    exit(1);
  }
  fprintf (fclassad,"%s",adbuffer.c_str());
  fclose(fclassad);
  
}
/* ============================================================================
 * log only basic file info 
 * ==========================================================================*/
void cpp_write_log_basic(char *request_id, char *targetfilename, char *logfilename, char *status)
{

  // -------- ClassAd stuff ------------------------------

  //create ClassAd
  ClassAd *classad = new ClassAd;
  ClassAdParser parser;
  ExprTree *expr = NULL;

  //insert attributes to ClassAd
  classad->Insert("TimeStamp",Literal::MakeAbsTime());

  if ( !parser.ParseExpression(targetfilename, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("FileName",expr);

  if ( !parser.ParseExpression(status, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("Status",expr);

  if ( !parser.ParseExpression(request_id, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("request_id",expr);

  //parse the ClassAd
  PrettyPrint unparser;
  string adbuffer = "";
  unparser.Unparse(adbuffer, classad);

  //destroy the ClassAd
  delete expr;
  delete classad;
  // -------- End of ClassAd stuff ------------------------------

  FILE *flog, *fclassad;

  //write the classad to log_classad file
  if ((fclassad = fopen(logfilename,"a+")) == NULL){
    dprintf(D_ALWAYS,
                "cannot open classad_logfile :%s...\n",logfilename);
    exit(1);
  }
  fprintf (fclassad,"%s",adbuffer.c_str());
  fclose(fclassad);

}

/* ============================================================================
 * write fid to the logfile 
 * ==========================================================================*/
void write_log_initial(char *request_id, char *targetfilename, char *logfilename, long filesize, char *status)
{


  // -------- ClassAd stuff ------------------------------

  //create ClassAd
  ClassAd *classad = new ClassAd;
  ClassAdParser parser;
  ExprTree *expr = NULL;

  //insert attributes to ClassAd
  classad->Insert("TimeStamp",Literal::MakeAbsTime());
  classad->InsertAttr("FileSize",(double)filesize,Value::NO_FACTOR);

  if ( !parser.ParseExpression(targetfilename, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("FileName",expr);

  if ( !parser.ParseExpression(status, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("Status",expr);

  if ( !parser.ParseExpression(request_id, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("request_id",expr);

  //parse the ClassAd
  PrettyPrint unparser;
  string adbuffer = "";
  unparser.Unparse(adbuffer, classad);

  //destroy the ClassAd
  delete expr;
  delete classad;
  // -------- End of ClassAd stuff ------------------------------

  FILE *flog, *fclassad;

  //write the classad to log_classad file
  if ((fclassad = fopen(logfilename,"a+")) == NULL){
    dprintf(D_ALWAYS,
                "cannot open classad_logfile :%s...\n",logfilename);
    exit(1);
  }
  fprintf (fclassad,"%s",adbuffer.c_str());
  fclose(fclassad);
  
}






