/* ============================================================================
 * ReqEx_Server.C - Request Executer Server
 * Reliable File Transfer through SRB-FTP-GsiFTP
 * by Tevfik Kosar <kosart@cs.wisc.edu>
 * University of Wisconsin - Madison
 * 2000-2001
 * ==========================================================================*/

#include "reqex_server.h"
#include "user_log.c++.h"
#include "nest_speak.h"  
#include "reqex_classad_reader.h"
#include "reqex_logger.h"
#include "reqex_scheduler.h"
#include "reqex_server_interface.h"


extern "C" int get_fileinfo(char *filename, long  *filesize, char *location);

char logfilename[MAXSTR];
char targetdirname[MAXSTR];
char inputfilename[MAXSTR];
int  daemon_std[3];
float nestfreespace, freediskspace ;
long lines_read_from_input = 0;

Scheduler scheduler;
long next_schedule = 0 , current_time = 0;
char scheduled_request[MAXSTR] = "-1";
long scheduled_time;
char hostfile [MAXSTR];



/* ==========================================================================
 * determine amount of initial available disk space in the system
 * ==========================================================================*/
void init_free_disk_space(char *targetdirname)
{
  freediskspace = sysapi_disk_space( targetdirname ) * 1000;

  //freediskspace = 87032000;
  printf("freediskspace = %f\n", freediskspace);
}

/* ==========================================================================
 * reserve disk space
 * ==========================================================================*/
int reserve_disk_space(float newspace)
{
  if (freediskspace >= newspace){
    freediskspace -= newspace;
    return REQEX_SUCCESS;
  }
  else  return REQEX_ERROR;
}

/* ==========================================================================
 * release disk space
 * ==========================================================================*/
void release_disk_space(float newspace)
{
  freediskspace += newspace;
}

/* ==========================================================================
 * return free disk space
 * ==========================================================================*/
float return_free_space()
{
  return freediskspace;
}


/* ==========================================================================
 * open daemon core file pointers
 * ==========================================================================*/
void open_daemon_core_file_pointers()
{
  char f1[MAXSTR], f2[MAXSTR], f3[MAXSTR];

  snprintf(f1,MAXSTR,"%s/x","/tmp");
  snprintf(f2,MAXSTR,"%s/y","/tmp");
  snprintf(f3,MAXSTR,"%s/z","/tmp");

  //set stdin, stdout and stderr file pointers
  daemon_std[0] = open(f1, O_CREAT | O_RDWR);  //stdin
  daemon_std[1] = open(f2, O_CREAT | O_RDWR);  //stdout
  daemon_std[2] = open(f3, O_CREAT | O_RDWR);  //stderr

}

/* ==========================================================================
 * close daemon core file pointers
 * ==========================================================================*/
void close_daemon_core_file_pointers()
{
  close(daemon_std[0]);
  close(daemon_std[1]);
  close(daemon_std[2]);
}

/* ============================================================================
 * create cmnd_file for condor_submit
 * ==========================================================================*/
void create_cmnd_file(char *fname, char *condorjobname)
{
  char cmndfilename[MAXSTR];
  FILE *cmndfile;

  snprintf(cmndfilename,MAXSTR,"%s/cmndfile_%s",SHARED_DIR,fname);
  cmndfile = fopen(cmndfilename,"w");
  if (cmndfile == NULL){
    dprintf(D_ALWAYS,"Cannot create file:%s\n",cmndfilename);
    exit(1);
  }
  
  fprintf(cmndfile,"Initialdir = %s\n",SHARED_DIR);
  fprintf(cmndfile,"Executable = %s/%s\n",SHARED_DIR, condorjobname);
  fprintf(cmndfile,"Arguments = %s\n",fname);
  fprintf(cmndfile,"Universe = vanilla\n");
  fprintf(cmndfile,"MemoryRequirements = 64\n");
  fprintf(cmndfile,"Requirements = ((OpSys == \"SOLARIS27\")||(OpSys == \"SOLARIS26\")) && ((FileSystemDomain == \".cs.wisc.edu\")||(FileSystemDomain == \"cs.wisc.edu\"))\n");
  fprintf(cmndfile,"Log = %s/condor.log_%s\n",SHARED_DIR, fname);
  fprintf(cmndfile,"Notification = error\n");
  fprintf(cmndfile,"Queue\n");

  fclose(cmndfile);

}

/* ============================================================================
 * check whether the file is already logged or not 
 * ==========================================================================*/
int already_logged(char *targetfilename, char *logfilename, char *status)
{
  char lastfilename[MAXSTR];

  ClassAd_Reader adreader2(logfilename);
  if (adreader2.lookFor(targetfilename,"status",status)) {
    return 1; //already logged
  }
  else return 0; //not logged
}

/* ============================================================================
 * check if all files are trnasfered or not
 * ==========================================================================*/
int check_for_completion(char *infilename, char *logfilename)
{
  FILE *infile;
  char targetfilename[MAXSTR];
  int transfer_complete = 1;

  infile = fopen( infilename, "r" );
  if( infile == NULL ) {
    dprintf( D_ALWAYS, "ERROR: can't open input file %s (%s)\n",
	     infilename, strerror( errno ) );
    exit( 1 );
  }
  
  while( fgets( targetfilename, MAXSTR, infile ) ) {
    chomp( targetfilename );
    if (!already_logged(targetfilename,logfilename,"transfered")){
      transfer_complete = 0;
      break;
    }
  }

  fclose(infile);

  return transfer_complete;
}
/* ============================================================================
 * check for transfered files
 * ==========================================================================*/
void check_for_transfered_files()
{

  char fname[MAXSTR], fsize[MAXSTR], flocation[MAXSTR];
  char status[MAXSTR], tempstr1[MAXSTR], tempstr2[MAXSTR], cmndfile[MAXSTR];
  char logfilename[MAXSTR], condorjobname[MAXSTR], request_id[MAXSTR];
  int pid;

  snprintf(logfilename,MAXSTR,"%s.log",inputfilename);

  ClassAd_Reader adreader(logfilename);
  ClassAd_Reader adreader2(logfilename);

  
  while (1){
    if (!adreader.queryName(fname,"status","transfered")) break;
    if (!adreader2.lookFor(fname,"status","job_submitted")) {
      printf("submitting file:%s to Condor\n",fname);
      adreader2.returnFirst("FileSize",fsize,"FileName",fname);
      adreader2.returnFirst("Location",flocation,"FileName",fname);
      adreader2.returnFirst("request_id",request_id,"FileName",fname);

      ClassAd_Reader adreader3(inputfilename);
      adreader3.returnFirst("request_id",request_id,"request_id",request_id);
      ClassAd *currentAd = NULL;
      currentAd = adreader3.returncurrentAd();
      get_subAdval(currentAd, "job", "jobname",condorjobname);
      if (currentAd != NULL) delete currentAd;

      create_cmnd_file(fname, condorjobname);
      cpp_write_log_basic(request_id, fname,logfilename,"job_submitted");      

      //----------
      struct stat filestat;
      char tempstr[MAXSTR];
      long filesize;

      snprintf(tempstr,MAXSTR,"%s/%s",SHARED_NEST_SPACE,fname);
      if (!stat(tempstr,&filestat)){ //if file exists
	filesize = filestat.st_size;
	printf("file size:%d\n",filesize);
      }
      else printf("file:%s does not Exist!!\n",tempstr);

      //--------

      //create a new process to transfer the files
      snprintf(tempstr1,MAXSTR,"%s/condor_submit",CONDOR_BIN);     
      snprintf(tempstr2,MAXSTR,"condor_submit %s/cmndfile_%s",SHARED_DIR,fname);     

      // chdir (SHARED_DIR); //done in the submitfile...
      pid = daemonCore->Create_Process(tempstr1,	// exec
				       tempstr2,	        // argv
				       PRIV_ROOT,
				       2,               //reaper 
				       FALSE,NULL,NULL,FALSE,NULL,
				       daemon_std             //std.file ids 
				       );

    }
  }// while 
  
}

/* ============================================================================
 * check for stage-out completion
 * ==========================================================================*/
void check_for_completed_stage_outs()
{

  char fname[MAXSTR];
  char status[MAXSTR], tempstr[MAXSTR];
  char logfilename[MAXSTR], request_id[MAXSTR];
  struct stat filestat;
  long filesize, outfilesize;

  snprintf(logfilename,MAXSTR,"%s.log",inputfilename);

  ClassAd_Reader adreader(logfilename);
  ClassAd_Reader adreader2(logfilename);

  while (1){
    if (!adreader.queryName(fname,"status","stage_out_completed")) break;
    if (!adreader2.lookFor(fname,"status","output_file_removed")) {
      adreader2.returnFirst("request_id",request_id,"FileName",fname);

      //get filesize info from request file
      ClassAd_Reader adreader_in(inputfilename);
      ClassAd *currentAd = NULL;
      adreader_in.returnFirst("request_id",request_id,"request_id",request_id);
      currentAd = adreader_in.returncurrentAd();
      get_subAdval(currentAd, "stage_out", "filesize", tempstr);
      outfilesize = atol(tempstr);

      //unlink transfered file
/*
      snprintf(tempstr,MAXSTR,"%s/%s",SHARED_NEST_SPACE,fname);
      if (!stat(tempstr,&filestat)){ //if file exists
	filesize = filestat.st_size;
	printf("* unlink %s: size:%d (reserved:%d)\n",
	       tempstr,filesize,outfilesize);
	unlink(tempstr);	
	release_disk_space(outfilesize);
	current_time = scheduler.remove(request_id);
	if (current_time < 0) fprintf(stderr,"error in scheduler_remove!\n");
	printf("after remove...\n");
	scheduler.print();
      }
*/
      cpp_write_log_basic(request_id, fname,logfilename,"output_file_removed");      
    }
  }// while 
  
}
/* ============================================================================
 * transfer file to ftp
 * ==========================================================================*/
void transfer_file_to_FTP(char *fname)
{
  char command[MAXSTR], tempstr[MAXSTR];
  FILE * commandfile;
  int pid;
  
  //dynamically create a shell script to execute
  snprintf(command,MAXSTR,"%s/ftpput",WORK_DIR);
  
  //create a new process to transfer the files
  printf("\n\nftpput hpss.cacr.caltech.edu ux421034 h2gamma %s %s . %s\n\n",
	 SHARED_NEST_SPACE, fname, fname);     

  snprintf(tempstr,MAXSTR,
	   "ftpput hpss.cacr.caltech.edu ux421034 h2gamma %s %s . %s",
	   SHARED_NEST_SPACE, fname, fname);     

  pid = daemonCore->Create_Process(
				   command,	// exec
				   tempstr,	        // argv
				   PRIV_ROOT,
				   2,               //reaper 
				   FALSE,NULL,NULL,FALSE,NULL,
				   daemon_std             //std.file ids 
				   );

}
/* ============================================================================
 * transfer file from ftp
 * ==========================================================================*/
void transfer_file_from_FTP(char *fname)
{
  char command[MAXSTR], tempstr[MAXSTR];
  FILE * commandfile;
  int pid;
  
  //dynamically create a shell script to execute
  snprintf(command,MAXSTR,"%s/ftpget",WORK_DIR);
  
  //create a new process to transfer the files
  snprintf(tempstr,MAXSTR,
	   "ftpget hpss.cacr.caltech.edu ux421034 h2gamma . %s %s/%s",
	   fname, SHARED_NEST_SPACE, fname);     
  
  pid = daemonCore->Create_Process(
				   command,	// exec
				   tempstr,	        // argv
				   PRIV_ROOT,
				   2,               //reaper 
				   FALSE,NULL,NULL,FALSE,NULL,
				   daemon_std             //std.file ids 
				   );
  
}

/* ============================================================================
 * transfer file from gsiftp
 * ==========================================================================*/
void transfer_file_from_GSIFTP(char *hostname, char *dir,char *fname, char *request_id)
{
  
  char command[MAXSTR], tempstr[MAXSTR];
  FILE * commandfile;
  int pid;
  
  //dynamically create a shell script to execute
  snprintf(command,MAXSTR,"%s/reqex_GSIFTP_transfer",WORK_DIR);
  
  //create a new process to transfer the files
  snprintf(tempstr,MAXSTR,"reqex_GSIFTP_transfer IN %s %s %s %s %s",
	   hostname, dir, fname, logfilename, request_id);     
  pid = daemonCore->Create_Process(
				   command,	// exec
				   tempstr,	        // argv
				   PRIV_ROOT,
				   3,               //reaper 
				   FALSE,NULL,NULL,FALSE,NULL,
				   daemon_std             //std.file ids 
				   );

}


/* ============================================================================
 * transfer file to gsiftp
 * ==========================================================================*/
void transfer_file_to_GSIFTP(char *hostname, char *dir, char *fname, char *request_id)
{
  
  char command[MAXSTR], tempstr[MAXSTR];
  FILE * commandfile;
  int pid;
  
  //dynamically create a shell script to execute
  snprintf(command,MAXSTR,"%s/reqex_GSIFTP_transfer",WORK_DIR);
  
  //create a new process to transfer the files
  snprintf(tempstr,MAXSTR,"reqex_GSIFTP_transfer OUT %s %s %s %s %s",
	   hostname, dir, fname, logfilename, request_id);     
  pid = daemonCore->Create_Process(
				   command,	// exec
				   tempstr,	        // argv
				   PRIV_ROOT,
				   3,               //reaper 
				   FALSE,NULL,NULL,FALSE,NULL,
				   daemon_std             //std.file ids 
				   );

}

/* ============================================================================
 * third party GSIFTP transfer
 * ==========================================================================*/
void thirdparty_GSIFTP_transfer(char *src_host, char *src_dir,char *src_file, char *dest_host, char *dest_dir, char *dest_file, char *request_id, char *log_message)
{
  
  char command[MAXSTR], tempstr[MAXSTR];
  FILE * commandfile;
  int pid;
  
  //dynamically create a shell script to execute
  snprintf(command,MAXSTR,"%s/reqex_THIRDPARTY_transfer",WORK_DIR);
  
  //create a new process to transfer the files
  snprintf(tempstr,MAXSTR,"reqex_THIRDPARTY_transfer %s %s %s %s %s %s %s %s %s",
	   src_host, src_dir, src_file, dest_host, dest_dir, dest_file, logfilename, request_id, log_message);     
  pid = daemonCore->Create_Process(
				   command,	// exec
				   tempstr,	        // argv
				   PRIV_ROOT,
				   3,               //reaper 
				   FALSE,NULL,NULL,FALSE,NULL,
				   daemon_std             //std.file ids 
				   );


}

/* ============================================================================
 * transfer file to srb
 * ==========================================================================*/
void transfer_file_to_SRB(char *fname, long filesize, char *request_id)
{
  char command[MAXSTR], tempstr[MAXSTR];
  FILE * commandfile;
  int pid;
  
  //dynamically create a shell script to execute
  snprintf(command,MAXSTR,"%s/reqex_SRB_NEST_transfer",WORK_DIR);
  
  //create a new process to transfer the files
  snprintf(tempstr,MAXSTR,"reqex_SRB_NEST_transfer OUT %s %s %d %s",
	   fname, logfilename, filesize, *request_id);     
  pid = daemonCore->Create_Process(
				   command,	// exec
				   tempstr,	        // argv
				   PRIV_ROOT,
				   1,               //reaper 
				   FALSE,NULL,NULL,FALSE,NULL,
				   daemon_std             //std.file ids 
				   );
  
}
/* ============================================================================
 * transfer file from SRB
 * ==========================================================================*/
void transfer_file_from_SRB(char *fname, long filesize, char* request_id)
{
  char command[MAXSTR], tempstr[MAXSTR];
  FILE * commandfile;
  int pid;

  snprintf(command,MAXSTR,"%s/reqex_SRB_NEST_transfer",WORK_DIR);
  
  //create a new process to transfer the files
  snprintf(tempstr,MAXSTR,"reqex_SRB_NEST_transfer IN %s %s %d %s",
	   fname, logfilename, filesize, request_id);     

  pid = daemonCore->Create_Process(
					 command,	// exec
					 tempstr,	        // argv
					 PRIV_ROOT,
					 1,               //reaper 
					 FALSE,NULL,NULL,FALSE,NULL,
					 daemon_std             //std.file ids 
					 ); 

}

/* ============================================================================
 * check for completed jobs
 * ==========================================================================*/
void check_for_completed_jobs()
{

  char fname[MAXSTR], outfname[MAXSTR];
  char tempstr[MAXSTR], request_id[MAXSTR];
  char logfilename[MAXSTR], dest_host[MAXSTR], dest_dir[MAXSTR];
  int pid;
  struct stat filestat;
  long filesize, infilesize;
  char Protocol[10];

  snprintf(logfilename,MAXSTR,"%s.log",inputfilename);

  ClassAd_Reader adreader(logfilename);
  ClassAd_Reader adreader2(logfilename);

  while (1){
    if (!adreader.queryName(fname,"status","job_completed")) break;
    adreader.getValue("request_id", request_id);
    
    ClassAd_Reader adreader3(inputfilename);
    adreader3.returnFirst("request_id",request_id,"request_id",request_id);
    ClassAd *currentAd = NULL;
    currentAd = adreader3.returncurrentAd();
    get_subAdval(currentAd, "stage_out", "filename",outfname);
    get_subAdval(currentAd, "stage_out", "destination",dest_host);
    get_subAdval(currentAd, "stage_out", "dest_dir",dest_dir);
    get_subAdval(currentAd, "stage_in", "filesize",tempstr);
    infilesize = atol(tempstr);
    
    if (currentAd != NULL) delete currentAd;

    ClassAd_Reader host_adreader(hostfile);
    host_adreader.returnFirst("protocol",Protocol,"host_name", dest_host);

    //    snprintf(outfname,MAXSTR,"%s.out",fname);

    if (!adreader2.lookFor(outfname,"status","stage_out_requested")) {
      
      //deallocate space allocated for the input file
      if (storage_type == STORE_TO_NEST){

	/*--->

	NestConnection nestcon;
	OpenNestConnection(nestcon, NEST_HOST);
	RemoveNestFile(fname, nestcon);
	CloseNestConnection(nestcon);

	<--*/

      }
      else{
	snprintf(tempstr,MAXSTR,"%s/%s",SHARED_NEST_SPACE,fname);
	printf("checking file: %s/%s\n",SHARED_NEST_SPACE,tempstr);
	if (!stat(tempstr,&filestat)){ //if file exists
	  printf("unlink %s: size:%d (reserved:%d)\n",
		 tempstr,filesize,infilesize);
	  unlink(tempstr);	
	  release_disk_space(infilesize);
	}
      }

      filesize = 0; //not used for out_transfer yet.. ??
      
      printf("PROTOCOL : %s\n",Protocol);
      printf("host: %s\n", dest_host);

      //write log to the logfile
      cpp_write_log_basic(request_id,outfname,logfilename,"stage_out_requested");      
      //transfer the output file
      if (!strcmp(Protocol,"SRB")){
	printf("Before calling Transfer to SRB.\n");
	printf("outfname : %s\n",outfname);
	printf("filesize : %d\n",filesize);
	printf("request_id:%s\n",request_id);
	transfer_file_to_SRB(outfname, filesize, request_id );
	printf("Transfer to SRB called ..\n");
      }
      else if (!strcmp(Protocol,"FTP"))
	transfer_file_to_FTP(outfname);
      else if (!strcmp(Protocol,"GSIFTP"))
	//	transfer_file_to_GSIFTP(dest_host, dest_dir, outfname, request_id);
	thirdparty_GSIFTP_transfer(NEST_HOST,SHARED_NEST_SPACE,outfname, dest_host, dest_dir,outfname, request_id ,"stage_out_completed");
    }
  }// while 

}

/* ============================================================================
 * check for condor log files for completed processes
 * ==========================================================================*/
void check_for_condor_logfiles()
{

  int fd;
  struct stat buf;

  char classadfilename[MAXSTR], condor_logfile[MAXSTR], cmndfile[MAXSTR];
  char status[MAXSTR], fname[MAXSTR], fsize[MAXSTR], flocation[MAXSTR];
  char outfile[MAXSTR], request_id[MAXSTR];

  //open classad file
  snprintf(classadfilename,MAXSTR,"%s.log",inputfilename);
  ClassAd_Reader adreader(classadfilename);
  ClassAd_Reader adreader2(classadfilename);
  
  bool completed = false;
  while (!completed){
    //query for submitted but not completed jobs
    if (!adreader.queryName(fname,"status","job_submitted")) break;
    adreader.getValue("request_id", request_id);
    if (!adreader2.lookFor(fname,"status","job_completed")) {
      
      printf("%s.log:\n",fname);

      //check condor.log files for submitted jobs
      chdir (SHARED_DIR); //done in the submitfile...
      snprintf(condor_logfile,MAXSTR,"condor.log_%s",fname);
      

      ReadUserLog rcondorlog;
      rcondorlog.initialize(condor_logfile);
      
      if ( (fd = rcondorlog.getfd()) == 0){
	printf("Cannot get fd for CondorLogFile!\n");
	exit(0);
      }

      if (fstat(fd, &buf) <0) continue; //if logfile doesn't exist
      
      //read condor log files for possible events..
      ULogEvent *event;
      bool done = false;
      while(!done){
	ULogEventOutcome outcome = rcondorlog.readEvent(event);

	switch(outcome){
	case ULOG_NO_EVENT:
	  done = true;
	  break;

	case ULOG_OK:
	  switch(event->eventNumber){
	  case ULOG_SUBMIT:
	    break;
	    
	  case ULOG_EXECUTE:
	    printf("\texecuting the job..\n");
	    break;
	    
	  case ULOG_JOB_TERMINATED:
	    {
	      printf("\tjob terminated:%s - ",fname);
	      JobTerminatedEvent * termEvent = (JobTerminatedEvent*) event;
	      printf("with code: %d\n",termEvent->returnValue);
	      if (termEvent->normal){
		// write to class ad log -> job_completed
		adreader2.returnFirst("FileSize",fsize,"FileName",fname);
		adreader2.returnFirst("Location",flocation,"FileName",fname);
		cpp_write_log_basic(request_id, fname,logfilename,"job_completed");
	      
		unlink(condor_logfile);
		snprintf(cmndfile,MAXSTR,"cmndfile_%s",fname);
		unlink(cmndfile);
	      }//if normal termination
		
	    }
	    break;
	    
	  case ULOG_JOB_ABORTED:
	    printf("\tjob is aborted!\n");
	    break;

	  default: 
	    printf("\tunknown event:%d\n",event->eventNumber);
	    
	  }//switch(event->eventNumber)
	  break;
	  
	default: 
	  printf("unknown outcome:%d\n",outcome);
	  done = true;

	}//switch(outcome)
      }//while (!done)

      free (event);

    }//if not job_completed
  }//while (!completed)


}

/* ============================================================================
 * main body of the condor_srb_reqex
 * ==========================================================================*/
void initializations()
{
  init_free_disk_space(SHARED_NEST_SPACE);
  pthread_t thread1;
  char *pptr;  //void
  int tid;

  //  pthread_create(&thread1, NULL, &listen_to_requests,(void*)pptr);

  tid = daemonCore->Create_Thread(&listen_to_requests, (void*)pptr, NULL, 1);
  printf("Thread created!\n");
  //sleep(2);

}

/* ============================================================================
 * main body of the condor_srb_reqex
 * ==========================================================================*/
int call_main()
{

    int i,j;
    int status;
    char command[MAXSTR], request_id[MAXSTR];
    char lastfilename[MAXSTR];
    char targetfilename[MAXSTR], targetlocation[MAXSTR];
    struct stat filestat;
    FILE *logfile, *configfile,*commandfile;
    long filesize;
    int pid;
    int std[3];
    char Protocol[10];
    long duration;
    char tempstr[MAXSTR];

    open_daemon_core_file_pointers();

    //set the logfilename
    snprintf(logfilename,MAXSTR,"%s.log",inputfilename);    
    snprintf(hostfile, MAXSTR,"%s/host.list",WORK_DIR);

    //initalize ClassAd reader
    ClassAd_Reader adreader(inputfilename);
    ClassAd *currentAd = NULL;

    lines_read_from_input++;
    
    //if there is a scheduled request
    if (strcmp(scheduled_request, "-1")){
      printf("current_time:%d, scheduled_time:%d\n",
	     current_time, scheduled_time);
      if (current_time != scheduled_time){ 
	for (i=1; i < lines_read_from_input; i++)
	  if (!adreader.readAd()){
	  close_daemon_core_file_pointers();
	  return 0;
	  }
      }
      else {
	lines_read_from_input = 0;
	strcpy(scheduled_request,"-1");
      }
    }
    
    //read the fids from input file 
    while(1) {
      if ( !adreader.readAd()){
	close_daemon_core_file_pointers();
	return 0;
      }
      
      currentAd = adreader.returncurrentAd();
      adreader.getValue("request_id", request_id);

      adreader.getValue("duration", tempstr);
      duration = atol(tempstr);

      //if there is a scheduled request, check duration of candidates
      if (strcmp(scheduled_request, "-1"))
	  if (current_time + duration > scheduled_time) continue;

      get_subAdval(currentAd, "stage_in", "filename",targetfilename);

      if (!already_logged(targetfilename,logfilename,"transfer_requested"))
	break;
      else if (currentAd != NULL) delete currentAd;

    }


    char source_host[MAXSTR], source_dir[MAXSTR];
    char dest_host[MAXSTR], dest_dir[MAXSTR], dest_file[MAXSTR];
    long outfilesize;
    

    filesize = 0;
    if (get_subAdval(currentAd, "stage_in", "filesize", tempstr))
      filesize = atol(tempstr);
    printf("given file size: %d\n", filesize);
    

    get_subAdval(currentAd, "stage_out", "filesize", tempstr);
    get_subAdval(currentAd, "stage_in", "source", source_host);
    get_subAdval(currentAd, "stage_in", "source_dir", source_dir);
    get_subAdval(currentAd, "stage_out", "destination", dest_host);
    get_subAdval(currentAd, "stage_out", "dest_dir", dest_dir);
    get_subAdval(currentAd, "stage_out", "filename", dest_file);

    outfilesize = atol(tempstr);


    printf("SOURCE : %s\n",source_host);

    if (currentAd != NULL) delete currentAd;

    ClassAd_Reader host_adreader(hostfile);
    host_adreader.returnFirst("protocol",Protocol,"host_name", source_host);
  
    printf("PROTOCOL : %s\n",Protocol);



    //------------------- SRB -----------------------------
    if (!strcmp(Protocol,"SRB")){

      //if filesize is not specified by user, get filesize info from MCAT
	printf("targetfilename: %s\n",targetfilename);
	printf("targetlocation: %s\n",targetlocation);
      if (filesize <= 0 ){ 
	printf("filesize: %d\n",filesize);
	
	if ( get_fileinfo(targetfilename, &filesize, targetlocation) 
	     != REQEX_SUCCESS) 
	  dprintf(D_ALWAYS,
		  "Error in getting fileinfo for:%s\n",targetfilename);
	else printf("get_fileinfo returns success!!\n");
      }
      printf("actual file size = %d\n",filesize);

      //====== STORE_TO_NEST ========
      if (storage_type == STORE_TO_NEST){

	/*-->

	//open nest connection
	NestConnection nestcon;
	OpenNestConnection(nestcon, NEST_HOST);
	AvailableNestSpace(nestfreespace, nestcon);
      
	if (nestfreespace < filesize + outfilesize){
	  printf("Not enough NEST diskspace to transfer file : %s, size = %d\n",
		 targetfilename, filesize);
	  CloseNestConnection(nestcon);
	}
	else { //we have available space
	  NestReservationHandle *handle;
	  AddNestGroup(nestcon, "condor");
	  AddUserToGroup(nestcon, "kosar", "condor");
	  ReserveUserSpace(handle, nestcon, "size", "time");
	  
	  write_log_initial(request_id,targetfilename,logfilename,filesize, 
			    "transfer_requested");
	  transfer_file_from_SRB(targetfilename, filesize, request_id);
	  CloseNestConnection(nestcon);
	}//else

	<--*/

      }//store_to_nest

      //====== STORE_TO_DISK ========
      else if (storage_type == STORE_TO_DISK){
	write_log_initial(request_id,targetfilename,logfilename,filesize, 
			  "transfer_requested");
	transfer_file_from_SRB(targetfilename, filesize, request_id);

      }//store_to_disk

    }//if Protocol ==  SRB


    
    //--------------------------- FTP --------------------------
    else if (!strcmp(Protocol,"FTP")){

      //if filesize is not specified by user.......?????
      if (filesize <= 0 ){ 
	filesize = 1000000;
      }
      
      if (reserve_disk_space(filesize + outfilesize) == REQEX_SUCCESS){
	write_log_initial(request_id,targetfilename,logfilename,filesize,
			  "transfer_requested");
	printf("get %s ..\n",targetfilename);
	transfer_file_from_FTP(targetfilename);
      }
      else{
	printf("Not enough space to transfer the file:%s\n",targetfilename);
	//continue;
      }
    }//else if Protocol == FTP

    //--------------------------- GSIFTP -----------------------

    else if (!strcmp(Protocol,"GSIFTP")){

      //if filesize is not specified by user.......?????
      if (filesize <= 0 ){ 
	filesize = 1000000;
      }      

      if (reserve_disk_space(filesize + outfilesize) == REQEX_SUCCESS){

 	scheduler.insert(request_id, current_time + duration, 
			 filesize + outfilesize);
	scheduler.print();

	write_log_initial(request_id,targetfilename,logfilename,filesize,
			  "transfer_requested");
	printf("get %s ..\n",targetfilename);
	//	transfer_file_from_GSIFTP(source_host, source_dir,targetfilename, request_id);
	thirdparty_GSIFTP_transfer(source_host,source_dir,targetfilename, NEST_HOST, SHARED_NEST_SPACE, targetfilename, request_id, "transfered");
      }
      else{
	printf("Not enough space to transfer the file:%s (%d)\n",
	       targetfilename,filesize);
	
	printf("scheduled_request:%s for time:%ld\n",
	       scheduled_request,scheduled_time);
	if (!strcmp(scheduled_request, "-1")){
	  strcpy(scheduled_request, request_id);
	  scheduled_time = scheduler.schedule(return_free_space(), 
					      filesize + outfilesize);
	
	  printf("Request:%s is scheduled for time :%ld\n",
		 scheduled_request, scheduled_time);
	}

      }
    }//else if Protocol == GSIFTP

    return TRUE;
}

/* ============================================================================
 * write incoming requests to file: <files-to-process>
 * ==========================================================================*/
void write_requests_to_file(int sockfd)
{
  ssize_t n;
  char line[MAX_TCP_BUF];
  FILE *inputfile;

  inputfile = fopen(inputfilename,"a+");
  if (inputfile == NULL){
    printf("Cannot create file:%s\n",inputfilename);
    exit(1);
  }

  for ( ; ; ){
    if ((n = recv(sockfd, line, MAX_TCP_BUF, 0)) == -1){
      printf("server: recv error!\n");
      exit(1);
    }
    else if (n == 0) break;
    
    line[n]='\0';
    //    printf("client: %s",line);
    fprintf(inputfile,"%s",line);
    fflush(inputfile);
  }
  
  fclose(inputfile);
}

/* ============================================================================
 * listen to the incoming requests from clients
 * ==========================================================================*/
//static void* listen_to_requests(void* pptr)
int listen_to_requests(void* pptr, Stream *sock)
{
  int listenfd, connfd;
  pid_t childpid;
  socklen_t clilen;
  struct sockaddr_in serv_addr, cli_addr;

  if ( (listenfd = socket(AF_INET, SOCK_STREAM, 0)) < 0){ 
    perror("server:cannot open socket!\n");	
    exit(1);			     		
  }				     		
  
  bzero(&serv_addr, sizeof(serv_addr)); 	
  serv_addr.sin_family      = AF_INET;   	
  serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  serv_addr.sin_port        = htons(SERV_TCP_PORT);

  if(bind(listenfd,(struct sockaddr*) &serv_addr, sizeof(serv_addr)) < 0){
    perror("reqex_server:cannot bind to local address!\n");
    exit(1);
  }		

  if (listen(listenfd, LISTENQ) < 0){
    perror("reqex_server:error in listen!\n");
    exit(1);
  }	
      
  printf("listening to requests from clients..\n");

  for ( ; ; ){
    clilen = sizeof(cli_addr);
    if ((connfd = accept(listenfd,(struct sockaddr*) &cli_addr, &clilen)) <0){
      perror("reqex_server:error in accept!\n");
      exit(1);
    }	
    
    //new connection
    if ((childpid = fork()) == 0){         //child process
      close(listenfd);
      write_requests_to_file(connfd);
      exit(0);
    }
    close(connfd);
  }


}

/* ============================================================================
 * reaper functions:
 * ==========================================================================*/
int srb_transfer_reaper(Service *,int pid,int exit_status)
{
  close_daemon_core_file_pointers();
  return TRUE;
}

int send_condor_job_reaper(Service *,int pid,int exit_status)
{
  close_daemon_core_file_pointers();
  return TRUE; 
}

int transfer_to_srb_reaper(Service *,int pid,int exit_status)
{
  close_daemon_core_file_pointers();
  return TRUE; 
}

int transfer_to_NEST_reaper(Service *,int pid,int exit_status)
{
  close_daemon_core_file_pointers();
  return TRUE; 
}

