#include "condor_common.h"
#include "dap_constants.h"
#include "dap_client_interface.h"
#include "dap_error.h"

/* ============================================================================
 * establish connection to the DaP server
 * ==========================================================================*/
int dap_open_connection(int & dap_conn, char * hostname)
{
  struct sockaddr_in serv_addr; 
  struct hostent* server;
  char * hostIP;
  
  if ( (server = gethostbyname(hostname)) == NULL ){
    fprintf(stderr, "Cannot get address for host %s\n",hostname);
    exit(1);	
  }
  
  if ( (dap_conn = socket(AF_INET,SOCK_STREAM,0)) < 0 ){
    fprintf(stderr, "Client:cannot open socket!\n");
    exit(1);
  }

  bzero(&serv_addr, sizeof(serv_addr)); 
  serv_addr.sin_family      = AF_INET;		
  memcpy(&(serv_addr.sin_addr.s_addr), server->h_addr, server->h_length);  
  serv_addr.sin_port        = htons(SERV_SUBMIT_TCP_PORT);

  hostIP = (char *)malloc(16*sizeof(char));
  hostIP = (char *)inet_ntoa(serv_addr.sin_addr);
  fprintf(stdout, "Trying %s..\n",hostIP);
  
  if ( connect(dap_conn, (struct sockaddr*) &serv_addr, sizeof(serv_addr)) != 0){
    fprintf(stderr, "Client:cannot connect to %s!\n", hostname);
    exit(1);
  }
  
  fprintf(stdout, "Connected to %s..\n",hostname);

  free(hostIP);
  
  return DAP_SUCCESS;
}

/* ============================================================================
 * close connection to the DaP server
 * ==========================================================================*/
int dap_close_connection(int dap_conn)
{
  if ( close(dap_conn) != 0){
    fprintf(stderr, "Error in closing the connection!\n");
    exit(1);
  }

  fprintf(stdout, "Connection closed..\n");
  return DAP_SUCCESS;
}

/* ============================================================================
 * send a given request through the socket dap_conn
 * ==========================================================================*/

unsigned long dap_send_request(int dap_conn, const char* request_str)
{
  
  char line[MAX_TCP_BUF];
  int n;

  if ( send (dap_conn, request_str, strlen(request_str),0) == -1){
    fprintf(stderr, "Client: send error!\n");
    exit(1);
  }

  if ((n = recv(dap_conn, line, MAX_TCP_BUF, 0)) == -1){
    fprintf(stderr, "Client: recv error!\n");
    exit(1);
  }
  line[n] = '\0';

  fprintf(stdout, "Request accepted by the server and assigned a dap_id: %s\n", line);
  
  //  return DAP_SUCCESS;
  return atol(line);

}









