/* ============================================================================
 * ReqEx_daemon_main.C - Request Executer Daemon
 * Reliable File Transfer through SRB
 * by Tevfik Kosar <kosart@cs.wisc.edu>
 * University of Wisconsin - Madison
 * March 2000
 * ==========================================================================*/

#include "condor_common.h"
#include "condor_debug.h"
#include "../condor_daemon_core.V6/condor_daemon_core.h"
#include "basename.h"
#include "dap_server.h"

// --------------
char* mySubSystem = "DATA_MANAGER";		// used by Daemon Core
// --------------

// command line arguments are read into these variables
char WORK_DIR[MAXSTR];
char * logfilename = NULL;
char * xmllogfilename = NULL;
char * userlogfilename = NULL;

extern int  submit_thread_reaper_id, status_thread_reaper_id;
extern int  transfer_dap_reaper_id, reserve_dap_reaper_id, release_dap_reaper_id;



/* ==========================================================================
 * ==========================================================================*/
void Usage()
{
  dprintf(D_ALWAYS,
	  "==============================================================\n");
  dprintf(D_ALWAYS,"USAGE: dap_server\n"); 
  dprintf(D_ALWAYS,"            -Daplog  <daplog filename>    //daplog in ClassAds\n");
  dprintf(D_ALWAYS,"          [ -Xmllog  <xmllog filename>  ] //daplog in XML format\n");
  dprintf(D_ALWAYS,"          [ -Userlog <userlog filename> ] //userlog in XMLformat\n");
  dprintf(D_ALWAYS,
	  "==============================================================\n");
  DC_Exit( 1 );
  
}

/* ==========================================================================
 * Parse command line arguments
 * ==========================================================================*/
void parse_arguments(int argc, char **argv)
{

  //check number of arguments
  if (argc < 2 ){
    Usage();
  }

  for (int i = 1; i < argc; i++) {
    if (!strcmp("-Daplog", argv[i])) {
      i++;
      if (argc <= i) {
	dprintf( D_ALWAYS, "No daplog file specified!\n" );
	Usage();
      }
      logfilename = argv[i];
    }
    else if (!strcmp("-Xmllog", argv[i])) {
      i++;
      if (argc <= i) {
	dprintf( D_ALWAYS, "No xmllog file specified!\n" );
	Usage();
      }
      xmllogfilename = argv[i];
    }
    else if (!strcmp("-Userlog", argv[i])) {
      i++;
      if (argc <= i) {
	dprintf( D_ALWAYS, "No userlog file specified!\n" );
	Usage();
      }
      userlogfilename = argv[i];
    }
    else{
      Usage();
    }
    
  }
      


  //dump the configuration
  dprintf(D_ALWAYS,
	  "==============================================================\n");
  dprintf(D_ALWAYS,
	  "              CONDOR_DATA_MANAGER CONFIGURATION:                 \n");
  dprintf(D_ALWAYS,
	   "==============================================================\n");
  dprintf(D_ALWAYS,
	  "WORK_DIR       : %s\n",WORK_DIR);
  
  //read the logfilename
  dprintf(D_ALWAYS,"DaP log file   : %s\n", logfilename);
  dprintf(D_ALWAYS,"Userlog file   : %s\n", userlogfilename);
  dprintf(D_ALWAYS,"XML log file   : %s\n", xmllogfilename);
//  sprintf(logfilename,"%s",argv[1]);
  
  dprintf(D_ALWAYS,
	  "==============================================================\n");
}


/* ============================================================================
 * initializations for the reqex_daemon
 * ==========================================================================*/
int main_init(int argc, char **argv)
{
  //change the working dir to the intial dir from which the program is actually called
  chdir(WORK_DIR);

  parse_arguments(argc, argv);

  //register timers (start_time,period,....)
  daemonCore->Register_Timer(0,
			     (TimerHandler)initializations,
			     "initializations");
  daemonCore->Register_Timer(1, 1,
			     (TimerHandler)call_main,
			     "call_main");

  daemonCore->Register_Timer(300, 300,
			     (TimerHandler)regular_check_for_requests_in_process,
			     "check_for_requests_in_process");
  daemonCore->Register_Timer(5, 5,
			     (TimerHandler)regular_check_for_rescheduled_requests,
			     "regular_check_for_rescheduled_requests");
    

  //register reaper functions
  transfer_dap_reaper_id = daemonCore->Register_Reaper("transfer_dap_reaper",(ReaperHandler)transfer_dap_reaper, "reaper for transfer DaP");

  reserve_dap_reaper_id = daemonCore->Register_Reaper("reserve_dap_reaper",(ReaperHandler)reserve_dap_reaper, "reaper for reserve DaP");

  release_dap_reaper_id = daemonCore->Register_Reaper("release_dap_reaper",(ReaperHandler)release_dap_reaper, "reaper for release DaP");

  return TRUE;
}

/* ============================================================================
 * reconfigure reqex_daemon
 * ==========================================================================*/
int main_config(bool v)
{
  dprintf(D_ALWAYS,"RECONFIGURING ......\n");
  return TRUE;
}

/* ============================================================================
 * shutdown reqex_daemon fast
 * ==========================================================================*/
int main_shutdown_fast()
{
  dprintf(D_ALWAYS,"SHUTDOWN FAST ....\n");
  
  DC_Exit(0);
  return TRUE;	// to satisfy c++
}

/* ============================================================================
 * shutdown reqex_daemon gracefully
 * ==========================================================================*/
int main_shutdown_graceful()
{
  dprintf(D_ALWAYS,"SHUTDOWN GRACEFULLY ....\n");
  exit(0);
  DC_Exit(0);
  return TRUE;	// to satisfy c++
}


void main_pre_dc_init(int argc, char **argv) {

  getwd(WORK_DIR);

}














