/* ============================================================================
 * dap_logger.C - DaP Scheduler Logging Utility
 * by Tevfik Kosar <kosart@cs.wisc.edu>
 * University of Wisconsin - Madison
 * 2001-2002
 * ==========================================================================*/

#include "dap_constants.h"
#include "dap_logger.h"
#include <pthread.h>

void write_dap_log(char *logfilename, char *status, char *param1, char *value1, char *param2 = NULL, char *value2 = NULL, char *param3 = NULL, char *value3 = NULL, char *param4 = NULL, char *value4 = NULL, char *param5 = NULL, char *value5 = NULL, char *param6 = NULL, char *value6 = NULL)
{
  
  // -------- ClassAd stuff ------------------------------
  
  //create ClassAd
  ClassAd *classad = new ClassAd;
  ClassAdParser parser;
  ExprTree *expr = NULL;

  //insert attributes to ClassAd
  classad->Insert("timestamp",Literal::MakeAbsTime());

  if ( !parser.ParseExpression(value1, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert(param1, expr);
  
  if ((param2 != NULL) && (value2 != NULL)){
    if ( !parser.ParseExpression(value2, expr) )
      dprintf(D_ALWAYS,"Parse error\n");
    classad->Insert(param2, expr);
  }

  if ((param3 != NULL) && (value3 != NULL)){
    if ( !parser.ParseExpression(value3, expr) )
      dprintf(D_ALWAYS,"Parse error\n");
    classad->Insert(param3, expr);
  }

  if ((param4 != NULL) && (value4 != NULL)){
    if ( !parser.ParseExpression(value4, expr) )
      dprintf(D_ALWAYS,"Parse error\n");
    classad->Insert(param4, expr);
  }

  if ((param5 != NULL) && (value5 != NULL)){
    if ( !parser.ParseExpression(value5, expr) )
      dprintf(D_ALWAYS,"Parse error\n");
    classad->Insert(param5, expr);
  }

  if ((param6 != NULL) && (value6 != NULL)){
    if ( !parser.ParseExpression(value6, expr) )
      dprintf(D_ALWAYS,"Parse error\n");
    classad->Insert(param6, expr);
  }

  if ( !parser.ParseExpression(status, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("status", expr);


  //parse the ClassAd
  PrettyPrint unparser;
  string adbuffer = "";
  unparser.Unparse(adbuffer, classad);

  //destroy the ClassAd
  //  delete expr;
  delete classad;

  // -------- End of ClassAd stuff ------------------------------

  FILE *flog;

  //write the classad to classad file
  if ((flog = fopen(logfilename,"a+")) == NULL){
    dprintf(D_ALWAYS,
	    "cannot open logfile :%s...\n",logfilename);
    exit(1);
  }
  fprintf (flog,"%s",adbuffer.c_str());
  fclose(flog);

}

//--------------------------------------------------------------------------
void write_classad_log(char *logfilename, char *status, ClassAd *classad)
{

  ClassAdParser parser;
  ExprTree *expr = NULL;
  PrettyPrint unparser;
  string adbuffer = "";
  FILE *flog;

  printf("Logging the classad..\n");

  //insert attributes to ClassAd
  classad->Insert("timestamp",Literal::MakeAbsTime());

  printf("*1*\n");

  if ( !parser.ParseExpression(status, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("status", expr);

  printf("*2*\n");
  /*
  if (!(expr = classad->Lookup("src_url")) ){
    printf("%s not found!\n", "src_url");
    return;
  }
  unparser.Unparse(adbuffer,expr);
  printf("VALUE => %s\n",adbuffer.c_str());
  */

  //parse the ClassAd
  unparser.Unparse(adbuffer, classad);
  
  printf("*3*\n");

  //write the classad to classad file
  if ((flog = fopen(logfilename,"a+")) == NULL){
    dprintf(D_ALWAYS,
	    "cannot open logfile :%s...\n",logfilename);
    exit(1);
  }

  printf("*4*\n");

  fprintf (flog,"%s",adbuffer.c_str());

  printf("*5*\n");

  fclose(flog);
}

//--------------------------------------------------------------------------
void write_collection_log(ClassAdCollection *dapcollection, char *dap_id, 
			  const char *update)
{
  //log the new status of the request
  ClassAdParser parser;
  string key;
  key = "key = ";
  key += dap_id;
  
  string partial_s = "[ ";
  partial_s += update;
  partial_s += " ]";
  
  ClassAd* partialAd = parser.ParseClassAd(partial_s, true);
  partialAd->Insert("timestamp",Literal::MakeAbsTime());
    
  PrettyPrint unparser;
  string adbuffer = "";
  unparser.Unparse(adbuffer, partialAd);

  string modify_s ="[ Updates  = ";
  modify_s += adbuffer;
  modify_s += " ]";
  
  ClassAd* tmpad = parser.ParseClassAd(modify_s, true);

  if (!dapcollection->ModifyClassAd(key, tmpad)){
    dprintf(D_ALWAYS, "ModifyClassAd failed! dap_id:%s\n", dap_id);
    return;
  }

  //  printf("ClassAd modified! dap_id:%s\n", dap_id);
}
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
void write_xml_log(char *logfilename, ClassAd *classad, const char *status)
{

  ClassAdParser parser;
  ExprTree *expr = NULL;
  ClassAdXMLUnParser  xmlunparser;
  string adbuffer = "";
  FILE *flog;

  //insert attributes to ClassAd
  classad->Insert("timestamp",Literal::MakeAbsTime());

  //update status
  classad->Delete("status");
  if ( !parser.ParseExpression(status, expr) )
    dprintf(D_ALWAYS,"Parse error\n");
  classad->Insert("status", expr);

  if (!strcmp(status, "\"processing_request\"") || 
      !strcmp(status, "\"request_rescheduled\"")){
    classad->Delete("error_code");
  }
  
  //convert classad to XML
  xmlunparser.SetCompactSpacing(false);
  xmlunparser.Unparse(adbuffer, classad);
  
  //write the classad to classad file
  if ((flog = fopen(logfilename,"a+")) == NULL){
    dprintf(D_ALWAYS,
	    "cannot open logfile :%s...\n",logfilename);
    exit(1);
  }
  fprintf (flog,"%s",adbuffer.c_str());
  fclose(flog);
}

//--------------------------------------------------------------------------
void write_xml_user_log(char *logfilename,char *param1, char *value1, char *param2 = NULL, char *value2 = NULL, char *param3 = NULL, char *value3 = NULL, char *param4 = NULL, char *value4 = NULL, char *param5 = NULL, char *value5 = NULL, char *param6 = NULL, char *value6 = NULL, char *param7 = NULL, char *value7 = NULL,char *param8 = NULL, char *value8 = NULL )
{

  ClassAdParser parser;
  ExprTree *expr = NULL;
  ClassAdXMLUnParser  xmlunparser;
  string adbuffer = "";
  FILE *flog;

  //create ClassAd
  ClassAd *classad = new ClassAd;

  //insert attributes to ClassAd
  classad->Insert("EventTime",Literal::MakeAbsTime());

  if ( parser.ParseExpression(value1, expr) )
    classad->Insert(param1, expr);

   if ((param2 != NULL) && (value2 != NULL)){
    if ( parser.ParseExpression(value2, expr) )
      classad->Insert(param2, expr);
  }

  if ((param3 != NULL) && (value3 != NULL)){
    if ( parser.ParseExpression(value3, expr) )
      classad->Insert(param3, expr);
  }

  if ((param4 != NULL) && (value4 != NULL)){
    if ( parser.ParseExpression(value4, expr) )
      classad->Insert(param4, expr);
  }

  if ((param5 != NULL) && (value5 != NULL)){
    if ( parser.ParseExpression(value5, expr) )
      classad->Insert(param5, expr);
  }

  if ((param6 != NULL) && (value6 != NULL)){
    if ( parser.ParseExpression(value6, expr) )
      classad->Insert(param6, expr);
  }

  if ((param7 != NULL) && (value7 != NULL)){
    if ( parser.ParseExpression(value7, expr) )
      classad->Insert(param7, expr);
  }
  
  if ((param8 != NULL) && (value8 != NULL)){
    if ( parser.ParseExpression(value8, expr) )
      classad->Insert(param8, expr);
  }

  //convert classad to XML
  xmlunparser.SetCompactSpacing(false);
  xmlunparser.Unparse(adbuffer, classad);
  
  //write the classad to classad file
  if ((flog = fopen(logfilename,"a+")) == NULL){
    dprintf(D_ALWAYS,
	    "cannot open logfile :%s...\n",logfilename);
    exit(1);
  }
  fprintf (flog,"%s",adbuffer.c_str());
  fclose(flog);
}







