#include "condor_common.h"
#include "dap_constants.h"
#include "dap_server_interface.h"

int main(int argc, char **argv)
{
  char dap_id[MAXSTR], hostname[MAXSTR];
  int dap_conn;
  struct sockaddr_in serv_addr; 
  struct hostent* server;
  char * hostIP;

  //check number of arguments
  if (argc < 3){
    fprintf(stderr, "==============================================================\n");
    fprintf(stderr, "USAGE: %s <host_name> <dap_id>\n", argv[0]);
    fprintf(stderr, "==============================================================\n");
    exit( 1 );
  }
  strncpy(hostname, argv[1], MAXSTR);
  strncpy(dap_id, argv[2], MAXSTR);
  

  //open connection to the dap_server
  if ( (server = gethostbyname(hostname)) == NULL ){
    fprintf(stderr,"Cannot get address for host %s\n",hostname);
    exit(1);	
  }
  
  if ( (dap_conn = socket(AF_INET,SOCK_STREAM,0)) < 0 ){
    fprintf(stderr, "Client:cannot open socket!\n");
    exit(1);
  }

  bzero(&serv_addr, sizeof(serv_addr)); 
  serv_addr.sin_family      = AF_INET;		
  memcpy(&(serv_addr.sin_addr.s_addr), server->h_addr, server->h_length);  
  serv_addr.sin_port        = htons(SERV_REMOVE_TCP_PORT);

  hostIP = (char *)malloc(16*sizeof(char));
  hostIP = (char *)inet_ntoa(serv_addr.sin_addr);
  fprintf(stdout, "Trying %s..\n",hostIP);
  
  if ( connect(dap_conn, (struct sockaddr*) &serv_addr, sizeof(serv_addr)) != 0){
    fprintf(stderr, "Client:cannot connect to %s!\n", hostname);
    exit(1);
  }
  
  fprintf(stdout, "Connected to %s..\n",hostname);

  free(hostIP);


  //send status request to the dap server 
  
  if ( send (dap_conn, dap_id, strlen(dap_id),0) == -1){
    fprintf(stderr, "Client: send error!\n");
    exit(1);
  }

  //listen to the reply from dap server
  char line[MAX_TCP_BUF];
  int n;
  
  if ((n = recv(dap_conn, line, MAX_TCP_BUF, 0)) == -1){
    fprintf(stderr, "Client: recv error!\n");
    exit(1);
  }
  line[n] = '\0';

  fprintf(stdout, "===============\n");
  fprintf(stdout, "%s", line);
  fprintf(stdout, "===============\n");

  //close connection to the dap_server
  if ( close(dap_conn) != 0){
    fprintf(stderr, "error in closeing the connection!\n");
    exit(1);
  }
  
  fprintf(stdout, "Connection closed..\n");
  
}











