#include "dap_constants.h"
#include "condor_common.h"
#include "dap_client_interface.h"
#include "dap_classad_reader.h"
#include "dap_utility.h"

int check_dap_format(ClassAd *currentAd)
{
  char dap_type[MAXSTR];
  string adbuffer = "";
  ExprTree *attrexpr = NULL;
  ClassAdUnParser unparser;

  //should be context insensitive..
  //just check the format, not the content!
  //except dap_type ??
  
  if ( !(attrexpr = currentAd->Lookup("dap_type")) ) return 0;
  else{
    unparser.Unparse(adbuffer,attrexpr);

    char tmp_adbuffer[MAXSTR];
    strncpy(tmp_adbuffer, adbuffer.c_str(), MAXSTR);
    strcpy(dap_type,strip_str(tmp_adbuffer));

    if ( !strcmp(dap_type, "transfer") ){
      if ( !currentAd->Lookup("src_url") )  return 0;
      if ( !currentAd->Lookup("dest_url") ) return 0;
    }
    
    else if ( !strcmp(dap_type, "reserve") ){
      if ( !currentAd->Lookup("dest_host") )    return 0;
      if ( !currentAd->Lookup("reserve_size") ) return 0;
      if ( !currentAd->Lookup("duration") )     return 0;
      if ( !currentAd->Lookup("reserve_id") )   return 0;
    }

    else if ( !strcmp(dap_type, "release") ){
      if ( !currentAd->Lookup("dest_host") ) return 0;
      if ( !currentAd->Lookup("reserve_id") ) return 0;
    }
    
    else return 0;
  }
  
  
  return 1;
}
//============================================================================
void Usage(char *argv0)
{
  fprintf(stderr, "==============================================================\n");
  fprintf(stderr, "USAGE: %s <host_name> <submit_file> [<options]\n", argv0);
  fprintf(stderr, "\nValid options:\n");
  fprintf(stderr, "       -lognotes \"notes\"      add lognote to submit file before processing\n");
  fprintf(stderr, "==============================================================\n");
  exit( 1 );
}

/* ============================================================================
 * main body of dap_submit
 * ==========================================================================*/
int main(int argc, char **argv)
{
  int i; char c;
  string adstr="";
  ClassAdParser parser;
  ClassAd *currentAd = NULL;
  int leftparan = 0;
  FILE *adfile;
  char fname[MAXSTR], hostname[MAXSTR];
  DapConnection dap_conn;
  char *lognotes = NULL;

  //check number of arguments
  if (argc < 3){
    Usage(argv[0]);
  }

  strcpy(hostname, argv[1]);
  strcpy(fname, argv[2]);

  for (i = 3; i < argc; i++) {
    if (!strcmp("-lognotes", argv[i])) {
      i++;
      if (argc <= i) {
	fprintf(stderr, "No line to add to the submit file specified!\n" );
	Usage(argv[0]);
      }
      fprintf(stdout, "Lognotes: %s\n",argv[i]);
      
      lognotes = (char*) malloc(MAXSTR * sizeof(char));
      snprintf(lognotes, MAXSTR, "\"%s\"",argv[i]);
    }
    else{
      Usage(argv[0]);
    }
    
  }
  

  //open the submit file
  adfile = fopen(fname,"r");
  if (adfile == NULL) {
    fprintf(stderr, "Cannot open file:%s\n",fname);
    exit(1);
  }

  //open the connection to the DaP server
  dap_open_connection(dap_conn, argv[1]);
  
  //read the request classAds
  while(1){
    i =0;
    while (1){
      c = fgetc(adfile);
      if (c == ']'){ 
	leftparan --; 
	if (leftparan == 0) break;
      }
      if (c == '[') leftparan ++; 
      
      if (c == EOF) {
	dap_close_connection(dap_conn); 
	return 0;
      }
      
      adstr += c;
      i++;
    }
    adstr += c;
    
    //check the validity of the request
    currentAd = parser.ParseClassAd(adstr);
    if (currentAd == NULL) {
      fprintf(stderr, "Invalid input format! Not a valid classad!\n");
      dap_close_connection(dap_conn);    
      return 0;
    }

    //add lognotes to the submit classad 
    if (lognotes){
      ExprTree *expr = NULL;
      if ( !parser.ParseExpression(lognotes, expr) )
	fprintf(stderr,"Parse error in lognotes\n");
      currentAd->Insert("lognotes", expr);
    }
    
    //check format of the submit classad
    if ( !check_dap_format(currentAd)){
      fprintf(stderr, "========================\n");
      fprintf(stderr, "Not a valid DaP request!\nPlease check your submit file and then resubmit...\n");
      fprintf(stderr, "========================\n");
      dap_close_connection(dap_conn);    
      return 0;
    }
    
    //if input is valid, then send the request:
    PrettyPrint unparser;
    string adbuffer = "";
    unparser.Unparse(adbuffer, currentAd);
    fprintf(stdout, "================\n");
    fprintf(stdout, "Sending request:");
    fprintf(stdout, "%s\n", adbuffer.c_str());
    
    //    char tmp_adstr[MAXSTR];
    //strncpy(tmp_adstr, adstr.c_str(), MAXSTR);
    dap_send_request(dap_conn, adbuffer.c_str());
    fprintf(stdout, "================\n");

    //    printf("request sent!\n");
    adstr = "";
  }
  
  //close the connection to the server
  dap_close_connection(dap_conn);    
  
}











