#include "condor_common.h"
#include "dap_constants.h"
#include "dap_error.h"
#include "dap_utility.h"
#include "condor_string.h"
#include <unistd.h>
#include <string>


int transfer_from_to_fnalsrm(char *src_url, char *dest_url, 
			     char *arguments, char *error_str)
{
  FILE *pipe = 0, *outfile;
  char pipecommand[MAXSTR];
  char linebuf[MAXSTR];
  char status[MAXSTR];
  int ret;
  
  snprintf(pipecommand, MAXSTR, "%s/srmcp %s %s %s 2>&1| cat", 
	  FNALSRM_BIN_DIR, arguments, src_url, dest_url);

  pipe = popen (pipecommand, "r");

  if ( pipe == 0 ) {
    fprintf(stderr,"Error:couldn't open pipe to hrm-get.linux !\n");
    return -1;
  }

  while ( 1 ) {
    if ( fgets( linebuf, MAXSTR, pipe ) == 0 ) {
      break;
    }
    if (sscanf(linebuf, "copying ... %s", status ) > 0){
      if (!strcmp(status, "done")){
	return DAP_SUCCESS;
      }
    }
    
    fprintf(stdout, ">%s", linebuf);
  }
  
  
  pclose(pipe);

  strncpy(error_str, linebuf, MAXSTR);
  return DAP_ERROR;
}

int main(int argc, char *argv[])
{
  char src_url[MAXSTR], dest_url[MAXSTR], arguments[MAXSTR];
  char src_protocol[MAXSTR], src_host[MAXSTR], src_file[MAXSTR];
  char dest_protocol[MAXSTR], dest_host[MAXSTR], dest_file[MAXSTR];
  char error_str[MAXSTR];
  int status;
  
 if (argc < 3){
    fprintf(stdout, "==============================================================\n");
    fprintf(stdout, "USAGE: %s <src_url> <dest_url> [<arguments>]\n", argv[0]);
    fprintf(stdout, "==============================================================\n");
    exit(-1);
  }

  strncpy(src_url, argv[1], MAXSTR);
  strncpy(dest_url, argv[2], MAXSTR);
  if (argc >= 4) strncpy(arguments, argv[3], MAXSTR); 
  else strncpy(arguments, "", MAXSTR);

  parse_url(src_url, src_protocol, src_host, src_file);
  parse_url(dest_url, dest_protocol, dest_host, dest_file);
  
  if (!strcmp(src_protocol, "fnalsrm")){
    snprintf(src_url, MAXSTR, "srm://%s%s", src_host, src_file);
  }
  if (!strcmp(dest_protocol, "fnalsrm")){
    snprintf(dest_url, MAXSTR, "srm://%s%s", dest_host, dest_file);
  }
  
  fprintf(stdout, "Transfering from: %s to: %s with arguments: %s\n", 
	  src_url, dest_url, arguments);

  status = transfer_from_to_fnalsrm(src_url, dest_url, arguments, error_str); 

  //-- output to a temporary file....

  
  if (status != DAP_SUCCESS){
    unsigned int mypid;
    FILE *f;
    char fname[MAXSTR];
    
    mypid = getpid();
    snprintf(fname, MAXSTR, "out.%d", mypid);

    f = fopen(fname, "w");
    fprintf(f, "FNAL SRM error: %s", error_str);
    fclose(f);  
  }
  //--

  return status;
}







