#include "condor_common.h"
#include "dap_constants.h"
#include "dap_error.h"
#include "dap_utility.h"
#include "condor_string.h"
#include <unistd.h>
#include <string>


int transfer_globus_url_copy(char *src_url, char *dest_url, 
			     char *arguments, char *error_str)
{
  FILE *pipe = 0, *outfile;
  char pipecommand[MAXSTR];
  char linebuf[MAXSTR] = "";
  char status[MAXSTR];
  int ret;


  snprintf(pipecommand, MAXSTR, 
	   "%s/globus-url-copy %s %s %s 2>&1| cat",
	   GLOBUS_BIN_DIR, arguments, src_url, dest_url);
  
  //  "%s/globus-url-copy -tcp-bs 208896 -bs 1048576 -p 4

  pipe = popen (pipecommand, "r");

  if ( pipe == 0 ) {
    fprintf(stderr,"Error:couldn't open pipe to globus-url-copy !\n");
    return -1;
  }

  /*
  while ( 1 ) {
    if ( fgets( linebuf, MAXSTR, pipe ) == 0 ) {
      break;
    }
    
    if (sscanf(linebuf, "copying ... %s", status ) > 0){
      if (!strcmp(status, "done")){
	return DAP_SUCCESS;
      }
    }
    fprintf(stdout, ">%s", linebuf);
  }
  */
  
  ret = pclose(pipe);
  //  fprintf(stdout, "Cammand terminated with err_code: %d\n", ret);

  if (ret == 0)
    return DAP_SUCCESS;
  else {
    strncpy(error_str, linebuf, MAXSTR);
    return DAP_ERROR;
  }
  
}

int main(int argc, char *argv[])
{
  char src_url[MAXSTR], dest_url[MAXSTR], arguments[MAXSTR];
  char src_protocol[MAXSTR], src_host[MAXSTR], src_file[MAXSTR];
  char dest_protocol[MAXSTR], dest_host[MAXSTR], dest_file[MAXSTR];
  char error_str[MAXSTR];
  int status;
  
 if (argc < 3){
    fprintf(stderr, "==============================================================\n");
    fprintf(stderr, "USAGE: %s <src_url> <dest_url> [<arguments>]\n", argv[0]);
    fprintf(stderr, "==============================================================\n");
    exit(-1);
  }

  strncpy(src_url, argv[1], MAXSTR);
  strncpy(dest_url, argv[2], MAXSTR);
  if (argc >= 4) strncpy(arguments, argv[3], MAXSTR); 
  else strncpy(arguments, "", MAXSTR);
  
  fprintf(stdout, "Transfering from: %s to: %s with arguments: %s\n", 
	  src_url, dest_url, arguments);

  status = transfer_globus_url_copy(src_url, dest_url, arguments, error_str); 

  //-- output to a temporary file....

  
  if (status != DAP_SUCCESS){
    unsigned int mypid;
    FILE *f;
    char fname[MAXSTR];
    
    mypid = getpid();
    snprintf(fname, MAXSTR, "out.%d", mypid);

    f = fopen(fname, "w");
    fprintf(f, "GLOBUS error: %s", error_str);
    fclose(f);  
  }
  //--

  return status;
}







