#include "condor_common.h"
#include "dap_constants.h"
#include "dap_utility.h"
#include "dap_error.h"
#include "dap_logger.h"
#include "nest_speak.h"
#include <unistd.h>

#define return_error {free(buf);NestCloseConnection(nest_con_in);NestCloseConnection(nest_con_out);return DAP_ERROR;} 

int transfer_from_nest_to_nest(char *src_host, char *src_file, char *dest_host, char *dest_file, NestReplyStatus & status)
{
    char *buf;
    long src_filesize, dest_filesize;
    int bytes_read;
    NestConnection nest_con_in, nest_con_out;

    //allocate memeory for buf
    buf = (char*)malloc(sizeof(char) * BUF_SIZE);
    if (buf == NULL) return DAP_ERROR;

    //open connection to src host
    status = NestOpenConnection(nest_con_in, src_host);
    if (status != NEST_SUCCESS) {
      free(buf);
      return DAP_ERROR;
    }

    //open connection to dest host
    status = NestOpenConnection(nest_con_out, dest_host);
    if (status != NEST_SUCCESS) {
      free(buf);
      NestCloseConnection(nest_con_in);
      return DAP_ERROR;
    }
    
    //get the filesize info for the src file
    status = NestFilesize(src_filesize, src_file, nest_con_in);
    if (status != NEST_SUCCESS) return_error;


    //transfer file from src to dest
    status = NestReadBytes(nest_con_in, src_file, 0, buf, BUF_SIZE , bytes_read);
    if (status != NEST_SUCCESS) return_error;

    int i = 1;
    while (bytes_read > 0){
      status = NestWriteBytes(nest_con_out, dest_file, (i-1)*BUF_SIZE ,
			      buf, bytes_read, 1 );
      if (status != NEST_SUCCESS) return_error;

      status = NestReadBytes(nest_con_in, src_file, i*BUF_SIZE, 
			     buf, BUF_SIZE, bytes_read);
      if (status != NEST_SUCCESS) return_error;

      i++;
    }

    //get the filesize info for the src file
    status = NestFilesize(dest_filesize, src_file, nest_con_in);
    if (status != NEST_SUCCESS) return_error;

    if (src_filesize != dest_filesize) return_error; 

    //otherwise file is transfered succesfully..
    free(buf);
    NestCloseConnection(nest_con_in);
    NestCloseConnection(nest_con_out);
    return DAP_SUCCESS;

}

int main(int argc, char *argv[])
{

  char src_protocol[MAXSTR], src_host[MAXSTR], src_file[MAXSTR];
  char dest_protocol[MAXSTR], dest_host[MAXSTR], dest_file[MAXSTR];
  char src_url[MAXSTR], dest_url[MAXSTR];
  int status;

  if (argc < 3){
    fprintf(stderr, "==============================================================\n");
    fprintf(stderr, "USAGE: %s <src_url> <dest_url>\n", argv[0]);
    fprintf(stderr, "==============================================================\n");
    exit(-1);
  }

  strncpy(src_url, argv[1], MAXSTR);
  strncpy(dest_url, argv[2], MAXSTR);
 
  parse_url(src_url, src_protocol, src_host, src_file);
  parse_url(dest_url, dest_protocol, dest_host, dest_file);
  
  fprintf(stdout, "Transfering file:%s from host %s to file:%s on host %s..\n",
	  src_file, src_host, dest_file, dest_host);
  
  NestReplyStatus nest_status;
  status = transfer_from_nest_to_nest(src_host, src_file, dest_host, 
				      dest_file, nest_status);
  
  //-- output to a temporary file....
  if (status != DAP_SUCCESS){
    unsigned int mypid;
    FILE *f;
    char fname[MAXSTR];
    
    mypid = getpid();
    snprintf(fname, MAXSTR, "out.%d", mypid);
    
    f = fopen(fname, "w");
    fprintf(f, "NeST error: %d", nest_status);
    fclose(f);  
  }
  //--
  
  return status;
  
}





