#include "condor_common.h"
#include "dap_constants.h"
#include "dap_error.h"
#include "dap_utility.h"
#include <unistd.h>

int transfer_from_srb_to_file(char *src_file, char *dest_file)
{

  FILE *pipe = 0, *outfile;
  char pipecommand[MAXSTR];
  char linebuf[MAXSTR];
  char cacheurl[MAXSTR], targeturl[MAXSTR];
  int ret;
  
  snprintf(pipecommand, MAXSTR, 
	   "%s/Sget -f %s %s 2>&1| cat", SRB_BIN_DIR, 
	   src_file, dest_file);
  

  pipe = popen (pipecommand, "r");
  if ( pipe == 0 ) {
    fprintf(stderr,"Error:couldn't open pipe to Sget!\n");
    return -1;
  }

  /*
  while ( 1 ) {
    if ( fgets( linebuf, MAXSTR, pipe ) == 0 ) {
      break;
    }
    if (sscanf(linebuf, "HRM_GRIDFTP: file transfer completed from %s to %s",
	       cacheurl, targeturl ) > 0){
      if (!strcmp(targeturl, dest_url)){
	return DAP_SUCCESS;
      }
    }
    
    //fprintf(stdout, ">%s", linebuf);
  }
  */
  
  ret = pclose(pipe);
  fprintf(stdout, "Cammand terminated with err_code: %d\n", ret);
  
  if (ret == 0) 
    return DAP_SUCCESS;
  else 
    return DAP_ERROR;
}

int main(int argc, char *argv[])
{
  char src_url[MAXSTR], dest_url[MAXSTR];
  char src_protocol[MAXSTR], src_host[MAXSTR], src_file[MAXSTR];
  char dest_protocol[MAXSTR], dest_host[MAXSTR], dest_file[MAXSTR];
  int status;

  if (argc < 3){
    fprintf(stderr, "==============================================================\n");
    fprintf(stderr, "USAGE: %s <src_url> <dest_url>\n", argv[0]);
    fprintf(stderr, "==============================================================\n");
    exit(-1);
  }

  strncpy(src_url, argv[1], MAXSTR);
  strncpy(dest_url, argv[2], MAXSTR);

  parse_url(src_url, src_protocol, src_host, src_file);
  parse_url(dest_url, dest_protocol, dest_host, dest_file);
  
  fprintf(stdout, "FROM: %s:%s:%s\n",
	  src_protocol, src_host, src_file );
  fprintf(stdout, "TO  : %s:%s:%s\n",
	  dest_protocol, dest_host, dest_file);
  
  fprintf(stdout, "Transfering from: %s to: %s\n", src_url, dest_url);

  status = transfer_from_srb_to_file(strip_str(src_file),
				     strip_str(dest_file)); 

  //-- output to a temporary file....
  if (status != DAP_SUCCESS){
    unsigned int mypid;
    FILE *f;
    char fname[MAXSTR];
    
    mypid = getpid();
    snprintf(fname, MAXSTR, "out.%d", mypid);

    f = fopen(fname, "w");
    fprintf(f, "SRB error!");
    fclose(f);  
  }
  //--

  return status;
}







