// hrm_call_back_impl.C (2000-Nov-30)
//
// Peter Couvares <pfc@cs.wisc.edu>
// University of Wisconsin-Madison

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <OB/CORBA.h>

#include "hrmDefs.h"
#include "hrm.h"
#include "hrm_call_back_impl.h"
#include "demo.h"
#include "util.h"

#define DEBUG 1

CORBA::Boolean
client_call_back_impl::staged( const char* request_id,
							   const HRM::RETURN_STATUS_SET_T& status_set )
{
	CORBA::ULong i, j;
	
	printf( "Callback recieved for %lu file%s from request \"%s\":\n",
			status_set.length(), (status_set.length() > 1 ? "s" : ""),
			request_id );

	for( i=0; i < status_set.length(); i++ ) {
		cout << "\t" << status_set[i].fid << " (status " << status_set[i].code
			 << ": " << status_set[i].explanation << ")\n";

		const char* fname = status_set[i].fid;

		if( !strcmp( fname, "NO MORE FILES" ) ) {
			printf( "Project finished...\n" );
			return 1;
		}
	}

	HRM::FIDSET_T fset;
	fset.length( status_set.length() );
	for( i=0; i < status_set.length(); i++ ) {
		fset[i] = status_set[i].fid;
	}

	HRM::PROTOCOL_SET_T pset;
	pset.length(5);
	pset[0] = "scp";
	pset[1] = "gsiftp";
	pset[2] = "srb";
	pset[3] = "magic carpet";
	pset[4] = "camel";

	for( i=0; i < status_set.length(); i++ ) {
		if( status_set[i].code == 1 ) {
			if( DEBUG )
				cout << "Getting URL for " << status_set[i].fid << " ...\n";

			HRM::URL_INFO_SET_T* urlInfoSet =
				hrmServant -> getUrls( userId, request_id, pset, fset );

			for( j=0; j < urlInfoSet->length(); j++ ) {
				cout << "\t" << (*urlInfoSet)[j].url << endl;
			}

			if( DEBUG )
				fprintf( stdout,
						 "Transferring the file%s...\n",
						 (status_set.length() > 1 ? "s" : "" ) );

			char cmdbuf[1024];
			char *url, *token, *host_arg, *file_arg;
			for( j=0; j < urlInfoSet->length(); j++ ) {
				// extract the host & filename from the URL so we can
				// reformat it for the scp command
				url = (*urlInfoSet)[j].url;
				if( DEBUG )
					fprintf( stdout, "\t%s\n", url );
				token = strtok( url, "//" );
				host_arg = strtok( NULL, "/" );
				file_arg = strtok( NULL, "\0" );
				snprintf( cmdbuf, 1024, "scp sc2000@%s:/%s .", host_arg,
						  file_arg );

				// transfer the file
				system( cmdbuf );
			}

			if( DEBUG )
				fprintf( stdout, "Releasing the staged file%s...\n",
						 (urlInfoSet->length() > 1 ? "s" : "") );

			hrmServant -> release( userId, request_id, fset );
			
			if( DEBUG )
				fprintf( stdout, "Removing the local file cop%s...\n",
						 (urlInfoSet->length() > 1 ? "ies" : "y") );
			unlink( basename( file_arg ) );
		} 
	}

	if( DEBUG )
		fprintf( stdout, "Waiting for more callbacks...\n" );

	return 1;
}
