/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/
#ifndef __NETWORK_BANDWIDTH_H__
#define __NETWORK_BANDWIDTH_H__

#include "HashTable.h"
#include "router.h"
#include "netcapacity.h"

class NetworkBandwidthAllocator {
public:
	NetworkBandwidthAllocator() : network(255, addrhash), router(NULL),
		capacityinfo(NULL){}
	~NetworkBandwidthAllocator();

	void BindRouter(Router *r) { router = r; }
	void BindCapacity(NetworkCapacity *c) { capacityinfo = c; } 

		// returns 1 if request granted, 0 if denied, and -1 if error
		// minMbps and maxMbps are the input parameters for the
		// request; maxMbps returns the allocated maximum
		// if canDeny==false, at least minMbps will be allocated
	int RequestBandwidth(double minMbps, double &maxMbps, const char source[],
						 const char destination[], bool canDeny=true);

		// returns 0 on success and -1 on error
	int ReleaseBandwidth(double allocMbps, const char source[],
						 const char destination[]);

		// sets availMbps to the available bandwidth between source and dest
		// sets totMbps to the total bandwidth between source and dest
		// returns 0 on success and -1 on error
	int Bandwidth(double &availMbps, double &totMbps, const char source[],
				  const char destination[]);

private:
	HashTable<unsigned int, double> network;

	Router *router;
	NetworkCapacity *capacityinfo;
};

#endif
