/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "condor_debug.h"
#include "netcapacity.h"

int
NetworkCapacity::LoadNetworkCapacity(const char filename[])
{
	FILE *fp = NULL;
	if ((fp = fopen(filename, "r")) == NULL) {
		dprintf(D_ALWAYS, "NetworkCapacity: can't open \"%s\"\n", filename);
		return -1;
	}

		// start with a clean capacity table
	Cleanup();

	char line[80];
	int lc = 0;
	while (fgets(line, 80, fp)) {
		lc++;
		char *tok;
		if ((tok = strtok(line, " \t\n")) == NULL) continue;
		if (tok[0] == '#') continue;
		unsigned int addr = ntohl(inet_addr(tok));
		if ((tok = strtok(NULL, " \t\n")) == NULL) {
				// parse error
			dprintf(D_ALWAYS,
					"NetCapacity: parse error on line %d of \"%s\"\n",
					lc, filename);
			return -1;
		}
		double capacity = atof(tok);
		SetCapacity(addr, capacity);
	}


	return 0;
}

int
NetworkCapacity::SetCapacity(unsigned int network, double Mbps)
{
	return cap.insert(network, Mbps);
}

int
NetworkCapacity::GetCapacity(unsigned int network, double &Mbps)
{
	return cap.lookup(network, Mbps);
}
