/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "condor_debug.h"
#include "netcapacityalloc.h"

void
usage(char progname[])
{
	fprintf(stderr, "usage: %s [-DEBUG-FLAGS ...] "
			"[capacity-info-file route-info-file]\n", progname);
	exit(1);
}

int
main(int argc, char *argv[])
{
	if (argc == 2 && strncmp(argv[1], "-h", 2) == 0) usage(argv[0]);

	NetworkCapacityAllocator netcapalloc;
	netcapalloc.SetHorizon(30);
	Router router;
	NetworkCapacity capinfo;
	netcapalloc.BindRouter(&router);
	netcapalloc.BindCapacity(&capinfo);

	argv++;
	argc--;
	while (argc && strncmp(*argv, "-D", 2) == 0) {
		Termlog = 1;
		set_debug_flags((*argv)+1);
		argv++;
		argc--;
	}

	if (argc == 2) {
		if (capinfo.LoadNetworkCapacity(argv[1]) < 0) {
			fprintf(stderr, "network capacity load failed\n");
			exit(1);
		}
		if (router.LoadRoutingTable(argv[2]) < 0) {
			fprintf(stderr, "network routing table load failed\n");
			exit(1);
		}
		fprintf(stderr, "network load completed\n");
	}

	printf("Enter commands: r[equest] bytes source destination\n");
	printf("                c[ommit]\n");
	printf("                a[bort]\n");
	printf("                l[oad] capacity-info-file route-info-file\n");
	printf("                b[enchmark] iterations\n");
	printf("> ");

	char line[80];
	while (fgets(line, 80, stdin)) {
		char *tok;
		if ((tok = strtok(line, " \t\n")) == NULL) continue;
		switch (tok[0]) {
		case 'r': {
			if ((tok = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			float bytes = atof(tok);
			char *src, *dest;
			if ((src = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if ((dest = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			int rval = netcapalloc.RequestCapacity(bytes, src, dest, true,
												   false);
			switch (rval) {
			case 1:
				fprintf(stderr, "request granted\n");
				break;
			case 0:
				fprintf(stderr, "request denied\n");
				break;
			default:
				fprintf(stderr, "request operation failed\n");
			}				
			break;
		}
		case 'c': {
			double bytes;
			int num_hops;
			if (netcapalloc.Commit("", &bytes, &num_hops) < 0) {
				fprintf(stderr, "commit failed\n");
			} else {
				fprintf(stderr, "transaction committed: "
						"bytes=%.0f num_hops=%d\n", bytes, num_hops);
			}
			break;
		}
		case 'a':
			if (netcapalloc.Abort() < 0) {
				fprintf(stderr, "abort failed\n");
			} else {
				fprintf(stderr, "transaction aborted\n");
			}
			break;
		case 'l':
			char *capfile, *routefile;
			if ((capfile = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if ((routefile = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if (capinfo.LoadNetworkCapacity(capfile) < 0) {
				fprintf(stderr, "network capacity load failed\n");
				break;
			}
			if (router.LoadRoutingTable(routefile) < 0) {
				fprintf(stderr, "network routing table load failed\n");
				break;
			}
			fprintf(stderr, "network load completed\n");
			break;
		default:
			fprintf(stderr, "unrecognized command\n");
		}
		printf("> ");
	}

	printf("\n");

	return 0;
}
