/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef __NETMAN_H
#define __NETMAN_H

#include "condor_io.h"
#include "condor_classad.h"
#include "netusagealloc.h"
#include "netcapacityalloc.h"
#include "netbandwidthalloc.h"

#define NET_ALLOC_ICKPT		1
#define NET_ALLOC_RESTART	2
#define NET_ALLOC_PCKPT		3
#define NET_ALLOC_VCKPT		4

class NetworkManager {
public:
	NetworkManager();
	~NetworkManager();

	void Config();

		// returns 0 on success and -1 on failure
	int HandleNetworkRequest(Stream *stream);

		// do any necessary periodic maintenance before starting a new
		// round of job scheduling
	int PrepareForSchedulingCycle() { return CleanupOldNetAllocations(); }

		// request network capacity for job placement:
		// returns 2 if capacity is available to this user,
		//         1 if this user is already using her fair-share but capacity
		//           is otherwise available,
		//         0 if the network is already over capacity, and 
        //        -1 on error
	int RequestPlacement(const char user[], double share,
						 const char scheddIP[], int executableSize,
						 const char lastCkptServerIP[], int ckptSize,
						 ClassAd &request, ClassAd &offer,
						 ClassAd *preemptJob=NULL);

		// commit last placement request (and charge user for usage)
		// returns 0 on success and -1 on error
	int CommitPlacement(const char user[]);

		// request network capacity:
		// returns 2 if capacity is available to this user,
		//         1 if this user is already using her fair-share but capacity
		//           is otherwise available,
		//         0 if the network is already over capacity, and 
        //        -1 on error
	int RequestCapacity(const char user[], double share, double bytes,
						const char source[], const char destination[],
						bool canDeny=true, bool willReturn=false,
						bool goodputEligible=false);

	int ShowDeniedRequests(int debug_mask) 
		{ return netcapalloc.ShowDeniedRequests(debug_mask); }

private:
	int CleanupOldNetAllocations();

	bool active;				// true iff successfully configured 
	Router router;
	NetworkCapacity capinfo;
	NetworkCapacityAllocator netcapalloc;
	NetworkUsage netusage;
	NetworkUsageAllocator netusagealloc;
	int horizon;
	double goodput_reservation, goodput_job_limit;
	int usage_horizon;

	/* Needs to be public because other non-class related stuff uses this */
	public:
	struct NetAllocKey {
		int vm;
		int startdIP;
		int type;
		bool operator==(const NetAllocKey &k) {
			return (k.vm == vm) && (k.startdIP == startdIP) &&
				(k.type == type);
		}
	};

	private:
	struct NetAllocKey lastNetAllocKey;

	struct NetAlloc {
		int bytes;
		int endpoint;
		time_t timestamp, expiration_time;
	} lastICkptAlloc, lastRestartAlloc, lastVCkptAlloc;
	HashTable<NetAllocKey, NetAlloc *> NetAllocations;
	HashTable<HashKey, NetAllocKey> NetAllocationKeys;
};

#endif
