/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "condor_debug.h"
#include "netres.h"

void
usage(char progname[])
{
	fprintf(stderr, "usage: %s [-DEBUG-FLAGS ...] "
			"[capacity-info-file route-info-file]\n", progname);
	exit(1);
}

int
main(int argc, char *argv[])
{
	if (argc == 2 && strncmp(argv[1], "-h", 2) == 0) usage(argv[0]);

	NetworkReservations netres;
	Router router;
	NetworkCapacity capinfo;
	netres.BindRouter(&router);
	netres.BindCapacity(&capinfo);

	argv++;
	argc--;
	while (argc && strncmp(*argv, "-D", 2) == 0) {
		Termlog = 1;
		set_debug_flags((*argv)+1);
		argv++;
		argc--;
	}

	if (argc == 2) {
		if (capinfo.LoadNetworkCapacity(argv[1]) < 0) {
			fprintf(stderr, "network capacity load failed\n");
			exit(1);
		}
		if (router.LoadRoutingTable(argv[2]) < 0) {
			fprintf(stderr, "network routing table load failed\n");
			exit(1);
		}
		fprintf(stderr, "network load completed\n");
	}

	bool search_forward = true;
	NetworkReservations::reservation_preference pref =
		NetworkReservations::pref_first;

	printf("Enter commands: r[equest] [bytes] source destination start-time end-time [min-rate max-rate]\n");
	printf("                c[ancel] reservation-id \n");
	printf("                s[earch] f[rward]\n");
	printf("                s[earch] b[ackward]\n");
	printf("                p[refer] fi[rst]\n");
	printf("                p[refer] s[hortest]\n");
	printf("                p[refer] e[arliest-completion]\n");
	printf("                p[refer] l[atest-start]\n");
	printf("                l[oad] capacity-info-file route-info-file\n");
	printf("> ");

	char line[80];
	while (fgets(line, 80, stdin)) {
		char *tok;
		if ((tok = strtok(line, " \t\n")) == NULL) {
			printf("> ");
			continue;
		}
		switch (tok[0]) {
		case 'c': {
			if ((tok = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			int id = atoi(tok);
			if (netres.CancelReservation(id) < 0) {
				fprintf(stderr, "failed to cancel reservation %d\n", id);
			} else {
				fprintf(stderr, "cancelled reservation %d\n", id);
			}
			break;
		}
		case 's': {
			if ((tok = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			switch (tok[0]) {
			case 'f':
				search_forward = true;
				break;
			case 'b':
				search_forward = false;
				break;
			default:
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			break;
		}
		case 'p': {
			if ((tok = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			switch (tok[0]) {
			case 'f':
				pref = NetworkReservations::pref_first;
				break;
			case 's':
				pref = NetworkReservations::pref_shortest;
				break;
			case 'e':
				pref = NetworkReservations::pref_earliest_completion;
				break;
			case 'l':
				pref = NetworkReservations::pref_latest_start;
				break;
			}
			break;
		}
		case 'r': {
			double bytes=-1.0, min_rate=-1.0, max_rate=-1.0;
			char *src, *dest;
			if ((tok = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if (!strchr(tok, '.')) {
				bytes = atof(tok);
				if ((tok = strtok(NULL, " \t\n")) == NULL) {
					fprintf(stderr, "invalid request syntax\n");
					break;
				}
			}
			src = tok;
			if ((dest = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if ((tok = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			time_t start_time = atoi(tok);
			if ((tok = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			time_t end_time = atoi(tok);
			if ((tok = strtok(NULL, " \t\n")) != NULL) {
				min_rate = atof(tok);
				if ((tok = strtok(NULL, " \t\n")) == NULL) {
					fprintf(stderr, "invalid request syntax\n");
					break;
				}
				max_rate = atof(tok);
			}
			int id = netres.NewID();
			int rval;
			if (bytes < 0.0) {
				if (min_rate < 0.0) {
					fprintf(stderr, "invalid request syntax\n");
					break;
				} else {
					rval = netres.ReserveBandwidth(min_rate, &max_rate,
												   src, dest,
												   start_time, end_time, id);
				}
			} else {
				if (min_rate < 0.0) {
					rval = netres.ReserveCapacity(bytes, src, dest,
												  search_forward,
												  &start_time, &end_time, id,
												  "", &max_rate);
				} else {
					rval = netres.ReserveBandwidth(bytes, min_rate, &max_rate,
												   src, dest,
												   search_forward, pref,
												   &start_time, &end_time, id);
				}
			}

			switch (rval) {
			case 1:
				fprintf(stderr, "request granted: start_time=%d, end_time=%d, "
						"bandwidth=%0.2f Mb/s, id=%d\n", (int)start_time,
						(int)end_time, max_rate, id);
				break;
			case 0:
				fprintf(stderr, "request denied\n");
				break;
			default:
				fprintf(stderr, "request operation failed\n");
			}				
			break;
		}
		case 'l':
			char *capfile, *routefile;
			if ((capfile = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if ((routefile = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if (capinfo.LoadNetworkCapacity(capfile) < 0) {
				fprintf(stderr, "network capacity load failed\n");
				break;
			}
			if (router.LoadRoutingTable(routefile) < 0) {
				fprintf(stderr, "network routing table load failed\n");
				break;
			}
			fprintf(stderr, "network load completed\n");
			break;
		default:
			fprintf(stderr, "unrecognized command\n");
		}
		printf("> ");
	}

	printf("\n");

	return 0;
}
