/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef __NETSHAREALLOC_H__
#define __NETSHAREALLOC_H__

#include "router.h"
#include "netcapacity.h"
#include "list.h"

int reqhash(const int &id, int numBuckets);

class NetworkShareAllocator {
public:
	NetworkShareAllocator() : next_id(1), requests(101, reqhash),
		network(255, addrhash), router(NULL), capacityinfo(NULL) {}
	~NetworkShareAllocator();

	void BindRouter(Router *r) { router = r; }
	void BindCapacity(NetworkCapacity *c) { capacityinfo = c; } 

		// request bandwidth between source and destination for given
		// user; returns the id assigned to the request, which can be
		// used to retrieve the allocation with GetAllocation() after
		// each call to UpdateAllocations()
	int SubmitRequest(const char source[], const char destination[],
					  double bandwidth, const char user[]="");

		// returns 0 on success, -1 on failure
	int CancelRequest(int id);
	
		// returns 0 on success, -1 on failure
	int UpdateAllocations();

		// returns bandwidth currently allocated to the request
		// (between 0.0 and requested bandwidth)
	double GetAllocation(int id);

private:	
	int NewID() { return next_id++; }
	int next_id;				// id counter for reservations
	struct BandwidthRequest {
		BandwidthRequest() : src(0), dest(0), requested_rate(0.0),
			allocated_rate(0.0), user(NULL) {}
		BandwidthRequest(unsigned int s, unsigned int d, double r,
						 const char u[]) :
			src(s), dest(d), requested_rate(r), allocated_rate(0.0)
			{ if (u) user = strdup(u); else user = NULL; }
		~BandwidthRequest() { if (user) free(user); }
		unsigned int src, dest;
		double requested_rate;
		double allocated_rate;
		bool satisfied;
		char *user;
	};
	HashTable<int, BandwidthRequest *> requests;
	struct NetworkRecord {
		NetworkRecord() : addr(0) {}
		NetworkRecord(unsigned int a) : addr(a) {}
		unsigned int addr;
		List<BandwidthRequest> reqlist;
		void Add(BandwidthRequest *r);
		void Cancel(BandwidthRequest *r);
		int num_unsat;
		double usage;
	};
	HashTable<unsigned int, NetworkRecord *> network;

	NetworkRecord *GetNetworkRecord(unsigned int addr);

	Router *router;
	NetworkCapacity *capacityinfo;
};

#endif
