/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "condor_debug.h"
#include "netsharealloc.h"

void
usage(char progname[])
{
	fprintf(stderr, "usage: %s [-DEBUG-FLAGS ...] "
			"[capacity-info-file route-info-file]\n", progname);
	exit(1);
}

int
main(int argc, char *argv[])
{
	if (argc == 2 && strncmp(argv[1], "-h", 2) == 0) usage(argv[0]);

	NetworkShareAllocator netalloc;
	Router router;
	NetworkCapacity capinfo;
	netalloc.BindRouter(&router);
	netalloc.BindCapacity(&capinfo);

	argv++;
	argc--;
	while (argc && strncmp(*argv, "-D", 2) == 0) {
		Termlog = 1;
		set_debug_flags((*argv)+1);
		argv++;
		argc--;
	}

	if (argc == 2) {
		if (capinfo.LoadNetworkCapacity(*argv) < 0) {
			fprintf(stderr, "network capacity load failed\n");
			exit(1);
		}
		argv++;
		if (router.LoadRoutingTable(*argv) < 0) {
			fprintf(stderr, "network routing table load failed\n");
			exit(1);
		}
		fprintf(stderr, "network load completed\n");
	}

	printf("Enter commands: r[equest] source destination rate\n");
	printf("                c[ancel] request-id\n");
	printf("                u[pdate allocations]\n");
	printf("                g[et-allocation] request-id\n");
	printf("                l[oad] capacity-info-file route-info-file\n");
	printf("> ");

	char line[80];
	while (fgets(line, 80, stdin)) {
		char *tok;
		if ((tok = strtok(line, " \t\n")) == NULL) {
			printf("> ");
			continue;
		}
		switch (tok[0]) {
		case 'r': {
			double rate=-1.0;
			char *src, *dest;
			if ((src = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if ((dest = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if ((tok = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			rate = atof(tok);
			int id;
			id = netalloc.SubmitRequest(src, dest, rate);
			fprintf(stderr, "id=%d\n", id);
			break;
		}
		case 'c': {
			if ((tok = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			int id = atoi(tok);
			if (netalloc.CancelRequest(id) == 0) {
				fprintf(stderr, "request %d cancelled\n", id);
			} else {
				fprintf(stderr, "failed to cancel request %d\n", id);
			}
			break;
		}
		case 'u':
			if (netalloc.UpdateAllocations() == 0) {
				fprintf(stderr, "update succeeded\n");
			} else {
				fprintf(stderr, "update failed\n");
			}
			break;
		case 'g': {
			if ((tok = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			int id = atoi(tok);
			double rate = netalloc.GetAllocation(id);
			fprintf(stderr, "request %d allocated %.2f Mbps\n", id, rate);
			break;
		}
		case 'l':
			char *capfile, *routefile;
			if ((capfile = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if ((routefile = strtok(NULL, " \t\n")) == NULL) {
				fprintf(stderr, "invalid request syntax\n");
				break;
			}
			if (capinfo.LoadNetworkCapacity(capfile) < 0) {
				fprintf(stderr, "network capacity load failed\n");
				break;
			}
			if (router.LoadRoutingTable(routefile) < 0) {
				fprintf(stderr, "network routing table load failed\n");
				break;
			}
			fprintf(stderr, "network load completed\n");
			break;
		default:
			fprintf(stderr, "unrecognized command\n");
		}
		printf("> ");
	}

	printf("\n");

	return 0;
}
