/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "netusage.h"

// Allocate space for static vars
const int NetworkUsage::numbuckets = CONDOR_NETUSAGE_NUMBUCKETS;

int
NetworkUsage::AddUsage(const char user[], unsigned int network, double seconds)
{
	char keystr[500];
	sprintf(keystr, "%s.%x", user, network);
	UsageRecord *rec = NULL;
	HashKey key(keystr);
	if (usage.lookup(key, rec) < 0) {
		rec = new UsageRecord(&horizon);
		usage.insert(key, rec);
	}
	rec->AddUsage(seconds);
	return 0;
}

int
NetworkUsage::GetUsage(const char user[], unsigned int network,
					   double &seconds)
{
	char keystr[500];
	sprintf(keystr, "%s.%x", user, network);
	UsageRecord *rec = NULL;
	HashKey key(keystr);
	if (usage.lookup(key, rec) < 0) {
		seconds = 0.0;
		return 0;
	}
	seconds = rec->Usage();
	return 0;
}

NetworkUsage::UsageRecord::UsageRecord(int *i) : horizon(i), current_bucket(0)
{
	current_bucket_time = time(0)/(*horizon/numbuckets);

	for (int j=0; j < numbuckets; j++) {
		usage[j] = 0.0;
	}
}

double
NetworkUsage::UsageRecord::Usage()
{
	SetCurrentBucket(time(0));	// want to call *every* time???

	double total_usage = 0.0;
	for (int i=0; i < numbuckets; i++) {
		total_usage += usage[i];
	}
	return total_usage;
}

void
NetworkUsage::UsageRecord::AddUsage(double seconds)
{
	SetCurrentBucket(time(0));	// want to call *every* time???

	usage[current_bucket] += seconds;
}

void
NetworkUsage::UsageRecord::SetCurrentBucket(time_t current_time)
{
	int new_bucket_time = current_time/(*horizon/numbuckets);

	if (current_bucket_time < new_bucket_time - numbuckets) {
			// loop around at most once
		current_bucket_time = new_bucket_time - numbuckets;
	}
	while (new_bucket_time > current_bucket_time) {
		current_bucket++;
		if (current_bucket == numbuckets) {
			current_bucket = 0;
		}
		current_bucket_time++;
		usage[current_bucket] = 0.0;
	}
}
