/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "netusagealloc.h"

int
NetworkUsageAllocator::FairShareReceived(const char user[], double share,
										 const char source[],
										 const char destination[])
{
	unsigned int src = ntohl(inet_addr(source));
	unsigned int dest = ntohl(inet_addr(destination));

	if (src == dest) return 0;	// loopback
	if (!capacityinfo) return 0; // no capacity set: allow anything
	if (!netusage) return 0;	// not tracking usage: allow anything
	if (!router) return -1;		// error if we can't route!

	int rval = 1;
	for (unsigned int current_addr = src;
		 rval > 0;
		 rval = router->NextHop(current_addr, src, dest)) {
		double usage = 0.0;
		netusage->GetUsage(user, current_addr, usage);
		if (usage >= horizon*share) {
			return 1; // user is already at or over fair-share for this network
		}
	}

	if (rval < 0) return -1;

	return 0;
}

int
NetworkUsageAllocator::AddUsage(const char user[], double bytes,
								const char source[], const char destination[])
{
	unsigned int src = ntohl(inet_addr(source));
	unsigned int dest = ntohl(inet_addr(destination));

	if (src == dest) return 0;	// loopback
	if (!capacityinfo) return 0; // no capacity set: allow anything
	if (!netusage) return 0;	// not tracking usage: allow anything
	if (!router) return -1;		// error if we can't route!

	int rval = 1;
	for (unsigned int current_addr = src;
		 rval > 0;
		 rval = router->NextHop(current_addr, src, dest)) {
		double capacity = 0.0;
		if (capacityinfo->GetCapacity(current_addr, capacity) < 0 ||
			capacity <= 0.0) continue;
		double new_usage = bytes/131072.0/capacity;
		rval = netusage->AddUsage(user, current_addr, new_usage);
	}

	if (rval < 0) return -1;

	return 0;
}
