/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/
#ifndef __NETWORK_USAGE_ALLOC_H__
#define __NETWORK_USAGE_ALLOC_H__

#include "router.h"
#include "netcapacity.h"
#include "netusage.h"

class NetworkUsageAllocator {
public:
	NetworkUsageAllocator() : horizon(0), router(NULL), capacityinfo(NULL),
							  netusage(NULL) {}
	~NetworkUsageAllocator() {}

	void BindRouter(Router *r) { router = r; }
	void BindCapacity(NetworkCapacity *c) { capacityinfo = c; } 
	void BindUsage(NetworkUsage *n)
		{ netusage = n; netusage->SetHorizon(horizon); }

	int SetHorizon(int seconds) 
		{ int o = horizon; horizon = seconds;
		  if (netusage) netusage->SetHorizon(seconds); return o; }
	int GetHorizon() { return horizon; }

		// determine if a user is already using more than her
		// fair-share (a fraction between 0.0 and 1.0 that specifies
		// how much of total network this user is entitled to) on the
		// network segments in a proposed allocation; returns 1 if the
		// user is over the fair-share, 0 if she is isn't, and
		// -1 on error
	int FairShareReceived(const char user[], double share, 
						  const char source[], const char destination[]);

		// record new network usage; returns 0 on success and -1 on failure
	int AddUsage(const char user[], double bytes, const char source[],
				 const char destination[]);
	int AddUsage(const char user[], unsigned int network, double seconds) {
		if (netusage) return netusage->AddUsage(user, network, seconds);
		return 0;
	}

private:
	int horizon;
	Router *router;
	NetworkCapacity *capacityinfo;
	NetworkUsage *netusage;
};

#endif
