/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/
#ifndef __ROUTER_H__
#define __ROUTER_H__

#include "HashTable.h"

// unsigned int addr must be in host byte order
int addrhash(const unsigned int &addr, int numBuckets);
char *inet_itoa(unsigned int addr);

class Router {
public:
	Router() : last_dest(0), destnet(NULL),
			   network(255, addrhash), nummasks(0) {}
	~Router() { Cleanup(); }

		// returns 0 on success, -1 on failure
	int LoadRoutingTable(const char filename[]);

		// sets current to point to the next network hop in the route;
		// current should be equal to src on the first call and should
		// be the current hop otherwise; returns 1 if there is another
		// hop, 0 if there is no next hop, and -1 on error; the caller
		// should stop calling NextHop when current == dest; calls for
		// different routes can be interleaved -- for this reason,
		// this method does not detect routing loops
	int NextHop(unsigned int &current, unsigned int src, unsigned int dest);

	struct NetworkRecord {
		NetworkRecord() : addr(0), mask(0), hoplist(255, addrhash) {}
		NetworkRecord(unsigned int a, unsigned int m) :
			addr(a), mask(m), hoplist(255, addrhash) {}
		unsigned int addr;
		unsigned int mask;
		struct NextHop {
			NextHop() : net(0), addr(0), mask(0) {}
			unsigned int net;	// destination addresses which match
			unsigned int addr;	// addr when masked with mask have net
			unsigned int mask;	// as their next hop
		};
		HashTable<unsigned int, NextHop *> hoplist;
		struct _hopcache {
			_hopcache() : addr(0), nexthop(0) {}
			unsigned int addr;
			NextHop *nexthop;
		} hopcache;
	};

private:
	void Cleanup();
	unsigned int last_dest;
	NetworkRecord *destnet;
	HashTable<unsigned int, NetworkRecord *> network;
	int nummasks;
	unsigned int masklist[16];
};

#endif
