/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 

#define _POSIX_SOURCE
#include "condor_common.h"
#include "condor_io.h"

#include <sys/param.h>
#include "condor_debug.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include "sched.h"
#include "proc.h"
#include "request.h"
#include "joblist.h"
#include "internet.h"

#include "pvm3.h"

#ifdef PVM34
#include "pvmproto.h"
#else
#include "pvmsdpro.h"
#endif

#include "CoCheckExport.h"
#include "carmipro.h"

/* A hack if the executable name doesn't come through pvm messages */
extern char executable_name_hack[_POSIX_PATH_MAX];


int carmi_spawn(int, int);
int carmi_vacate_notify(int, int);
int carmi_cancel_req(int, int);
int carmi_req_url_for_file(int, int);
int carmi_notify(int, int);

static struct {
	char	*msg_nam;
	int		msg_tag;
	int		(*handler)(int, int);
} MsgNames[] = {
{ "CARMI_SPAWN", CARMI_SPAWN, carmi_spawn },
{ "CARMI_VACATE_NOTIFY", CARMI_VACATE_NOTIFY, carmi_vacate_notify },
{ "CARMI_CANCEL_REQ", CARMI_CANCEL_REQ, carmi_cancel_req },
{ "CARMI_REQ_URL_FOR_FILE", CARMI_REQ_URL_FOR_FILE, carmi_req_url_for_file },
{ "CARMI_NOTIFY", CARMI_NOTIFY, carmi_notify },
};


int
carmi_init_spanning_tree(int new_size);

#include "prototypes.h"
#include "../condor_ckpt_server/server_interface.h"

int
HandleCARMI( int msgtag, int msgtid, int req_id )
{
	int		i;
	int		handled_it = 0;

	for (i = 0; i < sizeof(MsgNames) / sizeof(MsgNames[0]); i++) {
		if (msgtag == MsgNames[i].msg_tag) {
			dprintf( D_ALWAYS, "CARMI message is %s from t%x\n", 
				   MsgNames[i].msg_nam, msgtid );
			MsgNames[i].handler(msgtid, req_id);
			handled_it = 1;
			break;
		}
	}

	return handled_it;
}


int
carmi_spawn(int msgtid, int req_id)
{
	char	a_out[MAXPATHLEN];
	int		arg_count;
	char	temp_arg[MAXPATHLEN];
	char	long_arg[MAXPATHLEN];
	char	dest_arch[100];
	int		count;
	int		exit_response;
	int		response;
	int		i;

	pvm_upkstr(a_out);
	dprintf ( D_FULLDEBUG, "Unpacking a_out in carmi_spawn: \"%s\".\n", a_out);
	pvm_upkint(&arg_count, 1, 1);

	long_arg[0] = '\0';

	for (i = 0; i < arg_count; i++) {
		pvm_upkstr(temp_arg);
		strcat(long_arg, temp_arg);
		strcat(long_arg, " ");
	}

	pvm_upkstr(dest_arch);
	pvm_upkint(&count, 1, 1);
	pvm_upkint(&exit_response, 1, 1);
	pvm_upkint(&response, 1, 1);

	carmi_init_spanning_tree(count);

	for (i = 0; i < count; i++) {
#ifdef PVM34
		DoSpawn( msgtid, req_id, a_out, long_arg, dest_arch, 
				PvmTaskArch, 0, 0, 0, 0, 0, 0,
			 response, SPAWN_CARMI);
#else
		DoSpawn( msgtid, req_id, a_out, long_arg, dest_arch, 
				PvmTaskArch, 0, 0, 0, 0, response, SPAWN_CARMI);
#endif
	}
	return 0;
}


int
carmi_vacate_notify(int msgtid, int req_id)
{
	Host		*target_host;


	target_host = FindHostByStarterTid(msgtid);
	if (target_host != 0) {
		SendNotification(REQ_VACATE_NOTIFY, target_host->pvmd_tid);
	}
	return 0;
}


int
carmi_cancel_req(int msgtid, int req_id)
{
	Request		*request;
	int			cancel_reqid;
	int			response;

	pvm_upkint(&cancel_reqid, 1, 1);
	pvm_upkint(&response, 1, 1);
	request = FindRequest(msgtid, -1, -1, cancel_reqid);
	if (request) {
		dprintf(D_ALWAYS, "Canceling request: ");
		dprintf_request(D_ALWAYS | D_NOHEADER, request);
		DeleteRequest(request);
		cancel_reqid = 1;
	} else {
		cancel_reqid = 0;
	}
	pvm_initsend(PvmDataDefault);
	pvm_pkint(&cancel_reqid, 1, 1);
	carmi_store_req_id(req_id);
	pvm_send(msgtid, response);
	return 0;
}


int
carmi_req_url_for_file(int msgtid, int req_id)
{
	char	fname[ _POSIX_PATH_MAX ];
	char	local_serve_url[ _POSIX_PATH_MAX ];
	char	dest_url[ _POSIX_PATH_MAX ];
	int		to_serve_count;
	int		file_len;
	int		get_file_rval;
	struct  sockaddr_in sin;
	static int get_url_calls = 0;
	struct stat buf;

	pvm_upkstr(fname);
	dprintf ( D_FULLDEBUG, "carmi_req_url_for_file unpacked filename"
			  "\"%s\".\n", fname );
	pvm_upkstr(local_serve_url);
	
	if (get_ckpt_server_count() > 0) {
		set_ckpt_server_number( get_url_calls % get_ckpt_server_count());
	}

		/* I'm going to be shot for doing this, but it's a hack for a 
		   bug that I suspect is coming from pvm.  I *think* that the
		   pvm starter is packing the message correctly, but it's not
		   getting here.  I'm going to use executable_name_hack, which 
		   was just aquired in send_job_info.  -MEY 8-30-99 */



	if ( ( fname == NULL ) || 
		 ( fname[0] == 0 ) ||
		 ( stat(fname, &buf) == -1 ) ) {
		dprintf ( D_ALWAYS, "FUBAR FILENAME in carmi_req_url_for_file.  "
				  "Packing \"%s\" instead.\n", executable_name_hack );
		strncpy ( fname, executable_name_hack, _POSIX_PATH_MAX );
	}

	get_url_calls++;
	get_file_rval = pseudo_get_file_stream(fname, (size_t *)&(file_len),
			     (unsigned int *)&(sin.sin_addr), &(sin.sin_port));
	if (get_file_rval < 0) {
		to_serve_count = PvmNoFile;
		dest_url[0] ='\0';
	} else {
		sin.sin_addr.s_addr = htonl(sin.sin_addr.s_addr);
		sin.sin_port = htons(sin.sin_port);
		sprintf(dest_url, "cbstp:%s", sin_to_string(&sin));
		to_serve_count = 0;
	}

	pvm_initsend(PvmDataDefault);
	pvm_pkint(&to_serve_count, 1, 1);
	pvm_pkint(&file_len, 1, 1);
	pvm_pkstr(dest_url);
	pvm_send(msgtid, CARMI_REQ_URL_FOR_FILE);
	dprintf(D_ALWAYS, 
			"Told t%x to get file %s from url %s and serve %d others\n",
			msgtid, fname, dest_url, to_serve_count);
	return 0;
}


int spanning_tree_size;
int new_spanning_tree;
char **url_list = 0;
int  url_list_len = 0;

#if defined(SPANNING_TREE)
#define SPANNING_TREE_DEGREE 2
int
carmi_do_spanning_tree(char *fname, int *flen, char *server_url, 
					   char *return_url)
{
	char		my_url[ MAXPATHLEN ];
	int			my_service_fd;
	int 		i;
	int			to_serve;
	int			file_fd;
	static int 	this_file_size;

	if (new_spanning_tree) {
		new_spanning_tree = 0;
		my_service_fd = create_socket_url(my_url, SPANNING_TREE_DEGREE);
		file_fd = open(fname, O_RDONLY);
		if (file_fd < 0) {
			dprintf(D_ALWAYS, "Unable to open %s to serve to clients\n", 
					fname);
			return -1;
		}
		this_file_size = file_size(file_fd);	/* This is currently in pseudo_ops.c */

		if (fork() == 0) {
			/* The Child */
			carmi_serve_multiple(file_fd, my_service_fd, 
							 MIN(spanning_tree_size, SPANNING_TREE_DEGREE));
			exit(0);
		}
		/* The parent */
		close(my_service_fd);
		close(file_fd);
		url_list_len = MIN(spanning_tree_size, SPANNING_TREE_DEGREE);
		for (i = 0; i < url_list_len; i++) {
			url_list[i] = strdup(my_url);
		}
	}
	to_serve = MIN(SPANNING_TREE_DEGREE, spanning_tree_size - url_list_len);
	to_serve = MAX(0, to_serve);
	strcpy(return_url, url_list[0]);
	free(url_list[0]);
	for (i = 1; i < url_list_len; i++) {
		url_list[i - 1] = url_list[i];
	}
	url_list_len--;
	for (i = 0; i < to_serve; i++) {
		url_list[url_list_len + i] = strdup(server_url);
	}
	url_list_len += to_serve;
	spanning_tree_size--;
	*flen = this_file_size;
	return to_serve;
}

carmi_serve_multiple(int file_fd, int sock_fd, int serve_count)
{
	int		dest_fds[SPANNING_TREE_DEGREE];
	int		len;

	wait_for_connections(sock_fd, serve_count, dest_fds);
	close(sock_fd);

	multi_stream_file_xfer(file_fd, serve_count, dest_fds, len);

	close(file_fd);
}

#endif

int
carmi_init_spanning_tree(int new_size)
{
	int rval;

	rval = spanning_tree_size;
	spanning_tree_size = new_size;
	new_spanning_tree = 1;
	if (url_list) {
		free(url_list);
	}
	url_list = (char **) malloc(new_size * sizeof(char *));
	return rval;
}

/* Must be consistent with carmi.h, ugh! */
#define NOTIFY_SUSPEND	1
#define NOTIFY_RESUME	2
#define NOTIFY_VACATE	3

int
carmi_notify(int msgtid, int req_id)
{
	int		target_tid;
	int		response;
	Request	*request;
	Host	*target_host;
	int		rval;
	int		what;
	
	pvm_upkint(&what, 1, 1);
	pvm_upkint(&target_tid, 1, 1);
	pvm_upkint(&response, 1, 1);
	target_host = FindHostByTid(target_tid);
	if (target_host == 0) {
		pvm_initsend(PvmDataDefault);
		rval = PvmNoHost;
		pvm_pkint(&rval, 1, 1);
		carmi_store_req_id(req_id);
		pvm_send(msgtid, response);
	} else {
		if (what == NOTIFY_VACATE) {
			pvm_initsend(PvmDataDefault);
			rval = 0;
			pvm_pkint(&rval, 1, 1);
			pvm_send(target_host->starter_tid, CARMI_VACATE_NOTIFY);
		}
		request = NewRequest(REQ_VACATE_NOTIFY, msgtid, req_id, response);
		request->on_tid = target_tid;		
	}
	return 0;
}
