/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "condor_debug.h"
#include "condor_io.h"
#include "shadow.h"

#include "joblist.h"
#include "request.h"
#include "pvm3.h"

#ifdef PVM34
#include "pvmproto.h"
#else
#include "pvmsdpro.h"
#endif

#include "pvm_global.h"

#include "../condor_syscall_lib/syscall_param_sizes.h"


#include "user_log.h"
#include "condor_attributes.h"
#include "condor_classad.h"
#include "condor_qmgr.h"

extern int JobStatus;
extern int ImageSize;
extern struct rusage JobRusage;
extern V2_PROC *Proc;
extern char CkptName[];
extern char ICkptName[];
extern char RCkptName[];
extern ClassAd *JobAd;
extern int InitJobAd(int cluster, int proc);
extern "C" int CompleteSpawn(Request *request, int new_tid, int pid);

extern int Procnum;

extern struct tagJobList	*JobList;

int UsePipes = 0;

char executable_name_hack[_POSIX_PATH_MAX];

#include "prototypes.h"

int connect_file_stream( int connect_sock );
size_t file_size( int fd );
int create_tcp_port( u_short *port, int *fd );
void get_host_addr( unsigned int *ip_addr );
void display_ip_addr( unsigned int addr );
int has_ckpt_file();

extern char My_Filesystem_Domain[];
extern char My_UID_Domain[];
extern int	UseAFS;

int
pseudo_reallyexit( int *status, struct rusage *use_p )
{
	int stat1 = *status;
	JobStatus = *status;
	JobRusage = *use_p;
	update_rusage( &(JobList->RemoteUsage), &JobRusage );

	dprintf(D_SYSCALLS, "Shadow: reallyexit(0x%x, 0x%x) = %d\n",
			&JobStatus, &JobRusage, 0 );

	dprintf(D_ALWAYS, "termsig = %d, coredump = %d, retcode = %d\n",
			WTERMSIG(stat1), WCOREDUMP(stat1), WEXITSTATUS(stat1));
	dprintf( D_ALWAYS, "ru_utime = %d.%06d\n",
			JobRusage.ru_utime.tv_sec, JobRusage.ru_utime.tv_usec );
	dprintf( D_ALWAYS, "Total ru_utime = %d.%06d\n",
			JobList->RemoteUsage.ru_utime.tv_sec, 
			JobList->RemoteUsage.ru_utime.tv_usec );
	dprintf( D_ALWAYS, "ru_stime = %d.%06d\n",
			JobRusage.ru_stime.tv_sec, JobRusage.ru_stime.tv_usec );

	dprintf ( D_ALWAYS, "NOT CALLING REMOVEPROC\n" );
//	RemoveProc( Procnum );
	return( -12 );
}



#include "startup.h"

/*
  Gather up the stuff needed to satisfy a request for work from the
  condor_starter.
*/
int
pseudo_work_request( ClassAd *&ad, char *a_out, char *targ, char *orig, 
					int *kill_sig )
{
	int         new_process;
	int			host_num;
	WorkItem	*item;
	Request		*request;
	int			rval;

	host_num = JobList->procs[Procnum].host_num;
	if ( JobList->hosts[host_num].status == 	PVM_STATE_SEND_PVM ) {
		rval = PVMD_MAGIC_PROC;
		send_pvmd_job_info( ad, a_out, targ, orig, kill_sig, host_num );
		
	} else if ( JobList->hosts[host_num].status >= 
			   PVM_STATE_PENDING ) {
		request = FindRequest(-1, JobList->hosts[host_num].starter_tid,
							  REQ_SPAWN, -1);
		if ( request == NULL ) {
			dprintf( D_ALWAYS, "Work request, but nothing to do!\n" );
			errno = EAGAIN;
			rval = -1;
		} else {
			item = (WorkItem *) request->extra_data;
			if (item->process == 0) {
				new_process = AllocProc();
				item->process = &(JobList->procs[new_process]);
				item->process->a_out = strdup( item->Proc->cmd[0] );
			}
			item->process->host_num = host_num;
			dprintf(D_ALWAYS, "Arg String: %s\n", item->Proc->args[0]);
			rval = send_job_info(ad, a_out, targ, orig, kill_sig, item->Proc);
			if ( rval < 0 ) {
				RemoveHost(host_num);
			} else {
/*	MOVING INTO DoSpawn()		JobList->hosts[host_num].status++; */
/*				JobList->procs[new_process].pvm_ptid = item->ptid; */
				rval = item->process->vpid;
			}
		}
	} else {
		dprintf( D_ALWAYS, 
				"Work request from host %s in unexpected state %d\n",
				JobList->hosts[host_num].name, 
				JobList->hosts[host_num].status);
		errno = EAGAIN;
		rval = -1;
	}

	return rval;
}


int
pseudo_get_username( char *uname )
{
	strcpy( uname, JobList->Proc.owner );
	return 0;
}



int
pseudo_pvm_info(char *pvmd_reply, int starter_tid)
{
	int host_num;
	char *tmp, *p;

	host_num = JobList->procs[Procnum].host_num;

	if ( !pvmd_reply ) {
		dprintf(D_ALWAYS, "Problem with host %s\n", 
				JobList->hosts[host_num].name );
		pvm_initsend(PvmDataDefault);
		pvm_pkstr(JobList->hosts[host_num].name);
		pvm_send(JobList->local_proc_tid, 12);
		RemoveHost(host_num);
		return 0;
	}

	JobList->hosts[host_num].pvmd_reply = strdup(pvmd_reply);

	dprintf ( D_FULLDEBUG, "pvmd_reply = %s\n", pvmd_reply );
	
#ifdef PVM34
		/* rip out the hi_dsig element from the pvmd_reply */
	tmp = strdup( pvmd_reply );
	
	p = strstr( tmp, "dsig" );

	if ( !p ) {
			/* There was no element dsig<...>.  This bad. */
		dprintf ( D_ALWAYS,"************ PROBLEM! ************\n" );
		dprintf ( D_ALWAYS,"* There was no data signature element in the\n" );
		dprintf ( D_ALWAYS,"* pvmd's reply!  This almost certainly means\n" );
		dprintf ( D_ALWAYS,"* you've got a worker who is pvm version 3.3.x\n");
		dprintf ( D_ALWAYS,"* while the multishadow is version 3.4.x!\n" );
		dprintf ( D_ALWAYS,"* You should really fix this!\n" );
		pvm_initsend(PvmDataDefault);
		pvm_pkstr(JobList->hosts[host_num].name);
		pvm_send(JobList->local_proc_tid, 12);
		RemoveHost(host_num);
		free(tmp);
		return 0;
	}

	p += 5;
	int i;
	for ( i=0 ; p[i] != '>' ; i++ ) {
		;
	}
	p[i] = '\0';

	JobList->hosts[host_num].hi_dsig = atoi ( p );
	
	dprintf( D_FULLDEBUG, "dsig = %d\n", JobList->hosts[host_num].hi_dsig );
	free( tmp );
		/* done ripping out hi_dsig */
#endif

	JobList->hosts[host_num].starter_tid = starter_tid;
	if (JobList->hosts[host_num].starter_tid == -15) {
		dprintf(D_ALWAYS, "Looks like trouble on host %s\n", 
				JobList->hosts[host_num].name );
		pvm_initsend(PvmDataDefault);
		pvm_pkstr(JobList->hosts[host_num].name);
		pvm_send(JobList->local_proc_tid, 12);
		RemoveHost(host_num);
	} else {
		JobList->hosts[host_num].status = PVM_STATE_REPLIED;
		cancel_timer(JobList->hosts[host_num].host_timer);
		JobList->hosts[host_num].host_timer = -1;
		dprintf( D_ALWAYS, "Received PVM info from %s\n",
				JobList->hosts[host_num].name );
		SendPVMStartAck();
	}
	return 0;
}


int
pseudo_pvm_task_info(int pid, int task_tid)
{
	int		host_num;
	int		work_id;
	Request		*request;

	host_num = JobList->procs[Procnum].host_num;
	work_id = JobList->hosts[host_num].starter_tid;
	request = FindRequest(-1, work_id, REQ_SPAWN, -1);
	if (request != 0) {
		dprintf( D_ALWAYS, 
				"task_info from %s task_tid = %x, ack to t%x\n",
				JobList->hosts[host_num].name, 
				task_tid, request->for_tid);
		CompleteSpawn(request, task_tid, pid);
	} else {
		dprintf( D_ALWAYS, 
				"SPAWNACK from %s, w/ task_tid = %x, but no work record\n",
				JobList->hosts[host_num].name, task_tid );
		pvm_sendsig( task_tid, SIGKILL );
/*			RemoveProc( pid ); */
	}
	return 0;
}


int
pseudo_suspended(int suspended)
{
	int		host_num = JobList->procs[Procnum].host_num;

	JobList->hosts[host_num].is_suspended = suspended;
	if (suspended) {
		dprintf( D_ALWAYS, "Host %s has been Suspended\n",
				JobList->hosts[host_num].name );
		// Set was_ever_suspended to TRUE.  This flag is checked
		// before we send Resume notifications to the user.  We do
		// not send any Resumes until we have been suspended first at
		// least once in our lifetime; i.e. the user can assume that
		// the job starts in the Resume state.
		JobList->hosts[host_num].was_ever_suspended = 1;
		SendNotification( PvmHostSuspend, 
						 JobList->hosts[host_num].pvmd_tid | TIDPVMD);
	} else {
		dprintf( D_ALWAYS, "Host %s has Resumed\n",
				JobList->hosts[host_num].name );
		SendNotification( PvmHostResume, 
						 JobList->hosts[host_num].pvmd_tid | TIDPVMD);
	}
	return 0;
}


int
pseudo_subproc_status(int subproc, int *statp, struct rusage *rusagep)
{
	Request			*request;
	int				condor_id;
	int				host_num;
	Process			*dead_process;
	int stat1 = *statp;

	dprintf(D_ALWAYS,
			"Subproc %d exited, termsig = %d, coredump = %d, retcode = %d\n",
			subproc, WTERMSIG(stat1), WCOREDUMP(stat1), 
			WEXITSTATUS(stat1));
	dprintf( D_ALWAYS, "ru_utime = %d.%06d\n",
			rusagep->ru_utime.tv_sec, rusagep->ru_utime.tv_usec );
	dprintf( D_ALWAYS, "ru_stime = %d.%06d\n",
			rusagep->ru_stime.tv_sec, rusagep->ru_stime.tv_usec );

	update_rusage( &(JobList->RemoteUsage), rusagep );

	if (subproc == PVMD_MAGIC_PROC) {
		if (WCOREDUMP(stat1)) {
			JobList->pvmd_coredumps++;
		}
		return 0; // the remaining code is just for tasks, not pvmds
	}

	if (WCOREDUMP(stat1)) {
		JobList->coredumps++;
	}

	dead_process = &(JobList->procs[subproc]);

	dprintf(D_ALWAYS, "Procnum = %d, subproc = %d, vpid = %d\n",
			Procnum, subproc, dead_process->vpid);

	memcpy((void *) (&dead_process->Rusage), rusagep, sizeof( struct rusage));
	dead_process->exit_status = *statp;
	
	if (!CoCheckCompleteCkpt(dead_process)) {
		host_num = JobList->procs[Procnum].host_num;
		if (JobList->hosts[host_num].starter_proc == subproc) {
				// Since this is a RSC from the starter, subproc
				// should point to a user job process, not to the
				// starter itself.  I've seen some strange behavior
				// here though, so we explicitly check for this logic
				// error here, instead of letting it mysteriously kill
				// us when we return.
			EXCEPT("FATAL LOGIC ERROR: Starter reported itself exited!");
		}
		RemoveProc(subproc);
		/* See if this was a bad process */
		if ((WTERMSIG(stat1) == 0 && WCOREDUMP(stat1) != 0) ||
			(WTERMSIG(stat1) == 9 && WCOREDUMP(stat1) == 0)) {
			condor_id = JobList->hosts[host_num].starter_tid;
			request = FindRequest(-1, condor_id, REQ_SPAWN, -1);
			if (request != NULL) {
				dprintf( D_ALWAYS, "Sending NACK to task t%x\n",
						request->for_tid );
				CompleteSpawn(request, PvmNoFile, 0);
			}
		}
	}

	return 0;
}


int
send_job_info( ClassAd *&ad, char *a_out, char *targ, char *orig, 
					int *kill_sig, PROC *proc)
{
	char	*ickpt;

	dprintf( D_ALWAYS, 
			"Shadow: send_job_info( %d.%d, %s, %s, %s, %s )\n",
			proc->id.cluster, proc->id.proc,
			proc->in[0], proc->out[0], proc->err[0], proc->args[0] );
	
	dprintf(D_FULLDEBUG, "Shadow: send_job_info: RootDir = <%s>\n",
															proc->rootdir);

	/* get the corresponding classad */
	InitJobAd(proc->id.cluster, proc->id.proc);
	ad = new ClassAd(*JobAd);

	char buf[ ATTRLIST_MAX_EXPRESSION ];
	sprintf( buf, "%s = \"%s\"", ATTR_JOB_ARGUMENTS, proc->args[0]);
	ad->Insert( buf );

	*kill_sig = SIGTERM;

	*targ = '\0';
	if ( proc->cmd[0][0] == '/' ) {
		strcpy(a_out, proc->cmd[0]);
	} else {
		sprintf( a_out, "%s/%s", proc->iwd, proc->cmd[0] );
	}
	ickpt = orig;
	*ickpt = '\0';

	dprintf( D_ALWAYS, "Shadow: Sending name = '%s'\n", targ );

	dprintf( D_ALWAYS, "Shadow: Sending orig_name = %s\n", a_out );

	strcpy ( executable_name_hack, a_out );

#if defined(ALPHA)
	/* Don't need these values, and the Alpha sometimes barfs on this
	   when fields in the rusage get big */
	/* bzero(&(p->remote_usage[0]), sizeof(p->remote_usage[0])); */
#endif
	
	/* This does not appear to be used anywhere */
	/* p->n_pipes = 0; */
	return 0;
}


int
send_pvmd_job_info( ClassAd *&ad, char *a_out, char *targ, char *orig, 
					int *kill_sig, int host_num )
{
	char	*orig_name;
	char	*fix_up;
	static  char pvmd_arg_buf[200];
	char buf[ ATTRLIST_MAX_EXPRESSION ];

	dprintf( D_ALWAYS, "Shadow: send_pvm_job_info\n" );

	/* pvmd does not have an ad, so must build one */
	InitJobAd((JobList->Proc).id.cluster, (JobList->Proc).id.proc);
	ad = new ClassAd(*JobAd);

	sprintf(buf, "%s = %d", ATTR_CLUSTER_ID, -1);
	ad->Insert(buf);
	
	sprintf(buf, "%s = %d", ATTR_PROC_ID, -1);
	ad->Insert(buf);
	
	sprintf(buf, "%s = %d", ATTR_JOB_UNIVERSE, CONDOR_UNIVERSE_PVMD);
	ad->Insert(buf);
	
	sprintf(buf, "%s = \"%s\"", ATTR_JOB_CMD, "$(PVMD)");
	ad->Insert(buf);
	
	pvmd_arg_buf[0] = 0;
	strncpy(pvmd_arg_buf, 
		   strchr(JobList->hosts[host_num].pvmd_cmd, ' '), 
			sizeof(pvmd_arg_buf));

	sprintf(buf, "%s = \"%s\"", ATTR_JOB_ARGUMENTS, pvmd_arg_buf);
	ad->Insert(buf);
	
	sprintf(buf, "%s = \"%s\"", ATTR_JOB_INPUT, "");
	ad->Insert(buf);
	
	sprintf(buf, "%s = \"%s\"", ATTR_JOB_OUTPUT, "");
	ad->Insert(buf);
	
	sprintf(buf, "%s = \"%s\"", ATTR_JOB_ERROR, "");
	ad->Insert(buf);
	
	sprintf(buf, "%s = %d", ATTR_JOB_NOTIFICATION, pvm_mytid());
	ad->Insert(buf);
	
	*kill_sig = SIGTERM;

#if defined(ALPHA)
	/* Don't need these values, and the Alpha sometimes barfs on this
	   when fields in the rusage get big */
	/* memset(&(pvm_proc->remote_usage[0]), 0, sizeof(pvm_proc->remote_usage[0])); */
#endif

	dprintf( D_ALWAYS, "send_pvm_job_info: arg = %s\n", pvmd_arg_buf );

	orig_name = a_out;
	fix_up = strchr(JobList->hosts[host_num].name, '.');
	if (fix_up != NULL) {
		*fix_up = '\0';
	}
	char *host_part = strrchr(JobList->hosts[host_num].name, '@');
	if (host_part) {
		host_part++;
	} else {
		host_part = JobList->hosts[host_num].name;
	}
	sprintf( orig_name, "%s:$(PVMD)", host_part );
	if (fix_up != NULL) {
		*fix_up = '.';
	}
	
	*targ = '\0';
	*orig = '\0';

	FREE( JobList->hosts[host_num].pvmd_cmd );
	JobList->hosts[host_num].pvmd_cmd = NULL;	
	return 0;
}
