/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "sched.h"
#include "condor_debug.h"
#include "fileno.h"
#include "files.h"
#include "proc.h"
#include "exit.h"
#include "condor_io.h"
#include "shadow.h"
#include "condor_attributes.h"
#include "pvm3.h"

#include "joblist.h"
#include "condor_qmgr.h"
#include "prototypes.h"

extern struct tagJobList	*JobList;

extern PROC	*Proc;
extern char *schedd;

extern bool ShuttingDown;		// true if we're shutting down the job

ClassDef *
NewClass(int new_num, char *name, char *requirements, 
		 int min, int max, int current, ClassAd *thisad )
{
	ClassDef			*new_class;
	ClassDef			*insert_class;
	struct tagJobList	*this_job;

	this_job = JobList;

	new_class = (ClassDef *)MALLOC(sizeof(ClassDef));
	new_class->class_num = new_num;
	new_class->class_name = strdup(name);
	new_class->requirements = strdup(requirements);
	new_class->min_needed = min;
	new_class->max_needed = max;
	new_class->orig_max_needed = max;
	new_class->current = current;
	new_class->ad = thisad;
	new_class->next = NULL;
	
	if (this_job->class_list == NULL) {
		this_job->class_list = new_class;
	} else {
		for (insert_class = this_job->class_list; 
			 insert_class->next != NULL;
			 insert_class = insert_class->next) {
			;
		}
		insert_class->next = new_class;
	}
	this_job->class_count++;
	dprintf( D_ALWAYS, "Created class:\n");
	PrintClass( D_ALWAYS, new_class );
	return new_class;
}


int
PrintClass( int mask, ClassDef *classdef )
{
	dprintf(mask,  "#%d: %s (%d, %d) has %d\n",	classdef->class_num, 
			classdef->class_name, classdef->min_needed, 
			classdef->max_needed, classdef->current );
	return 0;
}


ClassDef	*
GetClassByName(char *class_name)
{
	ClassDef	*classdef;

	for (classdef = JobList->class_list; classdef != NULL; 
		 classdef = classdef->next) {
		if (!strcmp(class_name, classdef->class_name)) {
			return classdef;
		}
	}
	return NULL;
}


ClassDef	*
GetClassByNum(int class_num)
{
	ClassDef	*classdef;

	for (classdef = JobList->class_list; classdef != NULL; 
		 classdef = classdef->next) {
		if (class_num == classdef->class_num) {
			return classdef;
		}
	}
	return NULL;
}


int
StoreClassDef(ClassDef *classdef, bool qmgmt_connected)
{
	if (!qmgmt_connected) {
		if (ConnectQ(schedd, SHADOW_QMGMT_TIMEOUT) == 0) {
			return -1;
		}
	}

	dprintf ( D_FULLDEBUG, "Storing class def, curr hosts = %d (%x)\n", 
			  classdef->current, classdef->current );

	SetAttributeInt(JobList->Proc.id.cluster, classdef->class_num,
					ATTR_CURRENT_HOSTS, classdef->current);

		// If we're shutting down the computation, then make sure to
		// always set MaxHosts == CurrentHosts, so we don't request
		// any new hosts from the schedd.
	if (ShuttingDown) {
		SetAttributeInt(JobList->Proc.id.cluster, classdef->class_num,
						ATTR_MAX_HOSTS, classdef->current);
	} else {
		SetAttributeInt(JobList->Proc.id.cluster, classdef->class_num,
						ATTR_MAX_HOSTS, classdef->max_needed);
	}

	if (!qmgmt_connected) {
		DisconnectQ(0);
	}

	dprintf( D_ALWAYS, "Updated class:\n");
	PrintClass( D_ALWAYS, classdef );
	return 0;
}



int
StoreAllClassDefs(bool qmgmt_connected)
{
	ClassDef	*classdef;

	if (!qmgmt_connected) {
		if (ConnectQ(schedd, SHADOW_QMGMT_TIMEOUT) == 0) {
			return -1;
		}
	}

	for (classdef = JobList->class_list; classdef; 
		 classdef = classdef->next ) {
		StoreClassDef(classdef, true);
	}

	if (!qmgmt_connected) {
		DisconnectQ(0);
	}

	return 0;
}
