/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 

#define _POSIX_SOURCE
#include "condor_common.h"
#include "condor_io.h"

#include <sys/param.h>
#include <netinet/in.h>
#include "condor_debug.h"
#include "debug.h"
#include "sched.h"
#include "proc.h"
#include "joblist.h"
#include "request.h"

#include "pvm3.h"

#ifdef PVM34
#include "pvmproto.h"
#else
#include "pvmsdpro.h"
#endif

#include "CoCheckExport.h"
#include "carmipro.h"

#include "prototypes.h"
#include "../condor_ckpt_server/server_interface.h"

extern "C" {
	void CoCheckCompleteSpawn(WorkItem  work_ptr, int new_tid);
}

#if defined(CoCheck)

extern char *Spool;
extern PROC *Proc;

extern struct tagJobList	*JobList;
extern int  ProcTimeout;
int		proc_start_timeout();


int co_check(int, int);
int co_check_restart(int, int);
int co_check_exec_name(int, int);
int co_check_rejoin(int, int);
int co_check_prt(int, int);

int co_check_spawn(int, int);
int co_check_ckpt(int, int);
int co_check_restart(int, int);
int co_check_migrate(int, int);
int co_check_rm_ckpt(int, int);

static struct {
	char	*msg_nam;
	int		msg_tag;
	int		(*handler)();
} MsgNames[] = {
{ "CO_CHECK_REQ", CO_CHECK_REQ, co_check },
{ "CO_CHECK_RESTART", CO_CHECK_RESTART, co_check_restart },
{ "CO_CHECK_RJN", CO_CHECK_RJN, co_check_rejoin },
{ "CO_CHECK_EXEC_NAME", CO_CHECK_EXEC_NAME, co_check_exec_name },
{ "CO_CHECK_PRT", CO_CHECK_PRT, co_check_prt },
{ "CO_CHECK_REQ_CKPT", CO_CHECK_REQ_CKPT, co_check_ckpt },
{ "CO_CHECK_REQ_RESTART", CO_CHECK_REQ_RESTART, co_check_restart },
{ "CO_CHECK_REQ_MIGRATE", CO_CHECK_REQ_MIGRATE, co_check_migrate },
{ "CO_CHECK_RM_CKPT", CO_CHECK_RM_CKPT, co_check_rm_ckpt },
};


int HandleCoCheck( int msgtag, int msgtid, int req_id )
{
	int		i;
	int		handled_it = 0;

	for (i = 0; i < sizeof(MsgNames) / sizeof(MsgNames[0]); i++) {
		if (msgtag == MsgNames[i].msg_tag) {
			dprintf( D_ALWAYS, "CoCheck message is %s from t%x\n", 
				   MsgNames[i].msg_nam, msgtid );
			MsgNames[i].handler(msgtid, req_id);
			handled_it = 1;
			break;
		}
	}

	return handled_it;
}


int co_check(int msg_tid, int req_id)
{
}

int co_check_exec_name(int msg_tid, int req_id)
{
}


static void CoCheckSendSyn(
int ckptMethod,
char *taskCkptFile,
int n,
int *tids,
int target_tid)
{
	int		syns_sent;
	int		MyTid = pvm_mytid();

	pvm_initsend(PvmDataDefault);
	pvm_pkint(&MyTid, 1, 1);
	pvm_pkint(&ckptMethod, 1, 1);
	if( ckptMethod == CO_CHECK_METHOD_FILE ) {
		pvm_pkstr(taskCkptFile);
	}

	pvm_pkint(&n, 1, 1);
	pvm_pkint(tids, n, 1);
#ifdef DEBUG
  fprintf(stderr, "Sending CO_CHECK_SYN to t%x\n", rec);
#endif
	pvm_send(target_tid, CO_CHECK_SYN);
/*	set_task_state(target_tid, state);   /* set new task state */
}

// in pvm3.4 we do not support respawn 
ReSpawn(
Process	*process,
int		for_whom,
int		req_id,
char	*arg_line,
char	*dest_arch,
int		where_flag,
int		out_tid,
int		out_cod,
int		trc_tid,
int		trc_cod,
int		response,
int		spawn_type)
{
	Request	*request;
	WorkItem	*work;

#ifdef PVM34	
	// in pvm3.4 we do not support respawn 
#else
	DoSpawn(for_whom, req_id, process->a_out, arg_line, dest_arch,
			where_flag, out_tid, out_cod, trc_tid, trc_cod, response, 
			spawn_type);
#endif
	request = FindRequest(for_whom, -1, REQ_SPAWN, req_id);
	if (request == 0) {
		dprintf(D_ALWAYS, "ReSpawn() cannot find request made by DoSpawn()\n");
	} else {
		work = (WorkItem *) request->extra_data;
		work->process = process;
	}
}


extern int  ProcTimeout;

void CoCheckCompleteSpawn(Request *request, int new_tid)
{
	int		zero = 0;
	int		one = 1;	
	char	*ckpt_name;
	Process	*target_process;
	WorkItem	*work_ptr;
	Request	*new_request;
	int		*new_timer_num;
	int		rejoin_timeout();

	work_ptr = (WorkItem *) request->extra_data;

	if (work_ptr->spawn_type == RESTART_SIMPLE ||
		work_ptr->spawn_type == MIGRATE_SIMPLE) {
	}
	new_request = NewRequest(REQ_RJN_MSG, request->for_tid, 
							 request->req_id, request->resp_msg);
	new_request->on_tid = new_tid;
	new_timer_num = (int *) malloc(sizeof(int));
	new_request->extra_data = (void *) new_timer_num;
	*new_timer_num = new_timer( ProcTimeout * 2, rejoin_timeout );
}


int CoCheckCompleteCkpt(Process *dead_process)
{
	Request		*request;
	int			i;
	GeneralCkptReq	*ckpt_req;
	int			host_num;

	dprintf(D_ALWAYS, "subproc_status request list\n");
	DumpRequestList(D_ALWAYS);

	/* There's really gotta be a loop around this whole thing to find
	   all potential reqs.  I'll get to that someday. */

	request = FindRequest(-1, dead_process->pvm_tid, REQ_CKPT, -1);
	if (!request) {
		request = FindRequest(-1, dead_process->vpid, REQ_MIGRATE,
							  -1);
	}
	if (!request) {
		request = FindRequest(-1, dead_process->pvm_tid, REQ_CKPT_TASK_TO_FILE,
							  -1);
	}
	if (request) {
		/* If this was a checkpoint, we complete the request here, if it
		   is a migration, we just drop the migrate request and wait for
		   the ReSpawn request to complete to send the response to the
		   application */
		dprintf(D_ALWAYS, "Ckpt request is: ");
		dprintf_request(D_ALWAYS | D_NOHEADER, request);
		if (request->req_type == REQ_CKPT) {
			i = 1;		
			pvm_initsend(PvmDataDefault);
			pvm_pkint(&i, 1, 1);
			pvm_pkint(&(dead_process->pvm_tid), 1, 1);
			SendResponse( request );
			dead_process->process_state = PROCESS_CHECKPOINTED;
		}

		DeleteRequest( request );
		if (request->req_type != REQ_CKPT_TASK_TO_FILE) {
			/* Now, we must decrement the number of processes running on this
			   host. */
			host_num = dead_process->host_num;
			JobList->hosts[host_num].status--;
			dead_process->host_num = -2;
		} else {
			request = 0;	/* Set this to zero, so pseudo_subproc_status
							   will clean up this process' list entry */
		}
	} else {
		request = FindRequest(-1, -1, REQ_CKPT, -1);
		if (request) {
			ckpt_req = (GeneralCkptReq *) request->extra_data;
			for (i = 0; i < ckpt_req->ckpt_cnt; i++) {
				if (ckpt_req->ckpt_list[i] == dead_process->pvm_tid) {
					break;
				}
			}

			/* Did we find this task in the list of ckpting tasks
			   for this request? */
			if (i < ckpt_req->ckpt_cnt) {
				dead_process->process_state = PROCESS_CHECKPOINTED;
				host_num = dead_process->host_num;
				JobList->hosts[host_num].status--;
				dead_process->host_num = -2;

				ckpt_req->ckpt_cnt--;
				/* Now, check to see if all the required processes have
				   checkpointed */
				if (ckpt_req->ckpt_cnt > 0) {
					ckpt_req->ckpt_list[i] = 
						ckpt_req->ckpt_list[ckpt_req->ckpt_cnt];
				} else {
					i = 1;		
					pvm_initsend(PvmDataDefault);
					pvm_pkint(&i, 1, 1);
					pvm_pkint(&(dead_process->pvm_tid), 1, 1);
					SendResponse( request );
					free(ckpt_req->ckpt_list);
					free(ckpt_req->consistent_list);
					DeleteRequest( request );
				}
			} else {
				/* If we didn't find this task in the list, we might as
				   well have not found the request at all */
				request = 0;
			}
		}
	}
	return (request != 0);
}


int
co_check_rejoin(int msg_tid, int req_id)
{
	int		otid;
	int		ctid;
	Request	*request;
	int		ack_count;
	char	*ckpt_name;
	Process	*target_process;

	pvm_upkint(&otid, 1, 1);
	pvm_upkint(&ctid, 1, 1);

	dprintf(D_ALWAYS, "Task with orig tid %x is now t%x\n", otid, ctid);
	dprintf(D_ALWAYS, "Sending empty CO_CHECK_LST to t%x\n", ctid);
	pvm_initsend(PvmDataDefault);
	ack_count = 1;
	pvm_pkint(&ack_count, 1, 1);
	pvm_pkint(&otid, 1, 1);
	pvm_pkint(&ctid, 1, 1);
	pvm_send(ctid, CO_CHECK_LST);
	target_process = FindProcessByTid(otid);
	if (target_process) {
		ckpt_name = GetProcessCkptName(target_process->vpid);
		RemoveLocalOrRemoteFile(JobList->Proc.owner, ckpt_name);
		target_process->pvm_tid = ctid;
		/* Since this was a simple spawn, let's just trash the 
		   orig name. */
		target_process->pvm_orig_tid = otid;
	}

	request = FindRequest(-1, msg_tid, REQ_RJN_MSG, -1);
	if (request) {
		cancel_timer(*((int *) request->extra_data));
		pvm_initsend(PvmDataDefault);
		ack_count = 1;
		pvm_pkint(&ack_count, 1, 1);
		pvm_pkint(&msg_tid, 1, 1);
		SendResponse(request);
		DeleteRequest(request);
	} else {
		dprintf(D_ALWAYS, "Cannot find request for this RJN msg.\n");
	}
}


rejoin_timeout(int timer_id)
{
	Request	*request;
	int		*req_timer_num;
	int		nack;

	for (request = FindRequest(-1, -1, REQ_RJN_MSG, -1);
		 request != 0;
		 request = FindNextRequest(request, -1, -1, REQ_RJN_MSG, -1)) {
		req_timer_num = (int *) request->extra_data;
		if ( *req_timer_num == timer_id ) {
			dprintf(D_ALWAYS, "Timed out waiting for RJN msg from t%x",
					request->on_tid);
			pvm_initsend(PvmDataDefault);
			nack = PvmNoTask;
			pvm_pkint(&nack, 1, 1);
			SendResponse(request);
			DeleteRequest(request);
			break;
		}
	}

	if (request == 0) {
		dprintf(D_ALWAYS, "Couldn't find rejoin timer #%d\n", timer_id);
	}
}


int
co_check_prt(int msg_tid, int req_id)
{
	Process		*target_process;
	struct sockaddr_in sin;
	int			socket_fd;
	char		*dest_file;
	int			dest_file_fd;
	int			len;
	Request		*request;
	char		ckpt_url[MAXPATHLEN];
	char		restart_arg[MAXPATHLEN];
	int			dest_host_tid;
	Host		*dest_host;
	int			spawn_flag;
	char		*spawn_where;
	int			spawn_type;

	target_process = FindProcessByTid(msg_tid);
	request = FindRequest(-1, target_process->vpid, REQ_MIGRATE, -1);
	spawn_type = MIGRATE;
	if (!request) {
		request = FindRequest(-1, target_process->vpid, 
							  REQ_MIGRATE, -1);
		spawn_type = MIGRATE_SIMPLE;
	}
	if (target_process && request) {
		if (target_process->process_state != PROCESS_MIGRATING) {
			/* This is weird, what should we do */
		}
		pvm_upkbyte((void *) &(sin.sin_addr), sizeof(sin.sin_addr), 1);
		pvm_upkbyte((void *) &(sin.sin_port), sizeof(sin.sin_port), 1);
		sprintf(ckpt_url, "cbstp:%s", sin_to_string(&sin));
		dprintf(D_ALWAYS, "CO_CHECK_PRT: reporting URL %s\n", ckpt_url);
		sprintf(restart_arg, "-_condor_restart %s", ckpt_url);
		dest_host_tid = *((int *) request->extra_data);
		if (dest_host_tid) {
			dest_host = FindHostByTid(dest_host_tid);
			if (dest_host) {
				spawn_flag = PvmTaskHost;
				spawn_where = dest_host->name;
			} else {
				/* Can't migrate where requested, gotta NACK */
			}
		} else {
			spawn_flag = PvmTaskArch;
			spawn_where = target_process->dest_arch;
		}
		ReSpawn( target_process, request->for_tid, request->req_id, 
				restart_arg, spawn_where, spawn_flag, 0, 0, 0, 0, 
				request->resp_msg, spawn_type);
		/* The spawn request now is the request to complete */
/*		DeleteRequest( request ); */
	}
}


int co_check_simple_ckpt_task_to_file(int msg_tid, int req_id)
{
	int		response;
	int		target_tid;
	char	filename[MAXPATHLEN];
	int		rval;
	Request	*request;

	pvm_upkint(&target_tid, 1, 1);
	pvm_upkstr(filename);
	pvm_upkint(&response, 1, 1);

	CoCheckSendSyn(CO_CHECK_METHOD_FILE, filename, 0, 0, target_tid);
	pvm_sendsig(target_tid, CO_CHECK_SIG);

#if 0
	pvm_initsend(PvmDataDefault);
	pvm_pkint(&rval, 1, 1);
	carmi_store_req_id(req_id);
	pvm_send(msg_tid, response);
#else
	request = NewRequest(REQ_CKPT_TASK_TO_FILE, msg_tid, req_id, response);
	request->on_tid = target_tid;
#endif
}


int co_check_restart_task_from_file(int msg_tid, int req_id)
{
	char	a_out[MAXPATHLEN];
	char	dest_arch[100];
	char	filename[MAXPATHLEN];
	int		response;
	char	restart_arg[MAXPATHLEN];

	pvm_upkstr(a_out);
	pvm_upkstr(dest_arch);
	pvm_upkstr(filename);
	pvm_upkint(&response, 1, 1);

	sprintf(restart_arg, "-_condor_restart %s", filename);
#ifdef PVM34  
	// in pvm3.4 we do not support respawn 
#else
	DoSpawn( msg_tid, req_id, a_out, restart_arg, dest_arch, 
			PvmTaskArch, 0, 0, 0, 0, response, RESTART_SIMPLE);
#endif	
}



int co_check_simple_ckpt_task(int msg_tid, int req_id)
{
	int		response;
	int		target_tid;
	int		rval;
	Request	*request;
	Process *target_process;
	char	*dest_file;
	char	ckpt_url[MAXPATHLEN];
	struct sockaddr_in sin;

	pvm_upkint(&target_tid, 1, 1);
	pvm_upkint(&response, 1, 1);

	target_process = FindProcessByTid(target_tid);
	if (!target_process) {
		pvm_initsend(PvmDataDefault);
		rval = PvmNoTask;
		pvm_pkint(&rval, 1, 1);
		carmi_store_req_id(req_id);
		pvm_send(msg_tid, response);
	} else {
		dest_file = GetProcessCkptName(target_process->vpid);
		pseudo_put_file_stream(dest_file, -1, &(sin.sin_addr.s_addr), 
							   &(sin.sin_port));
		sin.sin_addr.s_addr = htonl(sin.sin_addr.s_addr);
		sin.sin_port = htons(sin.sin_port);
		sprintf(ckpt_url, "cbstp:%s", sin_to_string(&sin));
		dprintf(D_ALWAYS, "Giving URL %s to task t%x\n", ckpt_url, target_tid);
		CoCheckSendSyn(CO_CHECK_METHOD_FILE, ckpt_url, 0, 0, target_tid);
		pvm_sendsig(target_tid, CO_CHECK_SIG);
		request = NewRequest(REQ_CKPT, msg_tid, req_id, response);
		request->on_tid = target_tid;
	}
}


int co_check_ckpt(int msg_tid, int req_id)
{
	GeneralCkptReq		*ckpt_req;
	Request				*request;
	int					response;
	Process				*target_process;
	int					i;
	int					all_list_cnt;
	int					*all_list;
	char				*dest_file;
	char				ckpt_url[MAXPATHLEN];
	struct sockaddr_in 	sin;
	int					one = 1;

	ckpt_req = (GeneralCkptReq *) malloc(sizeof(GeneralCkptReq));
	pvm_upkint(&(ckpt_req->ckpt_cnt), 1, 1);
	ckpt_req->ckpt_list = (int *) malloc(ckpt_req->ckpt_cnt * sizeof(int));
	pvm_upkint(ckpt_req->ckpt_list, ckpt_req->ckpt_cnt, 1);

	pvm_upkint(&(ckpt_req->consistent_cnt), 1, 1);
	ckpt_req->consistent_list = (int *) malloc(ckpt_req->consistent_cnt * 
											   sizeof(int));
	pvm_upkint(ckpt_req->consistent_list, ckpt_req->consistent_cnt, 1);

	pvm_upkint(&(ckpt_req->flush_cnt), 1, 1);
	ckpt_req->flush_list = (int *) malloc(ckpt_req->flush_cnt * 
										  sizeof(int));
	pvm_upkint(ckpt_req->flush_list, ckpt_req->flush_cnt, 1);

	pvm_upkint(&response, 1, 1);

	request = NewRequest(REQ_CKPT, msg_tid, req_id, response);
	request->extra_data = (void *) ckpt_req;

	/* Need to build a single list containing all of the tasks */
	all_list_cnt = ckpt_req->ckpt_cnt + ckpt_req->consistent_cnt + 
		ckpt_req->flush_cnt;

	all_list = (int *) malloc( all_list_cnt * sizeof(int));
	memcpy(all_list, ckpt_req->ckpt_list, ckpt_req->ckpt_cnt * sizeof(int));
	memcpy(&(all_list[ckpt_req->ckpt_cnt]), ckpt_req->consistent_list,
		   ckpt_req->consistent_cnt * sizeof(int));
	memcpy(&(all_list[ckpt_req->ckpt_cnt + ckpt_req->consistent_cnt]),
		   ckpt_req->flush_list,
		   ckpt_req->flush_cnt * sizeof(int));

	/* This is too lazy.  Really gotta validate each tid before putting it
	   into the list */

	for (i = 0; i < ckpt_req->ckpt_cnt; i++) {
		/* Copy from simple ckpt task, except send SYN should include
		   all the tasks */
		target_process = FindProcessByTid(ckpt_req->ckpt_list[i]);
		if (target_process) {
			dest_file = GetProcessCkptName(target_process->vpid);
			pseudo_put_file_stream(dest_file, -1, &(sin.sin_addr.s_addr), 
								   &(sin.sin_port));
			sin.sin_addr.s_addr = htonl(sin.sin_addr.s_addr);
			sin.sin_port = htons(sin.sin_port);
			sprintf(ckpt_url, "cbstp:%s", sin_to_string(&sin));
			dprintf(D_ALWAYS, "Giving URL %s to task t%x\n", ckpt_url, 
					ckpt_req->ckpt_list[i]);
			CoCheckSendSyn(CO_CHECK_METHOD_FILE, ckpt_url, all_list_cnt, 
						   all_list, ckpt_req->ckpt_list[i]);
			pvm_sendsig(ckpt_req->ckpt_list[i], CO_CHECK_SIG);
		} else {
			dprintf(D_ALWAYS, "Requested to ckpt unknown t%x\n",
					ckpt_req->ckpt_list[i]);
		}
	}

	for (i = 0; i < ckpt_req->consistent_cnt; i++) {
		/* Do SendSYN to everyone, but method is CO_CHECK_METHOD_NONE */
		target_process = FindProcessByTid(ckpt_req->consistent_list[i]);
		if (target_process) {
			CoCheckSendSyn(CO_CHECK_METHOD_NONE, 0, all_list_cnt, 
						   all_list, ckpt_req->consistent_list[i]);
			pvm_sendsig(ckpt_req->consistent_list[i], CO_CHECK_SIG);
		} else {
			dprintf(D_ALWAYS, "Requested to be consistent with unknown t%x\n",
					ckpt_req->consistent_list[i]);
		}
	}

	for (i = 0; i < ckpt_req->flush_cnt; i++) {
		/* Do SendSYN to everyone, but method is CO_CHECK_METHOD_NONE, and
		   immediately send a CO_CHECK_LST to get them rolling again. */
		target_process = FindProcessByTid(ckpt_req->flush_list[i]);
		CoCheckSendSyn(CO_CHECK_METHOD_NONE, 0, all_list_cnt, 
					   all_list, ckpt_req->flush_list[i]);
		pvm_sendsig(ckpt_req->flush_list[i], CO_CHECK_SIG);
		dprintf(D_ALWAYS, "Sending empty CO_CHECK_LST to t%x\n", 
				ckpt_req->flush_list[i]);
		pvm_initsend(PvmDataDefault);
		pvm_pkint(&one, 1, 1);
		pvm_pkint(&(ckpt_req->flush_list[i]), 1, 1);
		pvm_pkint(&(ckpt_req->flush_list[i]), 1, 1);
		pvm_send(ckpt_req->flush_list[i], CO_CHECK_LST);
	}

	ckpt_req->flush_cnt = 0;
	free(ckpt_req->flush_list);
	free(all_list);
}


int co_check_restart(int msg_tid, int req_id)
{
	int		response;
	int		target_tid;
	char	restart_arg[MAXPATHLEN];
	struct sockaddr_in sin;
	Process	*target_process;
	int		rval;
	char	*dest_file;
	char	ckpt_url[MAXPATHLEN];
	int		file_size;
	int		get_file_rval;

	pvm_upkint(&target_tid, 1, 1);
	pvm_upkint(&response, 1, 1);

	target_process = FindProcessByTid(target_tid);
	if (!target_process) {
		pvm_initsend(PvmDataDefault);
		rval = PvmNoTask;
		pvm_pkint(&rval, 1, 1);
		carmi_store_req_id(req_id);
		pvm_send(msg_tid, response);
		dprintf(D_ALWAYS, "No such task t%x to restart\n", target_tid);
	} else if (target_process->process_state != PROCESS_CHECKPOINTED ) {
		pvm_initsend(PvmDataDefault);
		rval = PvmNoFile;
		pvm_pkint(&rval, 1, 1);
		carmi_store_req_id(req_id);
		pvm_send(msg_tid, response);
		dprintf(D_ALWAYS, "Task t%x not ckpted to restart\n", target_tid);
	} else {
		dest_file = GetProcessCkptName(target_process->vpid);
		get_file_rval = pseudo_get_file_stream(dest_file, &file_size, 
											   &(sin.sin_addr), 
											   &(sin.sin_port));
		if (get_file_rval < 0) {
			pvm_initsend(PvmDataDefault);
			rval = PvmNoFile;
			pvm_pkint(&rval, 1, 1);
			carmi_store_req_id(req_id);
			pvm_send(msg_tid, response);
			dprintf(D_ALWAYS, "Cannot find server to restart task t%x\n", 
					target_tid);
		} else {
			sin.sin_addr.s_addr = htonl(sin.sin_addr.s_addr);
			sin.sin_port = htons(sin.sin_port);
			sprintf(ckpt_url, "cbstp:%s", sin_to_string(&sin));
			dprintf(D_ALWAYS, "Giving URL %s to task t%x\n", ckpt_url, 
					target_tid);
			sprintf(restart_arg, "-_condor_restart %s", ckpt_url);
			ReSpawn( target_process, msg_tid, req_id, restart_arg, 
					target_process->dest_arch, PvmTaskArch, 0, 0, 0, 0, 
					response, RESTART_SIMPLE);
		}
	}
}


int co_check_migrate(int msg_tid, int req_id)
{
	int		response;
	int		target_tid;
	int		target_host_tid;
	Process	*target_process;
	int		rval;
	Request	*request;

	pvm_upkint(&target_tid, 1, 1);
	pvm_upkint(&target_host_tid, 1, 1);
	pvm_upkint(&response, 1, 1);

	target_process = FindProcessByTid(target_tid);
	if (!target_process) {
		pvm_initsend(PvmDataDefault);
		rval = PvmNoTask;
		pvm_pkint(&rval, 1, 1);
		carmi_store_req_id(req_id);
		pvm_send(msg_tid, response);
	} else {
		CoCheckSendSyn(CO_CHECK_METHOD_PORT, 0, 0, 0, target_tid);
		pvm_sendsig(target_tid, CO_CHECK_SIG);
		request = NewRequest(REQ_MIGRATE, msg_tid, req_id,
							 response);
		request->on_tid = target_tid;
		request->extra_data = (void *) MALLOC(sizeof(int));
		*((int *) request->extra_data) = target_host_tid;
	}
}


int co_check_rm_ckpt(int msg_tid, int req_id)
{
	int		tid;
	int		one = 1;
	int		response;
	Process	*target_process;

	pvm_upkint(&tid, 1, 1);
	pvm_upkint(&response, 1, 1);

	target_process = FindProcessByTid(tid);

	if (target_process) {
		if (target_process->process_state == PROCESS_CHECKPOINTED) {
			RemoveProc(target_process->vpid);
		} else {
			dprintf(D_ALWAYS, "RmCkpt: Task t%x is not ckpted (state = %d)\n",
					tid, target_process->process_state);
			tid = PvmNoFile;
		}
	} else {
		dprintf(D_ALWAYS, "RmCkpt: No such task t%x\n", tid);
		tid = PvmNoTask;
	}

	if (response != 0) {
		pvm_initsend(PvmDataDefault);
		pvm_pkint(&one, 1, 1);
		pvm_pkint(&tid, 1, 1);
		carmi_store_req_id(req_id);
		pvm_send(msg_tid, response);
	}
}

#else /* !CoCheck */

int
HandleCoCheck( int msgtag, int msgtid, int req_id )
{
	return 0;
}

void CoCheckCompleteSpawn(WorkItem  work_ptr, int new_tid)
{
}

int
CoCheckCompleteCkpt(Process *dead_process)
{
	return 0;
}

#endif
