/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "condor_classad.h"
#include "condor_attributes.h"
#define MAX_PVM_HOSTS	1024
#include "errno.h"
#include "sched.h"   /* For the PORTS struct */
#include "condor_debug.h"
#include "my_hostname.h"
#include "fileno.h"
#include "proc.h"
#include "condor_io.h"
#include "condor_uid.h"
#include "shadow.h"
#include "pvm3.h"
#include "job_report.h"

#ifdef PVM34
#include "pvmproto.h"
#else
#include "pvmsdpro.h"
#endif

#include "pvm_global.h"


#include "joblist.h"
extern struct tagJobList	*JobList;
#include "request.h"

#include "user_log.c++.h"
extern "C"  void log_termination(struct rusage *, struct rusage *);

FILE	*fdopen();

extern int	PipeToParent;
extern int	LogPipe;
extern int	LockFd;
extern int	ImageSize;
extern ClassAd *JobAd;
extern char	CurrentWorkingDir[ _POSIX_PATH_MAX ];

int	Procnum;	/* Process number of process doing this syscall */

extern int  ProcTimeout;
extern int 	HostTimeout;

extern PROC	*Proc;
extern char	*Spool;

extern ReliSock *syscall_sock;

extern int JobStatus;
extern struct rusage JobRusage;
extern int ChildPid;
extern int MainSymbolExists;

extern struct rusage LocalUsage;
extern struct rusage RemoteUsage;

extern ReliSock		stdin_relisock;
extern int		maxjob;
extern float TotalBytesSent, TotalBytesRecvd;
extern float BytesSent, BytesRecvd;

// the address of the schedd.
extern char *schedd;

#include "prototypes.h"
#include "condor_qmgr.h"
#include "get_daemon_addr.h"

extern "C" {
	void NotifyUser( char *buf, PROC *proc );
	void CompleteSpawn(Request *request, int new_tid, int pid);
	extern void get_local_rusage( struct rusage *bsd_rusage );
}

extern char *my_hostname();
extern int do_REMOTE_syscall(ReliSock *sock, int process);

void
HandleMsg()
{
#ifdef PVM34
     struct pvmminfo info;
#endif
	int		msg;
	int		for_whom;
	int		byte_count, bufid;
	int		req_id;

	/* Watch out for non-empty buffer at PVM level */
	while ( (bufid = pvm_nrecv( -1, -1 )) != 0 ) {
		if (bufid < 0) {
			EXCEPT( "Looks like we lost the PVMd!" );
		}
		pvm_bufinfo( bufid, &byte_count, &msg, &for_whom );
#ifdef PVM34
		// replace pvm_getmwid with pvm_getminfo() in pvm3.4
		pvm_getminfo(bufid, &info);
		req_id = info.wid;
#else
		req_id = pvm_getmwid(bufid);
#endif
		if ( !HandlePVM( msg, for_whom, req_id ) ) {
			if ( !HandleCARMI( msg, for_whom, req_id ) ) {
				if ( !HandleCoCheck( msg, for_whom, req_id ) ) {
					dprintf(D_ALWAYS, 
							"Unknown message with tag %d(0x%x) from t%x\n",
							msg, msg, for_whom);
				}
			}
		}
	}
}

/*
** Handle a system call for process process_num for job
*/

void
multi_HandleSyscalls(int process_num)
{
    int         syscall_ret;

    Proc = &(JobList->Proc);

    set_user_priv();

    dprintf(D_SYSCALLS, "multi_HandleSyscalls on process %d\n",
			process_num);

	strcpy(CurrentWorkingDir, JobList->procs[process_num].cwd);
    dprintf(D_FULLDEBUG, "multi_HandleSyscalls: about to chdir(%s)\n", 
			CurrentWorkingDir);
    if( chdir(CurrentWorkingDir) < 0 ) {
		job_report_store_error( "Can't access \"%s\"", CurrentWorkingDir );
		EXCEPT("chdir(%s)", CurrentWorkingDir );
    }

    errno = 0;

    /* No longer want to do forever, just go handle one, then we can return */

    JobStatus = JobList->JobStatus[process_num];
    ImageSize = JobList->procs[process_num].ImageSize;
/*    JobRusage = JobList->JobRusage[process_num]; */

	Procnum = process_num;
	syscall_sock = JobList->procs[process_num].syscall_sock;

	dprintf ( D_SYSCALLS, "Doing remote syscall, %0x %d\n", 
			  JobList->procs[process_num].syscall_sock, process_num );

	char stackthingy[10000];
	memset ( stackthingy, 55, 10000 );
    syscall_ret = do_REMOTE_syscall(JobList->procs[process_num].syscall_sock,
									process_num);

	syscall_sock = NULL;		// don't keep a stale pointer around
	CurrentWorkingDir[0] = '\0'; // don't keep stale cwd around

	for ( int q=0 ; q < 10000 ; q++ ) {
		if ( stackthingy[q] != 55 ) {
			dprintf ( D_ALWAYS, "STACK FUBAR\n" );
			EXCEPT( "The stack is fubar.\n" );
		}
	}

        /* Take back normal condor privileges */
	set_condor_priv();

	if ( JobList->JobStatus != NULL ) {
		JobList->JobStatus[process_num] = JobStatus;
		JobList->procs[process_num].ImageSize = ImageSize;
/*		JobList->JobRusage[process_num] = JobRusage; */
	}
}

int readline(int fd, char    *buf)
{
	int		count = 0;
	int		rval;

	for(;;) {
		if ((rval = read(fd, buf, 1)) < 0) {
			return rval;
		}
		if (*buf != '\n') {
			count++;
			buf++;
		} else {
			break;
		}
	}
	return	count;
}

int
HandleSchedd( int StdinIsAscii )
{
	char 	host[50], arch[50];
	char	addr_port[50], cap[50];
	char	*host_ptr, *arch_ptr;
	int		cluster=0, proc=0, machine_count=0;
	int		i;
	int		new_job;
	int		host_num;
	int		old_hosts = 0;
	ClassDef		*classdef = NULL;
	char	linebuf[200];

    memset ( linebuf, 0, 200 );

	if( StdinIsAscii ) {
		dprintf( D_ALWAYS, "Shadow reading via ASCII\n");
		if ( readline(0, linebuf) < 0 ) {
			dprintf( D_ALWAYS, "Pipe to Schedd is closed\n" );
			return -2;
		}
		sscanf(linebuf, "%d %d %d", &cluster, &proc, &machine_count);
		if (cluster == -1) {
			dprintf( D_ALWAYS, "Magic Cluster Number, exiting...\n" );
			multi_CleanupAll(0,0,NULL);
		}
	} else {
		dprintf( D_ALWAYS, "Shadow reading via ReliSock\n");
		stdin_relisock.decode();
		ASSERT( stdin_relisock.code(cluster) );
		ASSERT( stdin_relisock.code(proc) );
		ASSERT( stdin_relisock.code(machine_count) );
		ASSERT( stdin_relisock.end_of_message() );
		host_ptr = &(host[0]);
		arch_ptr = &(arch[0]);
	}
	
	dprintf( D_ALWAYS, "First Line: %d %d %d\n", cluster, proc,
			machine_count );

	if ((new_job = GetJobNum( cluster, proc )) != 0) {
		if (InitJob( cluster, proc ) != 0) {
			dprintf(D_ALWAYS, "InitJob() failed\n");
			return -1;
		}
		new_job = 1;
	}

	Proc = &(JobList->Proc);
	if (new_job == 1)
		init_user_ids( Proc->owner );

	if ( machine_count == -1 ) {
//		RemoveJob();
		return 0;
	}
	for (i = 0; i < machine_count; i++) {
		if( StdinIsAscii ) {
			if ( readline(0, linebuf) < 0 ) {
//				RemoveJob();
				return -2;
			}
			sscanf(linebuf, "%s %s %s %s", addr_port, cap, arch, host);

			dprintf(D_ALWAYS,"Machine from schedd: %s %s %s\n",
						addr_port, cap, arch);
		} else {
			ASSERT( stdin_relisock.code(host_ptr) );
			ASSERT( stdin_relisock.code(arch_ptr) );
			ASSERT( stdin_relisock.end_of_message() );
		}

		/* Be very careful, if we get a host, and we think that same
		   host is already being used, they end up on the starting
		   list forever.  Just drop hosts that we think we're already
		   using */
		if (FindHostByName(host) != NULL) {
			dprintf( D_ALWAYS, "Already have host %s, ignoring\n", host);
			old_hosts++;
		} else {
			if (classdef == NULL) {
				classdef = GetClassByName( arch );
				host_num = NewHost( host, classdef->class_num, addr_port, cap );
			} else {
				if ( classdef != GetClassByName(arch) ){
					dprintf( D_ALWAYS, 
						"Unexpected class id %s for host %s, ignoring\n",
							arch, host );
					old_hosts++;
				} else {
					host_num = NewHost( host, 
							   classdef->class_num, addr_port, cap );
				}
			}
		}
		dprintf( D_ALWAYS, "Machine Line: %s %s\n", host, arch );
	}

	machine_count -= old_hosts;

	/* Don't bother updating anything if we didn't end up with any new
	   hosts */
	if ( machine_count > 0 ) {
		machine_count += classdef->current;
		dprintf( D_ALWAYS, 
				"Machines now cur = %d desire = %d\n", machine_count,
				classdef->max_needed );
		classdef->current = machine_count;
		StoreClassDef(classdef);

		if ( new_job ) {
			machine_count = multi_start_job();
		} else {
			StartWaitingHosts();
		}
	}

	return 0;
}

/*
** Send the reschedule command to the local schedd to get the jobs running
** as soon as possible.
** This function taken from submit.c.  Probably should go in condor_util.
*/
int
reschedule()
{
        int                     cmd;

        /* Connect to the schedd */
        ReliSock sock;
        if(!sock.connect(schedd)) {
			dprintf( D_ALWAYS, "Can't connect to condor_scheddr (%s)\n",
					 schedd );
			return -1;
        }

        sock.encode();
        cmd = RESCHEDULE;
        if( !sock.code(cmd) || !sock.eom() ) {
			dprintf( D_ALWAYS,
					 "Can't send RESCHEDULE command to condor_schedd\n");
			return -1;
        }
		return 0;
}

void
HandleCompletion()
{
	char	notification[ BUFSIZ ];

	JobList->Proc.status = COMPLETED;

	struct rusage local;
	get_local_rusage(&local);
	multi_update_job_status( &local, NULL );
	MainSymbolExists = TRUE;
	multi_handle_termination( 0, notification, JobList->Proc.iwd );
		
		// write into log file that the job has terminated
	log_termination(&JobList->Proc.local_usage, &JobList->Proc.remote_usage[0]);
		
	dprintf( D_ALWAYS, "Notification = \"%s\"\n", notification );
	if( notification[0] ) {
		JobList->Proc.remote_usage[0] = JobList->RemoteUsage;
			// We don't properly keep track of network activity in the 
			// pvm shadow, so it's better to just not report anything
			// than report bad values.  So, we zero out all of the 
			// network counters here before calling NotifyUser().
		TotalBytesSent = BytesSent = TotalBytesRecvd = BytesRecvd = 0;
		NotifyUser( notification, &(JobList->Proc) );
	}
}

int
proc_start_timeout( int timer_id )
{
	WorkItem	*item;
	Host		*to_whom;
	int	hold_status;
	Host		*old_whom;
	Request		*request;

	dprintf( D_ALWAYS, "Proc Start TimeOut!\n" );
	for (request = FindRequest(-1, -1, REQ_SPAWN, -1);
		 request != 0;
		 request = FindNextRequest(request, -1, -1, REQ_SPAWN, -1)) {

		item = (WorkItem *) request->extra_data;
		if (item->timer == timer_id) {
			dprintf( D_ALWAYS, "timed out on condor_id 0x%x, resending.\n",
					request->on_tid);
			if (item->process != 0 && item->process->vpid != -1) {
				RemoveProc( item->process->vpid );
			}

			to_whom = GetMachine( item->where, item->spawn_flag );
			if (to_whom && to_whom->starter_tid == request->on_tid) {
				signal_startd( to_whom->name, RELEASE_CLAIM );

				hold_status = to_whom->status;
				to_whom->status = PVM_STATE_EXITED;
				old_whom = to_whom;
				to_whom = GetMachine( item->where, item->spawn_flag );
			} 
			if ( to_whom != 0 ) {
				signal_startd( to_whom->name, REQ_NEW_PROC );
				request->on_tid = to_whom->starter_tid;
				item->timer = new_timer( ProcTimeout, proc_start_timeout );
			} else {
				dprintf( D_ALWAYS, "No machine for timed out proc.\n" );
				CompleteSpawn(request, PvmNoHost, 0);
			}
			break;
		}
	}
	return 0;
}


int
host_timeout(int timer_id)
{
	int		host_num;

	dprintf( D_ALWAYS, "Got a host Time Out\n" );
	if (maxjob > 0) {
		for( host_num = 0; host_num < JobList->pvm_machine_count;
			host_num++) {
			if ( JobList->hosts[host_num].host_timer == timer_id ) {
				dprintf( D_ALWAYS, "Timed out on host %s\n",
						JobList->hosts[host_num].name );
				RemoveHost( host_num );
				dprintf( D_ALWAYS, "Completed time-out handler\n" );
				return 0;
			}
		}
	}
	dprintf( D_ALWAYS, "Could not find host with timer id %d\n", timer_id );
	return 0;
}
