/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "sched.h"
#include "condor_debug.h"
#include "fileno.h"
#include "files.h"
#include "proc.h"
#include "exit.h"
#include "condor_io.h"
#include "shadow.h"
#include "pvm3.h"
#include "condor_classad.h"
#include "pvm_global.h"
#include "condor_attributes.h"
#include "util_lib_proto.h"

/* Increment for expanding process list per Job */
#define ALLOCPROCLISTINC 20

#include "joblist.h"
struct tagJobList	*JobList;

#include "request.h"

int maxjob = 0;
#define ABS(x) (((x) < 0) ? -(x) : (x))

extern PROC	*Proc;

extern char *Spool;
extern ClassAd *JobAd;
extern int MakeProc(ClassAd *ad, PROC *p);

#include "prototypes.h"
#include "../condor_ckpt_server/server_interface.h"

extern "C" {
	Request * NewNotification( int kind, int to_task,
							   int msg_id, int on_task );
	int strcmp_until(const char *, const char *, const char);
	int unlink_local_or_ckpt_server( char *file );
}

#include "condor_qmgr.h"

extern char *schedd;

/* JCP
** Expand the size of the job list
** Changed this to always allocate array of just one element - Aug 7, 97
**
*/
void
AllocJobList()
{
    if (JobList == NULL) {
	JobList = (struct tagJobList *) MALLOC(sizeof(struct tagJobList));
        memset(JobList, 0, sizeof(struct tagJobList));
    }
    else {
        EXCEPT("Fatal error - joblist is not NULL\n");
    }
}

/* JCP
** Allocate a new processes in Job's process list
*/
int
AllocProc()
{
	int		proc;
	int		oldsize = JobList->max_processes;
	int		newsize;
	int 	i;	

	Proc = &(JobList->Proc);

	if (oldsize == 0) {
		JobList->procs = (Process *) MALLOC(ALLOCPROCLISTINC * 
												   sizeof(Process));
		JobList->JobStatus = (int *) MALLOC(ALLOCPROCLISTINC * 
											sizeof(int));
		memset(JobList->JobStatus, 0, ALLOCPROCLISTINC * sizeof(int));
		for (i = 0; i < ALLOCPROCLISTINC; i++) {
			JobList->procs[i].host_num = -1;
			JobList->procs[i].socketfd = NULL;
			JobList->procs[i].errsock = 0;
			JobList->procs[i].syscall_sock = NULL;
		}
		JobList->max_processes = ALLOCPROCLISTINC;
	}

	for ( proc = 0; proc < JobList->max_processes; proc++) {
		if ( proc != PVMD_MAGIC_PROC &&
			 JobList->procs[proc].host_num == -1) {
			break;
		}
	}
	dprintf( D_ALWAYS, "New process for proc %d\n", proc );

	while (proc >= JobList->max_processes) {
		newsize = JobList->max_processes + ALLOCPROCLISTINC;
		dprintf( D_ALWAYS, "Job's process list growing to %d entries\n",
				newsize );
		JobList->procs = (Process *) REALLOC(JobList->procs,
											 newsize * sizeof(Process));
		
		memset ( &(JobList->procs[JobList->max_processes]), 0, 
				 (newsize - JobList->max_processes) * sizeof(Process) );

		JobList->JobStatus = (int *) 
			REALLOC(JobList->JobStatus, newsize * sizeof(int));
		for (i = JobList->max_processes; i < newsize; i++) {
			JobList->procs[i].host_num = -1;
			JobList->procs[i].socketfd = NULL;
			JobList->procs[i].errsock = 0;
			JobList->procs[i].syscall_sock = NULL;
		}
		JobList->max_processes = newsize;
	}

	JobList->num_processes++;
		// appease purify...
	memset(&JobList->procs[proc], 0, sizeof(Process));
	JobList->procs[proc].socketfd = (int *) malloc(sizeof(int));
	*JobList->procs[proc].socketfd = 0;
	strcpy((JobList->procs[proc].cwd), Proc->iwd);
	strcpy((JobList->procs[proc].crootdir), Proc->rootdir );

	JobList->procs[proc].vpid = proc;
	JobList->procs[proc].host_num = -1;
	JobList->procs[proc].pvm_ptid = 0;
	JobList->procs[proc].a_out = NULL;
	JobList->procs[proc].dest_arch = NULL;

	dprintf(D_ALWAYS, "AllocProc() returning %d\n", proc);
	return proc;
}

/* JCP
** Remove a processes from Job's process list
*/
void
RemoveProc(int proc)
{
	int 	i;
	int		host_num;
	char	*ckpt_file;
	Request	*request;
	int		request_failed;
	Process	*this_process;

	if( proc < 0 || proc >= JobList->max_processes || 
	   JobList->procs[proc].host_num == -1 ) {
		return;
	}

	this_process = &(JobList->procs[proc]);

	dprintf( D_ALWAYS, "Removing Proc %d(t%x) from Job\n", proc, 
			this_process->pvm_tid);

	/* If this is the starter proc for this host, we may as well delete
	   the host */
	host_num = this_process->host_num;
	if (JobList->hosts[host_num].starter_proc == proc) {
		dprintf( D_ALWAYS, 
				"remove starter for host %s, removing the host too\n",
				JobList->hosts[host_num].name );
		/* Force the kill signal to be sent */
		RemoveHost( host_num );
			// don't return yet, we've still got some cleaning up to do
	}

	if ( this_process->pvm_tid != 0 ) {
		SendNotification(PvmTaskExit, this_process->pvm_tid);
		CancelAllTaskNotifys( this_process->pvm_tid );
	}

	for (request = FindRequest(-1, this_process->pvm_tid, -1, -1);
		 request != 0;
		 request=FindNextRequest(request, -1, this_process->pvm_tid, -1, -1)) {
		pvm_initsend(PvmDataDefault);
		request_failed = PvmHostFail;
		pvm_pkint(&request_failed, 1, 1);
		SendResponse(request);
		DeleteRequest(request);
	}

	if( this_process->ImageSize > JobList->Proc.image_size ) {
	   JobList->Proc.image_size = this_process->ImageSize;
	}

	if (*(this_process->socketfd) != 0) {
		close(ABS(*(this_process->socketfd)));
		*(this_process->socketfd) = 0;
		if ( this_process->syscall_sock != NULL ) {
			/* xdr_destroy( (this_process->xdr_syscall) ); */
			/* FREE( this_process->xdr_syscall ); */
			delete this_process->syscall_sock;
			this_process->syscall_sock = NULL;
		}
	}

	if (this_process->errsock != 0) {
		close(ABS(this_process->errsock));
		this_process->errsock = 0;
	}

	JobList->num_processes--;

	ckpt_file = GetProcessCkptName(proc);
	unlink_local_or_ckpt_server(ckpt_file);

	if (JobList->hosts[host_num].status > PVM_STATE_PENDING ) {
		JobList->hosts[host_num].status--;
		dprintf( D_ALWAYS, "Setting host #%d to running %d\n", host_num,
				JobList->hosts[host_num].status );
	}
	this_process->host_num = -1;
	if ( this_process->a_out != NULL ) {
		FREE( this_process->a_out );
		this_process->a_out = NULL;
	}

	if ( this_process->dest_arch != NULL ) {
		FREE( this_process->dest_arch );
		this_process->dest_arch = NULL;
	}

	if( ( JobList->num_processes == 0 ) && ( JobList->local_proc_pid == -1) ) {
		dprintf( D_ALWAYS, "All processes completed, job should be deleted\n");
	}
}


char *
GetProcessCkptName(int proc)
{
	char	*ckpt_file;
	static int		freq[3];
	static int 		num_calls = 0;
	int		ckpt_server_num;

	ckpt_file = gen_ckpt_name(Spool, JobList->Proc.id.cluster,
							  JobList->Proc.id.proc, 
							  JobList->procs[proc].pvm_orig_tid);

	if (get_ckpt_server_count() > 0) {
#if 0
		ckpt_server_num = proc % get_ckpt_server_count();
#else
		ckpt_server_num = config_hash(ckpt_file, get_ckpt_server_count());
#endif
	} else {
		ckpt_server_num = -1;
	}

	if (num_calls == 0) {
		freq[0] = freq[1] = freq[2] = 0;
	}

	set_ckpt_server_number(ckpt_server_num);

	if ( ckpt_server_num >= 0 ) { freq[ckpt_server_num]++; }
	num_calls++;
	if ((num_calls % 50) == 0) {
		dprintf(D_ALWAYS, "Checkpoint distribution: %d, %d, %d\n",
				freq[0], freq[1], freq[2]);
	}

	return ckpt_file;
}

Process	*
FindProcessByTid(int tid)
{
	int		i;
	Process	*pr;

	for (i = 0; i < JobList->max_processes; i++) {
		pr = &(JobList->procs[i]);
		if (pr->pvm_tid == tid || pr->pvm_orig_tid == tid) {
			return pr;
		}
	}
	return 0;
}


#if 0
/* JCP
** Remove a job from the JobList.  By setting its socketfd = 0, 
** we're signalling that this entry is free.
##
## This is a dangerous function to call, because it throws away our
** information about our remote machines.  It's better to call
** multi_CleanupAll() (or EXCEPT(), which calls multi_CleanupAll())
** and let it do a graceful shutdown.  Also, it is not safe to free()
** the JobList, since we stash pointers to it in other places! -Jim B.
*/

void
RemoveJob()
{
    int i, j;
	PROC	dummy_proc;

	multi_CleanupAll(0,0,NULL);

	FREE(JobList->procs);
	JobList->procs = NULL;
	FREE(JobList->JobStatus);
	JobList->JobStatus = NULL;

	JobList->max_processes = 0;
	JobList->num_processes = 0;

	if (JobList->pvm_list_size > 0) {
		FREE(JobList->hosts);
		JobList->hosts = NULL;
		JobList->pvm_list_size = 0;
		JobList->pvm_machine_count = 0;
	}
	if( JobList->Proc.universe == CONDOR_UNIVERSE_PVM ) {
/*		pvm_halt(); */
		if ( JobList->pvm_pid > 0 ) {
			kill( JobList->pvm_pid, SIGKILL );
			close(JobList->pvm_pipe);
			JobList->pvm_pipe = 0;
		}
		if ( JobList->local_proc_pid > 0 ) {
			kill( JobList->local_proc_pid, SIGKILL );
			JobList->local_proc_pid = -1;
		}
		if (JobList->pvm_arg_format != NULL) {
			FREE(JobList->pvm_arg_format);
			JobList->pvm_arg_format = NULL;
		}
	}

	maxjob = 0;
}
#endif

/* 
** Initialize the JobList
*/

void
NewJob()
{
	AllocJobList();
	if (maxjob <= 0) {
		maxjob = 1;
	}
	memset(JobList, 0, sizeof (struct tagJobList));
	JobList->num_processes = 0;
	JobList->max_processes = 0;
	JobList->pvm_pipe = 0;
	JobList->pvm_pid = 0;
	JobList->local_proc_pid = 0;
	JobList->pvm_machine_count = 0;
	JobList->pvm_machines_starting = 0;
	JobList->pvm_arg_format = NULL;
	JobList->hosts = NULL;
	JobList->pvm_list_size = 0;
/*	JobList->work_list = NULL; */
/*	JobList->notify_list = NULL; */
	JobList->class_list = NULL;
	JobList->class_count = 0;
}

int
GetJobNum( int cluster, int proc )
{
	if (maxjob > 0) {
		if ( JobList->Proc.id.cluster == cluster && 
			JobList->Proc.id.proc == proc ) {
			return 0;
		}
	}
	return -1;
}


int
InitJob( int cluster, int proc )
{
	int		i;
	int		process_num = 0;
	int		class_num;
	PROC	class_proc;
	char	class_name_buf[10];
	int		max_hosts=0;
	int		min_hosts=0;
	int		cur_hosts=0;
	char	reqs[ATTRLIST_MAX_EXPRESSION];

	if ( (i = GetJobNum( cluster, proc )) == 0 ) {
		return 0;
	}

	NewJob();

	Proc = &(JobList->Proc);
	Proc->id.cluster = cluster;
	Proc->id.proc = proc;

	if (ConnectQ(schedd, SHADOW_QMGMT_TIMEOUT, true) == 0) {
		return -1;
	}
	if (!JobAd) {
		JobAd = GetJobAd( cluster, proc );
	}
	MakeProc(JobAd, Proc);

#if !defined(HPUX8) && !defined(MULTI)
		if( Proc->status != RUNNING ) {
			dprintf( D_ALWAYS, 
                     "Shadow: Asked to run proc %d.%d, but status = %d\n",
					Proc->id.cluster, Proc->id.proc, Proc->status );
			EXCEPT("********** Shadow Parent Exiting **********\n");
		}
#endif

	sprintf(class_name_buf, "%d",proc);
	/* the min_needed field in the Proc structure stores the minimum needed
	   PVM machines in the least significant 16 bits, and the current
	   number of machines in the most significant 16 bits. Someday, when
	   classads completely get rid of the Proc structure, a current field
	   should be added in addition to the max_needed and min_needed fields
	*/
	NewClass(proc, class_name_buf, Proc->requirements, 
			 Proc->min_needed & 0xffff, Proc->max_needed, 
			 Proc->min_needed >> 16, JobAd);

	for(;;) {
		proc++;
		class_proc.id.cluster = cluster;
		class_proc.id.proc = proc;
		sprintf(class_name_buf, "%d", proc);
		
		ClassAd *nextad;

		if (GetAttributeInt(cluster, proc, ATTR_MAX_HOSTS, &max_hosts) >= 0) {

			if(GetAttributeInt(cluster, proc, ATTR_MIN_HOSTS, &min_hosts)<0) {
				dprintf (D_ALWAYS, "Failure to get MINHOSTS!!!!\n" );
				min_hosts = 1;
			}
			if(GetAttributeInt(cluster,proc,ATTR_CURRENT_HOSTS,&cur_hosts)<0) {
				dprintf ( D_ALWAYS, "Failure to get CURRENTHOSTS!!!!!\n" );
				cur_hosts = 1;
			}

			if (GetAttributeExpr(cluster, proc, ATTR_REQUIREMENTS, reqs)<0) {
				dprintf ( D_ALWAYS, "Failure to get Requirements!!!!\n" );
				reqs[0] = '\0';
			}

			nextad = GetJobAd(cluster, proc);

				/* Used to have cur_hosts as the last arg, but it is
				   one when it really *should* be zero. */

			NewClass(proc, class_name_buf, reqs, 
					 min_hosts, max_hosts, 0, nextad );
		} else {
			break;
		}
	}
		

	/* Call this here, so that JobLists will be dimensioned */
	process_num = AllocProc();
	JobList->num_processes--;

	JobList->LocalUsage = Proc->local_usage;
	JobList->procs[process_num].Rusage = Proc->remote_usage[process_num];
	JobList->procs[process_num].ImageSize = Proc->image_size;
	
	strcpy((JobList->procs[process_num].cwd), Proc->iwd);
	strcpy((JobList->procs[process_num].crootdir), Proc->rootdir );
	
	DisconnectQ(0);

	multi_get_client_ids( Proc, 0 );
	
	return 0;
}


/* Create a new host structure for job.  If this is for PVM, and not for 
   the local machine set the new host to the waiting state, otherwise, 
   set host as ready to receive work.
*/
int
NewHost( char *host, int class_num, char *addr_port, char *cap )
{
	int		new_host;
	char	localhost[80];

	if ( JobList->pvm_list_size == 0 ) {
		JobList->hosts= (Host *) 
			MALLOC(ALLOCPROCLISTINC * sizeof(Host));
		memset(JobList->hosts, 0, ALLOCPROCLISTINC * sizeof(Host));
		JobList->pvm_list_size = ALLOCPROCLISTINC;
	}

	new_host = JobList->pvm_machine_count;
	if ( new_host == JobList->pvm_list_size - 1 ) {
		JobList->pvm_list_size += ALLOCPROCLISTINC;		
		JobList->hosts = (Host *) REALLOC(JobList->hosts,
							 JobList->pvm_list_size * sizeof(Host));
	}

	memset( &(JobList->hosts[new_host]), 0, sizeof( JobList->hosts[new_host]));

	JobList->hosts[new_host].name = strdup( host );
	JobList->hosts[new_host].addr_and_port = strdup( addr_port );
	JobList->hosts[new_host].capability = strdup( cap );
	JobList->hosts[new_host].class_num = class_num;
	JobList->hosts[new_host].host_timer = -1;
	JobList->hosts[new_host].is_suspended = 0;
	JobList->hosts[new_host].was_ever_suspended = 0;
    JobList->hosts[new_host].pvmd_reply = NULL;
#ifdef PVM34
	JobList->hosts[new_host].hi_dsig = 0;
#endif
	if ( JobList->Proc.universe == CONDOR_UNIVERSE_PVM ) {
		JobList->hosts[new_host].status = PVM_STATE_WAITING;
#if 0
		gethostname( localhost, sizeof(localhost) );
		if (!strncmp(localhost, host, MIN(strlen(localhost), strlen(host)))) {
			strcat(localhost, "_worker");
			FREE(JobList->hosts[new_host].name);
			JobList->hosts[new_host].name = strdup( localhost );
		}
#endif
	} else {
		JobList->hosts[new_host].status = PVM_STATE_PENDING;
	}
	JobList->pvm_machine_count++;

	return new_host;
}


Host *
FindHostByName( char *host_name )
{
	int		i;
	Host	*this_host;

	dprintf ( D_FULLDEBUG, "FHBN: pvm_machine_count = %d\n", 
			  JobList->pvm_machine_count );
	for (i = 0; i < JobList->pvm_machine_count; i++) {
		this_host = &(JobList->hosts[i]);
		if ( this_host->status == PVM_STATE_EXITED ) {
			continue;
		}
		if (!strcmp(host_name, this_host->name)) {
			return this_host;
		}
			/* We'll try again with just the first part of the name...
			   we have to be careful, because it could be an ip address,
			   not a name.  Thus we only compare if it's not a digit... */
		if ( !isdigit( host_name[0] ) ) {
			if ( !strcmp_until( host_name, this_host->name, '.' ) ) {
				return this_host;
			}
		}
	}

	return NULL;
}

int
Find_Addr_and_Cap( char *host_name, char **addr_port, char **cap )
{
        int             i;
        Host    *this_host;

        for (i = 0; i < JobList->pvm_machine_count; i++) {
                this_host = &(JobList->hosts[i]);
                if (!strcmp(host_name, this_host->name)) {
                        if (this_host->status != PVM_STATE_EXITED) {
				*addr_port = strdup(this_host->addr_and_port);
				*cap = strdup(this_host->capability);
                                return 0;
                        }
                }
        }
        return -1;
}


Host *
FindHostByTid( int tid )
{
	int		i;
	Host	*this_host;

	for (i = 0; i < JobList->pvm_machine_count; i++) {
		this_host = &(JobList->hosts[i]);
//		dprintf(D_FULLDEBUG,
//                "in FindHostByTid: i=%d host_tid:%x tid:%x tid|TIDPVMD:%x\n",
//                i,this_host->pvmd_tid,tid,((tid | TIDPVMD) & ~(TIDLOCAL)));
		if ((this_host->pvmd_tid | TIDPVMD) == 
            ((tid | TIDPVMD) & ~(TIDLOCAL) )) {
			if (this_host->status != PVM_STATE_EXITED) {
				return this_host;
			}
		}
	}
	return NULL;
}


Host *
FindHostByStarterTid( int tid )
{
	int		i;
	Host	*this_host;

	for (i = 0; i < JobList->pvm_machine_count; i++) {
		this_host = &(JobList->hosts[i]);
		if (this_host->starter_tid == tid) {
			if (this_host->status != PVM_STATE_EXITED) {
				return this_host;
			}
		}
	}
	return NULL;
}


int
RemoveHost( int host_num )
{
	int		i;
	int		starter_proc;
	Host	*this_host;
	ClassDef	*classdef;

	if ( JobList->pvm_machine_count <= host_num ) {
		return -1;
	}

	this_host = &(JobList->hosts[host_num]);

	if (this_host->status == PVM_STATE_EXITED) {
		return 0;				// no need to remove twice!
	}

	classdef = GetClassByNum(this_host->class_num);
	classdef->current--;

	dprintf( D_ALWAYS, "RemoveHost: Sending HostDelete notify on t%x\n",
			this_host->pvmd_tid | TIDPVMD);
	// SendNotification( PvmHostResume, this_host->pvmd_tid | TIDPVMD);
	SendNotification( PvmHostDelete, this_host->pvmd_tid | TIDPVMD);

	signal_startd( this_host->name, RELEASE_CLAIM );
	if (this_host->status < PVM_STATE_PENDING) {
		JobList->no_starts++;
		this_host->status = PVM_STATE_EXITED;

		// To be compatible with pvm3.4,
		// this_host->pvmd_reply may not be NULL.
		// In any case we should check for NULL before 
		// re-assigning the pointer.  -Bin
		if (this_host->pvmd_reply != NULL) {
		     FREE( this_host->pvmd_reply );
		     this_host->pvmd_reply = NULL;
		} 

		this_host->pvmd_reply = strdup("PvmNoHost");
		if ( JobList->pvm_machines_starting > 0 ) { 
			SendPVMStartAck();
		}
		StartWaitingHosts();

		dprintf( D_FULLDEBUG, "RemoveHost: decreased active count by one.\n" );

	} else if (this_host->status >= PVM_STATE_PENDING) {
		starter_proc = this_host->starter_proc;
		for (i = 0; i < JobList->max_processes; i++) {
			if (JobList->procs[i].host_num == host_num && i != starter_proc) {
				RemoveProc(i);
			}
		}
		/* By reducing max_needed for the class, we're making sure
           that a new "request" is not generated as a by product of
           reducing current above.  If the host never got fully
           configured, we need to make the new request, so max_needed
           is not decremented in that case. */
		/* Also, if the user job never got started, then we really do need
		   to "request" a new host because we haven't fulfilled our 
		   requirement to the user, so they're not around to make the 
		   request */
		if (JobList->local_proc_pid > 0 && classdef->max_needed > 0) {
			classdef->max_needed--;
		}
	}
	StoreClassDef(classdef);

	this_host->status = PVM_STATE_EXITED;
	this_host->starter_proc = -1;
	if ( this_host->host_timer != -1 ) {
		cancel_timer(this_host->host_timer);
		this_host->host_timer = -1;
	}

/*	FREE( JobList->hosts[host_num].name );
	JobList->hosts[host_num].name = NULL; */
/*	FREE( JobList->hosts[host_num].arch );
	JobList->hosts[host_num].arch = NULL; */
	
/*	if (JobList->hosts[host_num].pvmd_reply != NULL) {
		FREE( JobList->hosts[host_num].pvmd_reply );
		JobList->hosts[host_num].pvmd_reply = NULL;
	} */
/*	JobList->hosts[host_num].pvmd_tid = 0; */

	return 0;
}

void
RemoveAllHosts()
{
	int		i;

	for(i = 0; i < JobList->pvm_machine_count; i++) {
		if ( JobList->hosts[i].status != PVM_STATE_EXITED ) {
			RemoveHost( i );
		}
	}
}

void
req_exit_all()
{
	int		i;

	for(i = 0; i < JobList->pvm_machine_count; i++) {
		if ( JobList->hosts[i].status != PVM_STATE_EXITED ) {
			signal_startd( JobList->hosts[i].name, RELEASE_CLAIM );
		}
	}
}


int
count_ready_hosts( int class_num )
{
	int	i, num_running;

	num_running = 0;

	for (i = 0; i < JobList->pvm_machine_count; i++) {
		if (JobList->hosts[i].status >= PVM_STATE_PENDING &&
			JobList->hosts[i].class_num == class_num ) {
			num_running++;
		}
	}
	return num_running;
}


void
AddToStartQueue( int hostnum )
{
	struct tagJobList	*this_job;

	this_job = JobList;

	if (this_job->pvm_starting_list == NULL) {
		this_job->pvm_starting_list = (int *) 
			MALLOC(this_job->pvm_machine_count * sizeof(int));
	}
	this_job->pvm_starting_list[this_job->pvm_machines_starting] = hostnum;
	this_job->pvm_machines_starting++;
}


int
PrintStartQueue( int debug_mask )
{
	int		i;
	struct	tagJobList	*this_job;

	this_job = JobList;
	if (this_job->pvm_machines_starting != 0) {
		dprintf( debug_mask, "{" );
		for (i = 0; i < this_job->pvm_machines_starting; i++) {
			dprintf( debug_mask | D_NOHEADER, " %s", 
					this_job->hosts[this_job->pvm_starting_list[i]].name );
		}
		dprintf( debug_mask | D_NOHEADER, "}\n" );
	}
	return 0;
}


Request *
NewNotification( int kind, int to_task, int msg_id, int on_task )
{
	int				i;
	Request			*new_request;
	Process			*target_process;
	Host			*target_host;

	/* if (kind == PvmHostAdd && on_task == -1) { */
	/* I think that 0 turns the notification off, instead of 1 */
	if (kind == PvmHostAdd && on_task == 0) {
		CancelNotification( kind, to_task, -1 );
		return NULL;
	}

	new_request = NewRequest(kind, to_task, 0, msg_id);
	new_request->on_tid = on_task;

    dprintf( D_FULLDEBUG, "Made new request:\n" );
    dprintf_request ( D_FULLDEBUG, new_request );

	dprintf( D_FULLDEBUG, "Set notification to t%x of type %d w/ msg_id %d\n",
			to_task, kind, msg_id );


	/* Trigger notification right away of task does not exist */
	if ( kind == PvmTaskExit && on_task != JobList->local_proc_tid) {
		target_process = FindProcessByTid( on_task );
		if ( target_process == 0 || target_process->host_num == -1 ) {
			dprintf( D_ALWAYS, "No such task %x, sending notification\n",
					on_task );
			SendNotification( kind, on_task );
		}
	} else if ( kind == PvmHostDelete ) {
		target_host = FindHostByTid( on_task );
		if ( target_host == 0 || target_host->status == PVM_STATE_EXITED ) {
			dprintf( D_ALWAYS, "No such host %x, sending notification\n",
					on_task );
			SendNotification( kind, on_task );
		}
	} else if ( kind == PvmHostSuspend || kind == PvmHostResume ) {
		target_host = FindHostByTid( on_task );
		if ( target_host == 0 || target_host->status == PVM_STATE_EXITED ) {
			dprintf( D_ALWAYS, "No such host %x, canceling notification\n",
					on_task );
			CancelNotification( kind, to_task, on_task );
		} else if ( target_host && 
				   ((target_host->is_suspended && kind == PvmHostSuspend) ||
				   (!target_host->is_suspended && target_host->was_ever_suspended
					 && kind == PvmHostResume))) {
			SendNotification( kind, on_task );
		}
	}
	return new_request;
}

int
CancelNotification( int kind, int to_task, int on_task )
{
	Request			*request;

	dprintf( D_ALWAYS, "Cancelling notification type %d to t%x\n",
			kind, to_task );
	if (to_task == 0) {
		return -1;
	}

	for (request = FindRequest(to_task, on_task, kind, -1);
		 request != 0;
		 request = FindNextRequest(request, to_task, on_task, kind, -1)) {
		DeleteRequest(request);
	}

	return 0;
}

void
CancelAllTaskNotifys( int task )
{
	CancelNotification( PvmTaskExit, task, -1 );
	CancelNotification( PvmHostDelete, task, -1 );
	CancelNotification( PvmHostAdd, task, -1 );
	CancelNotification( PvmHostSuspend, task, -1 );
	CancelNotification( PvmHostResume, task, -1 );
}

void
pvm_pkhostinfo(Host *host)
{
	ClassDef	*classdef;

	if (host == NULL) {
		dprintf(D_ALWAYS, "Trying to pack host info for null host!\n");
	} else {
		pvm_pkint( &(host->pvmd_tid), 1, 1 );
		pvm_pkstr( host->name );
		classdef = GetClassByNum(host->class_num);
		pvm_pkstr( classdef->class_name );
		pvm_pkint(&(host->status), 1, 1);
#ifdef PVM34
		pvm_pkint(&(host->hi_dsig), 1, 1);
#endif

		dprintf ( D_FULLDEBUG, "pvm_pkhostinfo packing: "
				  "%x, \"%s\", \"%s\", %d, %d\n", host->pvmd_tid, host->name, 
				  classdef->class_name, host->status, host->hi_dsig );

	}
}

void
SendNotification( int kind, int tid )
{
	Host				*notify_host;
	Process				*notify_process;
	Request				*request;
	int					search_tid;

	dprintf(D_ALWAYS, "SendNotification(kind = %d, tid = t%x)\n", kind, tid);
//    dprintf(D_FULLDEBUG, "Existing requests:\n" );
//    DumpRequestList( D_FULLDEBUG );


	/* If this is a host add, we're passed the tid of the new host, but
	   we're happy to fulfull this request for anyone */
	if (kind == PvmHostAdd) {
		search_tid = -1;
	} else {
		search_tid = tid;
	}

        /* The way the following is written (poorly, that is) we will
           get a FMR - Free Memory Read - every time we delete an element
           in the list and then keep searching through it.  It shouldn't
           be harmful, though.  Ugh. -MEY */

	for (request = FindRequest(-1, search_tid, kind, -1);
		 request != 0;
		 request = FindNextRequest(request, -1, search_tid, kind, -1)) {

		pvm_initsend(PvmDataDefault);
		/* Need to put the request id into the msg. header */
		if (kind == PvmTaskExit) {
			pvm_pkint(&tid, 1, 1);
			notify_process = FindProcessByTid(tid);
			if (notify_process) {
				pvm_pkint(&(notify_process->exit_status), 1, 1);
			}
		} 
		else {
			notify_host = FindHostByTid(tid);
			pvm_pkhostinfo(notify_host);
		}

        dprintf ( D_FULLDEBUG, "Handling request:\n " );
        dprintf_request ( D_FULLDEBUG, request );

		dprintf( D_ALWAYS, 
				"Sending notify: type %d to t%x on t%x tag %d\n",
				request->req_type, request->for_tid, tid, request->resp_msg );
		
			/* I'm not going to delete Suspend and Resume requests.
			   The reason being is that I have seen the pvm_notify()
			   requests fail to show up via select().  The result was 
			   that some resume or suspend notifications would not happen.
			   Not removing the notifies here is a much simpler solution, 
			   and implements the API the way *I* think it should be - 
			   once you set a Suspend|Resume notify on a tid, it should 
			   stay there! (MEY) 8-6-99 */

		SendResponse( request );
		if ( (kind != PvmHostAdd) &&
			 (kind != PvmHostSuspend) &&
			 (kind != PvmHostResume) ) {
            dprintf ( D_FULLDEBUG, "Deleting request:\n" );
            dprintf_request ( D_FULLDEBUG, request );
			DeleteRequest(request);
		}

		/* 
		 * For PvmHostAdd, if the count is > 0, we should
		 * decrease the count by one, and delete the request
		 * if the count is 0.  If the count is -1, we leave it
		 * there.  -Bin
		 */
        else { /* is PvmHostAdd */
            if (request->on_tid > 0) {
                
                    /* for PvmHostAdd, request->on_tid stores the count */
                request->on_tid--;
                if (request->on_tid == 0) {
                    dprintf ( D_FULLDEBUG, "deleting request:\n" );
                    dprintf_request ( D_FULLDEBUG, request );
                    DeleteRequest(request);
                }
            }        
        }
    }
}

// prints out some contents of a host.  Written as diagnostic by MEY.
void dprintf_Host ( int level, int num, Host* host ) {

	dprintf ( level, "Host number %d:\n", num );
	dprintf ( level, "\tname\t\"%s\"\n", host->name );
	dprintf ( level, "\tsin\t\"%s\"\n", host->addr_and_port );
	dprintf ( level, "\tstat\t%d\n", host->status );
	dprintf ( level, "\ttid\t%x\n", host->pvmd_tid );
	dprintf ( level, "\n" );
};
