/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/
#if !defined(_JOBLIST_H)
#define _JOBLIST_H

#include "condor_common.h"
#include "condor_classad.h"


/* Spawn Types */
#define SPAWN_PVM         1
#define SPAWN_CARMI       2
#define RESTART_SIMPLE 	  3
#define MIGRATE_SIMPLE 	  4
#define MIGRATE	          5
#define SPAWN_LOCAL       6


/* Process States */
#define PROCESS_STARTING      1
#define PROCESS_RUNNING	      2
#define PROCESS_CHECKPOINTED  3
#define PROCESS_MIGRATING     4

#define PVMD_MAGIC_PROC	32767

typedef struct {
  int      *socketfd;	/* Make this a pointer so that when the process list
                           gets realloc()'d, it doesn't change the real 
                           address of the socketfd which has previously
                           been handed to XDR during the xdr_create(). */
  int      errsock;
  /*XDR	   *xdr_syscall;*/
  ReliSock *syscall_sock;
  int      host_num;
  int      ImageSize;
  char     cwd[ _POSIX_PATH_MAX ];
  char     crootdir[_POSIX_PATH_MAX];
  int      pvm_tid;
  int      pvm_orig_tid; /*Current and orig. may differ after migration */
  int      process_state;
  int      pvm_ptid;
  int      vpid;
  char     *a_out;
  int      spawn_flag;
  char	   *dest_arch;
  struct rusage Rusage;
  int	   exit_status;
} Process;


typedef struct {
  char  *name;
  char  *addr_and_port;		/* <IP_addr:port> */
  char  *capability;		/* to present to startd */
  int  	class_num;
  int   status;
  int   is_suspended;
  int   was_ever_suspended;
  int   starter_proc;
  int   host_timer;
  int   pvmd_tid;
  int   starter_tid;
  char	*pvmd_cmd;
  char	*pvmd_reply;
#ifdef PVM34
  int   hi_dsig;
#endif
} Host;



typedef struct tagClassDef {
  int  	class_num;
  char	*class_name;
  char	*requirements;
  int  	min_needed;
  int  	max_needed;
  int  	orig_max_needed;
  int   current;
  ClassAd *ad;
  struct tagClassDef *next;
} ClassDef;


struct tagJobList {
  PROC 	Proc;
  int   *JobStatus;
  struct rusage LocalUsage;
  struct rusage RemoteUsage;
  char	*pvm_arg_format;
  int	pvm_master_tid;
  int  	pvm_pipe;
  int  	pvm_pid;
  int  	local_proc_pid;
  int  	local_proc_tid;
  int  	pvm_machine_count;
  int  	no_starts;
  int	coredumps;
  int	pvmd_coredumps;
  int  	pvm_machines_starting;
  int  	*pvm_starting_list;
  int  	hoststart_wid;
  Process *procs;
  Host	  *hosts;
  ClassDef *class_list;
  int	class_count;
  char 	CkptName[_POSIX_PATH_MAX], ICkptName[_POSIX_PATH_MAX], 
        RCkptName[_POSIX_PATH_MAX], TmpCkptName[_POSIX_PATH_MAX];
  int   pvm_list_size;
  int  	num_processes, max_processes;
};

/* PVM machine states */

#define PVM_STATE_START		(-1)
#define PVM_STATE_PENDING	0
#define PVM_STATE_RUNNING	(-2)
#define PVM_STATE_EXITED	(-3)
#define PVM_STATE_WAITING	(-4)   /* Must wait until PVM ready to be
                                          started */
#define PVM_STATE_SEND_PVM	(-5)   /* Need to send PVMd in response to
                                          next work request */
#define PVM_STATE_REPLIED	(-6)   /* Got PVMd info from starter */

/* joblist PROTOTYPES */
Host    *FindHostByName(char *);
int     Find_Addr_and_Cap(char *, char **, char **);
Host    *FindHostByTid(int);
Host    *FindHostByStarterTid(int);
Host    *GetMachine(char *, int);  /* Find a machine to spawn on */
Process *FindProcessByTid(int);
int     CancelNotification(int, int, int);
void    CancelAllTaskNotifys(int);
void    pvm_pkhostinfo(Host *);
void    SendNotification(int, int);
char    *GetProcessCkptName(int);

void    dprintf_Host( int, int, Host* );

/* class Prototypes */
ClassDef  *GetClassByName(char *);
ClassDef  *GetClassByNum(int);

#endif /* _JOBLIST_H */

