/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/
#include "pvm3.h"
#include "carmi.h"
#include "carmipro.h"

static int carmi_next_req_id = 0;


int
carmi_new_req_id()
{
	int rval = carmi_next_req_id++;
	return rval;
}


int
carmi_send_request(req_type, response_msg)
int req_type;
int response_msg;
{
	int		rm_tid;
	int		req_id;
	int		old_opt_val;

	req_id = carmi_new_req_id();
	carmi_store_req_id(req_id);
	rm_tid = pvm_get_rm_tid();
	if (rm_tid != 0) {
		if (response_msg == 0) {
			response_msg = req_type;
		}
		pvm_pkint(&response_msg, 1, 1);
		old_opt_val = pvm_setopt(PvmResvTids, 1);
		pvm_send(rm_tid, req_type);
		pvm_setopt(PvmResvTids, old_opt_val);
	} else {
		return -1;
	}
	return req_id;
}


static int sbufs[10];
static int rbufs[10];
static int buf_tops = 0;

static void
carmi_push_bufs(int encoding)
{
	if (buf_tops < sizeof(sbufs) / sizeof(sbufs[0])) {
		sbufs[buf_tops] = pvm_getsbuf();
		pvm_setsbuf(pvm_mkbuf(encoding));
		rbufs[buf_tops] = pvm_setrbuf(0);
		buf_tops++;
	}
}


static void
carmi_pop_bufs()
{
	int		old_buf;

	if (buf_tops > 0) {
		buf_tops--;
		old_buf = pvm_setsbuf(sbufs[buf_tops]);
		if (old_buf) {
			pvm_freebuf(old_buf);
		}
		old_buf = pvm_setrbuf(rbufs[buf_tops]);
		if (old_buf) {
			pvm_freebuf(old_buf);
		}
	}
}


int
carmi_spawn(char *a_out, char **av, char *res_class, int count, int response,
			int exit_response)
{
	int		new_req_id;
	int		i, n;

	carmi_push_bufs(PvmDataDefault);
	if (av == 0) {
		n = 0;
	} else {
		for (i = 0; av[i]; i++) ;
		n = i;
	}
	pvm_pkstr(a_out);
	pvm_pkint(&n, 1, 1);
	for (i = 0; i < n; i++) {
		pvm_pkstr(av[i]);
	}
	pvm_pkstr(res_class);
	pvm_pkint(&count, 1, 1);
	pvm_pkint(&exit_response, 1, 1);
	new_req_id = carmi_send_request(CARMI_SPAWN, response);
	carmi_pop_bufs();
	return new_req_id;
}


int
carmi_respawn(tid, response)
int tid;
int response;
{
	int		new_req_id;

	carmi_push_bufs(PvmDataDefault);
	pvm_pkint(&tid, 1, 1);
	new_req_id =  carmi_send_request(CARMI_RESPAWN, response);
	carmi_pop_bufs();
	return new_req_id;
}


int
carmi_cancel_req(reqid, response)
int		reqid;
int		response;
{
	int		new_req_id;

	carmi_push_bufs(PvmDataDefault);
	pvm_pkint(&reqid, 1, 1);
	pvm_pkint(&response, 1, 1);
	new_req_id =  carmi_send_request(CARMI_CANCEL_REQ, response);
	carmi_pop_bufs();
	return new_req_id;
}


int carmi_ckpt(int ckpt_cnt, int *ckpt_list, int consistent_cnt, 
			   int *consistent_list, int flush_cnt, int *flush_list, 
			   int response)
{
	int		new_req_id;

	carmi_push_bufs(PvmDataDefault);
	pvm_pkint(&ckpt_cnt, 1, 1);
	pvm_pkint(ckpt_list, ckpt_cnt, 1);
	pvm_pkint(&consistent_cnt, 1, 1);
	pvm_pkint(consistent_list, consistent_cnt, 1);
	pvm_pkint(&flush_cnt, 1, 1);
	pvm_pkint(flush_list, flush_cnt, 1);
	pvm_pkint(&response, 1, 1);
	new_req_id =  carmi_send_request(CO_CHECK_REQ_CKPT,
									 response);
	carmi_pop_bufs();
	return new_req_id;
}


int carmi_restart(int restart_cnt, int *restart_list, int consistent_cnt, 
				  int *consistent_list, int response)
{
	int		new_req_id;

	carmi_push_bufs(PvmDataDefault);
	pvm_pkint(&restart_cnt, 1, 1);
	pvm_pkint(restart_list, restart_cnt, 1);
	pvm_pkint(&consistent_cnt, 1, 1);
	pvm_pkint(consistent_list, consistent_cnt, 1);
	pvm_pkint(&response, 1, 1);
	new_req_id =  carmi_send_request(CO_CHECK_REQ_RESTART,
									 response);
	carmi_pop_bufs();
	return new_req_id;
}


int carmi_migrate(int ckpt_cnt, int target_tid, int host_tid, 
				  int consistent_cnt, int *consistent_list, 
				  int flush_cnt, int *flush_list, 
				  int response)
{
	int		new_req_id;

	carmi_push_bufs(PvmDataDefault);
	pvm_pkint(&target_tid, 1, 1);
	pvm_pkint(&host_tid, 1, 1);
	pvm_pkint(&consistent_cnt, 1, 1);
	pvm_pkint(consistent_list, consistent_cnt, 1);
	pvm_pkint(&flush_cnt, 1, 1);
	pvm_pkint(flush_list, flush_cnt, 1);
	pvm_pkint(&response, 1, 1);
	new_req_id =  carmi_send_request(CO_CHECK_REQ_MIGRATE, response);
	carmi_pop_bufs();
	return new_req_id;
}


int
carmi_rm_ckpt(tid, response)
int		tid;
int		response;
{
	int		new_req_id;

	carmi_push_bufs(PvmDataDefault);
	pvm_pkint(&tid, 1, 1);
	pvm_pkint(&response, 1, 1);
	new_req_id = carmi_send_request(CO_CHECK_RM_CKPT, response);
	carmi_pop_bufs();
	return new_req_id;
}


int
carmi_notify(what, host_id, response)
int		what;
int		host_id;
int		response;
{
	int		new_req_id;

	carmi_push_bufs(PvmDataDefault);
	pvm_pkint(&what, 1, 1);
	pvm_pkint(&host_id, 1, 1);
	new_req_id = carmi_send_request(CARMI_NOTIFY, response);
	carmi_pop_bufs();
	return new_req_id;
}
