/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"

#define START_PVMD
#define INCLUDE_STATUS_NAME_ARRAY

#include "sched.h"
#include "condor_debug.h"
#include "condor_config.h"
#include "fileno.h"
#include "files.h"
#include "proc.h"
#include "exit.h"
#include "condor_classad.h"
#include "condor_io.h"
#include "condor_uid.h"
#include "condor_attributes.h"
#include "shadow.h"
#include "pvm3.h"
#include "internet.h"
#include "condor_socket_types.h"
#include "condor_environ.h"

#ifdef PVM34
#include "pvmproto.h"
#else
#include "pvmsdpro.h"
#endif

#include "condor_socket_types.h"
#include "pvm_global.h"
#include "user_log.c++.h"

UserLog	ULog;


char 	*getenv(), *index(), *strcat(), *strcpy();

extern int part_send_job( int test_starter, char *host, int &reason,
                          char *capability, char *schedd, PROC *proc, int &sd1,
                          int &sd2, char **name);

extern int send_cmd_to_startd(char *sin_host, char *capability, int cmd);

extern "C" void initializeUserLog ();

FILE	*popen(), *fdopen();

extern int	LockFd;
extern char	MsgBuf[];

extern int     	MyPid;
extern int     	LogPipe;
extern int     	ImageSize;
extern int     	PipeToParent;
extern PROC	*Proc;
extern char	*Spool;

extern int     	Procnum;

int		ProcTimeout;
int		HostTimeout;
int		ShutdownTimeout;

extern char	QueueName[MAXPATHLEN];

#include "condor_qmgr.h"

extern struct rusage LocalUsage;
extern struct rusage RemoteUsage;

extern int JobStatus;
extern struct rusage JobRusage;
extern int ExitReason;

ReliSock *RSC_ShadowInit();

extern int MainSymbolExists;

ReliSock *RSC_multi_ShadowInit(int);

#include "joblist.h"
extern struct tagJobList *JobList;

extern int maxjob;
#define ABS(x) (((x) < 0) ? -(x): (x))

int	StdinIsAscii = TRUE;

bool ShuttingDown = false;		// true if we're shutting down the job
bool JobCompleted = false;		// true if we need to call HandleCompletion()
								// before exit()
int ShutdownTimer = 0;

ReliSock stdin_relisock;

char *schedd = NULL, *scheddName = NULL;

int Shadow_start_pvm();
//display_fd_set( int debug_level, char *string, fd_set *fdset, int max_fd );

#include "prototypes.h"

int sigbus_handler()
{
	dprintf( D_ALWAYS, "Got SIGBUS!!!!\n");
	multi_CleanupAll(0,0,NULL);
	abort();
	return 0;
}

extern "C" int pvmputenv(char*);

/* JCP
** Do all function for the multi_shadow.  It watches stdin for new jobs, and 
** all the connections to starter processes and handles input from each one.
** It never returns.
*/

int
multi_shadow( int argc, char *argv[] )
{
  fd_set  readfds, exceptfds;
  int    j;
  int    cnt;
  int    max_fd;
  int  	 sockfd, errsock;
  struct sockaddr_in from;
  SOCKET_LENGTH_TYPE len = sizeof from;
  int    handled_one;
  int    stdinfd = fileno( stdin );
  void	 sigchild_handler(int);
  void	 sigusr1_handler(int);
  void   sigterm_handler(int);
  struct sigaction sig_act;
  sigset_t sig_set;
  char	 *tmp;
  struct timeval select_to;

  char stack_check[1024];
  memset( stack_check, 43, 1024 );
  
#undef WAIT_FOR_DEBUGGER
#if defined(WAIT_FOR_DEBUGGER)
  int do_wait = 1;
  while (do_wait);
#endif

  argc--;
  argv++;
  
  /* At this point there used to be code to take an initial argument "-d"
     which was redundant. Removed from schedd as well as here */
  StdinIsAscii = TRUE;
  if( !StdinIsAscii ) {
    stdin_relisock.attach_to_file_desc( stdinfd );
  }
  /* Above 3 lines of code are redundant - communication is though ascii
     across a pipe. Someday if we decide to use ReliSocks, code is provided
  */
  
  /* The address of the schedd, being passed as the 1st argument */
  if (argv[0] != 0) {
    schedd = argv[0];
  }
  scheddName = getenv( EnvGetName( ENV_SCHEDD_NAME ) );
  
  /* Set-up handler for SIGCHLD && SIGALRM */
  sigemptyset(&sig_set);
  sigaddset(&sig_set, SIGCHLD);
  sigaddset(&sig_set, SIGALRM);
  sigaddset(&sig_set, SIGUSR1);
  sigaddset(&sig_set, SIGTERM);
  sigprocmask(SIG_BLOCK, &sig_set, (sigset_t *) NULL);
  sig_act.sa_handler = sigchild_handler;
  sig_act.sa_mask = sig_set;
  sig_act.sa_flags = 0;
  sigaction(SIGCHLD, &sig_act, (struct sigaction *) 0);
  sig_act.sa_handler = sigalrm_handler;
  sigaction(SIGALRM, &sig_act, (struct sigaction *) 0);
  sig_act.sa_handler = sigusr1_handler;
  sigaction(SIGUSR1, &sig_act, (struct sigaction *) 0);
  sig_act.sa_handler = sigterm_handler;
  sigaction(SIGTERM, &sig_act, (struct sigaction *) 0);

  if ( (tmp=param("PROC_TIMEOUT")) == NULL ) {
    ProcTimeout = 180;
  } else {
    ProcTimeout = atoi(tmp);
  }
  
  if ( (tmp=param("HOST_TIMEOUT")) == NULL ) {
    HostTimeout = 180;
  } else {
    HostTimeout = atoi(tmp);
  }

  if ( (tmp=param("SHUTDOWN_FAST_TIMEOUT")) == NULL ) {
    ShutdownTimeout = 300;
  } else {
    ShutdownTimeout = atoi(tmp);
  }

  for(;;) {

	  for ( int s=0 ; s<1024 ; s++ ) {
		  if ( stack_check[s] != 43 ) {
			  dprintf ( D_ALWAYS, "STACK HOSED!\n" );
			  break;
		  }
	  }

    FD_ZERO(&readfds);
	if (!ShuttingDown) FD_SET(0, &readfds);	/* Check stdin for new job */
	max_fd = 0;

    if (maxjob > 0) { /* there is a job in the system */
      if (JobList->pvm_pipe > 0) { /* watch pipe to pvmd */
        FD_SET(JobList->pvm_pipe, &readfds);
        if (JobList->pvm_pipe > max_fd) {
          max_fd = JobList->pvm_pipe;
        } 
      }
      if (JobList->num_processes != 0) {
        for (j = 0; j < JobList->max_processes; j++) {
          /* Watch out for negative fd's indicating a passive port */
          if (JobList->procs[j].socketfd != NULL) {
            sockfd = ABS(*(JobList->procs[j].socketfd));
          } else {
            sockfd = 0;
          }
          errsock = ABS(JobList->procs[j].errsock);
          /* add the process's sockfd and errsock
             to the fd set */
          if( sockfd != 0 ) {
            FD_SET(sockfd, &readfds);
            if (sockfd > max_fd) {
              max_fd = sockfd;
            }
          }
          if( errsock != 0 ) {
            FD_SET(errsock, &readfds);
            if (errsock > max_fd) {
              max_fd = errsock;
            }
          }
        }
      }
    } /* end if (maxjob > 0) */

		// When shutting down, we (unfortunately) need to wait and
		// field all of the final RSCs from the jobs we killed so they
		// won't get hung up.  So, we can't break out of our loop
		// until all of the remote sockets are closed.  Note that our
		// pvm_pipe and schedd pipe were closed by multi_CleanupAll.
	if (ShuttingDown && max_fd == 0) break;

    /* now we have setup a set of fds to select on */
    display_fd_set( D_FULLDEBUG, "Selecting on", &readfds, max_fd );
    
    sigprocmask(SIG_UNBLOCK, &sig_set, (sigset_t *) NULL);
    select_to.tv_sec = 15;
    select_to.tv_usec = 0;
    cnt = select(max_fd + 1, (fd_set *) &readfds, (fd_set *) 0, 
                 (fd_set *) 0, &select_to);
    sigprocmask(SIG_BLOCK, &sig_set, (sigset_t *) NULL);
    dprintf(D_FULLDEBUG, "cnt = %d\n", cnt);

		// max_fd == 0 indicates that we're only watching stdin, i.e.,
		// we're not running any pvmds or job processes.  This is the
		// case on startup, but the schedd should immediately give us
		// at least one allocated resource on stdin.  If the select()
		// timed out, then the schedd doesn't have any resources for us
		// and we should exit.
	if ( cnt == 0 && max_fd == 0 ) {
		dprintf( D_ALWAYS, "No allocated machines.\n");
		break;
	}

    if ( cnt == -1) { /* error occurred on select */
      if ( errno != EINTR && errno != EBADF) {
        EXCEPT( "select failed, errno = %d", errno );
      }
      if ((errno == EBADF) && (FD_ISSET(0, &readfds)) && !ShuttingDown) {
				/* check stdin */
        if ( HandleSchedd( StdinIsAscii ) == -2 ) {
			EXCEPT( "lost our connection to schedd" );
        }
      }
      continue;
    }
    
    /* Kludge, but some times the PVM messages don't seem to be showing up
       via select()????? 
       Jim Pruyne says he once thought of the reason this occurs -
       some race condition - but he forgot */
    if ( cnt == 0 && JobList->pvm_pipe > 0 ) {
      HandleMsg(); /* handle a PVM message */
      continue;
    }
    handled_one = FALSE;
    
    display_fd_set( D_FULLDEBUG, "Found input on", &readfds, max_fd );
    
    /* First, check stdin.  The schedd passes new jobs
       to the shadow on stdin. */
    if (FD_ISSET(0, &readfds) && !ShuttingDown) {
      if ( HandleSchedd( StdinIsAscii ) == -2 ) {
		  EXCEPT(" lost our connection to schedd" );
      }
    }
    
    /* See if anything from a PVMd */
    if (maxjob > 0) { /* there is a job */
      if (JobList->pvm_pipe > 0  && 
          FD_ISSET(JobList->pvm_pipe, &readfds)) {
        Proc = &(JobList->Proc);
        HandleMsg();
        handled_one = TRUE;
      }
    }
    
    if( handled_one ) {
      continue;
    }
    
    /* Now, check each job's socket to see if Any errors */
    if (maxjob > 0) {
      if (JobList->num_processes != 0) {
        for (j = 0; j < JobList->max_processes; j++) {
          errsock = JobList->procs[j].errsock;
          if (errsock && FD_ISSET(ABS(errsock), &readfds)) {
            if (errsock > 0) {
              char buffer[275];
              int len;
              len = read(errsock, buffer, sizeof(buffer)-1);
              if( len == 0 ) {
                dprintf( D_ALWAYS, "Job %d, process %d logsock "
                         "appears to be closed, removing..\n", 0, j );
                RemoveProc(j);
              } 
              else {
                int old_count, cnter;
                buffer[len] = '\0';
                Proc = &(JobList->Proc);
                dprintf(D_ALWAYS, 
                        "On LogSock for host %s:\n",
                        JobList->hosts[JobList->procs[j].host_num].name );
                /* print the message on logsock */
                old_count = 0;
                for (cnter = 0; cnter <= len; cnter++){
                  if ((buffer[cnter] == '\n' || 
                       buffer[cnter] == '\0') &&
                      cnter - old_count > 1) {
                      buffer[cnter] = '\0';
                      dprintf( D_ALWAYS | D_NOHEADER, 
                               "-> %s\n", 
                               &(buffer[old_count]));
                      old_count = cnter + 1;
                  }
                }
              }
            } else { /* errsock < 0 (== 0 taken care of in outer loop) */
              /* -ve fd indicates passive port */
              int tempfd;
              tempfd = accept(ABS(errsock), 
                              (struct sockaddr *)&from, &len);
              if (tempfd < 0) {
                EXCEPT("accept errsock");
              }
              close(ABS(errsock));
              JobList->procs[j].errsock = tempfd;
              dprintf(D_FULLDEBUG, "Just accepted errsock for "
                      "Job %d, Process %d, fd = %d\n", 0, j, tempfd);
            }
            handled_one = TRUE;
            break;
          } /* end if (errsock && ...) */
        } /* end for */
      } /* end if (JobList->num_processes != 0) */
    } /* end if (maxjob > 0) */
      
    if( handled_one ) {
      continue;
    }
    
    /* Now, check each job's socket to see if Any syscalls ready */
    if (maxjob > 0) {
      if (JobList->num_processes != 0) {
        for (j = 0; j < JobList->max_processes; j++) {
          if (JobList->procs[j].socketfd != NULL) {
            sockfd = *(JobList->procs[j].socketfd);
          } else {
            sockfd = 0;
          }
          if (sockfd && FD_ISSET(ABS(sockfd), &readfds)) {
            handled_one = TRUE;
            if (sockfd > 0) {
              Proc = &(JobList->Proc);
              multi_HandleSyscalls(j);
            } else { /* sockfd < 0 -> passive port */
              int tempfd;
              tempfd = accept(ABS(sockfd), 
                              (struct sockaddr *)&from, &len);
              if (tempfd < 0) {
                EXCEPT("accept syscall_sock");
              }
              close(ABS(sockfd));
              *(JobList->procs[j].socketfd) = tempfd;
              dprintf(D_ALWAYS, "Just accepted syscallsock "
                      "for Job %d, Process %d, fd = %d\n", 0, j, tempfd );
              RSC_multi_ShadowInit(j);
            }
            break;
          } /* end if (sockfd && ...) */
        } /* end for */
      } /* end if (JobList->num_processes != 0) */
    } /* end if (maxjob > 0) */
    
  } /* for(;;) */
  dprintf( D_ALWAYS, "MultiShadow Exiting!!!\n" );
  if (JobCompleted) HandleCompletion();
  return 0;
} /* function multi_shadow() */

/*
**	Check that we actually started up with at least the minimum number of
**	hosts specified in the submit file. If so, start pvm, and then start
**      other hosts
*/
int
multi_start_job()
{
  int host_num = 0;
  int i;
  int prev_machine_count;
  int new_proc;
  int new_job;
  int new_count;
  int good_starts;
  int host_timeout(int);
  struct tagJobList *this_job = JobList;
  
  Proc = &(this_job->Proc);
  good_starts = 100000;
  /* the min_needed field in the Proc structure stores the minimum needed
     PVM machines in the least significant 16 bits, and the current
     number of machines in the most significant 16 bits. Someday, when
     classads completely get rid of the Proc structure, a current field
     should be added in addition to the max_needed and min_needed fields
  */
  if (good_starts >= (this_job->Proc.min_needed & 0xffff)) {
    if ( Proc->universe == CONDOR_UNIVERSE_PVM ) {
      Shadow_start_pvm();
      StartWaitingHosts();
      /* Only needed when only host is local host */
      if ( this_job->pvm_machines_starting == 0 ) {
        StartLocalProcess();
      }
    }
    return good_starts;
  } else {
    RemoveAllHosts();
    return 0;
  }
}

/*
**	Start any hosts who had to wait until PVM confirmed ready.
**	Return the number of hosts actually added to the pool.
*/
int
StartWaitingHosts()
{
  int i;
  int new_proc;
  int rval;
  struct tagJobList *this_job;
  char name_to_send[500];
  
  dprintf( D_ALWAYS, "Entered StartWaitingHosts()\n");
  /* First, make sure we're not still in the starting state */
  
  this_job = JobList;
  if (this_job->pvm_machines_starting != 0) {
    dprintf( D_ALWAYS, "Can't start new machines now" );
    PrintStartQueue( D_ALWAYS | D_NOHEADER );
    return 0;
  }
  
  dprintf( D_ALWAYS, "Ok to start waiting hosts\n" );
  rval = 0;
  
  for (i = 0; i < this_job->pvm_machine_count; i++) {
    if ( this_job->hosts[i].status == PVM_STATE_WAITING ) {
      AddToStartQueue(i);
    }
  }
  
  if (this_job->pvm_machines_starting > 0) {
    pvm_initsend(PvmDataFoo);
    pvm_pkint(&this_job->pvm_machines_starting, 1, 1);
    for (i = 0; i < this_job->pvm_machines_starting; i++) {

      sprintf(name_to_send, "%s ep=$PVM_ROOT wd=$PVM_ROOT ip=%s", 
              this_job->hosts[this_job->pvm_starting_list[i]].name,
			  string_to_ipstr(this_job->hosts
							  [this_job->pvm_starting_list[i]].addr_and_port));

#if 0
/* This was put here by Bin for 3.4.1, but it doesn't work!  Using
   old sprintf above works fine. */
      sprintf(name_to_send, "%s ep=. wd=.", 
              this_job->hosts[this_job->pvm_starting_list[i]].name);
#endif

	dprintf ( D_FULLDEBUG, "StartWaitingHosts: about to pack \"%s\".\n",
			  name_to_send );

      pvm_pkstr(name_to_send);
    }

    pvm_send(this_job->pvm_master_tid, SM_ADD);
#if 0
/* This was also put here by Bin for 3.4.  It doesn't work either.
   We just use the old SM_ADD...(above) */
    pvm_send(this_job->pvm_master_tid, TM_ADDHOST);
#endif
  }
  return this_job->pvm_machines_starting;
}

int
multi_get_client_ids( register PROC *proc, int process)
{
  struct passwd *pwd;
  
  (void)sprintf( JobList->CkptName, "%s/job%06d.ckpt.%d",
                 Spool, proc->id.cluster, proc->id.proc );
  
  (void)sprintf( JobList->TmpCkptName, "%s.tmp", 
                 JobList->CkptName );
  
  (void)sprintf( JobList->ICkptName, "%s/job%06d.ickpt",
                 Spool, proc->id.cluster );

  if( strcmp(proc->rootdir, "/") == 0 ) {
    /* Return the checkpoint in place */
    (void)strcpy(JobList->RCkptName, JobList->CkptName);
  } else {
    (void)sprintf( JobList->RCkptName, "/job%06d.ckpt.%d",
                   proc->id.cluster, proc->id.proc );
  }
  return 0;
}

/* Need to put this here for below part_send_job hack. */
extern ClassAd *JobAd;

/*
** Connect to the scheduler on the remote host and ask to run a job.  The
** connection to the starter will become our main connection to the
** remote starter, and the scheduler will send back an aux port number to
** be used for out of band (error) traffic.
*/
int
multi_send_job( PROC *proc, int process, char *host)
{
  sigset_t           sig_set, old_set;
  struct sigaction   old_action, new_action;
  unsigned int       alarm_time;
  StartdRec          stRec;
  char               *capability;
  char	             *addr_port;
  int                reason, retval, sd1, sd2;
  Host               *this_host;
  char               *name = NULL;

  dprintf( D_ALWAYS, "Shadow: Entering multi_send_job(%s)\n", host );

  /* old hostnames need to be converted to <IP:port> and capabilities */
  if (Find_Addr_and_Cap(host, &addr_port, &capability) < 0) {
    dprintf( D_ALWAYS, "Shadow: Can't find addr and capability\n");
    return -1;
  }

	  /* Now things get annoying.  This is a hack to make multi-class
		 jobs work.  part_send_job is part of shadow_common.C, and 
		 therefore can't be changed.  It looks up the (global!) JobAd
		 (if it isn't there, it gets it from the schedd) and uses
		 it to talk to the startd with.  However, we don't always
		 want to use the JobAd (of the FIRST proc), we sometimes
		 want to use the 2nd or third (etc) ad.  Here, we mangle
		 JobAd to be what we want. MEY 8-13-99 */
  ClassAd *tempAd = JobAd;
	  /* Note both ridiculously inefficient lookups  */
  this_host = FindHostByName( host );
  ClassDef *tempclassdef = GetClassByNum( this_host->class_num );
  JobAd = tempclassdef->ad;

  retval = part_send_job(1, addr_port, reason, capability, schedd, 
                         proc, sd1, sd2, NULL);

	  /* Now we're done - set JobAd back to what it was. */
  JobAd = tempAd;

  if (retval == -1) {
    return -1;
  }

  FREE(addr_port);
  
  this_host->name = strdup(host);

  *(JobList->procs[process].socketfd) = sd1;
  dprintf( D_ALWAYS, "Shadow: RSC_SOCK connected, fd = %d\n", sd1 );
  
  dprintf( D_ALWAYS, "Multi_Shadow: CLIENT_LOG connected, fd = %d\n", sd2 );
  JobList->procs[process].errsock = sd2;
  
  RSC_multi_ShadowInit( process );

  return 0;
}


int
multi_update_job_status( struct rusage *localp, struct rusage *remotep)
{
  PROC	proc;
  int	status;
  float	usage;
  float utime = 0.0;
  float stime = 0.0;
    
  dprintf( D_ALWAYS, "Entering multi_update_job_status()\n");
  
  proc.id.cluster = JobList->Proc.id.cluster;
  proc.id.proc = JobList->Proc.id.proc;
  
  if (ConnectQ(schedd, SHADOW_QMGMT_TIMEOUT) == 0 ||
	  GetAttributeInt(proc.id.cluster, proc.id.proc, ATTR_JOB_STATUS,
					  &status) < 0) {
    dprintf(D_ALWAYS, "Unable to contact Qmgr\n");
    return -1;
  }
  
  if( status == REMOVED ) {
    dprintf( D_ALWAYS, "Job %d.%d has been removed by condor_rm\n",
             proc.id.cluster, proc.id.proc );
  } 
  else {
    if (localp != NULL) {
      update_rusage( &(JobList->LocalUsage), localp );
      JobList->Proc.local_usage = JobList->LocalUsage;
    }
    if (remotep != NULL) {
      update_rusage( &(JobList->RemoteUsage), remotep );
      JobList->Proc.remote_usage[0] = JobList->RemoteUsage;
      dprintf( D_ALWAYS, "Remote user Time: %s\n", 
               d_format_time(JobList->Proc.remote_usage[0].ru_utime.tv_sec) );
    }
    
    usage = rusage_to_float(JobList->Proc.local_usage, &utime, &stime);
    SetAttributeFloat(proc.id.cluster, proc.id.proc, ATTR_JOB_LOCAL_USER_CPU,
					  utime);
    SetAttributeFloat(proc.id.cluster, proc.id.proc, ATTR_JOB_LOCAL_SYS_CPU,
					  stime);
    usage = rusage_to_float(JobList->Proc.remote_usage[0], &utime, &stime);
    SetAttributeFloat(proc.id.cluster, proc.id.proc, ATTR_JOB_REMOTE_USER_CPU,
					  utime);
    SetAttributeFloat(proc.id.cluster, proc.id.proc, ATTR_JOB_REMOTE_SYS_CPU,
					  stime);
    dprintf( D_ALWAYS, "Shadow: marked job status \"%s\"\n", 
             JobStatusNames[ JobList->Proc.status ] );
  }
  
  DisconnectQ(0);
  
  dprintf( D_ALWAYS, "multi_update_job_status() returns %d\n", 0 );
  return 0;
}


/* Dummy function called by handle_termination() - was there in shadow.V6, not
 * present in multi_shadow, so made a dummy here
 */
extern "C" int DoCleanup() { return TRUE; }

void
multi_handle_termination( int process_num, char	*notification, char *coredir)
{
  dprintf(D_FULLDEBUG, "multi_handle_termination() called.\n");
  
  /* call the common code */
  handle_termination( &(JobList->Proc), notification,
                      &(JobList->JobStatus[process_num]), coredir );
  
  if (ExitReason == JOB_EXITED) {
    /* take care of the one case where we print additional info */
    (void)sprintf(notification, "\n%s%d%s%d%s%d%s", 
                  "exited with status ",
                  WEXITSTATUS(JobList->JobStatus[process_num]),
                  ", and touched ", JobList->pvm_machine_count,
                  " machines.\nStart-up was unsuccessful on ", 
                  JobList->no_starts, " machines.");
    if (WEXITSTATUS(JobList->JobStatus[process_num]) == 255) {
      char	*bad_exec = "\nFile may not be an executable.";
      strcat(notification, bad_exec);
    }
	if (JobList->pvmd_coredumps) {
		char buf[_POSIX_PATH_MAX+100];
		sprintf(buf, "\t\tPVM daemon crashed %d times.  Core file written to "
				"%s/core.-1.-1.", JobList->pvmd_coredumps, coredir);
		strcat(notification, buf);
	}
	if (JobList->coredumps) {
		char buf[_POSIX_PATH_MAX+100];
		sprintf(buf, "\t\tPVM task(s) crashed %d time(s).  Core file(s) "
				"written to %s/core.%d.*.", JobList->coredumps, coredir, 
				JobList->Proc.id.cluster);
		strcat(notification, buf);
	}
  }
  else if (ExitReason == JOB_NOT_CKPTED) {
    /* probably killed the job because it was "condor_rm-ed" :-) */
    notification[0] = '\0'; /* to prevent email being sent */
  }
}

int
shutdown_timeout(int timer_id)
{
	dprintf(D_ALWAYS, "Shutdown timeout expired.  MultiShadow Exiting!!!\n");
	if (JobCompleted) HandleCompletion();
	dprintf(D_ALWAYS, "********** Shadow Parent Exiting(%d) **********\n",
			ExitReason);
	exit( ExitReason );
	return 0;					// avoid compiler warning
}

/* This routine is called whenever the multishadow needs to exit. 
   It sets ShuttingDown to true to break us out of the multi_shadow
   select loop. */
int
multi_CleanupAll(int,int,char*)
{
  int   process;
  PROC  *proc;
  PROC  dummy_proc;
  int	i;
  priv_state old_priv;

  if (ShuttingDown) {
	  dprintf( D_ALWAYS, "multi_CleanupAll() called again\n");
	  return 0;
  }
  
  dprintf( D_ALWAYS, "Multi_Shadow: Shutting down...\n" );
  ShuttingDown = true;

  // first thing, update MaxHosts so we don't request any new hosts
  // from the schedd
  StoreAllClassDefs();

  proc = &(JobList->Proc);
  
  old_priv = set_user_priv();
  
  // tell all the remote starters to kill the jobs
  req_exit_all();
  
  // kill the local process
  set_root_priv();
  if (JobList->local_proc_pid > 0) {
	dprintf(D_FULLDEBUG, "Killing Local proc at pid %d\n", 
			  JobList->local_proc_pid);
    if (kill(JobList->local_proc_pid, SIGKILL) < 0)
      dprintf(D_ALWAYS, "kill() failed on %d, errno = %d\n", 
              JobList->local_proc_pid, errno);
  }		
  if (JobList->pvm_pid > 0) {
	dprintf(D_FULLDEBUG, "Killing PVM at pid %d\n", JobList->pvm_pid);
    // if (kill(JobList->pvm_pid, SIGTERM) < 0)
    if (kill(JobList->pvm_pid, SIGKILL) < 0)
      dprintf(D_ALWAYS, "kill() failed on PVMD, errno = %d\n",
              errno);
	close(JobList->pvm_pipe);
	JobList->pvm_pipe = 0;
  }

	  // We're going to start ignoring the schedd pipe now, since we
	  // don't want any new machines.  We close the pipe here, so it
	  // won't fill up and block the schedd.  In theory, the schedd
	  // should be smart, notice that the pipe is closed, and stop
	  // trying to send us more machines.  (We also set Maxhosts ==
	  // CurrentHosts above, so the schedd should know we don't want
	  // anything.)  Currently, the schedd keeps blindly writing to
	  // the pipe.
  close(0);
  
  set_priv(old_priv);

  ShutdownTimer = new_timer( ShutdownTimeout, shutdown_timeout );
  
  return TRUE;
}


/* sigusr1 means we're supposed to abort this job! */
void
sigusr1_handler(int dummy)
{
  dprintf(D_ALWAYS, "Got SIGUSR1\n");
  multi_CleanupAll(0,0,NULL);
}

/* SIGTERM also means we're supposed to go away.  This added by Mike Yoder
   1-13-98, because during a graceful shutdown of the schedd, the multishadow
   wouldn't go away.  The schedd now sends us a SIGTERM, which we catch 
   here and do cleanup... */
void
sigterm_handler( int dummy )
{
	dprintf( D_ALWAYS, "Got SIGTERM, going away...\n" );
	multi_CleanupAll(0,0,NULL);
}


ReliSock *
RSC_multi_ShadowInit( int process )
{
  ReliSock *sock;
  
  JobList->procs[process].syscall_sock = new ReliSock();
  
  sock = JobList->procs[process].syscall_sock;
  sock->attach_to_file_desc(*(JobList->procs[process].socketfd));
  
  sock->encode();
  dprintf( D_FULLDEBUG,
           "Initialized Syscall ReliSock stream: process %d, fd = %d\n",
           process, *(JobList->procs[process].socketfd) );
  
  
  return( sock );
}

int
display_fd_set( int debug_level, char *string, fd_set *fdset, int max_fd )
{
	int		i;
/* The old way...	
	dprintf( debug_level | D_NOHEADER, "%s: ", string );
	for( i=0; i<=max_fd; i++ ) {
		if( FD_ISSET(i,fdset) ) {
			dprintf( debug_level | D_NOHEADER, "%d ", i );
		}
	}
	dprintf( debug_level | D_NOHEADER, "[max = %d]\n", max_fd );
*/

/* The new way: dprintf calls are expensive (open, write, close).  
   Let's do this in one big glorp. */
	
	char fds[4096];
	int pos = 0;

/* This one takes up too much space... 

	memset ( fds, 0, 4096 );
	pos += sprintf ( fds, "%s: ", string );
	for ( i=0 ; i<=max_fd ; i++ ) {
		if ( FD_ISSET( i, fdset ) ) {
			pos += sprintf ( &fds[pos], "%d ", i );
		}
	}
	
	pos += sprintf ( &fds[pos], "[max = %d]\n", max_fd );
	dprintf ( debug_level, "%s", fds );

*/
/* This method is fast and takes up the least amount of space in the 
   log files. */

	int start = 0;
	int prev = -1;
	int n = 0;
	memset ( fds, 0, 4096 );
	pos = 0;
	pos += sprintf ( fds, "%s: ", string );
	for ( i=0 ; i<=max_fd ; i++ ) {
		if ( FD_ISSET( i, fdset ) ) {
			n++;
			if ( i != prev + 1 ) { 
				if ( start == prev ) {
					pos += sprintf ( &fds[pos], "%d, ", start );
				} else {
					if ( prev != -1 ) {
						pos += sprintf ( &fds[pos], "%d-%d, ", start, prev );
					}
				}
				start = i;
			}
			prev = i;
		}
	}

		/* print that last set... */
	if ( start == prev ) {
		pos += sprintf ( &fds[pos], "%d, ", start );
	} else {
		if ( prev != -1 ) {
			pos += sprintf ( &fds[pos], "%d-%d, ", start, prev );
		}
	}

	pos -= 2; /* get rid of last ", " */
	pos += sprintf ( &fds[pos], " [%d,%d]\n", n, max_fd );
	dprintf ( debug_level, "%s", fds );
	return 0;
}

extern	int		loclsock;    /* socket used for talking with PVMd
                                                 created by pvm library */

int
Shadow_start_pvm()
{
  char 	*pvmd = param("PVMD");
  char 	*argv[4];
  char	localhost[80];
  int	i;
  int  	pvm_tid;
  struct tagJobList *this_job = JobList;
  int	*pvm_fd;
  struct pvmhostinfo *master_info;
  int	sleep_time;
  int   pvmd_file_fd;
  char	*pvmsock;
  char	pvm_rendevous_file[30];
  int  	pfds[2];
  priv_state priv;
  
  if ( pvmd == NULL ) {
    EXCEPT("PVMD macro not defined");
  }
  
  argv[0] = pvmd;
  argv[1] = "-d0x11c"; 
  argv[2] = NULL;

  dprintf(D_ALWAYS, "Starting pvmd:");
  for(i = 0; argv[i] != NULL; i++) {
    dprintf(D_ALWAYS | D_NOHEADER, " %s", argv[i]);
  }
  dprintf(D_ALWAYS | D_NOHEADER, "\n");
  
  set_root_priv();
  sprintf( pvm_rendevous_file, "/tmp/pvmd.%d", get_user_uid() );
  unlink( pvm_rendevous_file );
  sprintf( pvm_rendevous_file, "/tmp/pvmd.%d", get_condor_uid() );
  unlink( pvm_rendevous_file );
  set_user_priv();
  
  pipe(pfds);
  
  if ( this_job->pvm_pid = fork() ) { /* The Parent */
    dprintf(D_ALWAYS, "PVM is pid %d\n", this_job->pvm_pid);
    sleep_time = 2;
    
    pvmd_file_fd = pfds[0];
    close(pfds[1]);
    
		// read the first line of stdout from pvmd
	int offset = 0;
	char c;
    do {
		int rval = read(pvmd_file_fd, &c, 1);
		if (rval < 0) {
			EXCEPT("read error on pvm pipe");
		}
		localhost[offset++] = c;
		if (offset == sizeof(localhost)) {
			EXCEPT("pvmd output overflows buffer!");
		}
    } while (c != '\n');
    localhost[offset - 1] = '\0'; /* chop of the \n at the end */
    
	dprintf ( D_ALWAYS, "pvmd response: %s\n", localhost );

    pvmsock = (char *) malloc(10+strlen(localhost));
    sprintf(pvmsock, "PVMSOCK=%s", localhost);
    close(pvmd_file_fd);

    pvmputenv(pvmsock);
   
    dprintf(D_ALWAYS, "%s\n", pvmsock);
    
    pvm_tid = pvm_mytid();
    if (pvm_tid < 0) {
		EXCEPT("pvm_mytid() failed");
    }
    
    if (pvm_reg_rm(&master_info) < 0) {
      EXCEPT("Cannot connect to PVMd");
    }
    
    this_job->pvm_master_tid = pvm_tidtohost(pvm_tid) | TIDPVMD;
    pvm_reg_hoster();
    pvm_setopt(PvmRoute, PvmDontRoute);
    pvm_setopt(PvmResvTids, 1);
    pvm_getfds(&pvm_fd);
    this_job->pvm_pipe = pvm_fd[0];
    dprintf(D_ALWAYS, "pvm_fd = %d, mytid = t%x\n", 
            this_job->pvm_pipe, pvm_tid);
#ifdef START_PVMD
    unlink( pvm_rendevous_file );
#endif
  } 
  else { /* The Child */
    extern	char	**environ;
#ifndef START_PVMD
    exit(0);
#endif
    set_user_priv_final();		// can't dprintf() after this
    dup2(pfds[1], 1);
    for(i = sysconf(_SC_OPEN_MAX) - 1; i > 2; i--) {
      close(i);
    }
    putenv("PVM_ROOT=/");
    /* Set the pvmd's home directory to be the jobs iwd so that any 
       processes run by the pvmd live there
    */
    pvmsock = (char *) malloc(6+strlen(JobList->Proc.iwd));
    sprintf(pvmsock, "HOME=%s", JobList->Proc.iwd);
    pvmputenv(pvmsock);
    
    execve(pvmd, argv, environ);
    exit( EXECFAILED );
  }
  return 0;
}

Host *
GetMachine(char *arch, int flag)
{
  int          i;
  int          min_load;
  Host         *min_host = 0;
  ClassDef     *classdef;
  Host         *this_host;
  
  if (flag == PvmTaskArch) {
    dprintf( D_ALWAYS, "Searching for Arch %s\n", arch );
    classdef = GetClassByName(arch);
    if (classdef == NULL) {
      dprintf( D_ALWAYS, "No such class defined: %s\n", arch);
      return 0;
    }
    min_load = MAXINT;
    /* let status go above 1 in a round-robin, i.e. allow the user
     * to spawn more tasks than there are hosts */
    for (i = 0; i < JobList->pvm_machine_count; i++) {
      this_host = &(JobList->hosts[i]);
      /* just allow one task per host for now... */
      if ( (this_host->status /*  >=  */ ==  PVM_STATE_PENDING) &&
           (this_host->status < min_load) &&
           (this_host->class_num == classdef->class_num) ) {
        min_host = this_host;
			// We always prefer to spawn on a host that isn't
			// suspended, but we've got to take what we can get.  If
			// the only available hosts are suspended, we try to spawn
			// there anyway, or else the pvm master is going to get
			// confused and wonder why it can't spawn on the remaining
			// host.
        min_load = (this_host->is_suspended) ? MAXINT : this_host->status;
      }
    }
    if (min_host == 0) {
      dprintf( D_ALWAYS, "No machine in class %s available\n",
               arch );
    }
  } 
  else if (flag == PvmTaskHost) {
    this_host = FindHostByName( arch );
    if (this_host && this_host->status >= PVM_STATE_PENDING) {
      min_host = this_host;
    }
  } 
  else {
    min_host = 0;
  }
  
  return min_host;
}


/* Start the master process of a PVM job on the local host rather than
   remotely.  Set up ids, fds, and exec.  This process must be linked
   as a normal program, not as a Condor program.  See starter.c for format
   of argv */
int
StartLocalProcess()
{
  char *av0;
  char *job_args[ MAXPATHLEN / 2 ];
  char arg_buf[MAXPATHLEN];
  int  ChildPid;
  int  i;
  int  fd0, fd1, fd2;
  int  argc;
  long open_max;
  ClassDef *classdef;

  pid_t pidlist[10];

  dprintf( D_ALWAYS, "StartLocalProcess: = %s < %s > %s >& %s\n",
           JobList->Proc.cmd[0],
           JobList->Proc.in[0],
           JobList->Proc.out[0],
           JobList->Proc.err[0] );
  if ( JobList->local_proc_pid != 0 ) {
    dprintf( D_ALWAYS, "Local process already running!\n" );
    return -1;
  }
  
  /* Should loop here over all classes */
  /* This loop isn't right since current does not mean ready */
  for (classdef = JobList->class_list; classdef != NULL;
       classdef = classdef->next) {
    if (count_ready_hosts( classdef->class_num ) < 
        classdef->min_needed) {
      dprintf( D_ALWAYS, 
               "Not enough machines in class %s to start local proc.\n",
               classdef->class_name);
      return -1;
    }
  }
  
  ChildPid = fork();
  if (ChildPid < 0) {
    dprintf( D_ALWAYS, "Unable to start local PVM process\n");
    return -1;
  }
  if ( ChildPid ) {	/* The Parent */
    JobList->local_proc_pid = ChildPid;
    dprintf ( D_ALWAYS, "Local PVM process pid = %d\n", ChildPid );
    initializeUserLog ();
  } 
  else {		/* The Child */
    open_max = sysconf(_SC_OPEN_MAX);
    dprintf(D_ALWAYS, "open_max = %d\n", open_max);
    set_user_priv_final();
    chdir(JobList->Proc.iwd);
    sprintf( arg_buf, "%s", JobList->Proc.args[0] );
    job_args[0] = JobList->Proc.cmd[0];
    mkargv( &argc, &job_args[1], arg_buf);
    fd0 = open(JobList->Proc.in[0], O_RDONLY, 0666);
    dup2(fd0, 0);
    
    fd1 = open(JobList->Proc.out[0], O_WRONLY | O_CREAT | O_TRUNC, 0666);
    dup2(fd1, 1);
    close(fd1);
    lseek(1, 0, SEEK_END);	/* go to the end instead of overwriting */
    
    fd2 = open(JobList->Proc.err[0], O_WRONLY | O_CREAT | O_TRUNC, 0666);
    dup2(fd2, 2);
    close(fd2);
    lseek(2, 0, SEEK_END);	/* go to the end instead of overwriting */
    
    for(i = open_max; i > 2; i--) {
      close(i);
    }
    execv( job_args[0], job_args );
    exit( EXECFAILED );
  }
  return 0;
}

int
DeadLocalProcess( int deadpid, int status, struct rusage rusage )
{
  int i;
  pid_t pidlist[10];
  
  if (maxjob > 0) {
    if ( deadpid == JobList->pvm_pid ) {
      dprintf( D_ALWAYS, "Lost local pvmd termsig = %d, retcode = %d\n", 
               WTERMSIG(status), WEXITSTATUS(status));
      JobList->pvm_pid = 0;
#ifdef START_PVMD
      JobList->pvm_pipe = 0;
#endif
      return 0;
    }
    if ( deadpid != JobList->local_proc_pid ) {
      dprintf( D_ALWAYS, "unknown process id %d died\n", deadpid );
      return -1;
    }
  }
  else {
    dprintf( D_ALWAYS, "unknown process id %d died\n", deadpid );
    return -1;
  }
  
  dprintf( D_ALWAYS, "Local process for job %d.%d died with status 0x%x\n", 
           JobList->Proc.id.cluster,
           JobList->Proc.id.proc,
           status);

  initializeUserLog ();

  JobStatus = status;  // Used in log_termination()
  
  SendNotification( PvmTaskExit, JobList->local_proc_tid );
  JobList->local_proc_pid = -1;
  /*	update_rusage( &(JobList->Proc.local_usage), &rusage ); */
  JobList->JobStatus[0] = status;
  JobCompleted = true;
  multi_CleanupAll(0,0,NULL);
  return 0;
}

void
sigchild_handler(int dummy)
{
  int deadpid;
#if defined(HPUX9)
  int status;
  int rusage;
  sigset_t sig_set;
  struct sigaction sig_act;
#else
  int 	        status;
  struct rusage	rusage;
#endif

  do {
#if defined(HPUX9)
    deadpid = wait3(&status, WNOHANG, NULL);
#else
    deadpid = wait3(&status, WNOHANG, &rusage);
#endif
    dprintf( D_ALWAYS, "deadpid = %d\n", deadpid );
    if (deadpid > 0) {
#if defined(HPUX9)
      int my_status;
      struct rusage my_rusage;
      my_status.w_status = status;
      DeadLocalProcess(deadpid, my_status, my_rusage);
#else
      DeadLocalProcess(deadpid, status, rusage);
#endif
    } 
    else {
      dprintf( D_ALWAYS, "No more dead processes(errno = %d)\n", errno);
    }
  } while (deadpid > 0 || (deadpid == -1 && errno == EINTR));
}

/*
  Connect to the startd on the remote host and send it a signal.
*/
int
signal_startd( char *host, int cmd )
{
  char *addr_port;
  char *capability;
  int retval;
  
  dprintf( D_ALWAYS, "signal_startd( %s, %d )\n", host, cmd );
  if (host == NULL || *host == 0) {
    return 0;
  }
  
  /* old hostnames need to be converted to <IP:port> and capabilities */
  if (Find_Addr_and_Cap(host, &addr_port, &capability) < 0) {
    dprintf( D_ALWAYS, "Shadow: Can't find addr and capability\n");
    return -1;
  }
  
  /* send the startd the command */
  retval = send_cmd_to_startd(addr_port, capability, cmd);
  
  switch (retval) {
  case -1: 
    dprintf( D_ALWAYS, "Shadow: Can't connect to condor_startd on %s\n",
             addr_port);
    return -1;
    break;
  case -2: 
    dprintf( D_ALWAYS, "sock->code(%d) failed!", cmd );
	return -1;
    break;
  case -3: 
    dprintf( D_ALWAYS, "sock->put() failed!" );
	return -1;
    break;
  case -4: 
    dprintf( D_ALWAYS, "end_of_message failed" );
	return -1;
    break;
  case 0: 
  default:
    break;
  }

  free(addr_port);
  free(capability);
  return 0;
}

int
SendPVMStartAck()
{
#ifdef PVM34
  struct pvmminfo info;
#endif
  int  	i;
  int  	new_count = 0;
  int  	no_error = 0;
  int  	*new_hosts;
  int  	pvm_new_host;
  char  buf[20];
  int  	host_num;
  Host  *test_host;

  new_hosts = (int *)MALLOC(JobList->pvm_machine_count * sizeof(int));
  for (i = 0; i < JobList->pvm_machines_starting; i++) {
    host_num = JobList->pvm_starting_list[i];
    test_host = &(JobList->hosts[host_num]);
    if ( test_host->status == PVM_STATE_SEND_PVM || 
         test_host->status == PVM_STATE_WAITING) {
      dprintf( D_ALWAYS, 
               "Host %s still in SEND_PVM, or WAITING can't STARTACK\n",
               test_host->name );					 
      FREE(new_hosts);
      return 0;
    } else if ( test_host->status == PVM_STATE_REPLIED ||
                test_host->status == PVM_STATE_EXITED ) {
      dprintf( D_ALWAYS, "Adding host %s to STARTACK msg.\n",
               test_host->name );
      new_hosts[new_count] = host_num;
      new_count++;
    } else {
      dprintf( D_ALWAYS, "SendPVMStartAck:Host on starting list in "
               "unexpected state %d\n", test_host->status );
    }
  }
  
  if (new_count > 0) {
    pvm_initsend(PvmDataFoo);
    dprintf( D_ALWAYS, "Num Hosts to pack = %d\n", new_count );
    pvm_pkint(&new_count, 1, 1);
    for (i = 0; i < new_count; i++) {
      test_host = &(JobList->hosts[new_hosts[i]]);
      dprintf( D_ALWAYS, "Packing tid t%x with reply %s\n",
               test_host->pvmd_tid,
               test_host->pvmd_reply );
      pvm_pkint(&(test_host->pvmd_tid), 1, 1);
      if (test_host->pvmd_reply) {
        pvm_pkstr(test_host->pvmd_reply);
#ifdef PVM34
        // pvmd_reply may be needed for config in pvm3.4.
        // We should not free it now.  -Bin
#else
        FREE( test_host->pvmd_reply );
        test_host->pvmd_reply = NULL;
#endif
      } 
      else {
        pvm_pkstr("PvmNoHost");
      }
    }
    dprintf( D_ALWAYS, "Sending SM_STHOSTACK to PVMd\n");
#ifdef PVM34
    pvm_getminfo(pvm_getsbuf(), &info);
    info.wid = JobList->hoststart_wid;
    pvm_setminfo(pvm_getsbuf(), &info);
#else
    pvm_setmwid(pvm_getsbuf(), JobList->hoststart_wid);
#endif
    JobList->hoststart_wid = -1;
    pvm_send(JobList->pvm_master_tid, SM_STHOSTACK);
  } 
  else {
    dprintf( D_ALWAYS,"SendPVMStartAck:fell through with count 0!!!!\n" );
  }
  FREE(new_hosts);
  return 0;
}
