/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 

#include "condor_common.h"

#include <sys/param.h>

#include "condor_debug.h"
#include "condor_config.h"
#include "debug.h"
#include "sched.h"
#include "proc.h"
#include "condor_io.h"
#include "joblist.h"
extern struct tagJobList	*JobList;
#include "request.h"

#include "pvm3.h"
#include "pvm_global.h"

#ifdef PVM34
#include "pvmproto.h"
#else
#include "pvmsdpro.h"
#endif


extern int  ProcTimeout;
extern int 	HostTimeout;

/*
static struct {
	char	*msg_nam;
	int		msg_tag;
} MsgNames[] = {
{ "SM_SPAWN",	SM_SPAWN},
{ "SM_EXEC",	SM_EXEC},
{ "SM_EXECACK",	SM_EXECACK},
{ "SM_TASK",	SM_TASK},
{ "SM_CONFIG",	SM_CONFIG  },
{ "SM_ADDHOST",	SM_ADDHOST },
{ "SM_DELHOST",	SM_DELHOST },
{ "SM_ADD",	 	SM_ADD     },
{ "SM_ADDACK",	SM_ADDACK  },
{ "SM_NOTIFY",	SM_NOTIFY  },
{ "SM_TASKX",	SM_TASKX   },
{ "SM_HOSTX",	SM_HOSTX   },
{ "SM_HANDOFF",	SM_HANDOFF },
{ "SM_SCHED",	SM_SCHED   },
{ "SM_STHOST",	SM_STHOST   },
{ "SM_STHOSTACK", SM_STHOSTACK}
};
*/

static char *pvmerrnames[] = {
    "PvmOk",
    "",
    "PvmBadParam",
    "PvmMismatch",
    "PvmOverflow",
    "PvmNoData",
    "PvmNoHost",
    "PvmNoFile",
    "PvmDenied",
    "",
    "PvmNoMem",
    "",
    "PvmBadMsg",
    "",
    "PvmSysErr",
    "PvmNoBuf",
    "PvmNoSuchBuf",
    "PvmNullGroup",
    "PvmDupGroup",
    "PvmNoGroup",
    "PvmNotInGroup",
    "PvmNoInst",
    "PvmHostFail",
    "PvmNoParent",
    "PvmNotImpl",
    "PvmDSysErr",
    "PvmBadVersion",
    "PvmOutOfRes",
    "PvmDupHost",
    "PvmCantStart",
    "PvmAlready",
    "PvmNoTask",
    "PvmNotFound",
    "PvmExists",
    "PvmHostrNMstr",
    "PvmParentNotSet",
    0
};

char *pvmstrerror(int err)
{
	if (err < 0 && err >= -35) {
		return pvmerrnames[-err];
	}
	return "";
}

#include "prototypes.h"

/* for get_user_uid */
#include "condor_uid.h"

/* handle pvm messages... */
void handle_sm_spawn ( int src, int req_id );
void handle_sm_addack ( int src, int req_id );
void handle_sm_addhost ( int src, int req_id );
void handle_sm_delhost ( int src, int req_id );
void handle_sm_execack ( int src, int req_id );
void handle_sm_config ( int src, int req_id );
void handle_sm_notify ( int src, int req_id );
void handle_sm_sthost ( int src, int req_id );
void handle_sm_task ( int src, int req_id );
void handle_sm_sched ( int src, int req_id );
void handle_sm_hostx ( int src, int req_id );
void handle_sm_taskx ( int src, int req_id );

/* Sanity checking */
const int PADLENGTH = 64;
void prepare_buffer ( char *buf, int goodlen, int padlen );
void check_buffer ( char *buf, int goodlen, int padlen );


extern "C" {
int CoCheckCompleteSpawn( Request *r, int new_tid );
	int CompleteSpawn(Request *request, int new_tid, int pid);
	Request * NewNotification( int kind, int to_task,
                                                int msg_id, int on_task );
}

int
SpawnLocal(Request *spawn_request)
{
	WorkItem	*item;
	int			argc;
	char		*argv[50];
	int			dummy_arg;
	int			one = 1;
	int			i;
	int			tmp_pid;
	char		cmd_full_path[1024];
	
	item = (WorkItem *) spawn_request->extra_data;
	mkargv( &argc, argv, item->Proc->args[0]);
	pvm_initsend(PvmDataFoo);
	dummy_arg = atoi(argv[0]);	/* parent tid */
	pvm_pkint(&dummy_arg, 1, 1);

	/* if the spawned task is a pvm group server, use the
	 path in the config file. */
	if (strcmp("pvmgs", item->Proc->cmd[0]) == 0) {
	     sprintf(cmd_full_path, "%s", param("PVMGS"));
	     dprintf(D_ALWAYS, "Bin: %s\n", cmd_full_path);
	     pvm_pkstr(cmd_full_path);
	} else if (item->Proc->cmd[0][0] != '/') {
		sprintf(cmd_full_path, "%s/%s", item->Proc->iwd, 
				item->Proc->cmd[0]);
		dprintf ( D_FULLDEBUG, "SpawnLocal: about to pack \"%s\".\n", 
				  cmd_full_path );
		pvm_pkstr(cmd_full_path);
	} else {
		dprintf ( D_FULLDEBUG, "SpawnLocal is about to pack \"%s\".\n", 
				  cmd_full_path );
		pvm_pkstr(item->Proc->cmd[0]);
	}
	dummy_arg = atoi(argv[1]);	/* pvm_flags */
	pvm_pkint(&dummy_arg, 1, 1);
	pvm_pkint(&one, 1, 1);		/* How many to spawn */
	dummy_arg = argc - 6;
	pvm_pkint(&dummy_arg, 1, 1);		/* How many cmd. line args */
	for (i = 0; i < dummy_arg; i++) {
		pvm_pkstr(argv[i + 6]);
	}
#ifdef PVM34
	dummy_arg = atoi(argv[2]); /* Out tid */
	pvm_pkint(&dummy_arg, 1, 1);

	dummy_arg = atoi(argv[3]); /* Out ctx */
	pvm_pkint(&dummy_arg, 1, 1);

	dummy_arg = atoi(argv[4]); /* Out cod */
	pvm_pkint(&dummy_arg, 1, 1);
	dummy_arg = atoi(argv[5]); /* Trace tid */
	pvm_pkint(&dummy_arg, 1, 1);

	dummy_arg = atoi(argv[6]); /* Trace ctx */
	pvm_pkint(&dummy_arg, 1, 1);

	dummy_arg = atoi(argv[7]); /* Trace cod */
	pvm_pkint(&dummy_arg, 1, 1);

#else
	dummy_arg = atoi(argv[2]); /* Out tid */
	pvm_pkint(&dummy_arg, 1, 1);
	dummy_arg = atoi(argv[3]); /* Out cod */
	pvm_pkint(&dummy_arg, 1, 1);
	dummy_arg = atoi(argv[4]); /* Trace tid */
	pvm_pkint(&dummy_arg, 1, 1);
	dummy_arg = atoi(argv[5]); /* Trace cod */
	pvm_pkint(&dummy_arg, 1, 1);
#endif
	dummy_arg = 0;
	pvm_pkint(&dummy_arg, 1, 1);	/* No environment variables */

#ifdef PVM34
	// In pvm3.4, pvmd expected message format with tag SM_EXEC
	// (See exectasks() in file ddpro.c  -Bin
	pvm_pkint(&one, 1, 1); /* for pvm3.4: wxp->w_instance */
	pvm_pkint(&one, 1, 1); /* for pvm3.4: wxp->w_outof */
#endif

	pvm_send(TIDPVMD, SM_EXEC);

	pvm_recv(TIDPVMD, SM_EXECACK);
	
	pvm_upkint(&one, 1, 1);
	if (one != 1) {
		dprintf(D_ALWAYS, "Unexpected EXECACK, count = %d\n", one);
	}
	pvm_upkint(&dummy_arg, 1, 1);	/* The new tid */
	tmp_pid = AllocProc();
	item->process = &(JobList->procs[tmp_pid]);
	CompleteSpawn(spawn_request, dummy_arg, tmp_pid);
	RemoveProc(tmp_pid);
	return 0;
}

int
#ifdef PVM34
DoSpawn(
int		for_whom,
int		req_id,
char	*a_out,
char	*arg_line,
char	*dest_arch,
int		where_flag,
int		out_tid,
int out_ctx,  // out context -- pvm3.4
int		out_cod,
int		trc_tid,
int trc_ctx,  // trace context -- pvm3.4
int		trc_cod,
int		response,
int		spawn_type)
#else
DoSpawn(
int		for_whom,
int		req_id,
char	*a_out,
char	*arg_line,
char	*dest_arch,
int		where_flag,
int		out_tid,
int		out_cod,
int		trc_tid,
int		trc_cod,
int		response,
int		spawn_type)
#endif
{
	Host	*to_whom;
	WorkItem	*new_work;
	Request		*request;
	char		*buf;

	to_whom = GetMachine( dest_arch, where_flag );
	
	request = NewRequest(REQ_SPAWN, for_whom, req_id, response);

	if (to_whom != 0) {
		request->on_tid = to_whom->starter_tid;
		to_whom->status++;
	} else {
		request->on_tid = 0;
	}

	new_work = (WorkItem *) MALLOC(sizeof(WorkItem));
	request->extra_data = (void *) new_work;
	
	new_work->Proc = ConstructProc( 1, &(JobList->Proc) );

	new_work->Proc->cmd[0] = strdup( a_out );
	
#ifdef PVM34
	buf = (char *) malloc (sizeof(int) * 8 + 
			       (strlen(arg_line) + 10) * sizeof(char));
#else
	buf = (char *) malloc (sizeof(int) * 6 + 
			       (strlen(arg_line) + 10) * sizeof(char));
#endif
	if (buf==0) {
	     EXCEPT("Dospawn out of mem");
	}

#ifdef PVM34
	sprintf(buf, "%d %d %d %d %d %d %d %d %s",
		for_whom, where_flag, out_tid, out_ctx, out_cod, 
		trc_tid, trc_ctx, trc_cod, arg_line);
#else
	sprintf(buf, "%d %d %d %d %d %d %s",
			for_whom, where_flag, out_tid, out_cod, trc_tid,
			trc_cod, arg_line);
#endif
	new_work->Proc->args[0] = strdup( buf );
	free(buf);

	new_work->Proc->in[0] = "/dev/null";
	new_work->Proc->out[0] = "/dev/null";
	new_work->Proc->err[0] = "/dev/null";
	new_work->Proc->exit_status[0] = 0;
	new_work->timer = new_timer( ProcTimeout, 
								proc_start_timeout );
	new_work->where = strdup( dest_arch );
	new_work->spawn_flag = where_flag;
	new_work->spawn_type = spawn_type;
	new_work->process = 0;

	if ( to_whom != 0 ) {
		signal_startd( to_whom->name, REQ_NEW_PROC );
		
	} else if (where_flag == PvmTaskHost && (dest_arch[0] == '.')) {
		SpawnLocal(request);
	} 
	/* If the flag is PvmMppFront, we spawn the task on the 
	   same machine as where the shadow is.  In this way, the
	   group server will be spawned on the local machine. */
	else if (where_flag == PvmMppFront) {
	     SpawnLocal(request);

	} else {
		/* Compose & send Nack to sender */
		CompleteSpawn(request, PvmNoHost, 0);
	}
	return 0;
}

int
CompleteSpawn(Request *request, int new_tid, int pid)
{
	int		ack_count;
	WorkItem *work_ptr;
	Process	 *new_process;
	Host		*spawn_host;
	ClassDef	*spawn_class;

	work_ptr = (WorkItem *) request->extra_data;
	new_process = work_ptr->process;

	if (new_tid < 0) {
		if ( pid != 0) {
			RemoveProc( pid );
		}
	} else if (new_process != 0) {
		/* I'm not sure how this can remain zero, but it has happened... */
		new_process->pvm_tid = new_tid;
		if (new_process->pvm_orig_tid == 0) {
			new_process->pvm_orig_tid = new_tid;
		}

		if (work_ptr->spawn_flag == PvmTaskArch) {

		     new_process->spawn_flag = work_ptr->spawn_flag;
		     if (new_process->dest_arch == NULL) {
			  new_process->dest_arch = strdup(work_ptr->where);
			}
		} else {
			/* If the user didn't request a particular arch., we need to 
			   determine the arch. used in case of a checkpoint restart */

		     new_process->spawn_flag = PvmTaskArch;

		     /*
		     spawn_host = &(JobList->hosts[new_process->host_num]);
		     */
		     /* The host should the the same as the local host -Bin */
		     
		     spawn_host = &(JobList->hosts[0]); 

		     spawn_class = GetClassByNum(spawn_host->class_num);
		     new_process->dest_arch = strdup(spawn_class->class_name);
		}

		if (new_process->a_out == NULL) {
			dprintf(D_ALWAYS, 
					"Uh Oh, new_process t%x doesn't know its a.out\n",
					new_process->pvm_tid);
			new_process->a_out = strdup( work_ptr->Proc->cmd[0] );
		}

		new_process->process_state = PROCESS_RUNNING;
		dprintf(D_ALWAYS, 
		"CompleteSpawn(): for t%x, new_tid = t%x, orig_tid = t%x, vpid = %d\n",
				request->for_tid, new_tid, new_process->pvm_orig_tid, pid);


	} else {
		dprintf(D_ALWAYS, 
				"CompleteSpawn() doesn't know what process spawned!!\n");
		new_tid = PvmBadParam;	/* Force an error to be returned */
	}

	if (work_ptr->spawn_type != SPAWN_PVM && 
		work_ptr->spawn_type != SPAWN_CARMI && 
		new_tid > 0) {
		CoCheckCompleteSpawn(request, new_tid);
	} else {
		ack_count = 1;
		pvm_initsend(PvmDataDefault);
		pvm_pkint(&ack_count, 1, 1);
		pvm_pkint(&new_tid, 1, 1);
		SendResponse(request);
	}

	cancel_timer( work_ptr->timer );
	FREE( work_ptr->Proc->cmd[0] );
	FREE( work_ptr->Proc->args[0] );
	DestructProc( work_ptr->Proc );
	if (work_ptr->where != NULL) {
		FREE( work_ptr->where );
	}
	DeleteRequest( request );
	return 0;
}

int
HandlePVM( int msg_tag, int src, int req_id )
{

/*
	for (i = 0; i < sizeof(MsgNames) / sizeof(MsgNames[0]); i++) {
		if (msg_tag == MsgNames[i].msg_tag) {
			dprintf( D_ALWAYS, "PVMd message is %s from t%x\n", 
					MsgNames[i].msg_nam, src );
			break;
		}
	}
*/
	char stack_check[10000];
	memset( stack_check, 42, 10000 );

	switch(msg_tag) {
	case SM_SPAWN:
		dprintf ( D_ALWAYS, "PVMd message is SM_SPAWN from t%x\n", src );
		handle_sm_spawn( src, req_id );
		break;
	case SM_ADDACK:
		dprintf ( D_ALWAYS, "PVMd message is SM_ADDACK from t%x\n", src );
		handle_sm_addack( src, req_id );
		break;
	case SM_ADDHOST:
		dprintf ( D_ALWAYS, "PVMd message is SM_ADDHOST from t%x\n", src );
		handle_sm_addhost( src, req_id );
		break;
	case SM_DELHOST:
		dprintf ( D_ALWAYS, "PVMd message is SM_DELHOST from t%x\n", src );
		handle_sm_delhost( src, req_id );
		break;
	case SM_EXECACK:
		dprintf ( D_ALWAYS, "PVMd message is SM_EXECACK from t%x\n", src );
		handle_sm_execack( src, req_id );
		break;
	case SM_CONFIG:
		dprintf ( D_ALWAYS, "PVMd message is SM_CONFIG from t%x\n", src );
		handle_sm_config( src, req_id );
		break;
	case SM_NOTIFY:
		dprintf ( D_ALWAYS, "PVMd message is SM_NOTIFY from t%x\n", src );
		handle_sm_notify( src, req_id );
		break;
	case SM_STHOST:
		dprintf ( D_ALWAYS, "PVMd message is SM_STHOST from t%x\n", src );
		handle_sm_sthost( src, req_id );
		break;
	case SM_TASK:
		dprintf ( D_ALWAYS, "PVMd message is SM_TASK from t%x\n", src );
		handle_sm_task( src, req_id );
		break;
	case SM_SCHED:
		dprintf ( D_ALWAYS, "PVMd message is SM_SCHED from t%x\n", src );
		handle_sm_sched( src, req_id );
		break;
	case SM_HOSTX:
		dprintf ( D_ALWAYS, "PVMd message is SM_HOSTX from t%x\n", src );
		handle_sm_hostx( src, req_id );
		break;
	case SM_TASKX:
		dprintf ( D_ALWAYS, "PVMd message is SM_TASKX from t%x\n", src );
		handle_sm_taskx( src, req_id );
		break;
	default:
		dprintf( D_FULLDEBUG, "PVMd message unknown (0x%x) from t%x\n", 
				 msg_tag, src );
		return 0;
	}

	for ( int i=0 ; i<10000 ; i++ ) {
		if ( stack_check[i] != 42 ) {
			dprintf ( D_ALWAYS, "STACK FUCKED!\n" );
			break;
		}
	}

	return 1;
}

void
handle_sm_spawn( int src, int req_id ) {

/* see lpvmgen.c, function pvm_spawn for the place that packs this... */

#ifdef PVM34
	int  out_ctx; /* context in pvm3.4 */
	int trc_ctx;  /* trace context in pvm3.4 */
#endif
	int     i;
	char	cmd_buf[_POSIX_PATH_MAX + PADLENGTH];
	char	dest_arch[80 + PADLENGTH];
	char	args[_POSIX_PATH_MAX + PADLENGTH];
	char	**env;
	char	temp_arg[_POSIX_PATH_MAX + PADLENGTH];
	int		count;
	int		flag;
	int		arg_count;
	int		env_count;
	int		out_tid;
	int		out_cod;
	int		trc_tid;
	int		trc_cod;

	prepare_buffer( cmd_buf, _POSIX_PATH_MAX, PADLENGTH );
	pvm_upkstr( cmd_buf );
	check_buffer( cmd_buf, _POSIX_PATH_MAX, PADLENGTH );

	pvm_upkint( &flag, 1, 0 );
	ASSERT( (flag >= 0) && (flag < 128 ) );

	prepare_buffer( dest_arch, 80, PADLENGTH );
	pvm_upkstr( dest_arch );
	check_buffer( dest_arch, 80, PADLENGTH );

	pvm_upkint( &count, 1, 0 );
	ASSERT( count == 1 );

	pvm_upkint( &arg_count, 1, 0 );
	ASSERT( (arg_count >= 0) && (arg_count < 1000) );

	args[0] = '\0';
	for ( ; arg_count > 0; arg_count--) {
		prepare_buffer( temp_arg, _POSIX_PATH_MAX, PADLENGTH );
		pvm_upkstr( temp_arg );
		check_buffer( temp_arg, _POSIX_PATH_MAX, PADLENGTH );

		strcat( temp_arg, " " );
		strcat( args, temp_arg );
	}
/* I'm not going to bother checking these, because the accepted
   values can be so darned variable... */
#ifdef PVM34
	pvm_upkint( &out_tid, 1, 1 );
	pvm_upkint( &out_ctx, 1, 1 ); /* context pvm3.4 */
	pvm_upkint( &out_cod, 1, 1 );
	pvm_upkint( &trc_tid, 1, 1 );
	pvm_upkint( &trc_ctx, 1, 1 ); /* context pvm3.4 */
	pvm_upkint( &trc_cod, 1, 1 );
#else
	pvm_upkint( &out_tid, 1, 1 );
	pvm_upkint( &out_cod, 1, 1 );
	pvm_upkint( &trc_tid, 1, 1 );
	pvm_upkint( &trc_cod, 1, 1 );
#endif

	pvm_upkint( &env_count, 1, 1 );
	ASSERT( (env_count >= 0) && (env_count < 1000) );

	env = (char **) MALLOC((env_count+1) * sizeof(char *));
	for ( i = 0; i < env_count; i++) {
		prepare_buffer( temp_arg, _POSIX_PATH_MAX, PADLENGTH );
		pvm_upkstr( temp_arg );
		check_buffer( temp_arg, _POSIX_PATH_MAX, PADLENGTH );
		env[i] = strdup( temp_arg );
	}
	env[i] = NULL;
	
	for ( i = 0; i < env_count; i++) {
		FREE( env[i] );
	}
	FREE( env );
#ifdef PVM34
	DoSpawn( src, req_id, cmd_buf, args, 
			 dest_arch, flag, out_tid, out_ctx, 
			 out_cod, trc_tid, trc_ctx, trc_cod,
			 SM_SPAWN, SPAWN_PVM );
#else
	DoSpawn( src, req_id, cmd_buf, args, 
			 dest_arch, flag, out_tid, out_cod, 
			 trc_tid, trc_cod,
			 SM_SPAWN, SPAWN_PVM );
#endif
	
}

void
handle_sm_addack( int src, int req_id ) {

/* see ddpro.c, function startack for sending.... */

	int  pvmd_tid;
	int	 new_count=0;
	int  narches;
	int	 i;
	char host_name[200 + PADLENGTH];
	char host_arch[50 + PADLENGTH];
	int	 host_speed;
	Host *new_host;
	int  machine_num;
	int  hd_dsig;
	
	pvm_upkint(&new_count, 1, 1);
	ASSERT( (new_count >= 0) && (new_count < 1000) );

	pvm_upkint(&narches, 1, 1);
	ASSERT( narches == 0 );  /* zero in 3.4.1... */

	for( i = 0; i < new_count; i++) {
		new_host = NULL;
		pvm_upkint(&pvmd_tid, 1, 1);
		pvmd_tid |= TIDPVMD;

		prepare_buffer( host_name, 200, PADLENGTH );
		pvm_upkstr(host_name);
		check_buffer( host_name, 200, PADLENGTH );

		prepare_buffer( host_arch, 50, PADLENGTH );
		pvm_upkstr(host_arch);
		check_buffer( host_arch, 50, PADLENGTH );

		/* These next two can have lots of values, won't check... */
		pvm_upkint(&host_speed, 1, 1);
#ifdef PVM34
		pvm_upkint(&hd_dsig, 1, 1);  /* New in 3.4.1.... */
#endif
		for (machine_num = 0; 
			 machine_num < JobList->pvm_machine_count; 
			 machine_num++) {
			if (pvmd_tid == 
				(JobList->hosts[machine_num].pvmd_tid | TIDPVMD)) {
				new_host = &(JobList->hosts[machine_num]);
				break;
			}
		}
		if (new_host != NULL) {
			new_host->pvmd_tid = pvmd_tid;
			dprintf( D_ALWAYS, 
					 "Host #%d(%s) has been added to PVM, pvmd_tid = %x\n",
					 machine_num, 
					 new_host->name,
					 pvmd_tid );
			if ( new_host->status != PVM_STATE_REPLIED ) {
				dprintf(D_ALWAYS,
						"This machine in unexpected state %d\n",
						new_host->status);
				continue;
			}
			new_host->status = PVM_STATE_PENDING;
			SendNotification( PvmHostAdd, pvmd_tid  );
			
				/* Make the starter the scheduler for that machine */
			pvm_initsend(PvmDataFoo);
			pvm_pkint(&new_host->starter_tid, 1, 1);
			pvm_send(pvmd_tid, SM_HANDOFF);
		} else {
			if (pvmd_tid < 0) {
				dprintf( D_ALWAYS, "pvmd reports error %d on SM_ADDACK: %s\n",
						 pvmd_tid, pvmstrerror(pvmd_tid) );
			} else {
				dprintf( D_ALWAYS, 
						 "No host has tid t%x, must be an error\n",
						 pvmd_tid );
			}
		}
	}
	if (JobList->pvm_machines_starting != new_count) {
		if ( new_count == PvmDSysErr ) {
			dprintf ( D_ALWAYS, "Shame on you, you caused a PvmDSysErr.\n");
		} else {
			dprintf( D_ALWAYS, "WARNING!!! Start Count from PVM (%d) does not "
					 "match expected %d\n", 
					 new_count, JobList->pvm_machines_starting );
		}
		new_count = JobList->pvm_machines_starting;
	}
	JobList->pvm_machines_starting -= new_count;
	dprintf( D_ALWAYS, 
			 "pvm_machines_starting = %d(should be 0)\n",
			 JobList->pvm_machines_starting );
	if (JobList->pvm_machines_starting == 0 ) {
		FREE( JobList->pvm_starting_list ); 
		JobList->pvm_starting_list = NULL;
		StartLocalProcess();
	}
	StartWaitingHosts();
}

void
handle_sm_addhost ( int src, int req_id ) {

/* See lpvmgen.c, function pvm_addhosts */

		/* This is asynch. lib is synch */				
#ifdef PVM34
	struct pvmminfo info;
#endif
	int      i;
	int      num_req;
	char     cmd_buf[_POSIX_PATH_MAX + PADLENGTH];
	int      num_active = 0;
	int      cur_req;
	ClassDef *classdef;
	ClassDef *loop_class;
	Host     *this_host;

#ifdef Solaris
/* It appears that Solaris 26 LIES to us, saying we can have 1024
   open fds at once, when in reality we can only have 256.  So...
   Set this to a safer value - let's try (256-10)/2 = 123 */
	int max_hosts = 123;
#else	
	int      max_hosts = (getdtablesize() - 9) / 2;
		/* Why -9?  Well, let's see:
		   0-2:		stdin, stdout, stderr
		   3:		JobQueue
		   4:		Connection to PVM
		   5:		dprintf
		   So, I'm missing 3, but I get FD PANIC's with smaller
		   values...
		*/
#endif

	pvm_upkint(&num_req, 1, 0);
		/* Can't check num_req...might be an error (neg number...) */
	for ( i = 0; i < num_req; i++) {
		prepare_buffer( cmd_buf, _POSIX_PATH_MAX, PADLENGTH );
		pvm_upkstr(cmd_buf);
		check_buffer( cmd_buf, _POSIX_PATH_MAX, PADLENGTH );
	}
	classdef = GetClassByName( cmd_buf );
	if (classdef == NULL) {
		dprintf(D_ALWAYS, "No such class %s\n", cmd_buf);
		return;
	}
	dprintf( D_ALWAYS, "Got a request for %d new hosts, id=%d\n", 
			 num_req, req_id );
	for (i = 0; i < JobList->pvm_machine_count; i++) {
		this_host = &(JobList->hosts[i]);
		if (this_host->status != PVM_STATE_EXITED ) {
			num_active++;
		}
	}
	
		/* This needs to be total outstanding requests !!! */
	cur_req = 0;
	for (loop_class = JobList->class_list;
		 loop_class != NULL; 
		 loop_class = loop_class->next ) {
		cur_req += loop_class->max_needed - loop_class->current;
	}
	
	dprintf ( D_FULLDEBUG, "%d + %d + %d = %d ?<? %d\n", num_active, num_req, 
			  cur_req, num_active+num_req+cur_req, max_hosts );

	if (num_active + num_req + cur_req < max_hosts) {
		classdef->max_needed += num_req;
		StoreClassDef(classdef);
		reschedule();
		dprintf( D_ALWAYS, 
				 "cur = %d, desired = %d, active = %d, max = %d\n",
				 classdef->current, classdef->max_needed, 
				 num_active, max_hosts );
	} else {
		num_req = 0;
	}
	pvm_initsend(PvmDataDefault);
	
#ifdef PVM34
		// pvm_setmwid is replace by pvm_setminfo in pvm3.4
	pvm_getminfo(pvm_getsbuf(), &info);
	info.wid = req_id;
	pvm_setminfo(pvm_getsbuf(), &info);
#else
	pvm_setmwid(pvm_getsbuf(), req_id);
#endif
	pvm_pkint(&num_req, 1, 1);
	pvm_pkint(&num_req, 1, 1);
	for (i = 0; i < num_req; i++) {
		int		zero = 0;
		pvm_pkint(&zero, 1, 1);   // status vector:0!
		pvm_pkstr("blah");        // not checked
		pvm_pkstr("blah");
		pvm_pkint(&zero, 1, 1);   // fake speed (not checked)
		pvm_pkint(&zero, 1, 1);   // fake dsig (not checked)
	}
	pvm_send(src, SM_ADDHOST);
}

void
handle_sm_delhost( int src, int req_id ) {

/* See lpvmgen.c, function pvm_delhosts */

	int     i;
	int		del_req;
	int		cur_count;
	char	host_name[255 + PADLENGTH];
	Host	*this_host;
	
	pvm_upkint(&i, 1, 0);
	pvm_initsend(PvmDataDefault);

#ifdef PVM34
	struct pvmminfo info;
	pvm_getminfo(pvm_getsbuf(), &info);
	info.wid = req_id;
	pvm_setminfo(pvm_getsbuf(), &info);
#else
	pvm_setmwid(pvm_getsbuf(), req_id);
#endif

	pvm_pkint(&i, 1, 1);
	for (cur_count = 0; cur_count < i; cur_count++) {
		prepare_buffer( host_name, 255, PADLENGTH );
		pvm_upkstr(host_name);
		check_buffer( host_name, 255, PADLENGTH );

		this_host = FindHostByName( host_name );
		if (this_host != NULL) {
			signal_startd( this_host->name, RELEASE_CLAIM );
			del_req = 1;
/*			CancelNotification( PvmHostDelete, src, this_host->pvmd_tid );
			NewNotification( PvmHostDelete, src, MY_HOSTX,
			   this_host->pvmd_tid | TIDPVMD ); */
		} else {
			dprintf( D_ALWAYS, "Can't find host %s to kill\n",
					 host_name );
			del_req = -6; // -6 = PvmNoSuchHost
		}
		pvm_pkint(&del_req, 1, 1);
	}
	pvm_send(src, SM_DELHOST);
}

void
handle_sm_execack( int src, int req_id ) {

/* See ddpro.c, bottom of function exectasks() */

	int	local_tid;
	int	count;

	pvm_upkint(&count, 1, 1);
	if (count != 1 || src != TIDPVMD) {
		dprintf( D_ALWAYS, "Unexpected EXECACK from t%x with count %d\n",
				 src, count );
	} else {
		pvm_upkint(&local_tid, 1, 1);
		if (JobList->local_proc_tid != 0) {
			dprintf( D_ALWAYS, "EXECACK: already have local tid as t%x\n",
					 JobList->local_proc_tid);
		} else {
			char	unlink_buf[ _POSIX_PATH_MAX ];
			dprintf( D_ALWAYS, "Setting local tid to t%x\n", local_tid );
			JobList->local_proc_tid = local_tid;
			sprintf( unlink_buf, "/tmp/pvmd.%d", get_user_uid() );
			unlink( unlink_buf );
		}
	}
}

void
handle_sm_config( int src, int req_id ) {

/* See lpvmgen.c, function pvm_config() */

	int	     machines_active = 0;
	int      i;
	Host     *host;
	ClassDef *classdef;

	dprintf ( D_FULLDEBUG, "In SM_CONFIG case, pvm_machine_count=%d\n",
			  JobList->pvm_machine_count );
	
	for (i = 0; i < JobList->pvm_machine_count; i++) {
		if (JobList->hosts[i].status >= 
			PVM_STATE_PENDING ) {
			machines_active++;
		}
	}
	
	dprintf ( D_FULLDEBUG, "Hosts w/ state >=PENDING = %d.\n",machines_active);
	
	pvm_initsend(PvmDataDefault);
#ifdef PVM34
	struct pvmminfo info;
	pvm_getminfo(pvm_getsbuf(), &info);
	info.wid = req_id;
	pvm_setminfo(pvm_getsbuf(), &info);
#else
	pvm_setmwid(pvm_getsbuf(), req_id);
#endif
	pvm_pkint(&machines_active, 1, 1);
	
	machines_active = 0;
	for (classdef = JobList->class_list;
		 classdef != NULL;
		 classdef = classdef->next) {
		machines_active++;
	}
	
	dprintf ( D_FULLDEBUG, "# members in class_list = %d.\n", 
			  machines_active );
	
	pvm_pkint(&machines_active, 1, 1);
	machines_active = 1;

	for (i = 0; i < JobList->pvm_machine_count; i++) {
		dprintf_Host ( D_FULLDEBUG, i, &(JobList->hosts[i]) );
		if (JobList->hosts[i].status >= PVM_STATE_PENDING ) {
			host = &(JobList->hosts[i]);
			pvm_pkhostinfo(host);
		}
	}

	pvm_send( src, SM_CONFIG );
}

void 
handle_sm_notify( int src, int req_id ) {

/* See lpvmgen.c, function pvm_notify() */

	int		kind;
	int		msg_id;
	int		count;
	int		tid;
	
		/* Won't bother checking these ints now...too much variation... */
	pvm_upkint(&kind, 1, 0);
#ifdef PVM34
	int ctx;
	pvm_upkint(&ctx, 1, 0);
#endif
	pvm_upkint(&msg_id, 1, 0);
	pvm_upkint(&count, 1, 0);
	
	dprintf ( D_FULLDEBUG, " kind %d, id %d, count %d\n", 
			  kind, msg_id, count );
	
	if ( kind == PvmHostAdd ) {
		NewNotification(kind, src, msg_id, count);
	} else {
		for (;count > 0; count--) {
			pvm_upkint(&tid, 1, 0);
			if ( kind == PvmHostDelete  ||
				 kind == PvmHostSuspend ||
				 kind == PvmHostResume ) {
				tid |= TIDPVMD;
			}
				/* First, cancel any notification requests already
				   outstanding for this task on this host/task */
			
			CancelNotification( kind, src, tid );
			
			if (msg_id != 0) {
				NewNotification(kind, src, msg_id, tid);
			}
		}
	}
}

void 
handle_sm_sthost( int src, int req_id ) {

/* See ddpro.c, function addhosts() */

	int		count;
	char 	host_name[200 + PADLENGTH];
	int		i;
	int 	pvmd_tid;
	char 	sopts[200 + PADLENGTH];
	char 	cmd[_POSIX_PATH_MAX + PADLENGTH];
	char    windows_bullshit[_POSIX_PATH_MAX + PADLENGTH];
	Host	*this_host;
	int		new_proc;
	
	pvm_upkint(&count, 1, 1);
	
#ifdef PVM34
		// pvm_getmwid is replace by pvm_getminfo in pvm3.4
	struct pvmminfo info;
	pvm_getminfo(pvm_getrbuf(), &info);
	JobList->hoststart_wid = info.wid;
#else
	JobList->hoststart_wid = pvm_getmwid(pvm_getrbuf());
#endif
	
	if (count != JobList->pvm_machines_starting) {
		dprintf( D_ALWAYS, "bad count for SM_STHOST, got %d, expected %d\n",
				 count, JobList->pvm_machines_starting);
	}
	for (i = 0; i < count; i++) {
		pvm_upkint(&pvmd_tid, 1, 1);

		prepare_buffer( sopts, 200, PADLENGTH );
		pvm_upkstr(sopts);
		check_buffer( sopts, 200, PADLENGTH );

		prepare_buffer( host_name, 200, PADLENGTH );
		pvm_upkstr(host_name);
		check_buffer( host_name, 200, PADLENGTH );

		prepare_buffer( cmd, _POSIX_PATH_MAX, PADLENGTH );
		pvm_upkstr(cmd);
		check_buffer( cmd, _POSIX_PATH_MAX, PADLENGTH );

#ifdef PVM34
			/* New in 3.4.1... */
		prepare_buffer( windows_bullshit, _POSIX_PATH_MAX, PADLENGTH );
		pvm_upkstr( windows_bullshit );
		check_buffer( windows_bullshit, _POSIX_PATH_MAX, PADLENGTH );

		dprintf ( D_FULLDEBUG, "Windows stuff: \"%s\"\n", windows_bullshit );
#endif
		
		dprintf ( D_ALWAYS, "SM_STHOST: %x \"%s\" \"%s\" \"%s\"\n", 
				  pvmd_tid, sopts, host_name, cmd );

			// If there's a -n argument in cmd, then we want to use that
			// name instead of host_name.
		char *real_name = strstr(cmd, "-n");
		if (real_name) {
			char *end = strchr(real_name, ' ');
			if (end) {
				*end = '\0';
				strcpy(host_name, real_name+2);
				*end = ' ';
			}
		}

		this_host = FindHostByName(host_name);
		if (this_host == NULL) {
			dprintf( D_ALWAYS, "Couldn't find host %s\n", host_name);
			continue;
		}
		this_host->pvmd_tid = pvmd_tid;
		strcat(cmd, " -f");		/* Don't fork() option */
		this_host->pvmd_cmd = strdup(cmd);
		new_proc = AllocProc();
		this_host->status = PVM_STATE_SEND_PVM;
		if ( multi_send_job( &(JobList->Proc), new_proc, this_host->name)<0) {
			RemoveProc( new_proc );
			RemoveHost( JobList->pvm_starting_list[i]);
		} else {
			JobList->procs[new_proc].host_num = JobList->pvm_starting_list[i];
			this_host->starter_proc = new_proc;
		}
	}

/*	
	dprintf ( D_FULLDEBUG, "Printing hosts in SM_STHOST.\n" );
	for (int j = 0; j < JobList->pvm_machine_count; j++) {
		dprintf_Host ( D_FULLDEBUG, j, &(JobList->hosts[j]) );
	}
*/
		/* Now set timers for all the hosts, so we don't
		   penalize the first ones in the list */
	for (i = 0; i < JobList->pvm_machines_starting; i++) {
		this_host = &(JobList->hosts[JobList->pvm_starting_list[i]]);
		if (this_host->status != PVM_STATE_EXITED) {
			this_host->host_timer = new_timer( HostTimeout, host_timeout );
		}
	}
}

void
handle_sm_task ( int src, int req_id ) {

/* See lpvmgen.c, function pvm_tasks() */

	int		where;
	Host	*proc_host;
	Process	*process;
	int		proc;
	int		zero = 0;
	
	pvm_upkint(&where, 1, 0);
	
	pvm_initsend(PvmDataDefault);
#ifdef PVM34
	struct pvmminfo info;
	pvm_getminfo(pvm_getsbuf(), &info);
	info.wid = req_id;
	pvm_setminfo(pvm_getsbuf(), &info);
#else
	pvm_setmwid(pvm_getsbuf(), req_id);
#endif
	req_id = 0;
	pvm_pkint(&req_id, 1, 1);
	for (proc = 0; proc < JobList->max_processes; proc++) {
		process = &(JobList->procs[proc]);
		if (process->host_num != -1) {
			proc_host = &(JobList->hosts[process->host_num]);
			if (where == 0 || where == proc_host->pvmd_tid) {
				if (proc_host->starter_proc != proc && 
						/* In case the host is deleted... */
					proc_host->starter_proc != -1) {

					pvm_pkint(&process->pvm_tid, 1, 1);
					pvm_pkint(&process->pvm_ptid, 1, 1);
					pvm_pkint(&proc_host->pvmd_tid, 1, 1);
					pvm_pkint(&zero, 1, 1);
					dprintf ( D_FULLDEBUG, "SM_TASK: packing ");
					if (process->a_out) {
						dprintf( D_FULLDEBUG | D_NOHEADER, "\"%s\"\n", 
								 process->a_out );
						pvm_pkstr(process->a_out);
					} else {
						dprintf ( D_FULLDEBUG | D_NOHEADER, "\"\"\n" );
						pvm_pkstr("");
					}
					pvm_pkint(&zero, 1, 1);	/* No Unix pid */
				}
			}
		}
	}
	pvm_send(src, SM_TASK);
}

void
handle_sm_sched ( int src, int req_id ) {

/* See lpvmgen.c, function pvm_reg_rm() */
			/* Starter wants to become a PVM scheduler */

	pvm_initsend(PvmDataFoo);
	pvm_pkint(&src, 1, 1);
	pvm_send(pvm_tidtohost(src) | TIDPVMD, SM_HANDOFF);
	
		/* Beats me why pvm_packf is used here... it works, though. */
	pvm_packf("%+ %d %d %s %d %s", PvmDataDefault, pvm_mytid(), 5, 
			  "ADDR", 5, "Arch");
	
#ifdef PVM34
	struct pvmminfo info;
	pvm_getminfo(pvm_getsbuf(), &info);
	info.wid = req_id;
	pvm_setminfo(pvm_getsbuf(), &info);
#else
	pvm_setmwid(pvm_getsbuf(), req_id);
#endif
	pvm_send(src, SM_SCHED);
}

void
handle_sm_hostx ( int src, int req_id ) {

/* See ddpro.c, function hostfailentry() */

	int		pvmd_tid;
	
	pvm_upkint(&pvmd_tid, 1, 1);
	dprintf( D_ALWAYS, "PVM says we lost host t%x\n", pvmd_tid );
}

void
handle_sm_taskx ( int src, int req_id ) {

	int		task_tid;

	pvm_upkint(&task_tid, 1, 1);
	dprintf( D_ALWAYS, "PVM says we lost task t%x\n", task_tid );

	if (task_tid == JobList->local_proc_tid) {
		dprintf( D_ALWAYS, "The local task exited.\n");
	}
}

/* The following are sanity-checking routines to make sure that 
   the pvmd returns sane values. */

void prepare_buffer( char *buf, int goodlen, int padlen ) {
/* This function sets up a char * buffer for use with pvm_upkstr() 
   which will have some padding at the end filled with a certain 
   value.  The valid data is in 0-goodlen, and the padding is
   from index goodlen to goodlen+padlen.  Thus, the overall buffer
   size is goodlen+padlen.  */
	memset( buf, 0, goodlen );
	memset( &buf[goodlen], 0x55, padlen );  /* 01010101 */
}

void check_buffer( char *buf, int goodlen, int padlen ) {
/* Take a buffer prepared with prepare_buffer, and make sure that
   the padding at the end is still there.  If it isn't, we'll 
   do an ASSERT().
*/
	for ( int i=goodlen ; i<padlen ; i++ ) {
		ASSERT( buf[i] == 0x55 );
	}
}
