/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/


#include "condor_common.h"
#include "sched.h"
#include "condor_debug.h"
#include "fileno.h"
#include "files.h"
#include "proc.h"
#include "exit.h"
#include "condor_io.h"
#include "shadow.h"
#include "pvm3.h"
#include "request.h"
#include "prototypes.h"

/* All comments in the below code courtesy Mike Yoder.   */

static Request *requests = 0;
static int request_counter = 1;

/** This call effectively sets up a Request structure with the given
    information and pushes it onto the list pointed at by requests. 
*/
Request *
NewRequest(int req_type, int for_tid, int req_id, int resp_msg)
{
	Request *new_req;

	new_req = (Request *) MALLOC(sizeof(Request));
	new_req->req_type = req_type;
	new_req->for_tid = for_tid;
	new_req->on_tid = 0;
	new_req->req_id = req_id;
	new_req->resp_msg = resp_msg;
	new_req->local_req_id = request_counter++;
	new_req->next = requests;
	new_req->extra_data = 0;
	requests = new_req;
	return new_req;
}

/** Seaches the request list for del_req and deletes it. */
int
DeleteRequest(Request *del_req)
{
	Request *req;
	Request *back_req;

	if (del_req == 0) {
		return 0;
	}

	back_req = 0;
	for (req = requests; req != 0; req = req->next) {
		if (req == del_req) {
			break;
		}
		back_req = req;
	}
	if (req) {
		if (back_req == 0) {
			requests = req->next;
		} else {
			back_req->next = req->next;
		}
		if (req->extra_data) {
			FREE(req->extra_data);
		}
		FREE(req);
	}
	return 0;
}

/** Returns the request whose local_req_id matches that given.
    Null is returned on failure. */
Request *
FindRequestByReqId(int local_req_id)
{
	Request *req;

	for (req = requests; req != NULL; req = req->next) {
		if (req->local_req_id == local_req_id) {
			return req;
		}
	}
	return NULL;
}

/** Starting at the beginning of the request list, find & return 
    the request matching these numbers ( or -1 ) */
Request *
FindRequest(int for_tid, int on_tid, int type, int id)
{
	Request	dummy_req;

	dummy_req.next = requests;

	return FindNextRequest(&dummy_req, for_tid, on_tid, type, id);
}

/** Starting at req, search for & return the matching request.
    Note that the numbers can match or be -1 in the request...*/
Request *
FindNextRequest(Request *req, int for_tid, int on_tid, int type, int id)
{
    if ( req == NULL ) {
        return NULL;
    }

	for (req = req->next; req != NULL; req = req->next) {
		if ((for_tid == -1 || for_tid == req->for_tid) &&
			(on_tid == -1  || on_tid == req->on_tid  ) &&
			(type == -1    || type == req->req_type  ) &&
			(id == -1      || id == req->req_id      )) {
			return req;
		}
	}
	return NULL;
}

/** Sends the message buffer associated with request->resp_msg 
    to request->for_tid.  (I think). */
int
SendResponse(Request *request)
{
	if (request->resp_msg != 0) {
		carmi_store_req_id(request->req_id);
		pvm_send(request->for_tid, request->resp_msg);
	}
	return 0;
}

/** A nice, handy shortcut to dprintf a Request. */
void
dprintf_request(int dprintf_flag, Request *req)
{
	dprintf(dprintf_flag, 
			"type = %d, for t%x, on t%x, req_id = %d, resp = %d, ptr=%0x\n",
			req->req_type, req->for_tid, req->on_tid, req->req_id,
			req->resp_msg, req);
}

/** Prints the entire list of requests */
void
DumpRequestList(int dprintf_flag)
{
	Request		*req;

	for (req = FindRequest(-1, -1, -1, -1); req; 
		 req = FindNextRequest(req, -1, -1, -1, -1)) {
		dprintf_request(dprintf_flag, req);
	}
}
