/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 
#include "condor_common.h"
#include "condor_attributes.h"
#include "condor_version.h"
#include "my_hostname.h"
#include "sched.h"
#include "condor_debug.h"
#include "fileno.h"
#include "files.h"
#include "proc.h"
#include "exit.h"
#include "condor_io.h"
#include "condor_uid.h"
#include "shadow.h"
#include "condor_config.h"
#include "internet.h"
#include "sig_install.h"
#include "condor_environ.h"
#include "condor_distribution.h"

int ShadowBDate = -1;
int LastRestartTime = -1;
int LastCkptTime = -1;
int NumCkpts = 0;
int NumRestarts = 0;
int CommittedTime = -1;
char *ExecutingHost = NULL;

extern "C" {
	int update_job_status( struct rusage *localp, struct rusage *remotep );
	int update_job_rusage( struct rusage *localp, struct rusage *remotep );
	void MvTmpCkpt();
	int get_local_rusage( struct rusage *bsd_rusage );
	int     multi_CleanupAll(int,int,char*);
	int multi_shadow( int argc, char    *argv[] );
	int		whoami(FILE *fp);
	int log_execute(char *host);
}


extern int	LockFd;
extern char	MsgBuf[];

#include "joblist.h"
extern struct tagJobList	*JobList;


int		MyPid;
int		LogPipe;
int		ImageSize;
int		PipeToParent;
PROC	*Proc = NULL;
char	*Spool;

char	QueueName[MAXPATHLEN];

int MaxDiscardedRunTime = 3600;

struct rusage LocalUsage;
struct rusage RemoteUsage;

char	CkptName[ MAXPATHLEN ];		/* The name of the ckpt file */
char	ICkptName[ MAXPATHLEN ];	/* The name of the initial ckpt file */
char	RCkptName[ MAXPATHLEN ];	/* The name of the returned ckpt file*/
char	TmpCkptName[ MAXPATHLEN ];	/* The name of the temp ckpt file */

int JobStatus;
struct rusage JobRusage;
int ExitReason = JOB_EXCEPTION;            /* Schedd counts on knowing exit reason */
int JobExitStatus = 0;
int MainSymbolExists = TRUE;

char * My_Filesystem_Domain = NULL;
char * My_UID_Domain = NULL;
int  UseAFS;
int  UseNFS;
int  UseCkptServer;
int	 StarterChoosesCkptServer;
int  CompressPeriodicCkpt=0;
int  CompressVacateCkpt=0;
int  PeriodicSync=0;
int  SlowCkptSpeed=0;
char *CkptServerHost = NULL;
char *LastCkptServer = NULL;

int
usage()
{
	dprintf( D_ALWAYS, "Usage: shadow host cluster proc\n" );
	exit( JOB_SHADOW_USAGE );
}


void
printClassAd( void )
{
	printf( "%s = False\n", ATTR_IS_DAEMON_CORE );
	printf( "%s = True\n", ATTR_HAS_PVM );
	printf( "%s = \"%s\"\n", ATTR_VERSION, CondorVersion() );
}


/*ARGSUSED*/
main(int argc,char *argv[] )
{
	int p[2];
	int	log[2];
	char	*tmp;
	int		reserved_swap, free_swap;
	char    *my_fs_domain, *my_uid_domain, *use_afs, *use_nfs;
	char	*use_ckpt_server, *ckpt_server_host;

	myDistro->Init( argc, argv );
	if( argc == 2 && strincmp(argv[1], "-cl", 3) == MATCH ) {
		printClassAd();
		exit( 0 );
	}

	/* Start up with condor.condor privileges. */
	set_condor_priv();

	_EXCEPT_Cleanup = multi_CleanupAll;
	MyPid = getpid();
	
	config();

#if !defined(WIN32)
	install_sig_handler(SIGPIPE, SIG_IGN );
#endif

	dprintf_config( "SHADOW", SHADOW_LOG );
	DebugId = whoami;
	close( 1 );

	ShadowBDate = LastRestartTime = time(0);


	dprintf(D_ALWAYS, "********** Multi_Shadow starting up **********\n" );


	ExecutingHost = strdup(my_hostname());
	log_execute(ExecutingHost);

	if( (tmp=param("RESERVED_SWAP")) == NULL ) {
		reserved_swap = 5 * 1024;		/* 5 megabytes */
	} else {
	        reserved_swap = atoi( tmp ) * 1024;     /* Value specified in megabytes */
	}
	free_swap = reserved_swap + 1024;
	dprintf( D_FULLDEBUG, "*** Reserved Swap = %d\n", reserved_swap );
	dprintf( D_FULLDEBUG, "*** Free Swap = %d\n", free_swap );
	if( free_swap < reserved_swap ) {
		dprintf( D_ALWAYS, "Not enough reserved swap space\n" );
		exit( JOB_NO_MEM );
	}


	dprintf(D_ALWAYS, "uid=%d, euid=%d, gid=%d, egid=%d\n",
			getuid(), geteuid(), getgid(), getegid());

	Spool = param( "SPOOL" );
	if( Spool == NULL ) {
		EXCEPT( "Spool directory not specified in config file" );
	}
	(void)sprintf( QueueName, "%s/job_queue", Spool );

	My_Filesystem_Domain = param( "FILESYSTEM_DOMAIN" );
	dprintf( D_ALWAYS, "My_Filesystem_Domain = \"%s\"\n", 
			My_Filesystem_Domain );
	
	My_UID_Domain = param( "UID_DOMAIN" );
	dprintf( D_ALWAYS, "My_UID_Domain = \"%s\"\n", My_UID_Domain );
	
	use_afs = param( "USE_AFS" );
	if( use_afs && (use_afs[0] == 'T' || use_afs[0] == 't') ) {
		UseAFS = TRUE;
	} else {
		UseAFS = FALSE;
	}
	
	use_nfs = param( "USE_NFS" );
	if( use_nfs && (use_nfs[0] == 'T' || use_nfs[0] == 't') ) {
		UseNFS = TRUE;
	} else {
		UseNFS = FALSE;
	}

	use_ckpt_server = param( "USE_CKPT_SERVER" );
	if (CkptServerHost) free(CkptServerHost);
	CkptServerHost = param( "CKPT_SERVER_HOST" );
	if( !CkptServerHost ||
		(use_ckpt_server && (use_ckpt_server[0] == 'F' ||
							 use_ckpt_server[0] == 'f')) ) {
			// We don't have a ckpt server defined, or the user
			// explicitly configures USE_CKPT_SERVER = False.
		UseCkptServer = FALSE;
	} else {
			// We've got a checkpoint server, so let's use it.
		UseCkptServer = TRUE;
	}
	if (use_ckpt_server) free(use_ckpt_server);
	tmp = param( "STARTER_CHOOSES_CKPT_SERVER" );
	if (tmp && (tmp[0] == 'T' || tmp[0] == 't')) {
		StarterChoosesCkptServer = TRUE;
	} else {
		StarterChoosesCkptServer = FALSE;
	}
	if (tmp) free(tmp);

	tmp = param( "MAX_DISCARDED_RUN_TIME" );
	if (tmp) {
		MaxDiscardedRunTime = atoi(tmp);
		free(tmp);
	} else {
		MaxDiscardedRunTime = 3600;
	}

	struct rlimit rl;
	getrlimit(RLIMIT_NOFILE,&rl);
	rl.rlim_cur = rl.rlim_max;
	setrlimit(RLIMIT_NOFILE,&rl);

	dprintf(D_FULLDEBUG,"Max file descriptors = %d\n",getdtablesize());

	JobList = NULL;
	multi_shadow( argc, argv );
    if (My_UID_Domain) {
        free( My_UID_Domain );
    }
    if (My_Filesystem_Domain) {
        free( My_Filesystem_Domain );
    }
	dprintf(D_ALWAYS, "********** Shadow Parent Exiting(%d) **********\n",
		ExitReason);
	exit( ExitReason );
}


/* Just make this null function so we can share pseudo_ops.c with the
   V5 shadow.  This probably isn't necessary anyway since we end up
   updating the proc structure pretty regularly anyway.
*/

int
update_job_status( struct rusage *localp, struct rusage *remotep )
{
	return TRUE;
}

int
update_job_rusage( struct rusage *localp, struct rusage *remotep )
{
	return TRUE;
}
