/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "condor_debug.h"

typedef struct tagTimer {
  unsigned        when;
  int             id;
  int             (*handler)(int);
  struct tagTimer *next;
} Timer;

Timer      *timer_list = NULL;
static int timer_ids   = 0;

int dump_time_list();

int
new_timer(unsigned when, int (*handler)(int) )
{
  Timer    *new_timer=0;
  unsigned time_remaining;
  unsigned time_count;
  Timer    *timer_ptr=0;
  Timer    *trail_ptr=0;
  
  dprintf( D_ALWAYS, "in new_timer()\n" );
  time_remaining = alarm(0);

/*  if (time_remaining == 0) {
    time_remaining = 1;
  } */

  new_timer = (Timer *) MALLOC( sizeof(Timer) );
  if ( new_timer == NULL ) {
    dprintf( D_ALWAYS, "Unable to allocate new timer\n" );
    alarm(time_remaining);
    return -1;
  }
  
  new_timer->id = timer_ids++;
  new_timer->handler = handler;
  
  if ( timer_list == NULL ) {
    timer_list = new_timer;
    new_timer->next = NULL;
    new_timer->when = when;
    time_remaining = when;
  } 
  else {
    timer_list->when = time_remaining;
    if ( when < timer_list->when ) {
      new_timer->next = timer_list->next;
      timer_list->when -= when;
      new_timer->when = when;
      new_timer->next = timer_list;
      time_remaining = when;
      timer_list = new_timer;
    } 
    else {
      time_count = 0;
      for (timer_ptr = timer_list; timer_ptr != NULL; 
           timer_ptr = timer_ptr->next ) {
        time_count += timer_ptr->when;
        if (time_count > when) {
          break;
        }
        trail_ptr = timer_ptr;
      }
      if (timer_ptr == NULL) {
        when -= time_count;
        new_timer->next = NULL;
        trail_ptr->next = new_timer;
        new_timer->when = when;
      } 
      else {
        new_timer->next = trail_ptr->next;
        trail_ptr->next = new_timer;
        new_timer->when = timer_ptr->when - (time_count - when);
        timer_ptr->when = (time_count - when);
      }
    }
  }

  dump_time_list();
  alarm(time_remaining);
  return  new_timer->id;
}

int
cancel_timer( int id )
{
  Timer    *timer_ptr=0;
  Timer    *trail_ptr=0;
  unsigned time_remaining;

  time_remaining = alarm(0);

/*  if (time_remaining == 0) {
    time_remaining = 1;
  } */

  dprintf( D_ALWAYS, "In cancel_timer()\n");
  if (timer_list == NULL) {
    dprintf( D_ALWAYS, "Removing Timer from empty list!\n");
    alarm(time_remaining);
    return -1;
  }

  if (timer_list->id == id) {
    if (timer_list->next == NULL) {
      FREE( timer_list );
      timer_list = NULL;
    } 
    else {
      timer_ptr = timer_list;
      timer_list = timer_list->next;
      timer_list->when += time_remaining;
      alarm(timer_list->when);
      FREE( timer_ptr );
    }
    dump_time_list();
    return 0;
  }
  for (timer_ptr = timer_list; timer_ptr != NULL && timer_ptr->id != id; 
       timer_ptr = timer_ptr->next) {
    trail_ptr = timer_ptr;
  }

  if ( timer_ptr == NULL ) {
    dprintf( D_ALWAYS, "Timer %d not found\n" );
    alarm(time_remaining);
    return -1;
  }

  if (timer_ptr->next != NULL) {
    timer_ptr->next->when += timer_ptr->when;
  }

  trail_ptr->next = timer_ptr->next;
  FREE( timer_ptr );
  dump_time_list();
  alarm( time_remaining );
  return 0;
}

#if 0
cancel_all_timers()
{
  Timer  *timer_ptr;
  
  alarm(0);
  while( timer_list != NULL ) {
    timer_ptr = timer_list;
    timer_list = timer_list->next;
    FREE( timer_ptr );
  }
}
#endif

#if defined(VOID_SIGNAL_RETURN)
void
#else
int
#endif
sigalrm_handler(int dummy)
{
  Timer  *timer_ptr;
  int    current_id;
  
  dprintf( D_ALWAYS, "In Alarm Handler!\n");
  if (timer_list != NULL) {
    timer_list->when = 0;
  } 
  else {
    dprintf( D_ALWAYS, "Empty timer list, nothing to do\n" );
  }
  
  while( timer_list != NULL && timer_list->when == 0 ) {
    dump_time_list();
    current_id = timer_list->id;
    timer_list->handler( timer_list->id );
    /* Watch out for cancel_timer() called with only one item in list 
       which makes timer_list go to NULL */
    if (timer_list != NULL) {
      /* Here, watch out for a cancel_timer() of the timer which was
         currently being handled.  When this happens, timer_list has
         already been pushed forward, and without this check, we'd
         delete the new head of the timer list */
      if (timer_list->id == current_id) {
        timer_ptr = timer_list;
        timer_list = timer_list->next;
        FREE( timer_ptr );
      }
    }
  }
  dump_time_list();
  if (timer_list != NULL) {
    alarm(timer_list->when);
  }
  dprintf( D_ALWAYS, "Alarm Handler Complete\n");
}


int
dump_time_list()
{
  Timer  *timer_ptr;
  
  dprintf( D_ALWAYS, "Timer List\n" );
  dprintf( D_ALWAYS, "^^^^^ ^^^^\n" );
  for (timer_ptr = timer_list; timer_ptr != NULL; 
       timer_ptr = timer_ptr->next ) {
    dprintf( D_ALWAYS, "id = %d, when = %d\n", timer_ptr->id, 
             timer_ptr->when );
  }
  return 0;
}
