/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 

#define _POSIX_SOURCE

#include "condor_common.h"
#include "condor_debug.h"
#include <sys/param.h>

#include "pvm3.h"

#ifdef PVM34
#include "pvmproto.h"
#else
#include "pvmsdpro.h"
#endif

#include "carmipro.h"


extern int shadow_tid;

/*
** Transfer a remote file to the local machine.
*/
int
carmi_get_file( const char *remote, const char *local, int mode )
{
	int		remote_fd, local_fd;
	size_t	len;
	double	start, finish;
	int		num_to_serve;
	char	src_url[ MAXPATHLEN ];
	int		fsize;
	int		sock_fd;
	int		to_serve_fds[10];
	int		i;

	dprintf( D_ALWAYS, "Entering carmi_get_file( %s, %s, 0%o )\n", remote,
			local,mode);

	sock_fd = create_socket_url(src_url, 3);
	if (sock_fd < 0) {
		dprintf(D_ALWAYS, 
				"Failed to make socket for data movement, falling back\n");
		return get_file( remote, local, mode );
	}

	start = get_time();

	pvm_initsend(PvmDataDefault);
	pvm_pkstr(remote);
	pvm_pkstr(src_url);		/* This is really our service URL */
	pvm_send(shadow_tid, CARMI_REQ_URL_FOR_FILE);
	pvm_recv(shadow_tid, CARMI_REQ_URL_FOR_FILE);

	pvm_upkint(&num_to_serve, 1, 1);
	pvm_upkint(&fsize, 1, 1);
	pvm_upkstr(src_url);

		/* open the remote file */
	remote_fd = open_url( src_url, O_RDONLY, &fsize );

	dprintf(D_ALWAYS, "src_url = %s, fsize = %d\n", src_url, fsize);

	if( remote_fd < 0 ) {
		dprintf( D_ALWAYS, "open_url(%s,O_RDONLY,%d) failed\n",
				src_url, fsize );
		close(sock_fd);
		return get_file( remote, local, mode );
	}

		/* open the local file */
	if( (local_fd=open(local,O_WRONLY|O_CREAT|O_TRUNC,mode)) < 0 ) {
		dprintf( D_ALWAYS, "Failed to open \"%s\" locally, errno = %d\n",
				local, errno);
		close(sock_fd);
		close(remote_fd);
		return get_file( remote, local, mode );
	}

	dprintf(D_ALWAYS, "file xfer: src = %d, dest = %d, size = %d\n",
			remote_fd, local_fd, fsize);
		/* transfer the data */

	wait_for_connections(sock_fd, num_to_serve, to_serve_fds);
	to_serve_fds[num_to_serve] = local_fd;
	num_to_serve++;
	close(sock_fd);

	len = multi_stream_file_xfer(remote_fd, num_to_serve, to_serve_fds, fsize);

	finish = get_time();

		/* clean up */
	for (i = 0; i < num_to_serve; i++) {
		(void)close( to_serve_fds[i] );
	}
	(void)close( remote_fd );

	if (finish != start) {
		dprintf( D_ALWAYS,"Get_file() transferred %d bytes, %d bytes/second\n",
				len, (int)(len/(finish-start)) );
	}

	return len;
}

#ifdef WANT_NETMAN
void file_stream_progress_report(int bytes_moved) {}
#endif
