/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 

#define _POSIX_SOURCE

#include "condor_common.h"
#include "condor_debug.h"
#include "condor_constants.h"
#include <signal.h>
#include "condor_fix_stdio.h"
#include "starter.h"
#include <time.h>

extern int errno;
pid_t	StarterPid;

/* Prototypes for random functions */
#if defined (HPUX)
void srand( unsigned int );
int rand ( void );
#define random() rand()
#define srandom(x) srand(x)
#elif defined (Solaris)
/* no prototypes */
#else
void srandom( int );
int random( void );
#endif

typedef struct {
	EVENT	ev;
	char	*str;
	int		weight;
} MENU_ITEM;

MENU_ITEM	Menu[] = {
	{ SUSPEND,		 	"SUSPEND",			30	},
	{ CONTINUE,		 	"CONTINUE",			50	},
	{ DIE,		 		"DIE",				1	},
	{ VACATE,		 	"VACATE",			5	},
	{ CKPT_and_VACATE,	"CKPT_and_VACATE",	5	},
	{ -1,				NULL,				-1 	}
};
int		MenuSize;
int		MaxWeight;
int		Seed;

EVENT	get_command();
void	usage( char * );
void	send_sig( int sig );
pid_t	get_starter_pid( const char * );
void	check_starter();
int		contains( const char *, const char * );
pid_t	get_pid( const char * );
void	random_sleep( int max );
int		weighted_toss( int weight );


void
usage( char *my_name )
{
	fprintf( stderr, "Usage: %s [starter_pid]\n", my_name );
	exit( 1 );
}



int
main( int argc, char *argv[] )
{
	MENU_ITEM	*ptr;
	EVENT		command;
		
	if( (argc != 1) && (argc != 2) ) {
		usage( argv[0] );
	}

	if( argc == 2 ) {	// Seed supplied on the command line
		Seed = atoi( argv[1] );
	} else {			// Pick a seed 0 - 999
		Seed = ( (int)time(0) % 1000 );
	}
	srandom( Seed );

	StarterPid = get_starter_pid( argv[0] );

		// Calculate number of menu items
	MenuSize = 0;
	MaxWeight = 0;
	for( ptr=Menu; ptr->str; ptr++ ) {
		MenuSize++;
		if( ptr->weight > MaxWeight ) {
			MaxWeight = ptr->weight;
		}
	};

	while( StarterPid ) {
		check_starter();
		command = get_command();
		switch( command ) {
			case SUSPEND:
				send_sig( SIGUSR1 );
				break;
			case CONTINUE:
				send_sig( SIGCONT );
				break;
			case VACATE:
				send_sig( SIGTSTP );
				break;
			case ALARM:
				send_sig( SIGALRM );
				break;
			case DIE:
				send_sig( SIGINT );
				break;
			case CKPT_and_VACATE:
				send_sig( SIGUSR2 );
				break;
			case NO_EVENT:
				exit( 0 );
			default:
				printf( "Unknown command (%d)\n", command );
		}
	}
	return 0;
}

EVENT
get_command()
{
	MENU_ITEM	*ptr;
	char		ch;
	char		buf[1024];
	long		raw;
	int			index;
	long		toss;
	int			i;
	int			accept;

	random_sleep( 10 );

	for(;;) {
		raw = random();
		index = (int)(raw % MenuSize);
		// printf( "index = %d, weight = %d\n", index, Menu[index].weight );
		if( weighted_toss(Menu[index].weight) ) {
			// printf( "Accepted\n" );
			printf( "Sending %s\n", Menu[index].str );
			return Menu[index].ev;
		} else {
			// printf( "Rejected\n" );
		}
	}

}

int
weighted_toss( int weight )
{
	int		i;
	int		toss;

	for( i=0; i < weight; i++ ) {
		toss = random() % MaxWeight;
		// printf( "toss %d = %d\n", i, toss );
		if( toss == 0 ) {
			return TRUE;
		}
	}
	return FALSE;
}

void
random_sleep( int max )
{
	int		i;

	for( i = 0; i < max; i++ ) {
		if( random() % max == 0 ) {
			printf( "\n" );
			return;
		}
		sleep( 1 );
		check_starter();
		printf( "." );
		fflush( stdout );
	}
	printf( "\n" );
}

void
send_sig( int sig )
{
	if( sig != SIGCONT ) {
		if( kill(StarterPid,SIGCONT) < 0 ) {
			if( errno == ESRCH ) {
				StarterPid = 0;
				return;
			}
			fprintf( stderr, "Can't send SIGCONT to process %d", StarterPid );
			perror( " " );
			exit( 1 );
		}
	}
	// printf( "Sent signal SIGCONT to process %d\n", StarterPid );

	if( kill(StarterPid,sig) < 0 ) {
		if( errno == ESRCH ) {
			StarterPid = 0;
			return;
		}
		fprintf( stderr, "Can't send signal %d to process %d", sig, StarterPid );
		perror( " " );
		exit( 1 );
	}
	// printf( "Sent signal %d to process %d\n", sig, StarterPid );
}

const char	* const PROG_NAME = "condor_starter";

pid_t
get_starter_pid( const char *my_name)
{
	char	buf[1024];
	FILE	*fp;
	char	cmd[ 1024];
	pid_t	answer = 0;

#if defined(AIX32)	// really SysV
	sprintf( cmd, "ps -ef | egrep %s", PROG_NAME );
#else
	sprintf( cmd, "ps -ax | egrep %s", PROG_NAME );
#endif

	if( (fp=popen(cmd,"r")) == NULL ) {
		fprintf( stderr, "Can't execute \"%s\"\n", cmd );
		exit( 1 );
	}
	while( fgets(buf,sizeof(buf),fp) ) {
		if( contains(buf,"egrep") ) {
			continue;
		}
		if( contains(buf,my_name) ) {
			continue;
		}
		if( contains(buf,PROG_NAME) ) {
			if( answer ) {
				fprintf( stderr,
					"Multiple processes named \"%s\" exist\n", PROG_NAME
				);
				exit( 1 );
			}
			answer = get_pid(buf);
		}
	}
	(void)pclose( fp );
	if( answer ) {
		printf( "Process \"%s\" PID is %d\n", PROG_NAME, answer );
		return answer;
	} else {
		printf( "Can't find \"%s\"\n", PROG_NAME );
		return 0;
	}
}

void
check_starter()
{
	if( kill(StarterPid,0) < 0 ) {
		if( errno == ESRCH ) {
			printf( "Starter %d exited\n", StarterPid );
			printf( "Seed was %d\n", Seed );
			exit( 0 );
		} else {
			printf( "Can't send signal to starter %d\n", StarterPid );
			exit( 1 );
		}
	}
}

/*
  Return TRUE if the string contains the target, and false otherwise.
*/
int
contains( const char *str, const char *targ )
{
	int		len = strlen( targ );
	const char	*ptr;

	for( ptr=str; *ptr; ptr++ ) {
		if( strncmp(ptr,targ,len) == 0 ) {
			return TRUE;
		}
	}
	return FALSE;
}

pid_t
get_pid( const char *str )
{
	const char	*ptr;

	for( ptr=str; *ptr; ptr++ ) {
		if( isdigit(*ptr) ) {
			return atoi(ptr);
		}
	}
	return 0;
}

