/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 
#include "condor_common.h"
#include "condor_config.h"
#include "condor_debug.h"
#include "debug.h"
#include "condor_constants.h"
#include "condor_string.h"
#include "condor_uid.h"
#include "name_tab.h"
#include "proto.h"
#include "renice_self.h"

#include "fileno.h"
#include "list.h"
#include "starter.h"
#include "alarm.h"
#include "selector.h"

#include "starter_common.h"

#include "condor_sys.h"
#include "pvm_user_proc.h"
#include "pvm3.h"

#ifdef PVM34
#include "pvmproto.h"
#else
#include "pvmsdpro.h"
#endif 
/*
#if !defined(PVM33)
#include "sdpro.h"
#else
#include "pvmsdpro.h"
#endif
*/
#include "carmipro.h"

extern NameTable SigNames;
extern int Running_PVMD;
extern NameTable PvmMsgNames;

extern List<UserProc>	UProcList;		// List of user processes

int		shadow_tid;

extern void do_unlink( const char * );

extern char VirtualMachineName[];

extern "C" {
	void unblock_signal( int );
	int pvmputenv(char*);
}

int pvmdPid = -1;  /* warning: hack alert! */

void PVMdProc::execute()
{
	int		argc;
	char	*argv[ 2048 ];
	char	**argp;
	char	**envp;
	sigset_t	sigmask;
	long	arg_max;
	char	*tmp;
	char	*tmp2;
	char	a_out_name[ _POSIX_PATH_MAX ];
	int		user_syscall_fd;
	int		pipes[2];
	int		readlen;
	int		mytid;
	uid_t	old_uid;
	char	unlink_buf[ _POSIX_PATH_MAX ];
	extern char *Execute;

	dprintf ( D_ALWAYS, "In PVMdProc::execute().\n" );
	
		// We will use mkargv() which modifies its arguments in place
		// so we not use the original copy of the arguments
	if( (arg_max=sysconf(_SC_ARG_MAX)) == -1 ) {
		arg_max = _POSIX_ARG_MAX;
	}
	dprintf(D_ALWAYS, "arg_max = %d\n", arg_max);
	tmp = new char [arg_max];
	strncpy( tmp, args, arg_max );

	sprintf( a_out_name, "condor_exec.%d.%d", cluster, proc );

	argv[0] = a_out_name;
	mkargv( &argc, &argv[1], tmp );

	tmp2 = new char [ _POSIX_PATH_MAX ];
	sprintf(tmp2, "PVM_ROOT=%s/%s", Execute, local_dir);
	env_obj.add_string(tmp2);
	delete [] tmp2;

	env_obj.add_string(VirtualMachineName);

	// set up environment vector
	envp = env_obj.get_vector();

		// handle USER_JOB_WRAPPER param
	support_job_wrapper(a_out_name,&argc,argv);

		// print out arguments to execve
	dprintf( D_ALWAYS, "Calling execve( \"%s\"", a_out_name );
	for( argp = argv; *argp; argp++ ) {							// argv
		dprintf( D_ALWAYS | D_NOHEADER, ", \"%s\"", *argp );
	}
	dprintf( D_ALWAYS | D_NOHEADER, ", 0" );
	for( argp = envp; *argp; argp++ ) {							// envp
		dprintf( D_ALWAYS | D_NOHEADER, ", \"%s\"", *argp );
	}
	dprintf( D_ALWAYS | D_NOHEADER, ", 0 )\n" );

	priv_state priv;
	priv = set_root_priv();
        // if the pvmd finds these files, it'll have a fit.
	sprintf(unlink_buf, "/tmp/pvml.%d", get_user_uid());
	do_unlink(unlink_buf);
	sprintf(unlink_buf, "/tmp/pvmd.%d", get_user_uid());
	do_unlink(unlink_buf);
	set_priv(priv);
 
	pipe(pipes);
	dprintf( D_ALWAYS, "pipes: %d & %d\n", pipes[0], pipes[1] );

	if( (pid = fork()) < 0 ) {
		EXCEPT( "fork" );
	}

	if( pid == 0 ) {	// the child

		renice_self( "JOB_RENICE_INCREMENT" );

		if( chdir(local_dir) < 0 ) {
			EXCEPT( "chdir(%s)", local_dir );
		}

			// child process should have only it's submitting uid, and cannot
			// switch back to root or some other uid.
		set_user_priv_final();

		dup2(pipes[1], 1);
		if (pipes[1] != 1) {
			close(pipes[1]);
		}
		close(pipes[0]);

		close(RSC_SOCK);
		close(CLIENT_LOG);

		// Gotta free the signals

		unblock_signal( SIGCHLD );
		unblock_signal( SIGTERM );
		unblock_signal( SIGCONT );
		unblock_signal( SIGTSTP );

			// Everything's ready, start it up...
		errno = 0;

		execve( a_out_name, argv, envp );

			// A successful call to execve() never returns, so it is an
			// error if we get here.  A number of errors are possible
			// but the most likely is that there is insufficient swap
			// space to start the new process.  We don't try to log
			// anything, since we have the UID/GID of the job's owner
			// and cannot write into the log files...
		dprintf( D_ALWAYS, "Exec failed - errno = %d\n", errno );
		exit( EXECFAILED );
	}

		// The parent
    close ( pipes[1] );
	dprintf( D_ALWAYS, "Started pvmd - PID = %d\n", pid );
	pvmdPid = pid;

		// read the first line of stdout from pvmd
	int offset = 0;
	char c;
	int pvmd_file_fd = pipes[0];
    do {
		int rval = read(pvmd_file_fd, &c, 1);
		if (rval < 0) {
			EXCEPT("read error on pvm pipe");
		}
		tmp[offset++] = c;
		if (offset == arg_max) {
			EXCEPT("pvmd output overflows buffer!");
		}
    } while (c != '\n');
    tmp[offset - 1] = '\0'; /* chop of the \n at the end */
    
	dprintf( D_ALWAYS, "PVMd response: %s\n", tmp);

		// set PVMSOCK env var so we don't need to use roundevous file
	char pvmsockaddr[arg_max];
	offset = 0;
    do {
		int rval = read(pvmd_file_fd, &c, 1);
		if (rval < 0) {
			EXCEPT("read error on pvm pipe (PVMSOCK env var)");
		}
		pvmsockaddr[offset++] = c;
		if (offset == arg_max) {
			EXCEPT("pvmd output overflows buffer (PVMSOCK env var)!");
		}
    } while (c != '\n');
    pvmsockaddr[offset - 1] = '\0'; /* chop of the \n at the end */
	char *pvmsock = (char *)malloc(10+strlen(pvmsockaddr));
	sprintf(pvmsock, "PVMSOCK=%s", pvmsockaddr);
	pvmputenv(pvmsock);

    dprintf(D_ALWAYS, "%s\n", pvmsock);
    
	priv = set_user_priv();  /* we execed the pvmd as the user, so we must 
				  * run pvm_mytid() as the user.*/
#ifdef PVM34
	// to prevent circular waiting of msgs in pvm3.4
	pvm_setopt(PvmSelfTraceTid, -1);  
#endif
	mytid = pvm_mytid();	
	set_priv(priv);

	dprintf( D_ALWAYS, "Starter tid = t%x\n", mytid );

	(void)REMOTE_CONDOR_pvm_info( tmp, mytid );
	delete [] tmp;
	state = EXECUTING;
	shadow_tid = my_shadow_tid;
	dprintf( D_ALWAYS, "Shadow tid = t%x\n", shadow_tid);
	pvm_setopt(PvmRoute, PvmDontRoute);
	pvm_setopt(PvmResvTids, 1);
	Running_PVMD = TRUE;
}


void PVMdProc::delete_files()
{
	char	buf[80];
	uid_t	old_uid;

	priv_state priv;
	priv = set_root_priv();
	sprintf(buf, "/tmp/pvml.%d", get_user_uid() );
	do_unlink( buf );
	sprintf(buf, "/tmp/pvmd.%d", get_user_uid() );
	do_unlink( buf );
	UserProc::delete_files();
	set_priv(priv);
}


void PVMUserProc::execute()
{
	int		argc;
	char	*argv[ 2048 ];
	char	**argp;
	char	**envp;
	sigset_t	sigmask;
	long	arg_max;
	char	*tmp;
	char	a_out_name[ _POSIX_PATH_MAX ];
	int		user_syscall_fd;
	int		pipes[2];
	int		readlen;
	int		mytid;
	uid_t	old_uid;
	int		trc_tid;
	int		trc_cod;
	int		out_tid;
	int		out_cod;
#ifdef PVM34
	int trc_ctx;
	int out_ctx;
#endif	
	int env_index;
	int		pvm_parent;
	int		pvm_flags;
	int		arg_count;
	int		env_count;
	int		one = 1;
	struct pvmtaskinfo	*tinfo;
	extern int pvmschedtid;
	int		hold_schedtid;

	dprintf ( D_ALWAYS, "In PVMUserProc::execute(). \n" );

		// We will use mkargv() which modifies its arguments in place
		// so we not use the original copy of the arguments
	if( (arg_max=sysconf(_SC_ARG_MAX)) == -1 ) {
		arg_max = _POSIX_ARG_MAX;
	}
	tmp = new char [arg_max];
	strncpy( tmp, args, arg_max );

	sprintf( a_out_name, "../%s/condor_exec.%d.%d", local_dir, cluster, proc );

	argv[0] = a_out_name;
	mkargv( &argc, &argv[1], tmp );

	env_obj.add_string(VirtualMachineName);

	// set up environment vector
	envp = env_obj.get_vector();

	pvm_parent = atoi(argv[1]);
	pvm_flags = atoi(argv[2]);
#ifdef PVM34
	out_tid = atoi(argv[3]);
	out_ctx = atoi(argv[4]);
	out_cod = atoi(argv[5]);
	trc_tid = atoi(argv[6]);
	trc_ctx = atoi(argv[7]);
	trc_cod = atoi(argv[8]);
#else
	out_tid = atoi(argv[3]);
	out_cod = atoi(argv[4]);
	trc_tid = atoi(argv[5]);
	trc_cod = atoi(argv[6]);
#endif
		// print out arguments to execve
	dprintf( D_ALWAYS, "Sending SM_EXEC( \"%s\"", a_out_name );
	arg_count = 0;
#ifdef PVM34
	env_index = 9;
#else
	env_index = 7;
#endif
	for( argp = &argv[env_index]; *argp; argp++ ) {							// argv
		dprintf( D_ALWAYS | D_NOHEADER, ", \"%s\"", *argp );
		arg_count++;
	}
	dprintf( D_ALWAYS | D_NOHEADER, ", 0" );
	env_count = 0;
	for( argp = envp; *argp; argp++ ) {							// envp
		dprintf( D_ALWAYS | D_NOHEADER, ", \"%s\"", *argp );
		env_count++;
	}
	dprintf( D_ALWAYS | D_NOHEADER, ", 0 )\n" );

	pvm_initsend(PvmDataFoo);
	pvm_pkint(&pvm_parent, 1, 1);
	pvm_pkstr(a_out_name);
	pvm_pkint(&pvm_flags, 1, 1);
	pvm_pkint(&one, 1, 1);		// How many to spawn
	pvm_pkint(&arg_count, 1, 1);
	for ( argp = &argv[env_index]; *argp; argp++) {
		pvm_pkstr(*argp);
	}
	pvm_pkint(&out_tid, 1, 1);
#ifdef PVM34
	pvm_pkint(&out_ctx, 1, 1);
#endif
	pvm_pkint(&out_cod, 1, 1);
	pvm_pkint(&trc_tid, 1, 1);
#ifdef PVM34
	pvm_pkint(&trc_ctx, 1, 1);
#endif
	pvm_pkint(&trc_cod, 1, 1);
	pvm_pkint(&env_count, 1, 1);
	for( argp = envp; *argp; argp++ ) {							// envp
	    pvm_pkstr(*argp);
	}

#ifdef PVM34
	// In pvm3.4, pvmd expected message format with tag SM_EXEC
	// (See exectasks() in file ddpro.c  -Bin
	pvm_pkint(&one, 1, 1); /* for pvm3.4: wxp->w_instance */
	pvm_pkint(&one, 1, 1); /* for pvm3.4: wxp->w_outof */
#endif

	pvm_send( TIDPVMD, SM_EXEC );			/* Go To it! */

	pvm_recv( TIDPVMD, SM_EXECACK );
	dprintf( D_ALWAYS, "Got EXECACK!\n");
	state = EXECUTING;
	pvm_upkint( &one, 1, 1 );
	pvm_upkint( &pvm_tid, 1, 1);
	dprintf( D_ALWAYS, "New Tid = t%x\n", pvm_tid );

	REMOTE_CONDOR_pvm_task_info( v_pid, pvm_tid );
	delete [] tmp;

	hold_schedtid = pvmschedtid;
	pvmschedtid = 0;
	pvm_tasks(pvm_tid, &one, &tinfo);
	pvmschedtid = hold_schedtid;
	if (one == 1) {
		pid = tinfo[0].ti_pid;
		
		/* 
		 * send a message to the parent pvmd that the process
		 * has been spawned.  So that if pvm_catchout() is called
		 * by the parent before the spawn, when the parent calls 
		 * pvm_exit() later, the parent won't hang.  - Bin 
		 */
		if (out_tid) { 
		  int negone = -1; /* -1 signals spawn */
		  int status;
#ifdef PVM34
		  int ctx;
#endif
		  /* allow sending reserved messages */
		  pvm_setopt(PvmResvTids, 1);  

		  pvm_initsend(PvmDataFoo); 
		  pvm_pkint(&pvm_tid, 1, 1);    /* spawned task id */  
		  pvm_pkint(&negone, 1, 1);     /* -1 to signal spawn */
		  pvm_pkint(&pvm_parent, 1, 1); /* parent task id */
#ifdef PVM34
		  ctx = pvm_setcontext(out_ctx);
#endif
		  dprintf ( D_ALWAYS, "Sending 0x%x to t%x\n", out_cod, out_tid );
		  status = pvm_send(out_tid, out_cod);
#ifdef PVM34
		  pvm_setcontext(ctx);
#endif
		}

	} else {
		dprintf(D_ALWAYS, "Uh Oh, it looks like the spawn failed\n");
	}
#if 0
	respawn_for_tid = 0;
	respawn_req_id = 0;
	respawn_resp_msg = 0;
#endif
	dprintf( D_ALWAYS, "New process unix pid = %d\n", pid);
}


void PVMUserProc::request_ckpt()
{
	int		dummy_resp = 0;

	dprintf(D_ALWAYS, 
			"PVMUserProc::request_ckpt sending CARMI_VACATE_NOTIFY to t%x\n",
			shadow_tid);
	pvm_initsend(PvmDataDefault);
	pvm_pkint(&pvm_tid, 1, 1);
	pvm_pkint(&dummy_resp, 1, 1);
	pvm_send(shadow_tid, CARMI_VACATE_NOTIFY);
}


void sigalrm_handler(int signum)
{
	UserProc	*proc;

	set_root_priv();
	(void)REMOTE_CONDOR_pvm_info( "Warning Error", -15 );

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if ( proc->get_class() == CONDOR_UNIVERSE_PVMD ) {
			proc->request_exit();
		}
	}
	abort();
}


#if 0
void PVMUserProc::send_sig( int sig )
{
	Alarm	myalarm;
	void (*old_handler)(int);
	extern int DoDelays;
	int	old_delays;
	sigset_t	sigset;

	old_handler = signal(SIGALRM, sigalrm_handler);
	sigemptyset(&sigset);
	sigaddset(&sigset, SIGALRM);
	sigprocmask(SIG_UNBLOCK, &sigset, (sigset_t *) NULL);
	
	myalarm.set( 30 );


	dprintf( D_ALWAYS, "Sending Signal %s to PVM task t%x\n", 
			SigNames.get_name(sig), pvm_tid );
	pvm_sendsig( pvm_tid, sig );

	myalarm.cancel();
	signal(SIGALRM, old_handler);
}
#endif


#if 0
void
PVMUserProc::set_up_respawn(int for_tid, int req_id, int resp_tag)
{
	respawn_for_tid = for_tid;
	respawn_req_id = req_id;
	respawn_resp_msg = resp_tag;
	// Send hard kill signal to self!
	kill_forcibly();
}


void
PVMUserProc::do_respawn()
{
#ifdef PVM34
     struct pvmminfo info;
#endif
	int hold_for_tid;
	int hold_req_id;
	int hold_resp_msg;
	int old_my_tid;

	hold_for_tid = respawn_for_tid;
	hold_req_id = respawn_req_id;
	hold_resp_msg = respawn_resp_msg;
	old_my_tid = pvm_tid;

	execute();

	pvm_initsend(PvmDataDefault);
	pvm_pkint(&old_my_tid, 1, 1);
	pvm_pkint(&pvm_tid, 1, 1);

#ifdef PVM34
	pvm_getminfo(pvm_getsbuf(), &info);
	info.wid = hold_req_id;
	pvm_setminfo(pvm_getsbuf(), &info);
#else
	pvm_setmwid(pvm_getsbuf(), hold_req_id);
#endif
	pvm_send( hold_for_tid, hold_resp_msg );
}
#endif


/*
  Handle message from PVMd about death of a child process.
*/
int
pvm_reaper()
{
	int			st;
	pid_t		pid;
	PVMUserProc	*proc;
	int			bufid;
	int			tid;
	int			user_secs;
	int			user_usecs;
	int			system_secs;
	int			system_usecs;
	int			kill_rval;

	pvm_upkint(&tid, 1, 1);
	pvm_upkint(&st, 1, 1);
		// PVM always gives 0 rusage.  I don't know why.  Anyway, we're
		// going to ignore what PVM tells us and call times() ourself.
	pvm_upkint(&user_secs, 1, 1);
	pvm_upkint(&user_usecs, 1, 1);
	pvm_upkint(&system_secs, 1, 1);
	pvm_upkint(&system_usecs, 1, 1);

	dprintf( D_ALWAYS, "TASKX on %x, status = 0x%x, u_time = %d\n", 
			tid, st, user_secs );

		// Find the corresponding UserProc object
	dprintf( D_FULLDEBUG,
			"Task t%x exited, searching process list...\n", tid
			);

	UProcList.Rewind();
	while( proc = (PVMUserProc *) UProcList.Next() ) {
		dprintf( D_ALWAYS, "pvm_reaper looking at v_pid = %d\n", 
				proc->get_id() );
		if (proc->get_class() == CONDOR_UNIVERSE_PVM) {
			if( proc->get_tid() == tid ) {
#if 0
				if (proc->is_respawning()) {
					proc->do_respawn();
				}
#endif
				/* Use kill() to see if process is still living, if it is, 
				   we don't want to clean-up for it yet. */
				pid = proc->get_pid();
				errno = 0;
				kill_rval = kill(pid, 0);
				dprintf(D_ALWAYS, "kill(%d, 0) returns %d, errno = %d\n",
						pid, kill_rval, errno);
				// Sometimes, the process will die so quickly, we won't even
				// get a pid back from PVM.  In that case, the pid will look
				// like it is zero, so check for that case as well.  Also,
				// if the status != 0, then we can assume it is a real
				// status.  Finally, sometimes it looks like we get a bogus
				// pid of 2, probably when the process dies quickly (like
				// getting a pid of 0).
				if ((kill_rval < 0 && errno == ESRCH) || 
					(pid == 0) ||
					(pid == 2) || 
					(st != 0)) {
					break;
				}
			}
		}
	}

		// If we didn't find the process's object, bail out now
	if( proc == NULL ) {
		dprintf( D_ALWAYS,
				"received tid t%x, but no such process marked as EXECUTING\n",
				tid
				);
		// By rewinding, we force the next call to UProcList.Current() to
		// return NULL, which is what we want since we couldn't find the
		// Proc which apparently exited (perhaps it called pvm_exit() and
		// then the process exited which causes PVM to send 2 TASKX msgs.
		UProcList.Rewind();
		return 0;
	}

		// Ok, update the object
	dprintf( D_FULLDEBUG, "Found object for pvm task %x\n", tid );
	proc->handle_termination( st );
	return 0;
}


/*
  Received some PVM msg, decode it and see where to go next.
*/
int
read_pvm_msg()
{
#ifdef PVM34
     struct pvmminfo info;
#endif
	int		bufid, tag, src, buflen;
	int		i;
	static int req_id = 1;
	int		msg_req_id;
	char	*hold_buffer;
	static struct {
		int		starter_req_id;
		int		task_req_id;
		int		task_tid;
	} request_map[50];
	static int request_map_pos = 0;
	static struct {
		int		msg_id;
		int		task_msg_id;
		int		task_tid;
	} notifications[50];
	static int notifications_pos = 0;
	int	sent = 0;

	bufid = pvm_nrecv(-1, -1);
	if (bufid <= 0) {
		dprintf( D_ALWAYS, "read_pvm_msg: No PVM message ready\n");
		return DEFAULT;
	}
	pvm_bufinfo(bufid, &buflen, &tag, &src);
	dprintf( D_ALWAYS, "read_pvm_msg: Received msg %s(%d) from t%x\n",
			PvmMsgNames.get_name(tag), tag, src );
	switch( tag ) {
	    case SM_TASKX:
		    return PVM_CHILD_EXIT;
			break;

		case SM_NOTIFY:
			{
				int		kind;
				int		msg_id;
				int		count;
				int		tid;

				pvm_initsend(PvmDataDefault);
				pvm_upkint(&kind, 1, 0);
				pvm_pkint(&kind, 1, 0);
				pvm_upkint(&msg_id, 1, 0);
				pvm_pkint(&req_id, 1, 0);
				pvm_upkint(&count, 1, 0);
				pvm_pkint(&count, 1, 0);
				if ( kind != PvmHostAdd ) {
					for (;count > 0; count--) {
						pvm_upkint(&tid, 1, 0);
						pvm_pkint(&tid, 1, 0);
					}
				}
				pvm_pkint(&req_id, 1, 0);
				pvm_send(shadow_tid, SM_NOTIFY);
				notifications[notifications_pos].msg_id = req_id++;
				notifications[notifications_pos].task_msg_id = msg_id++;
				notifications[notifications_pos].task_tid = src;
			}
			return DEFAULT;
			break;

		case SM_SPAWN:
		case SM_TASK:
		case SM_CONFIG:
		case SM_ADDHOST:
		case SM_DELHOST:
		case SM_SCHED:
			hold_buffer = (char *) malloc(buflen);
			pvm_initsend(PvmDataFoo);
			if (src != shadow_tid) { /* From local, route to the shadow */
#ifdef PVM34
			     pvm_getminfo(pvm_getrbuf(), &info);
			     msg_req_id = info.wid;
#else
				msg_req_id = pvm_getmwid(pvm_getrbuf());
#endif
				pvm_upkbyte(hold_buffer, buflen, 1);
				pvm_pkbyte(hold_buffer, buflen, 1);
#ifdef PVM34
				pvm_getminfo(pvm_getsbuf(), &info);
				info.wid = req_id;
				pvm_setminfo(pvm_getsbuf(), &info);
#else
				pvm_setmwid(pvm_getsbuf(), req_id);
#endif
				request_map[request_map_pos].starter_req_id = req_id;
				request_map[request_map_pos].task_req_id = msg_req_id;
				request_map[request_map_pos].task_tid = src;
				req_id++;
				request_map_pos++;
			} else { /* From the shadow, route to local task */
#ifdef PVM34
			     pvm_getminfo(pvm_getrbuf(), &info);
			     msg_req_id = info.wid;
#else
				msg_req_id = pvm_getmwid(pvm_getrbuf());
#endif	
				pvm_upkbyte(hold_buffer, buflen, 1);
				for (i = 0; i < request_map_pos; i++) {
					if (msg_req_id == request_map[i].starter_req_id) {
						dprintf( D_ALWAYS, 
							"Mapping from req_id %d to task t%x, req_id %d\n",
								msg_req_id, request_map[i].task_tid,
								request_map[i].task_req_id);
						pvm_pkbyte(hold_buffer, buflen, 1);
#ifdef PVM34
						pvm_getminfo(pvm_getsbuf(), &info);
						info.wid = request_map[i].task_req_id;
						pvm_setminfo(pvm_getsbuf(), &info);
#else
						pvm_setmwid(pvm_getsbuf(),request_map[i].task_req_id);
#endif
						pvm_send(request_map[i].task_tid, tag);
						sent = 1;
						request_map[i] = request_map[request_map_pos];
						request_map_pos--;
						break;
					}
				}
				if (sent == 0) {
					dprintf( D_ALWAYS, "Couldn't find starter req_id %d\n",
							msg_req_id);
				}
			}
			free(hold_buffer);
			return DEFAULT;
			break;

		case CARMI_RESPAWN:
			{
				int		msg_req_id;
				int		old_tid;
				int		msg_resp_tag;
				PVMUserProc *proc;

				pvm_upkint(&old_tid, 1, 1);
				pvm_upkint(&msg_resp_tag, 1, 1);
#ifdef PVM34
				pvm_getminfo(pvm_getrbuf(), &info);
				msg_req_id = info.wid;
#else
				msg_req_id = pvm_getmwid(pvm_getrbuf());
#endif
				UProcList.Rewind();
				while( proc = (PVMUserProc *) UProcList.Next() ) {
					if ( proc->get_class() == CONDOR_UNIVERSE_PVM ) {
						if ( proc->get_tid() == old_tid ) {
#if 0
							proc->set_up_respawn(src, msg_req_id, 
												 msg_resp_tag);
#endif
						}
					}
				}
			}
			break;
			
		case CARMI_VACATE_NOTIFY:
			{
				int		target_tid;
				PVMUserProc *proc;

				pvm_upkint(&target_tid, 1, 1);

				dprintf(D_ALWAYS, "Got ckpt on vacate for t%x\n",
						target_tid);

				UProcList.Rewind();
				while( proc = (PVMUserProc *) UProcList.Next() ) {
					if ( proc->get_class() == CONDOR_UNIVERSE_PVM ) {
						if ( proc->get_tid() == target_tid ) {
							proc->set_checkpoint_on_vacate();
						}
					}
				}
			}
			break;
			
		default:	/* Could be a notification to forward to a user */
			for (i = 0; i < notifications_pos; i++) {
				if (tag == notifications[i].msg_id) {
					pvm_setsbuf(bufid);
					pvm_send(notifications[i].task_msg_id, 
							 notifications[i].task_tid);
					dprintf( D_ALWAYS, 
							"Sending notification to t%x w/ tag %d\n",
							notifications[i].task_tid,
							notifications[i].task_msg_id );
					sent = 1;
					notifications[i] = notifications[notifications_pos];
					notifications_pos--;
					break;
				}
			}
			if (sent == 0) {
				dprintf( D_ALWAYS, "Unexpected message tag %d\n", tag);
			}
			return DEFAULT;
			break;
	}
	return DEFAULT;
}


int
pvm_pause()
{
	static int	polling_freq = -1;
	static long		clock_tick = 0;
	char		*pvm_polling_freq;
	
	if( Running_PVMD ) {
		if (polling_freq == -1) {
			pvm_polling_freq = param("PVM_POLLING_FREQ");
			if (pvm_polling_freq != NULL) {
				polling_freq = atoi(pvm_polling_freq);
				if ( !clock_tick ) {
					clock_tick = sysconf( _SC_CLK_TCK );
				}
				polling_freq *= clock_tick;
			} else {
				polling_freq = 0;
			}
		}

		//	Check for a queued PVM message, and go handle it if	ready.
		for(;;) {
			int		*pvm_fd = NULL;
			Selector	sel;

			if (polling_freq == 0) {
				if (pvm_fd == NULL) {
					pvm_getfds(&pvm_fd);
				}
				if (pvm_fd == NULL) {
					dprintf ( D_FULLDEBUG, "pvm_fd is STILL NULL\n");
						/* Shoot pvmd! */
					kill ( pvmdPid, SIGTERM );
					return PVM_MSG;
				} else {
					sel.add_fd( pvm_fd[0], IO_READ );
				}
			} else {
				sel.set_timeout( polling_freq );
			}
			sel.execute();
			if ((sel.has_ready() && sel.fd_ready(pvm_fd[0], IO_READ)) ||
				sel.timed_out() ) {
				return PVM_MSG;
			}
		}
	} else {
		return pause();
	}
}
