/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "starter_common.h"
#include "condor_attributes.h"
#include "condor_version.h"

#include "proto.h"
#include "name_tab.h"
#include "selector.h"

#include "state_machine_driver.h"

#include "starter.h"
#include "fileno.h"
#include "ckpt_file.h"
#include "user_proc.h"
#include "startup.h"
#include "alarm.h"
#include "list.h"
#include "sig_install.h"
#include "../condor_sysapi/sysapi.h"
#include "condor_distribution.h"

#if !defined(X86)
typedef List<UserProc> listuserproc;
#endif

#if defined(LINK_PVM)
#include "pvm_user_proc.h"
#include "pvm3.h"
#ifdef PVM34
#include "pvmproto.h"
#else
#include "pvmsdpro.h"
#endif

/* #if !defined(PVM33)
#include "sdpro.h"
#else
#include "pvmsdpro.h"
#endif
*/
#endif


#undef ASSERT
#define ASSERT(cond) \
	if( !(cond) ) { \
		EXCEPT( "Assertion ERROR on (%s)\n", #cond ); \
	} else {\
		dprintf( D_FULLDEBUG, "Assertion Ok on (%s)\n", #cond ); \
	}

	// Global flags
int	QuitNow = FALSE;		// Quit after completing current checkpoint
int	XferCkpts = FALSE;		// Transfer ckpt files before exiting
int	Running_PVMD = FALSE;	// True if we are running a pvm job

	// Constants
const int	VacateTimer = 180;	// How long to wait for proc requested to exit
const int	CkptLimit = 300;	// How long to wait for proc requested to exit
const pid_t	ANY_PID = -1;		// arg to waitpid() for any process

ReliSock		*SyscallStream = NULL;	// ReliSock stream to shadow for remote system calls
List<UserProc>	UProcList;		// List of user processes
char	*Execute;				// Name of directory where user procs execute
int     ExecTransferAttempts;   // How many attempts at getting the initial ckpt
int		MinCkptInterval;		// Use this ckpt interval to start
int		MaxCkptInterval;		// Don't increase ckpt interval beyond this
int		DoDelays;				// Insert artificial delays for testing
char	*UidDomain=NULL;		// Machines we share UID space with

Alarm		MyAlarm;			// Don't block forever on user process exits
CkptTimer	*MyTimer;			// Timer for periodic checkpointing
char	*InitiatingHost;		// Machine where shadow is running

extern State StateTab[];
extern Transition TransTab[];
extern int EventSigs[];

extern NameTable ProcStates;
extern NameTable PvmMsgNames;

extern int shadow_tid;

// so local logging doesn't break:
char* mySubSystem = "STARTER";

	// instantiate templates
template class List<UserProc>;
template class Item<UserProc>;

	// Prototypes of local interest only
int update_one( UserProc *proc );
void send_final_status( UserProc *proc );
void resume_all();
void req_exit_all();
void determine_user_ids( uid_t &requested_uid, gid_t &requested_gid );


void
printClassAd( void )
{
	printf( "%s = False\n", ATTR_IS_DAEMON_CORE );
	printf( "%s = True\n", ATTR_HAS_PVM );
	printf( "%s = \"%s\"\n", ATTR_VERSION, CondorVersion() );
}


/*
  Main routine for condor_starter.  DEBUGGING can be turned on if we
  need to attach a debugger at run time.  Everything is driven by the
  finite state machine, so we just intiialize it and run it.
*/

int
main( int argc, char *argv[] )
{
	myDistro->Init( argc, argv );

	if( argc == 2 && strincmp(argv[1], "-cl", 3) == MATCH ) {
		printClassAd();
		exit( 0 );
	}

	StateMachine condor_starter( StateTab, TransTab, EventSigs, START, END );
	extern void sigalrm_handler(int);

#define DEBUGGING FALSE
#if DEBUGGING
	wait_for_debugger( TRUE );
#else
	wait_for_debugger( FALSE );
#endif

        if (argc == 2 && strcmp(argv[1], "-dot") == MATCH) {
                condor_starter.dot_print(stdout);
                exit(0);
        }
  
#if !defined(WIN32)
	install_sig_handler(SIGPIPE, SIG_IGN );
#endif

	set_posix_environment();
	initial_bookeeping( argc, argv );

/*	(void)signal(SIGSEGV, sigalrm_handler); */

	// condor_starter.display();
	condor_starter.execute();
	dprintf( D_ALWAYS, "********* STARTER terminating normally **********\n" );
	return 0;
}


/*
  If TESTING is set to TRUE, we delay approximately sec seconds for
  purposes of debugging.  If TESTING is not TRUE, we just return.
*/
void
delay( int sec )
{
	int		i;
	int		j, k;
#if defined(HPUX) 
	int		lim = 650000;
#endif
#if defined(SPARC) || defined(sun4m) || defined(Solaris)
	int		lim = 250000;
#endif
#if defined(R6000)
	int		lim = 250000;
#endif
#if defined(MIPS)
	int		lim = 200000;
#endif
#if defined(ALPHA)
	int		lim = 1800000;
#endif
#if defined(IRIX)
	int		lim = 650000;
#endif
#if defined(LINUX)
	int		lim = 200000;
#endif


	if( !DoDelays || sec == 0 ) {
		return;
	}

	for( i=sec; i > 0; i-- ) {
		dprintf( D_ALWAYS | D_NOHEADER,  "%d\n", i );
		for( j=0; j<lim; j++ ) {
			for( k=0; k<5; k++ );
		}
	}
	dprintf( D_ALWAYS | D_NOHEADER,  "0\n" );
}

/*
  Initialization stuff to be done before getting any use processes.
*/
int
init()
{
	move_to_execute_directory();
	sysapi_set_resource_limits();
	close_unused_file_descriptors();
	MyTimer = new CkptTimer( MinCkptInterval, MaxCkptInterval );
	return DEFAULT;
}


/*
  Suspend ourself and wait for a SIGCONT signal.
*/
int
susp_self()
{
	dprintf( D_ALWAYS, "Suspending self\n" );
	(void)REMOTE_CONDOR_suspended( 1 );
	kill( getpid(), SIGSTOP );
	(void)REMOTE_CONDOR_suspended( 0 );
	dprintf( D_ALWAYS, "Resuming self\n" );
	return 0;
}

/*
  We were called with incorrect set of command line arguments.  Print a
  message and exit.
*/
void
usage( char *my_name )
{
	dprintf( D_ALWAYS, "Usage: %s initiating_host\n", my_name );
	exit( 0 );
}

/*
  Wait up for one of those nice debuggers which attaches to a running
  process.  These days, most every debugger can do this with a notable
  exception being the ULTRIX version of "dbx".
*/
void wait_for_debugger( int do_wait )
{
	sigset_t	sigset;

	// This is not strictly POSIX conforming becuase it uses SIGTRAP, but
	// since it is only used in those environments where is is defined, it
	// will probably pass...
#if defined(SIGTRAP)
		/* Make sure we don't block the signal used by the
		** debugger to control the debugged process (us).
		*/
	sigemptyset( &sigset );
	sigaddset( &sigset, SIGTRAP );
	sigprocmask( SIG_UNBLOCK, &sigset, 0 );
#endif

	while( do_wait )
		;
}

/*
  We're born with two TCP connections to the shadow.  The connection at
  file descriptor #1 is for remote system calls, and the one at file
  descriptor #2 is for logging.  Initialize these and set them up to be
  accessed by the well known numbers used by all the remote system call
  and logging code.
*/
void
init_shadow_connections()
{
	int		scm;


	(void) dup2( 1, RSC_SOCK );
	(void) dup2( 2, CLIENT_LOG );

//	SyscallStream = RSC_Init( RSC_SOCK, CLIENT_LOG );
	SyscallStream = init_syscall_connection( FALSE );
	// xdrrec_skiprecord( SyscallStream );
}

/*
  Change directory to where we will run our user processes.
*/
void
move_to_execute_directory()
{
	if( chdir(Execute) ) {
		EXCEPT( "chdir(%s)", Execute );
	}
	dprintf( D_FULLDEBUG, "Done moving to directory \"%s\"\n", Execute );
}

/*
  Close any unnecessary open file descriptors which may be lying
  around.  This is important becuase our fd's will get duplicated in
  our user processes, and we don't want to fill up the kernel's fd
  table needlessly.
*/
void
close_unused_file_descriptors()
{
	long		open_max;
	long		i;


		/* first find out how many fd's are available on this system */
	errno = 0;
	if( (open_max=sysconf(_SC_OPEN_MAX)) == -1 ) {
		if( errno == 0 ) {
			open_max = RSC_SOCK;
			dprintf( D_ALWAYS, "OPEN_MAX is indeterminite, using %d\n",
																RSC_SOCK );
		} else {
			EXCEPT( "_SC_OPEN_MAX not recognized" );
		}
	}

		/* now close everything except the ones we use */
	for( i=0; i<open_max; i++ ) {

		if( i == fileno(stderr) || i == RSC_SOCK || i == CLIENT_LOG ) {
			continue;
		}

		(void) close( i );
	}
	dprintf( D_FULLDEBUG, "Done closing file descriptors\n" );
}

/*
  Grab initialization parameters and put them into global variables for
  quick and easy access.
*/
void
init_params()
{
	char	*tmp;

	if( (Execute=param("EXECUTE")) == NULL ) {
		EXCEPT( "Execute directory not specified in config file" );
	}

	if( (tmp=param("MIN_CKPT_INTERVAL")) == NULL ) {
		MinCkptInterval = 30 * MINUTE;
	} else {
		MinCkptInterval = atoi( tmp );
	}

	if( (tmp=param("MAX_CKPT_INTERVAL")) == NULL ) {
		MaxCkptInterval = 2 * HOUR;
	} else {
		MaxCkptInterval = atoi( tmp );
	}

	if( (tmp=param("STARTER_DELAYS")) == NULL ) {
		DoDelays = FALSE;
	} else {
		if( tmp[0] == 't' || tmp[0] == 'T' ) {
			DoDelays = TRUE;
		} else {
			DoDelays = FALSE;
		}
	}

		// find out domain of machines whose UIDs we honor
	UidDomain = param( "UID_DOMAIN" );

		// if the domain is null, don't honor any UIDs
	if( UidDomain == NULL || UidDomain[0] == '\0' ) {
		UidDomain = "Unknown";
	}

		// if the domain is "*", honor all UIDs - a dangerous idea
	if( UidDomain[0] == '*' ) {
		UidDomain[0] = '\0';
	}

	// We can configure how many times the starter wishes to attempt to
	// pull over the initial checkpoint
	if ( (tmp=param( "EXEC_TRANSFER_ATTEMPTS" )) == NULL)
	{
		ExecTransferAttempts = 3;
	}
	else
	{
		ExecTransferAttempts = atoi(tmp);
		// This catches errors on atoi(), and if the user did something stupid
		if (ExecTransferAttempts < 1)
		{
			dprintf( D_ALWAYS, "Please check your EXEC_TRANSFER_ATTEMPTS "
				"macro. It must be a number greater than or equal to one. "
				"Defaulting EXEC_TRANSFER_ATTEMPTS to 3.\n") ;
			ExecTransferAttempts = 3;
		}
		free(tmp);
	}

	
}

/*
  Get one user process from the shadow and append it to our list of
  user processes.  Note - this routine does not transfer any checkpoint
  or executable files.  That is done in a separate routine.
*/
int
get_proc()
{
	int	answer;
	UserProc *new_process;

	dprintf( D_ALWAYS, "Entering get_proc()\n" );
	if( new_process=get_job_info() ) {
		UProcList.Append( new_process );
		return SUCCESS;
	} else {
		return TRY_LATER;
	}
}


/*
   Set up for the transfer of the remote checkpoint to local on our way to
   get_exec().
*/
int
init_transfer()
{
	UserProc *new_process;

	new_process = UProcList.Current();
	new_process->init_fetch();
	return 0;
}


extern NameTable TransferStates;
/*
  Transfer a checkpoint or executable file from the submitting machine to
  our own file system.
*/
int
get_exec()
{
	UserProc	*new_process;

	dprintf( D_ALWAYS, "Entering get_exec()\n" );

	new_process = UProcList.Current();

	if (new_process->fetch_ckpt() != TRUE) {
		return FAILURE;
	}

	return SUCCESS;
}


/*
  We've been asked to leave the machine, and we may not create or
  update any more checkpoint files.  We may however transfer any
  existing checkpoint files back to the submitting machine.
*/
int
req_vacate()
{	
	req_exit_all();
	MyAlarm.set( CkptLimit );
	return 0;
}

/*
  We've been asked to leave the machine, and we may not create or
  update any more checkpoint files.  Also, we may not transfer any
  existing checkpoint files back to the submitting machine.
*/
int
req_die()
{
	req_exit_all();
	MyAlarm.set( VacateTimer );
	XferCkpts = FALSE;
	return 0;
}



/*
  Request every user process to exit now.
*/
void
req_exit_all()
{
	UserProc	*proc;
	PVMUserProc	*pvm_proc;

		// Request all the processes to exit
	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if ( proc->get_class() != CONDOR_UNIVERSE_PVMD ) {
			dprintf( D_ALWAYS, "req_exit_all: Proc %d in state %s\n", 
					proc->get_id(),	ProcStates.get_name(proc->get_state())
					);
			if( proc->is_running() || proc->is_suspended() ) {
				if (proc->ckpt_enabled()) {
					dprintf( D_ALWAYS, "Requesting Checkpoint on proc #%d\n",
							proc->get_id());
					if (proc->get_class() == CONDOR_UNIVERSE_PVM) {
						pvm_proc = (PVMUserProc *) proc;
						pvm_proc->request_ckpt();
					} else {
						proc->request_ckpt();
					}
				} else {
					dprintf( D_ALWAYS, "Requesting Exit on proc #%d\n",
							proc->get_id());
					proc->request_exit();
				}
			}
		}
	}
}


int
periodic_ckpt_all()
{
	return 0;					// PVM jobs don't do pckpts (for now)
}

/*
  Wait for every user process to exit.
*/
int
terminate_all()
{
	UserProc	*proc;

	// dprintf( D_ALWAYS, "Entering function terminate_all()\n" );

	// Previously, we didn't want to kill the PVMD until we had exit
	// status for every user process.  When we do have status from
	// every process, we can request exit on the PVMD, and wait
	// for it to terminate.

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if( (proc->is_running() || proc->is_suspended()) && 
		   proc->get_class() != CONDOR_UNIVERSE_PVMD) {
			return DO_WAIT;
		}
	}

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if (proc->get_class() == CONDOR_UNIVERSE_PVMD) {
			dprintf(D_ALWAYS, "terminate_all: PVMD state is %s\n",
					ProcStates.get_name(proc->get_state()));
			if (proc->is_running()) {
#if defined(LINK_PVM)
				pvm_exit();		/* Get out so PVMd doesn't kill us */
#endif
				proc->request_exit();
				return DO_WAIT;
			}
		}
	}

		// Cancel alarm
	MyAlarm.cancel();
	return DEFAULT;
}

/*
  Send final exit status and cleanup files for every user proc in the
  list.
*/
int
dispose_all()
{
	UserProc	*proc;

	// dprintf( D_ALWAYS, "Entering dispose_all()\n" );

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		send_final_status( proc );
		proc->delete_files();
		UProcList.DeleteCurrent();
	}

	return DEFAULT;
}


/*
  Send final image size, exit status and resource usage for a process
  to the shadow.
*/
void
send_final_status( UserProc *proc )
{
	int			image_size;
	void		*status;
	void		*rusage;
	int			id;
	int			cluster_no = proc->get_cluster();
	int			proc_no = proc->get_proc();


	dprintf( D_ALWAYS,
		"Sending final status for process %d.%d\n", cluster_no, proc_no
	);

		// update shadow with it's image size
	if( proc->ckpt_enabled() ) {
		image_size = proc->get_image_size();
		(void)REMOTE_CONDOR_image_size( image_size );
	}

		// update shadow with it's resource usage and exit status
	status = proc->bsd_exit_status();
	if( proc->get_class() == CONDOR_UNIVERSE_PVM ) {
			// times() isn't going to give us a valid usage value
			// until we reap the pvmd (our immediate child process).
			// Also, we don't want to double-count.  We could
			// potentially have many PVM tasks, but we never reset our
			// usage counter.  So, don't report any usage here.
			// Instead, report it all with the pvmd.
		struct rusage usage;
		memset(&usage, 0, sizeof(usage));
		id = proc->get_id();
		(void)REMOTE_CONDOR_subproc_status( (int)id, (int*)status, 
			(struct rusage*) &usage);
	} else if ( proc->get_class() == CONDOR_UNIVERSE_PVMD ) {
			// This assumes we never have more than one pvmd.
			// Otherwise, we'll be double-counting our CPU usage,
			// since UserProc::accumulated_rusage() gives us usage for
			// all children, not just the process(es) associated with
			// this proc.
		rusage = proc->accumulated_rusage();	// All resource usage
		id = proc->get_id();
		(void)REMOTE_CONDOR_subproc_status( (int)id, (int*)status, 
			(struct rusage*)rusage);
	} else if (proc->get_class() != CONDOR_UNIVERSE_PVMD) {
		rusage = proc->guaranteed_rusage();		// Only usage guaranteed by ckpt
		(void)REMOTE_CONDOR_reallyexit( (int*)status, (struct rusage*)rusage);
	}

	dprintf( D_FULLDEBUG,
		"Done sending final status for process %d.%d\n", cluster_no, proc_no
	);
		
}

/*
  Wait for some user process to exit, and update it's object in the
  list with the exit status information.
*/
int
reaper()
{
	int			st;
	pid_t		pid;
	UserProc	*proc;

		// Wait for a child process to exit
	while( (pid=waitpid(ANY_PID,&st,0)) == -1 && errno == EINTR )
		;

		// Should never happen...
	if( pid == -1 ) {
		EXCEPT( "wait()" );
	}

	
		// Find the corresponding UserProc object
	dprintf( D_FULLDEBUG,
		"Process %d exited, searching process list...\n", pid 
	);

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if( proc->get_pid() == pid ) {
			break;
		}
	}

		// If we didn't find the process's object, bail out now
	if( proc == NULL ) {
		EXCEPT(
			"wait() returned pid %d, but no such process marked as EXECUTING",
			pid
		);
	}

		// Ok, update the object
	dprintf( D_FULLDEBUG, "Found object for process %d\n", pid );
	proc->handle_termination( st );
	return 0;
}


/*
  Temporarily suspend the timer which controls periodic checkpointing.
*/
int
susp_ckpt_timer()
{
	MyTimer->suspend();
	return 0;
}

/*
  Set a global flags which says we may not transfer any checkpoints back
  to the initiating machine.
*/
int
unset_xfer()
{
	dprintf( D_FULLDEBUG, "Setting XferCkpts to FALSE\n" );
	XferCkpts = FALSE;
	return 0;
}

/*
  Suspend all user processes and ourself - wait for a SIGCONT.
*/
int
susp_all()
{
	MyTimer->suspend();
	stop_all();

	susp_self();

	resume_all();
	MyTimer->resume();
	return 0;
}

/*
  Set a global flag which says we must leave after completing and
  transferring the current batch of checkpoints.
*/
int
set_quit()
{
	// dprintf( D_ALWAYS, "Entering function set_quit()\n" );

	QuitNow = TRUE;
	return 0;
}

/*
  Suspend every user process.
*/
int
stop_all()
{
	UserProc	*proc;

	dprintf( D_FULLDEBUG, "Entering function stop_all()\n" );

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if( proc->is_running() && proc->get_class() != CONDOR_UNIVERSE_PVMD ) {
			proc->suspend();
			dprintf( D_ALWAYS, "\tRequested user job to suspend\n" );
		}
	}

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if( proc->is_running() && proc->get_class() == CONDOR_UNIVERSE_PVMD ) {
//			proc->suspend();
			dprintf( D_ALWAYS, "\tDidn't request PVMD to suspend\n" );
		}
	}
	return 0;
}

/*
  Resume every suspended user process.
*/
void
resume_all()
{
	UserProc	*proc;

	dprintf( D_FULLDEBUG, "Entering function resume_all()\n" );

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if( proc->is_suspended() && proc->get_class() == CONDOR_UNIVERSE_PVMD ) {
			proc->resume();
			dprintf( D_ALWAYS, "\tRequested user job to resume\n" );
		}
	}

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if( proc->is_suspended() && proc->get_class() != CONDOR_UNIVERSE_PVMD ) {
			proc->resume();
			dprintf( D_ALWAYS, "\tRequested user job to resume\n" );
		}
	}
}

/*
  Start up every runnable user process which isn't already running.
*/
int
spawn_all()
{
	UserProc	*proc;

	dprintf( D_ALWAYS, "Entering function spawn_all()\n" );

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if( proc->is_runnable() ) {
			dprintf ( D_ALWAYS, "Executing proc %d.\n", proc->get_id() );
			proc->execute();
		} else {
			dprintf( D_ALWAYS, "Proc %d not runnable in state %d\n", 
					proc->get_id(), proc->get_state() );
		}
		if( proc->is_suspended() ) {
			dprintf ( D_ALWAYS, "Resuming proc %d.\n", proc->get_id() );
			proc->resume();
		}
	}
	return 0;
}

/*
  Start up the periodic checkpoint timer, then wait for some asynchronous
  event.  The event could be the timer, but may also be caused by a
  user process exiting, a vacate request, etc.  If we run out of user
  processes, we will just sit here until we either get a GET_NEW_PROC
  or a VACATE or DIE request.
*/
int
supervise_all()
{
	UserProc	*proc;
	int			periodic_checkpointing = FALSE;


	dprintf( D_FULLDEBUG, "Entering function supervise_all()\n" );

#if 0
	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if( proc->ckpt_enabled() ) {
			periodic_checkpointing = TRUE;
			break;
		}
	}

	if( periodic_checkpointing ) {
		if( MyTimer->is_active() ) {
			MyTimer->resume();
		} else {
			MyTimer->start();
		}
	}
#endif

#if defined(LINK_PVM)
	if( Running_PVMD ) {
		static int	polling_freq = -1;
		static long		clock_tick = 0;
		char		*pvm_polling_freq;

		if (polling_freq == -1) {
			pvm_polling_freq = param("PVM_POLLING_FREQ");
			if (pvm_polling_freq != NULL) {
				polling_freq = atoi(pvm_polling_freq);
				if ( !clock_tick ) {
					clock_tick = sysconf( _SC_CLK_TCK );
				}
				polling_freq *= clock_tick;
			} else {
				polling_freq = 0;
			}
		}


		if (pvm_probe(-1, -1)) {
			dprintf(D_ALWAYS, "Already had a queued msg.\n");
			return PVM_MSG;
		}
		//	Check for a queued PVM message, and go handle it if	ready.
		for(;;) {
			int		*pvm_fd = NULL;
			Selector	sel;

			if (polling_freq == 0) {
				if (pvm_fd == NULL) {
					pvm_getfds(&pvm_fd);
				}
				sel.add_fd( pvm_fd[0], IO_READ );
			} else {
				sel.set_timeout( polling_freq );
			}
			sel.execute();
			if ((sel.has_ready() && sel.fd_ready(pvm_fd[0], IO_READ)) ||
				sel.timed_out() ) {
				return PVM_MSG;
			}
		}
	}
#endif

	for(;;) {
		pause();
	}

		// Can never get here
	return NO_EVENT;
}

/*
  Some user process exited while we were in SUPERVISE state.  Determine
  the cause of the exit, and jump to the appropriate new state.
*/
int
proc_exit()
{
	UserProc	*proc;
	PROC_STATE	state;
	int			pvm_orphans = FALSE;


		// Grab a pointer to proc which just exited
	proc = UProcList.Current();

	if (proc == NULL) {
		dprintf(D_ALWAYS, "proc_exit(): Current Proc is NULL, ignoring\n");
		return NO_CORE;
	}

	/* If the PVMd dies, we must make sure to clean up all sub processes
	   (there really shouldn't be any) */
	if (proc->get_class() == CONDOR_UNIVERSE_PVMD) {
		proc->delete_files();
		UProcList.DeleteCurrent();
		Running_PVMD = FALSE;

		UProcList.Rewind();
		while ( proc = UProcList.Next() ) {
			proc->kill_forcibly();		// This assumes signal not via PVMd
			proc->delete_files();
			UProcList.DeleteCurrent();
			pvm_orphans = TRUE;
		}
		if (pvm_orphans) {
			return PVMD_DIED;
		} else {
			/* Here the PVMD died unexpectedly, and yet there were no 
			 * PVM tasks running here.  We have two choices here: either
			 * clean up from the pvm daemon exit and go back to state
			 * SUPERVISE, _or_ clean up and have the starter exit.  The
			 * question is if we would ever envision it desireable to
			 * have the starter.carmi running when a pvmd is not.  
			 * If we want to stick around, return NO_CORE.  If we want
			 * to exit, remote PVMD_DIED.  For now, we return PVMD_DIED
			 * to make certain the condor_rm works properly all the time,
			 * but someday condor_rm could be handled more robustly in 
			 * the shadow.carmi.  -Todd 3/97 */
			// return NO_CORE;
			return PVMD_DIED;
		}
	}
		
	switch( state = proc->get_state() ) {

	  case CHECKPOINTING:
		return CKPT_EXIT; // Started own checkpoint, go to UPDATE_CKPT state
	  case ABNORMAL_EXIT:
		return HAS_CORE;

	  case NON_RUNNABLE:
	  case NORMAL_EXIT:
		return NO_CORE;

	  default:
		EXCEPT( "Unexpected proc state (%d)\n", state );
	}

		// Can never get here.
	return DEFAULT;
}

/*
  Dispose of one user process from our list.
*/
int
dispose_one()
{
	UserProc	*proc;

		// Grab a pointer to proc which just exited
	proc = UProcList.Current();

	if (proc == NULL) {
		return -1;
	}

		// Send proc's status to shadow
	send_final_status( proc );

		// Delete proc from our list
	proc->delete_files();
	UProcList.DeleteCurrent();

	return 0;
}

/*
  Some user process exited abnormally with a core.  Attempt to send that
  core back to the submitting machine so the user can debug it.
*/
int send_core()
{
	UserProc	*proc;

		// Grab a pointer to proc which just exited
	proc = UProcList.Current();
	proc->store_core();

	return DEFAULT;
}

/*
  Update checkpoint files for every process which wants checkpointing.
  At this point, every user process should be stopped or already exited
  with a core file for checkpointing.
*/
int
update_all()
{
	UserProc	*proc;

	MyTimer->clear();
	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		switch( proc->get_state() ) {
		  
		  case CHECKPOINTING:			// already exited with a core
			return UPDATE_ONE;

		  case SUSPENDED:				// request checkpoint now
			if( proc->ckpt_enabled() ) {
				proc->request_ckpt();
				return DO_WAIT;
				break;
			}
		}
	}

	if( QuitNow ) {
		return DO_QUIT;
	} else {
		MyTimer->update_interval();
		return SUCCESS;
	}
}

/*
  Wait for some asynchronous event.  We do the pause in a loop becuase it
  may return due to a SUSPEND/CONTINUE pair which is basically a no-op
  from the state machine's point of view.
*/
int
asynch_wait()
{

	for(;;) {
#if defined(LINK_PVM)
		extern int pvm_pause();
		
		dprintf(D_FULLDEBUG, "about to call pvm_pause()\n");
		return pvm_pause();
#else
		pause();
#endif
	}
		// Can never get here
	return NO_EVENT;
}


/*
  Update the checkpoint file for the process which just exited.  It's
  possible the process has just exited for some other reason, so we
  must handle the other cases here too.
*/
int
update_one()
{
	UserProc	*proc;
	PROC_STATE	state;

		// Grab a pointer to proc which just exited
	proc = UProcList.Current();

	switch( state = proc->get_state() ) {

	  case CHECKPOINTING:
		if( proc->update_ckpt() == FALSE ) {
			return FAILURE;
		}
		proc->commit_ckpt();
		return SUCCESS;

	  case ABNORMAL_EXIT:
		proc->store_core();
		return EXITED;

	  case NON_RUNNABLE:
	  case NORMAL_EXIT:
		return EXITED;

	  default:
		EXCEPT( "Unexpected proc state (%d)\n", state );
	}

		// Can never get here
	return SUCCESS;
}

/*
  User process has just been checkpointed.  Update the shadow with its
  accumulated CPU usage so that it the user process calls getrusage(),
  then usage from previous checkpointed runs will be included.
*/
int
update_cpu()
{
	UserProc	*proc;
	void		*rusage;

		// Grab a pointer to proc which just exited
	proc = UProcList.Current();

	rusage = proc->accumulated_rusage();
	(void)REMOTE_CONDOR_send_rusage( (struct rusage*)rusage );
	return 0;
}



/*
  Send all checkpoint files back to the submitting machine.
*/
int
send_ckpt_all()
{
	UserProc	*proc;

	// dprintf( D_ALWAYS, "Entering function send_ckpt_all()\n" );

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		proc->store_ckpt();
	}
	return DEFAULT;
}

/*
  Get information regarding one user process from the shadow.  Our
  protocal with the shadow calls for RPC's with us as the client and
  the shadow as the server, so we do the asking.
*/
UserProc *
get_job_info()
{
	char	buf[1024];

	char	a_out[ _POSIX_PATH_MAX ];
	char	orig_file[ _POSIX_PATH_MAX ];
	char	target_file[ _POSIX_PATH_MAX ];
	uid_t	uid;
	gid_t	gid;
	int		scm;
	UserProc	*u_proc;
	int		id = -1;
	condor_signal_t		soft_kill;
	struct passwd	*pwd_entry;

	ClassAd *ad = new ClassAd;
	id = REMOTE_CONDOR_work_request(ad, a_out, target_file, orig_file, 
		&soft_kill);

	uid = REMOTE_CONDOR_getuid();
	gid = REMOTE_CONDOR_getgid();

	determine_user_ids( uid, gid );

	dprintf( D_ALWAYS, "User uid set to %d\n", uid );
	dprintf( D_ALWAYS, "User gid set to %d\n", gid );

	set_user_ids( uid, gid );

	int universe;
	ad->LookupInteger(ATTR_JOB_UNIVERSE, universe);

	switch(universe) {
#if defined(LINK_PVM)
	    case CONDOR_UNIVERSE_PVMD:
		u_proc = new PVMdProc( ad, a_out, target_file, uid, 
							  gid, id, soft_kill);
	 	break;
	    case CONDOR_UNIVERSE_PVM:
		u_proc = new PVMUserProc( ad, a_out, target_file, 
						 uid, gid, id, soft_kill);
		break;
#endif
	    default:
		u_proc = new UserProc( ad, orig_file, target_file, uid, 
							  gid, id, soft_kill);
		break;
	}

	u_proc->display();

	return u_proc;
}

/*
  We have been asked to leave the machine, and have requested all of our
  user processes to exit.  Unfortunately however, one or more user
  processes hasn't exited after being sent a reqest and waiting a
  reasonable time.  Now we kill them all with SIGKILL (-9), delete
  their files, and remove them from our list.  We don't try to send
  any status's to the shadow becuase some process may have left our
  RPC stream to the shadow in an inconsitent state and we could hang
  if we tried using that.
*/
int
cleanup()
{
	UserProc	*proc;

	UProcList.Rewind();
	while( proc = UProcList.Next() ) {
		if ( proc->get_class() != CONDOR_UNIVERSE_PVMD ) {
			proc->kill_forcibly();
			proc->delete_files();
			UProcList.DeleteCurrent();
		}
	}
	UProcList.Rewind();
	while ( proc = UProcList.Next() ) {
		if ( proc->get_class() == CONDOR_UNIVERSE_PVMD ) {
			proc->kill_forcibly();
			proc->delete_files();
			UProcList.DeleteCurrent();
		}
	}
	return 0;
}

/*
  Clean up as well as we can in the face of an exception.  Make as few
  assumptions as possible about the state of things here - the
  exception could be called from anywhere.

  This function returns no useful value, and should be of type "void",
  but it has to get registered with the EXCEPT mechanism which is older
  and not typed correctly...
*/
int
exception_cleanup(int,int,char*)
{
	sigset_t	mask;

		// Don't want to be called recursively by any further exceptions
	_EXCEPT_Cleanup = 0;

	alarm( 0 );			// cancel any pending alarm
	(void)cleanup();	// get rid of all user processes

		// make sure we don't have alarm's blocked
	sigemptyset( &mask );
	sigaddset( &mask, SIGALRM );
	sigprocmask( SIG_UNBLOCK, &mask, 0 );

		// Hang around awhile to make sure our last log messages get out...
		// I don't think this should be needed, but it is - mike
	sleep( 10 );
	return 0;
}

/* int	AvoidNFS = 0; */

char Condor_CWD[ _POSIX_PATH_MAX ];

#if defined(OSF1) || defined(HPUX)
/*
  None of this stuff is ever used, but it must be here so that we
  can link with the remote system call library without getting
  "undefined" errors.
*/
int SyscallInProgress;
int CkptWanted;
int KillWanted;
extern "C" {
	void do_kill();
	void ckpt();
}
#endif

#if defined( NO_CKPT )
void do_kill(){}
void ckpt(){}
extern "C"	void _updateckpt( char *foo, char *bar, char *glarch ) {}
#endif
