/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "condor_common.h"
#include "condor_config.h"
#include "condor_debug.h"
#include "condor_string.h"
#include "condor_uid.h"
#include "name_tab.h"
#include "proto.h"
#include "condor_io.h"
#include "condor_attributes.h"
#include "condor_classad.h"
#include "fileno.h"
#include "renice_self.h"
#include "my_hostname.h"
#include "starter_common.h"

const mode_t REGULAR_FILE_MODE =
	S_IRUSR | S_IWUSR |					// rw- for user
	S_IRGRP |					        // r-- for group
	S_IROTH;						    // r-- for other

const mode_t EXECUTABLE_FILE_MODE =
	S_IRUSR | S_IWUSR | S_IXUSR |		// rwx for user
	S_IRGRP |           S_IXGRP |		// r-x for group
	S_IROTH |           S_IXOTH;		// r-x for other

const mode_t LOCAL_DIR_MODE =
	S_IRUSR | S_IWUSR | S_IXUSR |		// rwx for user
	S_IRGRP | S_IWGRP | S_IXGRP |		// rwx for group
	S_IROTH | S_IWOTH | S_IXOTH;		// rwx for other


#include "user_proc.h"
#include "condor_sys.h"

extern "C" {
	void _updateckpt( char *, char *, char * );
}

extern sigset_t	ChildSigMask;
extern NameTable SigNames;
extern NameTable TransferStates;
extern char *InitiatingHost;
extern ReliSock	*SyscallStream;			// ReliSock stream to shadow for remote system calls
extern int ExecTransferAttempts;// How many attempts at getting the initial ckpt

int UserProc::proc_index = 1;

extern NameTable JobClasses;
extern NameTable ProcStates;
extern int Running_PVMD;

UserProc::UserProc( ClassAd *ad, char *orig, char *targ, uid_t u, uid_t g, int id, int soft ) :
	uid( u ),
	gid( g ),
	v_pid( id ),
	pid( 0 ),
	exit_status_valid( FALSE ),
	exit_status( 0 ),
	soft_kill_sig( soft ),
	ckpt_wanted( FALSE ),
	state( NEW ),
	new_ckpt_created( FALSE ),
	ckpt_transferred( FALSE ),
	core_created( FALSE ),
	core_transferred( FALSE ),
	exit_requested( FALSE ),
	user_time( 0 ),
	sys_time( 0 ),
	guaranteed_user_time( 0 ),
	guaranteed_sys_time( 0 ),
	transfer_buf( 0 ),
	transfer_buf_size( 0 ),
	FD_from( -1 ),
	FD_to( -1 ),
	transfer_status( TRANSFER_UNINITIALIZED )
{
	if ( ad->LookupInteger(ATTR_CLUSTER_ID, cluster) != 1 ) {
		EXCEPT ( "NO %s in job ad!\n", ATTR_CLUSTER_ID );
	}
	if ( ad->LookupInteger(ATTR_PROC_ID, proc) != 1 ) {
		EXCEPT ( "NO %s in job ad!\n", ATTR_PROC_ID );
	}
	if ( ad->LookupInteger(ATTR_JOB_UNIVERSE, job_class) != 1 ) {
		EXCEPT ( "NO %s in job ad!\n", ATTR_JOB_UNIVERSE );
	}
	if ( ad->LookupInteger(ATTR_IMAGE_SIZE, image_size) != 1 ) {
		EXCEPT ( "NO %s in job ad!\n", ATTR_IMAGE_SIZE );
	}

	char	buf[ ATTRLIST_MAX_EXPRESSION ];
	char	*value;
	mode_t	omask;

	if ((ad->LookupString(ATTR_JOB_INPUT, buf) == TRUE) && (buf[0])){
		in = new char [ strlen(buf) + 1 ];
		strcpy( in, buf );
	} else {
		in = strdup("/dev/null");
	}

	if ((ad->LookupString(ATTR_JOB_OUTPUT, buf) == TRUE) && (buf[0])) {
		out = new char [ strlen(buf) + 1 ];
		strcpy( out, buf );
	} else {
		out = strdup("/dev/null");
	}

	if ((ad->LookupString(ATTR_JOB_ERROR, buf) == TRUE) && (buf[0])) {
		err = new char [ strlen(buf) + 1 ];
		strcpy( err, buf );
	} else {
		err = strdup("/dev/null");
	}

	if ( ad->LookupString(ATTR_JOB_ROOT_DIR, buf) != 1 ) {
		EXCEPT ( "No %s in ad!\n", ATTR_JOB_ROOT_DIR );
	}
	rootdir = new char [ strlen(buf) + 1 ];
	strcpy( rootdir, buf );

	if ( ad->LookupString(ATTR_JOB_CMD, buf) != 1 ) {
		EXCEPT ( "No %s in ad!\n", ATTR_JOB_CMD );
	}
	cmd = new char [ strlen(buf) + 1 ];
	strcpy( cmd, buf );

	if ( ad->LookupString(ATTR_JOB_ARGUMENTS, buf) != 1 ) {
		EXCEPT ( "No %s in ad!\n", ATTR_JOB_ARGUMENTS );
	}
	args = new char [ strlen(buf) + 1 ];
	strcpy( args, buf );

	if ( ad->LookupString(ATTR_JOB_ENVIRONMENT, buf) != 1 ) {
		EXCEPT ( "No %s in ad!\n", ATTR_JOB_ENVIRONMENT );
	}
	env = new char [ strlen(buf) + 1 ];
	strcpy( env, buf );

	orig_ckpt = new char [ strlen(orig) + 1 ];
	strcpy( orig_ckpt, orig );

	target_ckpt = new char [ strlen(targ) + 1 ];
	strcpy( target_ckpt, targ );

		// Generate a directory where process can run and do its checkpointing
	omask = umask(0);
	sprintf( buf, "dir_%d", getpid() );
	local_dir = new char [ strlen(buf) + 1 ];
	strcpy( local_dir, buf );
    
	if( mkdir(local_dir,LOCAL_DIR_MODE) < 0 ) {
        if ( errno != EEXIST ) {
            EXCEPT( "mkdir(%s,0%o), errno:%d", 
                    local_dir, LOCAL_DIR_MODE, errno );
        }
	}

	(void)umask(omask);

	sprintf( buf, "%s/condor_exec.%d.%d", local_dir, cluster, proc );
	cur_ckpt = new char [ strlen(buf) + 1 ];
	strcpy( cur_ckpt, buf );

	sprintf( buf, "%s/condor_exec.%d.%d.tmp", local_dir, cluster, proc );
	tmp_ckpt = new char [ strlen(buf) + 1 ];
	strcpy( tmp_ckpt, buf );

	sprintf( buf, "%s/core", local_dir );
	core_name = new char [ strlen(buf) + 1 ];
	strcpy( core_name, buf );

		// Find out if user wants checkpointing
#if DOES_CHECKPOINTING
	ad->LookupInteger(ATTR_WANT_CHECKPOINT, ckpt_wanted);
#else
	ckpt_wanted = FALSE;
	dprintf(D_ALWAYS,
			"This platform doesn't implement checkpointing yet\n"
	);
#endif

		// find out if user wants to limit size of coredumps
	env_obj.add_string( env );	// set up environment as an object
	value = env_obj.getenv( "CONDOR_CORESIZE" );
	if( value ) {
		coredump_limit_exists = TRUE;
		coredump_limit = atoi( value );
	} else {
		coredump_limit_exists = FALSE;
	}

	/* Port regulation for user job */
    char *low = NULL, *high = NULL;
    int low_port, high_port;

    if ( (low = param("LOWPORT")) != NULL ) {
        if ( (high = param("HIGHPORT")) != NULL ) {
            low_port = atoi(low);
            high_port = atoi(high);
            sprintf(buf, "_condor_LOWPORT=%d", low_port);
            env_obj.add_string(buf);
            sprintf(buf, "_condor_HIGHPORT=%d", high_port);
            env_obj.add_string(buf);
            free(low);
            free(high);
        } else {
            dprintf(D_ALWAYS, "LOWPORT is defined but HIGHPORT is not!\n");
            dprintf(D_ALWAYS, "LOWPORT will be ignored!\n");
            free(low);
        }
    }
    /* end - Port regulation for user job */
}

UserProc::~UserProc()
{
	delete [] in;
	delete [] out;
	delete [] err;
	delete [] rootdir;
	delete [] cmd;
	delete [] args;
	delete [] env;
	delete [] orig_ckpt;
	delete [] target_ckpt;
	delete [] local_dir;
	delete [] cur_ckpt;
	delete [] tmp_ckpt;
	delete [] core_name;
}

void
UserProc::display()
{
	dprintf( D_ALWAYS, "User Process %d.%d {\n", cluster, proc );

	dprintf( D_ALWAYS, "  in = %s\n", in );
	dprintf( D_ALWAYS, "  out = %s\n", out );
	dprintf( D_ALWAYS, "  err = %s\n", err );
	dprintf( D_ALWAYS, "  rootdir = %s\n", rootdir );
	dprintf( D_ALWAYS, "  cmd = %s\n", cmd );
	dprintf( D_ALWAYS, "  args = %s\n", args );
	dprintf( D_ALWAYS, "  env = %s\n", env );

	dprintf( D_ALWAYS, "  orig_ckpt = %s\n", orig_ckpt );
	dprintf( D_ALWAYS, "  target_ckpt = %s\n", target_ckpt );

	dprintf( D_ALWAYS, "  local_dir = %s\n", local_dir );
	dprintf( D_ALWAYS, "  cur_ckpt = %s\n", cur_ckpt );
	dprintf( D_ALWAYS, "  tmp_ckpt = %s\n", tmp_ckpt );
	dprintf( D_ALWAYS, "  core_name = %s\n", core_name );

	dprintf( D_ALWAYS, "  uid = %d, gid = %d\n", uid, gid );

	dprintf( D_ALWAYS, "  v_pid = %d\n", v_pid );
	if( pid ) {
		dprintf( D_ALWAYS, "  pid = %d\n", pid );
	} else {
		dprintf( D_ALWAYS, "  pid = (NOT CURRENTLY EXECUTING)\n" );
	}

	display_bool( D_ALWAYS, "  exit_status_valid", exit_status_valid );
	if( exit_status_valid ) {
		dprintf( D_ALWAYS, "  exit_status = 0x%x\n", exit_status );
	} else {
		dprintf( D_ALWAYS, "  exit_status = (NEVER BEEN EXECUTED)\n" );
	}

	display_bool( D_ALWAYS, "  ckpt_wanted", ckpt_wanted );
	display_bool( D_ALWAYS, "  coredump_limit_exists", coredump_limit_exists );
	if( coredump_limit_exists ) {
		dprintf( D_ALWAYS, "  coredump_limit = %d\n", coredump_limit );
	}

	dprintf( D_ALWAYS, "  soft_kill_sig = %d\n", soft_kill_sig );
	dprintf( D_ALWAYS, "  job_class = %s\n", JobClasses.get_name(job_class) );

	dprintf( D_ALWAYS, "  state = %s\n", ProcStates.get_name(state) );
	display_bool( D_ALWAYS, "  new_ckpt_created", new_ckpt_created );
	display_bool( D_ALWAYS, "  ckpt_transferred", ckpt_transferred );
	display_bool( D_ALWAYS, "  core_created", core_created );
	display_bool( D_ALWAYS, "  core_transferred", core_transferred );
	display_bool( D_ALWAYS, "  exit_requested", exit_requested );
	dprintf( D_ALWAYS, "  image_size = %d blocks\n", image_size );

	dprintf( D_ALWAYS, "  user_time = %d\n", user_time );
	dprintf( D_ALWAYS, "  sys_time = %d\n", sys_time );

	dprintf( D_ALWAYS, "  guaranteed_user_time = %d\n", guaranteed_user_time );
	dprintf( D_ALWAYS, "  guaranteed_sys_time = %d\n", guaranteed_sys_time );

	dprintf( D_ALWAYS, "}\n" );

}


/*
  We have been given a filename describing where we should get our
  executable from.  That name could be of the form "<hostname>:<pathname>",
  or it could just be <pathname>.  Also the <pathname> part could contain
  macros to be looked up in our config file.  We do that macro expansion
  here, and also check the <hostname> part.  If the hostname refers
  to our own host, we set "on_this_host" to TRUE and if it refers to
  the submitting host, we set it to FALSE.  For now, it is an error
  if it refers to some third host.  We then ruturn a pointer to the
  macro expanded pathname in a static data area.
*/
char *
UserProc::expand_exec_name( int &on_this_host )
{
	static char answer[ _POSIX_PATH_MAX ];
	char	*host_part;
	char	*path_part;
	char	*tmp;
	char    *ThisHost;

	ThisHost = my_full_hostname();

	if( strchr(orig_ckpt,':') ) {		// form is <hostname>:<path>
		host_part = strtok( orig_ckpt, " \t:" );
		if( host_part == NULL ) {
			EXCEPT( "Can't find host part" );
		}
		path_part = strtok( NULL, " \t:");
		if( path_part == NULL ) {
			EXCEPT( "Can't find path part" );
		}
		if( strincmp(host_part,ThisHost, 
					MIN(strlen(host_part), strlen(ThisHost) )) == MATCH ) {
			on_this_host = TRUE;
		} else {
			if( strincmp(host_part,InitiatingHost,
				MIN(strlen(host_part), strlen(InitiatingHost))) == MATCH ) {
				on_this_host = FALSE;
			} else {
				EXCEPT( "Unexpected host_part \"%s\", ThisHost %s", 
					   host_part, ThisHost );
			}
		}
	} else if( strncmp("/afs",orig_ckpt,4) == MATCH ) {
		on_this_host = TRUE;
		path_part = orig_ckpt;
	} else {	// form is <path>
		on_this_host = FALSE;
		path_part = orig_ckpt;
	}

		// expand macros in the pathname part
	tmp = macro_expand( path_part );
	strcpy( answer, tmp );
	FREE( tmp );

	if( on_this_host ) {
		dprintf( D_ALWAYS, "Executable is located on this host\n" );
	} else {
		dprintf( D_ALWAYS, "Executable is located on submitting host\n" );
	}

	dprintf( D_ALWAYS, "Expanded executable name is \"%s\"\n", answer );
	return answer;
}

/*
  The executable file is on this host already, so just make a symbolic
  link to it.
*/
int
UserProc::link_to_executable( char *src, int & error_code )
{
	int status;

	errno = 0;
	status =  symlink( src, cur_ckpt );
	if( status < 0 ) {
		dprintf( D_ALWAYS,
			"Can't create sym link from \"%s\" to \"%s\", errno = %d",
			src, cur_ckpt, errno
		);
	} else {
		dprintf( D_ALWAYS,
			"Created sym link from \"%s\" to \"%s\"\n", src, cur_ckpt
		);
	}
	error_code = errno;
	return status;
}

extern "C" int carmi_get_file( char *, char *, int );

/*
  The executable file is on the submitting host.  Transfer a copy
  to this machine.
*/
int
UserProc::transfer_executable( char *src, int &error_code )
{
	int		status;
	int		attempts = 0;
	int		tmp_errno;

	dprintf( D_ALWAYS, "Going to try %d %s at getting the inital executable\n",
		ExecTransferAttempts, ExecTransferAttempts==1?"attempt":"attempts" );

	do
	{
		errno = 0;
#if defined(LINK_PVM)
		status = carmi_get_file( src, cur_ckpt, EXECUTABLE_FILE_MODE );
#else
		status = get_file( src, cur_ckpt, EXECUTABLE_FILE_MODE );
#endif
		tmp_errno = errno;

		attempts++;
	
		if( status < 0 ) {
			dprintf( D_ALWAYS,
				"Failed to fetch orig ckpt file \"%s\" into \"%s\", "
				"attempt = %d, errno = %d\n", src, cur_ckpt, attempts, errno );
		} else {
			dprintf( D_ALWAYS,
				"Fetched orig ckpt file \"%s\" into \"%s\" with %d %s\n", 
				src, cur_ckpt, attempts, attempts==1?"attempt":"attempts" );
		}
	}
	while( (status < 0) && (attempts < ExecTransferAttempts) );
	
	error_code = tmp_errno;
	return status;
}

/*
  Check to see if an executable file is properly linked for execution
  with the Condor remote system call library.
*/
int
UserProc::linked_for_condor()
{
	if( sysapi_magic_check(cur_ckpt) < 0 ) {
		state = BAD_MAGIC;
		dprintf( D_ALWAYS, "magic_check() failed\n" );
		return FALSE;
	}

	// Don't look for sym tab in ckpt files or PVM processes
	if( this->is_restart() && job_class != CONDOR_UNIVERSE_PVM ) {	
		if( sysapi_symbol_main_check(cur_ckpt) < 0 ) {
			state = BAD_LINK;
			dprintf( D_ALWAYS, "symbol_main_check() failed\n" );
			return FALSE;
		}
	}

	dprintf( D_FULLDEBUG, "Done verifying executable file\n" );
	return TRUE;
}

int
UserProc::fetch_ckpt()
{
	int		on_this_host;
	char	*new_path;
	int		status;
	int		error_code;

	dprintf ( D_FULLDEBUG, "Entered fetch_ckpt()\n" );

	new_path = expand_exec_name( on_this_host );

	if( on_this_host ) {
		status = link_to_executable( new_path, error_code );
	} else {
		status = transfer_executable( new_path, error_code );
	}

	if( status < 0 ) {
		state = CANT_FETCH;
		exit_status = error_code;
		return FALSE;
	}

	switch( job_class ) {

	  case CONDOR_UNIVERSE_STANDARD:
		if( linked_for_condor() ) {
			state = RUNNABLE;
			return TRUE;
		} else {
			return FALSE;
		}
	  case CONDOR_UNIVERSE_VANILLA:
	  case CONDOR_UNIVERSE_PVM:
	  case CONDOR_UNIVERSE_PVMD:
		state = RUNNABLE;
		return TRUE;

	  default:
		EXCEPT( "Unknown job class (%d)", job_class );
	}

	// We can never get here
	return FALSE;
}

void
UserProc::check_if_runnable()
{
	switch( job_class ) {

	  case CONDOR_UNIVERSE_STANDARD:
		if( linked_for_condor() ) {
			state = RUNNABLE;
		}
		break;

	  case CONDOR_UNIVERSE_VANILLA:
	  case CONDOR_UNIVERSE_PVM:
	  case CONDOR_UNIVERSE_PVMD:
		state = RUNNABLE;
		break;

	  default:
		EXCEPT( "Unknown job class (%d)", job_class );
	}
}


void
UserProc::init_transfer(char *from_file, char *to_file, TRANSFER_MODE mode,
						int new_file_mode)
{
	char	*tmp;

	transfer_mode = mode;
	switch (mode) {
	    case REMOTE_TO_LOCAL:
		/* XXX I wonder if this type cast to open_flags_t is ok... -psilord */
		    FD_from = REMOTE_CONDOR_open( from_file, (open_flags_t)O_RDONLY, 0);
			FD_to = open( to_file, O_WRONLY|O_CREAT|O_TRUNC, 
						 new_file_mode );
		    
		    break;
		case LOCAL_TO_REMOTE:
		    FD_from = open( from_file, O_RDONLY, 0 );
		/* XXX I wonder if this type cast to open_flags_t is ok... -psilord */
			FD_to = REMOTE_CONDOR_open( to_file, 
						   (open_flags_t)(O_WRONLY | O_CREAT | O_TRUNC),
						   new_file_mode );
		    
			break;
	}

	transfer_status = TRANSFER_INITIALIZED;
	if ( (tmp=param("TRANSFER_BLOCK_SIZE")) == NULL ) {
		transfer_buf_size = 8192;
	} else {
		transfer_buf_size = atoi(tmp);
	}
	dprintf( D_ALWAYS, "transfer_buf_size set to %d\n", 
			transfer_buf_size );
//	transfer_buf = new char [transfer_buf_size];
	if (job_class == CONDOR_UNIVERSE_PVM) {
		dprintf( D_ALWAYS, "get_tid() = 0x%x\n", get_tid() );
	}
	transfer_buf = malloc(transfer_buf_size + 100);
	dprintf( D_ALWAYS, "transfer_buf_addr = 0x%lx\n", transfer_buf );
	if (job_class == CONDOR_UNIVERSE_PVM) {
		dprintf( D_ALWAYS, "get_tid() = 0x%x\n", get_tid() );
	}
	if (FD_from < 0 || FD_to < 0) {
		end_transfer( TRANSFER_FAILURE );
	}
}


void
UserProc::init_fetch()
{
	int		on_this_host;
	char	*new_path;
	int		status;
	int		error_code;

#if 0
	new_path = expand_exec_name( on_this_host );

	if( on_this_host ) {
		status = link_to_executable( new_path, error_code );
		transfer_status = TRANSFER_COMPLETE;
		check_if_runnable();
	} else {
		init_transfer( new_path, cur_ckpt, REMOTE_TO_LOCAL, 
					  EXECUTABLE_FILE_MODE );
	}
#endif

}


void
UserProc::end_transfer( TRANSFER_STATE completion_state )
{
	transfer_status = completion_state;
	dprintf( D_ALWAYS, "end_transfer: setting status to %s\n",
			TransferStates.get_name(completion_state));
//	delete [] transfer_buf;
	dprintf( D_ALWAYS, "pre-free:transfer_buf_addr = 0x%lx\n", transfer_buf );
	free( transfer_buf );
	if (job_class == CONDOR_UNIVERSE_PVM) {
		dprintf( D_ALWAYS, "get_tid() = 0x%x\n", get_tid() );
	}
	transfer_buf = 0;
	transfer_buf_size = 0;
	if (job_class == CONDOR_UNIVERSE_PVM) {
		dprintf( D_ALWAYS, "get_tid() = 0x%x\n", get_tid() );
	}
	if (FD_to != -1) {
		switch ( transfer_mode ) {
		    case REMOTE_TO_LOCAL:
			    close( FD_to );
				break;
			case LOCAL_TO_REMOTE:
				REMOTE_CONDOR_close( FD_to );
				break;
		}
		FD_to = -1;
	}
	if (FD_from != -1) {
		switch ( transfer_mode ) {
		    case REMOTE_TO_LOCAL:
			    REMOTE_CONDOR_close( FD_from );
				break;
			case LOCAL_TO_REMOTE:
				close( FD_from );
				break;
		}
		FD_from = -1;
	}
}


#define MATCH 0			/* result of strcmp */
#define EQUAL '='		/* chars looked for during parsing */
#define SPACE ' '
#define TAB '\t'
#define SEMI ';'

int
UserProc::is_restart()
{
	if( strcmp(orig_ckpt,target_ckpt) == MATCH ) {
		return FALSE;
	} else {
		return TRUE;
	}
}

void
UserProc::execute()
{
	int		argc;
	char	*argv[ 2048 ];
	char	**argp;
	char	**envp;
	sigset_t	sigmask;
	long	arg_max;
	char	*tmp;
	char	a_out_name[ _POSIX_PATH_MAX ];
	int		user_syscall_fd;
	ReliSock*	new_reli = NULL;

		// We will use mkargv() which modifies its arguments in place
		// so we not use the original copy of the arguments
	if( (arg_max=sysconf(_SC_ARG_MAX)) == -1 ) {
		arg_max = _POSIX_ARG_MAX;
	}
	tmp = new char [arg_max];
	strncpy( tmp, args, arg_max );

	sprintf( a_out_name, "condor_exec.%d.%d", cluster, proc );

		// Set up arg vector according to class of job
	switch( job_class ) {

	  case CONDOR_UNIVERSE_STANDARD:
		argv[0] = a_out_name;
		argv[1] = in;
		argv[2] = out;
		argv[3] = err;
		mkargv( &argc, &argv[4], tmp );
		break;

	  case CONDOR_UNIVERSE_PVM:
		argv[0] = a_out_name;
		argv[1] = "-1";
		argv[2] = in;
		argv[3] = out;
		argv[4] = err;
		mkargv( &argc, &argv[5], tmp );
		break;

	  case CONDOR_UNIVERSE_VANILLA:
		argv[0] = a_out_name;
		mkargv( &argc, &argv[1], tmp );
		break;
	}

		// set up environment vector
	envp = env_obj.get_vector();

		// We may run more than one of these, so each needs its own
		// remote system call connection to the shadow
	if( job_class == CONDOR_UNIVERSE_PVM ) {
		new_reli = NewConnection( v_pid );
		user_syscall_fd = new_reli->get_file_desc();
	}

		// handle USER_JOB_WRAPPER
	support_job_wrapper(a_out_name,&argc,argv);

		// print out arguments to execve
	dprintf( D_ALWAYS, "Calling execve( \"%s\"", a_out_name );
	for( argp = argv; *argp; argp++ ) {							// argv
		dprintf( D_ALWAYS | D_NOHEADER, ", \"%s\"", *argp );
	}
	dprintf( D_ALWAYS | D_NOHEADER, ", 0" );
	for( argp = envp; *argp; argp++ ) {							// envp
		dprintf( D_ALWAYS | D_NOHEADER, ", \"%s\"", *argp );
	}
	dprintf( D_ALWAYS | D_NOHEADER, ", 0 )\n" );



	if( (pid = fork()) < 0 ) {
		EXCEPT( "fork" );
	}

	if( pid == 0 ) {	// the child

			// Block only these 3 signals which have special meaning for
			// checkpoint/restart purposes.  Leave other signals ublocked
			// so that if we get an exception during the restart process,
			// we will get a core file to debug.
		sigemptyset( &sigmask );
		sigaddset( &sigmask, SIGUSR1 );
		sigaddset( &sigmask, SIGUSR2 );
		sigaddset( &sigmask, SIGTSTP );
		sigprocmask( SIG_SETMASK, &sigmask, 0 );

			// renice
		renice_self( "JOB_RENICE_INCREMENT" );

			// make certain the syscall sockets which are being passed
			// to the user job are setup to be blocking sockets.  this
			// is done by calling timeout(0) CEDAR method.
			// we must do this because the syscall lib does _not_ 
			// expect to see any failures due to errno EAGAIN...
		if ( SyscallStream ) {
			SyscallStream->timeout(0);
		}
		if ( new_reli ) {
			new_reli->timeout(0);
		}

		if( chdir(local_dir) < 0 ) {
			EXCEPT( "chdir(%s)", local_dir );
		}

			// child process should have only it's submitting uid, and cannot
			// switch back to root or some other uid.
		set_user_priv_final();

		switch( job_class ) {
		  
		  case CONDOR_UNIVERSE_PVM:
			dup2( user_syscall_fd, RSC_SOCK );
			break;

		  case CONDOR_UNIVERSE_VANILLA:
			(void)close( RSC_SOCK );
			close_unused_file_descriptors();	// shouldn't need this
			break;
		}


			// One last time, make sure we're not root
		if( getuid() == 0 ) {
				// EXCEPT( "We're about to start as root, aborting." );
				// We can't see this error message, so just exit.
			exit(4);
		}

			// Everything's ready, start it up...
		errno = 0;
		execve( a_out_name, argv, envp );

			// A successful call to execve() never returns, so it is an
			// error if we get here.  A number of errors are possible
			// but the most likely is that there is insufficient swap
			// space to start the new process.  We don't try to log
			// anything, since we have the UID/GID of the job's owner
			// and cannot write into the log files...
		dprintf( D_ALWAYS, "Exec failed - errno = %d\n", errno );
		exit( EXECFAILED );
	}

		// The parent
	dprintf( D_ALWAYS, "Started user job - PID = %d\n", pid );
	delete [] tmp;
	state = EXECUTING;

	if( new_reli ) {
		delete new_reli;
	}
}

// inline void
void
do_unlink( const char *name )
{
	while( unlink(name) < 0 ) {
		if( errno != ETXTBSY ) {
			if (errno == ENOENT) {
				dprintf( D_ALWAYS, "\"%s\" does not exist\n", name );
				return;
			} else {
				dprintf( D_ALWAYS,
						"Can't unlink \"%s\" - errno = %d\n", name, errno
						);
				return;
			}
		}
	}
	dprintf( D_ALWAYS, "Unlinked \"%s\"\n", name );
}


void
UserProc::delete_files()
{
	do_unlink( cur_ckpt );
	do_unlink( tmp_ckpt );
	do_unlink( core_name );

	if( rmdir(local_dir) < 0 ) {
		dprintf( D_ALWAYS,
			"Can't remove directory \"%s\" - errno = %d\n", local_dir, errno
		);
	} else {
		dprintf( D_ALWAYS, "Removed directory \"%s\"\n", local_dir );
	}

	if ( transfer_status == TRANSFER_IN_PROGRESS || 
		transfer_status == TRANSFER_INITIALIZED ) {
		if ( transfer_mode == REMOTE_TO_LOCAL && FD_from != -1 ) {
			dprintf( D_ALWAYS, 
					"UserProc::delete_files() forced to close remote fd\n" );
			(void)REMOTE_CONDOR_close( FD_from );
		} else if ( transfer_mode == LOCAL_TO_REMOTE && FD_to != -1 ) {
			dprintf( D_ALWAYS, 
					"UserProc::delete_files() forced to close remote fd\n" );
			(void)REMOTE_CONDOR_close( FD_to );
		}
	}
}

void
UserProc::handle_termination( int exit_st )
{
	struct stat	buf;

	exit_status = exit_st;
	exit_status_valid = TRUE;
	accumulate_cpu_time();

	if ( exit_requested ) {  // job exited by request
		dprintf(D_ALWAYS,"Process exited by request\n");
		state = NON_RUNNABLE;
	} else if( WIFEXITED(exit_status) ) {	// exited on own accord with some status
		dprintf( D_ALWAYS,
				"Process %d exited with status %d\n", pid, 
				WEXITSTATUS(exit_status)
		);
		if( WEXITSTATUS(exit_status) == EXECFAILED ) {
			dprintf( D_ALWAYS,
				"EXEC of user process failed, probably insufficient swap\n"
			);
			state = NON_RUNNABLE;
		} else {
			state = NORMAL_EXIT;
			commit_cpu_time();
		}
	} else {
		dprintf( D_ALWAYS,
			"Process %d killed by signal %d\n", pid, WTERMSIG(exit_status)
		);
		switch( WTERMSIG(exit_status) ) {
		  case SIGQUIT:				// exited for a checkpoint
			dprintf( D_ALWAYS, "Process eixted for checkpoint\n" );
			state = CHECKPOINTING;
			break;
		  case SIGUSR1:
		  case SIGKILL:				// exited by request - no ckpt
			dprintf( D_ALWAYS, "Process exited by request\n" );
			state = NON_RUNNABLE;
			break;
		  default:					// exited abnormally due to signal
			dprintf( D_ALWAYS, "Process exited abnormally\n" );
			state = ABNORMAL_EXIT;
			commit_cpu_time();
		}
			
	}
	pid = 0;

	switch( state ) {
	    case CHECKPOINTING:
        case ABNORMAL_EXIT:
		    if( core_is_valid(core_name) ) {
				dprintf( D_ALWAYS, "A core file was created\n" );
				core_created = TRUE;
				break;
			}
		default:
			dprintf( D_FULLDEBUG, "No core file was created\n" );
			core_created = FALSE;
			(void)unlink( core_name );	// remove any incomplete core
	}

	if( get_class() == CONDOR_UNIVERSE_PVMD ) {
		Running_PVMD = FALSE;
	}

}

void
UserProc::send_sig( int sig )
{
        priv_state      priv;

	if( !pid ) {
		dprintf( D_FULLDEBUG,
		"UserProc::send_signal() called, but user job pid NULL\n" );
		return;
	}

        priv = set_root_priv();

	if( sig != SIGCONT ) {
		if( kill(pid,SIGCONT) < 0 ) {
			if( errno == ESRCH ) {	// User proc already exited
				return;
			}
			EXCEPT( "kill(%d,SIGCONT)", pid  );
		}
	}

	if( kill(pid,sig) < 0 ) {
		if( errno == ESRCH ) {	// User proc already exited
			return;
		}
		EXCEPT( "kill(%d,%d)", pid, sig );
	}

	set_priv(priv);

	dprintf( D_ALWAYS, "Sent signal %s to user job %d\n", SigNames.get_name(sig), pid);
}

void
UserProc::commit_ckpt()
{
	// We'd like to just rename() the file, but if we don't do the unlink(),
	// then on some platforms the "tmp_ckpt" file gets removed from the
	// directory, but the disk space doesn't get freed.

	(void)unlink( cur_ckpt );

	if( rename(tmp_ckpt,cur_ckpt) < 0 ) {
		EXCEPT( "rename" );
	}
	state = RUNNABLE;
}

void
UserProc::store_ckpt()
{
	char	tmp_name[ _POSIX_PATH_MAX ];
	int		status;

	if( new_ckpt_created ) {
		sprintf( tmp_name, "%s.tmp", target_ckpt );
		delay( 15 );
		status = send_file( cur_ckpt, tmp_name, REGULAR_FILE_MODE );
		if( status < 0 ) {
			dprintf( D_ALWAYS, "File NOT xferred\n" );
			dprintf( D_ALWAYS, "(Insufficient space on submitting machine?)\n");
			dprintf( D_ALWAYS, "status = %d, errno = %d\n", status, errno );
			(void)REMOTE_CONDOR_unlink(tmp_name);  // rm partial file
			ckpt_transferred = FALSE;
		} else {
			if( REMOTE_CONDOR_rename(tmp_name,target_ckpt) < 0 ) {
				EXCEPT("Remote rename \"%s\" to \"%s\"", tmp_name, target_ckpt);
			}
			commit_cpu_time();
			ckpt_transferred = TRUE;
		}
	} else {
		dprintf( D_ALWAYS, "Checkpoint file never updated - not sending\n" );
		ckpt_transferred = FALSE;
	}
}


int
UserProc::update_ckpt()
{
	int		size_estimate;
	int		free_disk;
	int		answer;
	char	*old_brk, *new_brk;

#if 0
	free_disk = sysapi_disk_space( "." );

	if( !core_created ) {
		dprintf( D_ALWAYS, "No core file - checkpoint *NOT* updated\n" );
		image_size = free_disk;	// assume ran out of disk trying to creat core
		return FALSE;
	}

	size_estimate = estimate_image_size();
	dprintf( D_ALWAYS, "Estimating new ckpt needs %d kbytes\n", size_estimate );
	dprintf( D_ALWAYS, "Found %d kbytes of free disk\n", free_disk );

	if( free_disk > size_estimate ) {
		delay( 5 );
		dprintf( D_ALWAYS, "Updating checkpoint file...\n" );

		old_brk = (char *)sbrk(0);
		dprintf( D_ALWAYS, "before update break is 0x%x\n", old_brk );
		_updateckpt( tmp_ckpt, cur_ckpt, core_name );
		new_brk = (char *)sbrk(0);
		dprintf( D_ALWAYS, "after update break is 0x%x\n", new_brk );
		dprintf( D_ALWAYS,
				"difference is %d megabytes\n",
				(new_brk - old_brk) / (1024 * 1024)
                );

		image_size = physical_file_size( tmp_ckpt );  // Get real size
		if( image_size < 0 ) {
			answer = FALSE;
			image_size = free_disk;	// Assume we ran out of disk space
			dprintf( D_ALWAYS, "Checkpoint file *NOT* updated\n" );
			dprintf( D_ALWAYS, "Probably ran out of disk\n" );
		} else {
			answer = TRUE;
			new_ckpt_created = TRUE;
			dprintf( D_ALWAYS, "Checkpoint file *IS* updated\n" );
			dprintf( D_ALWAYS, "Actual ckpt file size %d kbytes\n", image_size);
		}
	} else {
		answer = FALSE;
		image_size = size_estimate;
		dprintf( D_ALWAYS, "Checkpoint file *NOT* updated\n" );
		dprintf( D_ALWAYS, "Estimated we would run out of disk\n" );
	}

	if( unlink(core_name) < 0 ) {
		EXCEPT( "unlink(%s)", core_name );
	}

	return answer;
#else
	return FALSE;
#endif
}

inline void
display_bool( int debug_flags, char *name, int value )
{
	dprintf( debug_flags, "%s = %s\n", name, value ? "TRUE" : "FALSE" );
}


void
UserProc::store_core()
{
	int		core_size;
	char	virtual_working_dir[ _POSIX_PATH_MAX ];
	char	new_name[ _POSIX_PATH_MAX ];
	int		free_disk;
	priv_state priv;

	if( !core_created ) {
		dprintf( D_ALWAYS, "No core file to send - probably ran out of disk\n");
		return;
	}

	priv = set_root_priv();
	core_size = physical_file_size( core_name );
	set_priv(priv);

	if( coredump_limit_exists ) {
		if( core_size > coredump_limit ) {
			dprintf( D_ALWAYS, "Core file size exceeds user defined limit\n" );
			dprintf( D_ALWAYS, "*NOT* sending core file.\n" );
			return;
		}
	}

	memset ( virtual_working_dir, 0, _POSIX_PATH_MAX );

	if( REMOTE_CONDOR_getwd(virtual_working_dir) < 0 ) {
		EXCEPT( "REMOTE_CONDOR_getwd(virtual_working_dir = %s)",
			virtual_working_dir != NULL?virtual_working_dir:"(null)" );
	}

	if ( virtual_working_dir[0] == '\0' ) {
		dprintf ( D_ALWAYS, "CONDOR_getwd had a problem.  errno %d\n", errno );
		return;
	}

	free_disk = REMOTE_CONDOR_free_fs_blocks( virtual_working_dir);
	if( free_disk < 0 ) {
		EXCEPT( "REMOTE_CONDOR_free_fs_blocks(virtual_working_dir = %s)",
			virtual_working_dir != NULL?virtual_working_dir:"(null)" );
	}

	dprintf( D_ALWAYS, "Core file size is %d kbytes\n", core_size );
	dprintf( D_ALWAYS, "Free disk on submitting machine is %d kbytes\n",
																free_disk );

	virtual_working_dir[0] = '.';
	virtual_working_dir[1] = '\0';
	
	if( free_disk > core_size) {
		sprintf( new_name, "%s/core.%d.%d", virtual_working_dir, cluster, proc);
		dprintf( D_ALWAYS, "Transferring core file to \"%s\"\n", new_name );
		delay( 15 );
		priv = set_root_priv();
		send_file( core_name, new_name, REGULAR_FILE_MODE );
		set_priv(priv);
		core_transferred = TRUE;
	} else {
		dprintf( D_ALWAYS, "*NOT* Transferring core file\n" );
		core_transferred = FALSE;
	}
}

void
UserProc::accumulate_cpu_time()
{
	struct tms	current;
	static struct tms	previous;

	(void)times( &current );
	user_time += current.tms_cutime - previous.tms_cutime;
	sys_time += current.tms_cstime - previous.tms_cstime;
	previous = current;
}

void
UserProc::commit_cpu_time()
{
	guaranteed_user_time = user_time;
	guaranteed_sys_time = sys_time;
}

void *
UserProc::accumulated_rusage()
{
	return bsd_rusage(user_time,sys_time);
}

void *
UserProc::guaranteed_rusage()
{
	return bsd_rusage(guaranteed_user_time, guaranteed_sys_time);
}

void *
UserProc::bsd_exit_status()
{
	return bsd_status( exit_status, state, ckpt_transferred, core_transferred );
}

void
UserProc::suspend()
{
	send_sig( SIGSTOP );
	state = SUSPENDED;
}

void
UserProc::resume()
{
	send_sig( SIGCONT );
	state = EXECUTING;
}

void
UserProc::request_ckpt()
{
	send_sig( SIGTSTP );
}

void
UserProc::request_exit()
{
	send_sig( soft_kill_sig );
}

void
UserProc::kill_forcibly()
{
	send_sig( SIGKILL );
}

extern "C"
{
int
pre_open( int, int, int ) { return 0; }
}


extern "C"      int MappingFileDescriptors();

extern "C" int
MarkOpen( const char *file, int flags, int fd, int is_remote )
{
        if( MappingFileDescriptors() ) {
                EXCEPT( "MarkOpen() called, but should never be used!" );
        }
        return fd;
}
