/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 

#ifndef USER_PROC_H
#define USER_PROC_H

#include "environ.h"
#include "condor_attributes.h"
#include "condor_classad.h"

#define EXECFAILED 255 // exit status if call to execv() fails

	// Reason user process exited (or didn't execute)
typedef enum {
	ABNORMAL_EXIT,	// process exited abnormally due to signal
	BAD_LINK,		// checkpoint file not linked with condor library
	BAD_MAGIC,		// bad magic number in checkpoint file
	CANT_FETCH,		// can't get executable file from submitting machine
	CHECKPOINTING,	// process terminated for checkpointing
	EXECUTING,		// currently executing
	NEW,			// haven't tried to execute it yet
	NON_RUNNABLE,	// process kicked off due to non-condor work
	NORMAL_EXIT,	// process exited normally with some status value
	RUNNABLE,		// ready to go, but not running
	SUSPENDED		// process is stopped
} PROC_STATE;

	// N.B. This name table *must* match the above enumerator.
#if defined(INSERT_TABLES)
static NAME_VALUE	ProcStateNames[] = {
	{ ABNORMAL_EXIT,	"ABNORMAL_EXIT",	},
	{ BAD_LINK,			"BAD_LINK",			},
	{ BAD_MAGIC,		"BAD_MAGIC",		},
	{ CHECKPOINTING,	"CHECKPOINTING",	},
	{ EXECUTING,		"EXECUTING"			},
	{ NEW,				"NEW"				},
	{ NON_RUNNABLE,		"NON_RUNNABLE",		},
	{ NORMAL_EXIT,		"NORMAL_EXIT",		},
	{ RUNNABLE,			"RUNNABLE",			},
	{ SUSPENDED,		"SUSPENDED",		},
	{ -1,				"(UNKNOWN)"			}
};
NameTable ProcStates( ProcStateNames );
#endif

typedef int JOB_CLASS;

	// N.B. This name table *must* match the above enumerator.
#if defined(INSERT_TABLES)
static NAME_VALUE	JobClassNames[] = {
	{ CONDOR_UNIVERSE_STANDARD,		"STANDARD"	},
	{ CONDOR_UNIVERSE_PVM,			"PVM",		},
	{ CONDOR_UNIVERSE_VANILLA,		"VANILLA",	},
    { CONDOR_UNIVERSE_PVMD,			"PVMD",		},
	{ -1,			"(UNKNOWN)"	}
};
NameTable JobClasses( JobClassNames );
#endif

	// State of file transfer for a process
typedef enum {
	TRANSFER_UNINITIALIZED,	// No transfer in progress
	TRANSFER_INITIALIZED,	// Ready to start transfer
	TRANSFER_IN_PROGRESS,	// In the middle of a transfer
	TRANSFER_COMPLETE,		// Transfer is all done
	TRANSFER_FAILURE		// Transfer failed
} TRANSFER_STATE;

	// N.B. This name table *must* match the above enumerator.
#if defined(INSERT_TABLES)
static NAME_VALUE	TransferStateNames[] = {
	{ TRANSFER_UNINITIALIZED,	"TRANSFER_UNINITIALIZED"	},
	{ TRANSFER_INITIALIZED,		"TRANSFER_INITIALIZED"	},
	{ TRANSFER_IN_PROGRESS,		"TRANSFER_IN_PROGRESS"	},
	{ TRANSFER_COMPLETE,		"TRANSFER_COMPLETE"		},
	{ TRANSFER_FAILURE,			"TRANSFER_FAILURE" },
	{ -1,				"(UNKNOWN)"		}
};
NameTable TransferStates( TransferStateNames );
#endif

typedef enum {
	REMOTE_TO_LOCAL,	// File transfer is going there to here
	LOCAL_TO_REMOTE		// File transfer is from here to there
} TRANSFER_MODE;

	// N.B. This name table *must* match the above enumerator.
#if defined(INSERT_TABLES)
static NAME_VALUE	TransferModeNames[] = {
	{ REMOTE_TO_LOCAL,	"REMOTE_TO_LOCAL"	},
	{ LOCAL_TO_REMOTE,	"LOCAL_TO_REMOTE"	},
	{ -1,				"(UNKNOWN)"		}
};
NameTable TransferModes( TransferModeNames );
#endif

class UserProc;

UserProc *get_job_info();
inline void display_bool( int debug_flags, char *name, int value );
void *bsd_rusage( clock_t user_time, clock_t sys_time );
void *bsd_status( int, PROC_STATE, int, int );

class UserProc {
public:
		// Constructors, etc.
	UserProc( ClassAd *, char *, char *, uid_t, uid_t, int, int );
	virtual ~UserProc();
	void display();

		// Handle files associated with a process
	int  fetch_ckpt();		// get checkpoint from submitting machine
	void store_ckpt();		// send checkpoint to submitting machine
	void store_core();		// send abnormal core to submitting machine
	int  update_ckpt();		// create "ckpt.tmp" from "ckpt" and "core"
	void commit_ckpt();		// mv "ckpt.tmp" to "ckpt"
	virtual void delete_files();	// clean up files on executing machine
	// Set-up for transfer of remote to local transfer
	void init_transfer(char *, char *, TRANSFER_MODE, int);
	void init_fetch();
	void move_next_block();	// Move next block remote <-> local
	void end_transfer( TRANSFER_STATE );	// Complete transfer
	void check_if_runnable();

		// Process control
	virtual void execute();
	virtual void suspend();
	virtual void resume();
	virtual void request_ckpt();
	virtual void request_exit();
	virtual void kill_forcibly();
	virtual void handle_termination( int exit_stat );

		// Access functions
	PROC_STATE	get_state() { return state; }
	pid_t	get_pid() { return pid; }
	int		get_id() { return v_pid; }
    virtual int get_tid() { return 0; }
	int		get_cluster() { return cluster; }
	int		get_proc() { return proc; }
	JOB_CLASS	get_class() { return job_class; }
	int		is_runnable() { return state == RUNNABLE; }
	int		is_running() { return state == EXECUTING; }
	int		is_suspended() { return state == SUSPENDED; }
	int		is_checkpointing() { return state == CHECKPOINTING; }
	int		exited_normally() { return state == NORMAL_EXIT; }
	int		exited_abnormally() { return state == ABNORMAL_EXIT; }
	int		non_runnable() { return state == NON_RUNNABLE; }
	int     ckpt_enabled() { return ckpt_wanted; }
	char    *get_env_param( char * );
	int	    get_image_size() { return image_size; }
	TRANSFER_STATE		get_transfer_status() { return transfer_status; }
	void	get_transfer_fds(int &to, int &from) { to = FD_to; from = FD_from;}
	void	set_transfer_fds(int to, int from) { FD_to = to; FD_from = from; }
	void	*bsd_exit_status();		// BSD style exit status for shadow
	void	*accumulated_rusage();	// accumulated so far
	void	*guaranteed_rusage();	// guaranteed by xfer of ckpt or termination
	void set_times(clock_t u_time, clock_t s_time) { user_time = u_time; sys_time = s_time; }


protected:
	
	int		cluster;
	int		proc;

	char	*in;
	char	*out;
	char	*err;
	char	*rootdir;
	char	*cmd;
	char	*args;
	char	*env;
	Environ	env_obj;

	char	*orig_ckpt;		// on submitting machine
	char	*target_ckpt;	// on submitting machine

	char	*local_dir;		// on executing machine
	char	*cur_ckpt;		// on executing machine
	char	*tmp_ckpt;		// on executing machine
	char	*core_name;		// on executing machine

	uid_t	uid;
	uid_t	gid;

	pid_t	v_pid;			// shadow's pid for the process
	pid_t	pid;			// local pid of the process

	JOB_CLASS	job_class;		// Normal Condor, PVM, Pipe, etc.
	PROC_STATE	state;

		// CPU accumulated by completed executions
	clock_t	user_time;
	clock_t sys_time;
	int		ckpt_wanted;	// TRUE if user wants periodic checkpointing


private:
	virtual void send_sig( int );
	int  estimate_image_size();
	int  is_restart();
	void accumulate_cpu_time();
	void commit_cpu_time();
	char *expand_exec_name( int &on_this_host );
	int link_to_executable( char *src, int & error_code );
	int transfer_executable( char *src, int & error_code );
	int  linked_for_condor();

	static int proc_index;

	int		exit_status_valid;	// TRUE only if process has run and exited
	int		exit_status;	// POSIX exit status

	int		coredump_limit_exists;
	int		coredump_limit;

	int		soft_kill_sig;	// Signal to request process to exit

	int		new_ckpt_created;
	int		ckpt_transferred;
	int		core_created;
	int		core_transferred;
	int		exit_requested;
	int		image_size;		// in 1024 bytes blocks

		// CPU guaranteed by transfer of ckpt or termination of job
	clock_t	guaranteed_user_time;
	clock_t	guaranteed_sys_time;

	    // Status of file transfer for this proc.
	void	*transfer_buf;
	int		transfer_buf_size;
	int		FD_from;
	int		FD_to;
	TRANSFER_STATE	transfer_status;
	TRANSFER_MODE	transfer_mode;
};


#endif
