#include "pool_info.h"
/************************************************************
ColInfo class
************************************************************/
ColInfo::ColInfo (string attribute_name,
		  COL_TYPE col_type)
{
    col_name = attribute_name;
    this->col_type = col_type;

    max_int_val = 0;
    min_int_val = 0;
    max_real_val = 0.0;
    min_real_val = 0.0;

    undefined = 0;
    error_num = 0;

    num_true = 0;
    num_false = 0;

    string_vals.clear();
}

void 
ColInfo::InsertCellInfo (const char * machine_name,
			 CellInfo * value)
{
    cell_values[machine_name]=value;
    
    if (value == NULL) return;

    // update column info 

    switch (value->attr_type) {
	
    case UNDEFINED_TYPE:
	undefined ++; 

    case INTEGER_TYPE:
	set_max_min_int (value->Value.int_val);
	break;

    case REAL_TYPE:
	set_max_min_real (value->Value.real_val);
	break;
	
    case BOOL_TYPE:
	add_bool (value->Value.bool_val);
	break;

    case ERROR_TYPE:
	add_error ();
	break;
	
    case STRING_TYPE:
	add_string (value->Value.string_val);
	break;

    case NULL_TYPE:
	add_null ();
	break;

    
    default:
	//printf("type not handled \n");
	break;
    };
}

CellInfo * 
ColInfo::Lookup (const char * machine_name)
{
    printf("ColInfo::Lookup %s\n", machine_name);
    
    if (cell_values.find(machine_name)!=cell_values.end()) {
	return (cell_values.find(machine_name))->second;
    }

    else return NULL;	     
}

string 
ColInfo::GetColName ()
{
    return col_name;
}

void 
ColInfo::PrintColumnInfo (PoolInfo * pool)
{
    printf("%s: ", col_name.c_str());
    PrintType (col_type);
    printf("\n----------------------------\n");
   
  
    hash_map <const char *, CellInfo *>::iterator itr;
    
    itr = cell_values.begin();
    for (int i=0; i<cell_values.size(); i++, itr++) 
    {
	printf("%s: ", itr->first);
	itr->second->PrintCell();
    }
    PrintColumnSummary (pool);
    printf("\n");

}


/* statistics functions */
void   
ColInfo::set_max_min_int (int ival) 
{
    if (cell_values.size() == 1) {
	max_int_val = ival;
	min_int_val = ival;
    }
    if (ival > max_int_val)
	max_int_val = ival;
    if (ival <min_int_val)
	min_int_val = ival; 
    
    int_vals.insert (ival);
}

void   
ColInfo::set_max_min_real (double fval)
{ 
    if (cell_values.size() == 1) {
	max_real_val = fval;
	min_real_val = fval;
    }
    if (fval >max_real_val)
	max_real_val = fval;
    if (fval <min_real_val)
	min_real_val = fval;

    real_vals.insert(fval);
}
 
void   
ColInfo::add_string (string sval)
{
    string_vals.insert(sval);
}

void   
ColInfo::add_undefined ()
{ 
    undefined ++;
}

void
ColInfo::add_null()
{
    null_num ++;
}

void   
ColInfo::add_error ()
{
    error_num ++;
}

void   
ColInfo::add_bool (bool bval)
{
    if (bval) {
	num_true++;
    }
    else {
	num_false++;
    }
}

int 
ColInfo::GetUndefined (PoolInfo * pool)
{
    return pool->GetMachineCount()-cell_values.size();
}


/* Is this column a candidate key to the table? 
   Only allow a column with string value to be candidate
   key, other values do not make semantic sense
*/

bool 
ColInfo::IsKey (int total_ads)
{
    if (col_type == STRING_TYPE &&
	string_vals.size() == total_ads) {
	    return true;
    }
    // Any other type cannot become a candidate key
    return false;
}

void 
ColInfo::PrintColumnSummary (PoolInfo *pool)
{
    switch (col_type) {
    case INTEGER_TYPE:
	printf("Range of values = (%d, %d)\n",
	       min_int_val, max_int_val);
	printf("There are %d startd ads with this attribute undefined\n",
	       GetUndefined(pool));
	break;
	
    case REAL_TYPE:
	printf("Range of values = (%f, %f)\n",
	       min_real_val, max_real_val);
	printf("There are %d startd ads with this attribute undefined\n",
	       GetUndefined(pool));
	break;

    case BOOL_TYPE:
	printf("There are %d startd ads with true value\n", 
	       num_true);
	printf("There are %d startd ads with false value\n",
	       num_false);
	printf("There are %d startd ads with this attribute undefined\n",
	       GetUndefined(pool));
	break;

    case STRING_TYPE:
	if (string_vals.size()<=1)
	   printf("There is %d unique string value\n",
		  string_vals.size()); 
	else
	    printf("There are %d unique string values\n",
		   string_vals.size());
	printf("There are %d startd ads with this attribute undefined\n",
	       GetUndefined(pool));
	break;

    case UNDEFINED_TYPE:
	printf("(undefined) ");
	break;
	
    case RELATIVE_TIME_TYPE:
	printf("(relative time)");
	break; 
	    
    case ABSOLUTE_TIME_TYPE:
	printf("(absolute time)");
	break; 

    case CLASSAD_TYPE:
	printf("(classad)");
	break;

    case LIST_TYPE:
	printf("(list)");
	break;

    case ATTR_REF_TYPE:
	printf("(attribute reference)");
	break;

    case OP_NODE_TYPE:
	printf("(op_node)");
	break;

    case FN_CALL_TYPE:
	printf("(fn_call)");
	break;

    case CLASSAD_NODE_TYPE:
	printf("(classad_node)");
	break;

    case LIST_NODE_TYPE:
	printf("(list_node)");
	break;

    default:
	printf("(unknown) ");
	break;	
    };
    
}
