/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2001 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
 ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

/* File:    constraint.C
   Author:  ningl
   Purpose: Class implementations for constraint class and DagConstraints
            class. Basic services for these two classes are as follows:
       Constraint: mainly keeps track of <attribute_value, frequency> pair
                   how many values have been used and how many are available
		   for use.
       DagConstraints: It processes constraints from the config file 
                   Stores the constraints in a hash table
		   It responds to requests to give out a proper value 
       		      to assign to a dag node 
 
   Date:    10/3/02
*/

#include "constraint.h"
       
// Utility functions
char* ignore_spaces (char * cur);
void print_constraint_format ();

// Constraint class

Constraint::Constraint (string value, float frequency)
{
    this->value = value;
    this->frequency = frequency;

    needed = 0;
    used = 0;
}

//DagCconstraints class
DagConstraints::DagConstraints(char * constr)
{
    if (!ProcessConstraints(constr)) {
	cout<<"Process constraints failed, did not initialize DagConstraints!\n";
	delete this;
    }
    
}

DagConstraints::~DagConstraints()
{
    values.clear();
}

bool DagConstraints::ProcessConstraints(char * constr)
{
    char * cur = constr;
    string attr = "";
    string requirement = "";
    list <string> * schema = NULL;

    cur = ignore_spaces(cur);

    if (*cur != '[') {
	cerr<<"Parse error: constraint's format should like like this"<<endl;
	print_constraint_format();
    }
    cur ++;
    cur=ignore_spaces(cur);
	
    // Get all the valid attributes from the pool
    if (pool) {
	schema = pool->GetSchema();
	//pool->PrintSchema();
    }
    else {
	cout<<"pool == Null"<<endl;
    }
    while (cur && *cur != '\0' && *cur!=']') {
	attr = "";
	while (*cur!= ' ' && *cur!='=' && *cur!='\t') {
	    attr+=*cur;
	    cur++;
	}
	if (!schema) {
	    cerr<<"No schema information is found\n"<<endl;
	}
	else {
	    if (find(schema->begin(), schema->end(), attr)==schema->end()) {
		cerr<<"Cannot find attribute "<<attr<<" in constraint"<<endl;
	    }
	}

	if (*cur=='=') {
	    cur ++;
	    if (*cur!='=') {
		cerr<<"Constraints should use == "<<endl;
		print_constraint_format();
		return false;
	    }
	}
	else {
	    cur=ignore_spaces(cur);
	
	    if (*cur!='=') {
		cerr<<"Constraints should use == "<<endl;
		print_constraint_format();
		return false;
	    }
	    else {
		cur ++;
		if (*cur!='=') {
		    cerr<<"Constraints should use == "<<endl;
		    print_constraint_format();
		    return false;
		}
		cur++;
	    }
	}

	cur = ignore_spaces(cur);
	requirement = "";
	

	while (*cur!=',' && *cur!=']') {
	    requirement+=*cur;
	    cur++;
	}

	if (!StoreConstraint(attr, ProcessRequirement(requirement))) {
	    cerr<<"Cannot insert requirement "<<requirement<<endl;
	}

	if (*cur==']') 
	    break;

	cur = ignore_spaces(cur);

	if (*cur==',') {
	    cur++;
	}
	cur=ignore_spaces(cur);
	cur++;
    }    
    return true;
}

Constraint * DagConstraints::ProcessRequirement (string sreq)
{
    string value; 
    string freq_int = "";
    string freq_float = "";
    bool percent;

    const char * creq = sreq.c_str();
    char * req = new char[sreq.size()+1];

    cout<<"requirement = ["<<sreq <<"] size = "<<sreq.size() <<endl;
    strncpy (req, creq, sreq.size());
    req[sreq.size()]='\0';

    cout<<"Looking for value part"<<endl;

    // Look for the value part 
    req = ignore_spaces (req);
    
    while (req && *req!='\0' && *req!=' ') {
	value += *req;
	req++;
    }

    cout<<"value = "<<value<<endl;
    req = ignore_spaces (req);



    cout<<"Look for the percent part"<<endl;
    
    while (req && *req>='0' && *req<='9') {
	freq_int += *req;
	req++;
    }
    if (req && *req == '.') {
	percent =false;
	req++;

	// If floating pt found, look for second half 
	while (req && *req>='0' && *req<='9') {
	    freq_float += *req;
	    req++;
	}
    }
    else if (req && *req == '%') {
	percent = true;
	req++;
    }
    
    req = ignore_spaces (req);
    if (*req != '\0') {
	cerr<<"Parse error on constraint in constraint requirement part!"<<endl;
	return NULL;
    }
    
    float frequency;
    if (value == "uniform") {
	frequency = 0.0;
    }
    
    int int_part, f_part;

    // construct constraint
    if (percent) {
	int_part = atoi(freq_int.c_str());
	frequency = ((float) int_part) /100.0;
    } else {
	int_part = atoi(freq_int.c_str());
	f_part = atoi(freq_float.c_str());

	frequency = ((float) f_part) /100.0 + ((float) int_part);
    }
    
    // parsing req into value and frequency pairs 
    Constraint * c = new Constraint (value, frequency);
    return c;
}

bool DagConstraints::StoreConstraint(string attribute_name, 
				     Constraint* constr)
{
    hashtab::iterator hash_itr = values.find(attribute_name);

    // A value has already been set for this constraint
    if (hash_itr != values.end()) {
	(hash_itr->second)->push_back(constr);
    } else {
	list <Constraint*> * nlist = new list<Constraint*>;
	nlist->push_back(constr);
	values[attribute_name]= nlist;
	printf("Inserting %s into values \n", attribute_name.c_str());
    }
    return true;
}

string DagConstraints::RequestValue (string attribute_name)
{

    hashtab::iterator itr=values.find(attribute_name);
    if (itr==values.end()) {
	return "";
    }

    constr_list * constr = itr->second;
    
    constr_list::iterator constr_itr;
    
    // for every constraint check which we should return
    for (constr_itr = constr->begin(); constr_itr!=constr->end(); 
	 constr_itr++) {

	Constraint * c = *constr_itr;

	// skip this value if we've already used up
	if (c->used == c->needed) {
	    continue;
	} else {
	    c->used++;
	    return c->value;
	}
    }
    // Cannot tell DDag what value to use for this constraint
    // DDag can decide on this 
    return "";
    
}
    
void DagConstraints::InitializeForDag (int nodenumber) 
{
    // For every attribute_value, initialize the number needed 
     constr_list::iterator constr_itr;
     hashtab::iterator table_itr = values.begin();
     constr_list *constr;

     while (table_itr!=values.end()) {
	 constr = table_itr->second;

	 for (constr_itr = constr->begin(); constr_itr!=constr->end(); 
	      constr_itr++) {
	     Constraint * c = *constr_itr;
	     c->used = 0;
	     c->needed = (int)  (c->frequency * nodenumber);
	     cout<<"++++++ Setting needed = "<<c->needed<<endl;
	     printf ("\tfrequency = %f, nodenumber = %d\n\t",
		     c->frequency, nodenumber);
	 }
	 table_itr++;
     }
}

void print_constraint_format ()
{
    cerr<<"CONSTRAINT = [  attrA == something 40%,\\\
                attrB == something 50%,\\\
                attrC == uniform]"<<endl;
    cerr<<"Each unfinished line should be followed by \\"<<endl;
}


char* ignore_spaces (char * cur)
{

    while (cur && cur!='\0') {
	if (*cur == ' ' || *cur=='\t' || *cur=='\t') {
	    cur++;
	    continue;
	}
	/* skip new line 
	else if (*cur=='\\') {
	    while (*cur!='\0' && (*cur==' ') || (*cur == '\n')) {
		cur++;
	    }
	}
	*/
	else {
	    break;
	}
	
	cur++;
    }
    return cur;
}
