/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2001 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
 ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

/* File:    constraint.h
   Author:  ningl
   Purpose: Class definitions to service constraints imposed on dagnodes.
            Such as what percentage of nodes should take on what values 
	    of a certain attribute
   Date:    10/3/02
*/

#ifndef _CONSTRAINT_H_
#define _CONSTRAINT_H_

#include "condor_common.h"
#include <list>
#if (__GNUC__<3)
#include <hash_map>
#else
#include <ext/hash_map>
#endif
#include <string>
#include <iostream.h>
#include "ddag.h"

// Utility functions
char* ignore_spaces (char * cur);
void print_constraint_format ();

using namespace std;
 
struct eqstring
{
    bool operator () (string s1, string s2)
    {
	return (s1==s2);
    }
};

class Constraint;           
typedef list <Constraint*> constr_list;
typedef hash_map <string, constr_list*, StringHash> hashtab;

class Constraint 
{
public:
    Constraint (string value, float frequency);
    
    float frequency;
    string value; 
    
    //Number of nodes still need to use this value
    int needed; 
    
    // Number of nodes already used this value as part of their 
    // constraint
    int used;   
    
    
};


class DagConstraints
{
public:

    DagConstraints(char * constraints);

    ~DagConstraints ();

 
    string RequestValue (string attribute_name);
    
    void InitializeForDag (int nodenumber);

private:
    
    bool ProcessConstraints (char * constr);

    bool StoreConstraint(string attribute_name, Constraint* constr);
    
    //Given requirement of the form: value 40%"
    // or value 0.4, turn this into a constraint
    Constraint* ProcessRequirement (string req);
    
    /* This is key to the whole structure - a hash map that is indexed
       by the name of the attribute, each contains bucket contains a 
       ptr to a list of constraints (or <value, frequency pairs>)
    */
    hashtab  values;

};


extern PoolInfo * pool;

#endif
