/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2001 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
 ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

/* File:    dag_property.h
   Author:  ningl
   Purpose: Class definitions to service constraints imposed on dags
   Date:    10/15/02
*/

#ifndef _DAG_PROPERTY_H
#define _DAG_PROPERTY_H

#include "condor_common.h"
#include <string>

#if (__GNUC__<3)
#include <hash_map>
#else
#include <ext/hash_map>
#endif


#include <vector>
#include <list>
#include <set>
#include "djob.h"
#include "../condor_classad.V6/common.h"

using namespace std;

/* Qualifiers for the level information */
enum qualifier {
    NUMBER,
    PARENT,
    CHILD,
    PROPERTY
};

class Level;
typedef hash_map <string, Level*, StringHash> level_map;
typedef hash_map <string, string, StringHash> property_map;
typedef hash_map <string, set <DagNode*>*, StringHash> level_nodes_map;

typedef struct lf_pair 
{
    string name;
    float fraction;

} level_nf_pair;


class Level 
{

private:
    Level (string name);
    ~Level ();

    void   set_node_num (int n) {number_of_nodes = n;};

    string name;
    int    number_of_nodes;
    vector <list <level_nf_pair> * > info;

    friend class DDag;
    friend class DagProperty;
};


class DagProperty 
{

public:
    DagProperty ();
    ~DagProperty();
    
    /* Read in the whole constraint string from config file
     and process it line by line*/
    bool ProcessConstraintString (string constraint);

private:
    friend class DDag;

    
    /* Process constraints for nodes of the dag of this form: 
       property1 = "target.OpSys == \"LINUX\"";
    */
    bool ProcessPropertyDefinition (string lhs, string rhs);

    /* Process definitions for a level of dag
       level1.number = 10;
       level2.child = 0.1*level3 + 0.5*level4;
       level2.parent = 1*level1;
       level1.property = 0.5*property1 + 0.5*property2;
    */
    bool ProcessLevelDefinition (string lhs, string rhs);


    /* Get property by name */
    string GetRequirement (string property);

    /* hash_map that contains mappings for property1 = "target.OpSys==\"LINUX\"" */
    property_map properties;

    /* hash_map that contains mappings for level information 
       level1.number = 30; ==> this means 30 nodes in level1
       level2.number = 10;
       level2.parent = 1*level1;  ==> 100% of level2's parents are in level 1
       level1.child = 0.1*level2 + 0.5*level3;
       ==> 10% of level1's children are in level2
           50% of ........................ level3
    */
    level_map * level_info;
    
    /* records level_name & set of DagNodes */
    level_nodes_map node_levels;
    
    void PrintLevelInfo ();
};


#endif
