/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/
#include "condor_common.h"
#include "pool_info.h"
#include "dagman_test_utils.h"

// pretty print ads
void 
printads (ClassAdList & adList) 
{
    ClassAd * ad;
    adList.Open();

    while ((ad=adList.Next())) {
	//ad->dPrint(D_ALWAYS);
    }
    adList.Close();
}

// Checks if machine is available to run jobs 
bool 
isAvailable (ClassAd * ad)
{
    double load_avg;
    double condor_load_avg;
    int   keyboard_idle;
    int   memory; 
    char  state [100];

    /* Important conditions to match
       (LoadAvg - CondorLoadAvg) <= 0.300000) && 
       (KeyboardIdle > 15 * 60) && 
       (TARGET.ImageSize <= ((Memory - 15) * 1024))
    */
    
    ad->EvaluateAttrString (ATTR_STATE, state, 100);
    //ad->LookupString(ATTR_STATE, state);
    
    if (strcasecmp(state, "Unclaimed")) {
	return false;
    }
    
    ad->EvaluateAttrReal(ATTR_LOAD_AVG, load_avg);
    ad->EvaluateAttrReal(ATTR_CONDOR_LOAD_AVG, condor_load_avg);
    ad->EvaluateAttrInt (ATTR_KEYBOARD_IDLE, keyboard_idle);
    ad->EvaluateAttrInt (ATTR_MEMORY, memory);

    /*
    fprintf(stderr, "load avg = %f\nc_load_avg = %f\nidle=%d\nmemory=%dK\n",
	    load_avg, condor_load_avg,
	    keyboard_idle, memory);
    */
    return ((load_avg-condor_load_avg)<=0.3 &&
	    (keyboard_idle>15*60));
    
}

// This method checks how many machines in the pool
// match our system requirement & whether these machines
// are busy
void 
sortads (ClassAdList & adList)
{
    ClassAd *  ad;
    ExprTree * requirement, * start;
    char **    req_string, ** start_string;
    int        available = 0, not_available=0, os_not_match = 0;
    struct utsname uname_info;
    char *     os_type = NULL;

    fprintf(stderr, "Reading classads ---- \n");

    
    // Find out current system information
    if (uname(&uname_info) != 0) {
	fprintf(stderr, "Could not get current machine info\n");
	return;
    }

    else {
	fprintf(stderr, "Current machine info: %s, %s, %s\n",
		uname_info.sysname,
		uname_info.version,
		uname_info.machine
		);
	os_type = uname_info.sysname;
    }

    adList.Open();
    while ((ad=adList.Next())) {
	// Check if machine is available
	if (isAvailable (ad)) {
	    available ++; 
	    //fprintf(stderr, "machine may be avaliable!\n");
	    
	    // getting more information about the machine
	    char * name = new char[35];
	    ad->EvaluateAttrString(ATTR_NAME, name, 35);
	    
	    char * arch = new char[30];
	    ad->EvaluateAttrString(ATTR_ARCH, arch, 35);

	    char * os = new char [30];
	    ad->EvaluateAttrString(ATTR_OPSYS, os, 30);
	
	    if (strcasecmp(os, os_type)) {
		//printf("os = %s, os_type = %s\n", os, os_type);
		available--;
		os_not_match++;
	    }
	    
	    /*
	    fprintf(stderr, "Machine info: %s (%s, %s)\n\n",
		    name, arch, os);
	    */
	    
	    delete [] name;
	    delete [] arch;
	    delete [] os;
	    
	}
	else {
	    not_available ++;
	    //fprintf(stderr, "machine is not avaliable\n\n");
	}

	//requirement->Display();
    }
    adList.Close();
    printf("There are %d machines available, %d not available and %d with different os\n",
	   available, not_available, os_not_match);
}



bool
obtainAdsFromCollector(ClassAdList & startdAds)
{
    QueryResult result;
    ClassAd *   ad;
    ExprTree *  startd_ad_req;
    char *      ad_content = new char[1024];
    char *      addr = NULL;
    

    // Initialize startdQuery 
    CondorQuery startdQuery(STARTD_AD);
    
    dprintf(D_ALWAYS, " Getting all startd ads ... \n");
   
    result = startdQuery.fetchAds(startdAds);
        
    if (result!=Q_OK) {
	char error_message[1000];
	char *log_directory, *collector_host;
	
	log_directory = param("LOG");
	
	if (addr != NULL) {
	    collector_host = addr; 
	} else {
	    collector_host = param("COLLECTOR_HOST");
	}
	sprintf(error_message, 
		"Error: Couldn't contact the condor_collector on %s.",
		collector_host ? collector_host : "your central manager");
	fprintf(stderr, "%s", error_message);
	
	dprintf(D_ALWAYS, "Couldn't fetch ads: %s\n", 
		getStrQueryResult(result));

	fprintf(stderr, "Couldn't fetch ads: %s\n", 
		getStrQueryResult(result));
	
	return false;
    }

    return true;
}

/*
// Make table information of all the classads 
void 
processAds (ClassAdList & allAds)
{
    ClassAd * ad;
    PoolInfo pool;

    allAds.Open();

    //while (ad=allAds.Next()) {
    for (int i=0; i<30; i++) {
	ad = allAds.Next();
	readAd (ad, &pool);
	//getAd (ad, &pool);
	//break;
    }
    allAds.Close();
    pool.PrintSchema ();
    pool.PrintDebugInfo();
    
}
*/



// Get the attribute type of an attribute 
COL_TYPE
GetAttributeType (ClassAd * ad,
		  ExprTree * input, 
		  Value::ValueType & type)
{
    Value v;
    ExprTree::NodeKind nodekind = input->GetKind();

    switch (nodekind) {
    case ExprTree::LITERAL_NODE:

	ad->EvaluateExpr(input, v);
	type = v.GetType();
	//printf("type = %d", GetValueType(type));
	return GetValueType (type);
	break;

    case ExprTree::ATTRREF_NODE:
	//printf("attribute reference: ");
	return ATTR_REF_TYPE;
	break;

    case ExprTree::OP_NODE:
	//printf("op_node: ");
	return OP_NODE_TYPE;

    case ExprTree::FN_CALL_NODE:
	//printf("function call node: ");
	return FN_CALL_TYPE;

    case ExprTree::CLASSAD_NODE:
	//printf("classad node: ");
	return CLASSAD_NODE_TYPE;

    case ExprTree::EXPR_LIST_NODE:
	//printf ("expr list node: ");
	return LIST_NODE_TYPE;
    };
}

// Translate from ValueType to COL_TYPE
COL_TYPE
GetValueType (Value::ValueType type) 
{
    switch (type) {
    case Value::NULL_VALUE:
	return NULL_TYPE;

    case Value::ERROR_VALUE:
	//	printf("error \n");
	return ERROR_TYPE;

    case Value::UNDEFINED_VALUE:
	// printf("undefined \n");
	return UNDEFINED_TYPE;

    case Value::BOOLEAN_VALUE:
//	printf("bool \n");
	return BOOL_TYPE;

    case Value::INTEGER_VALUE:
//	printf("int \n");
	return INTEGER_TYPE;

    case Value::REAL_VALUE:
//	printf("real \n");
	return REAL_TYPE;

    case Value::RELATIVE_TIME_VALUE:
//	printf("relative time \n");
	return RELATIVE_TIME_TYPE;
	
    case Value::ABSOLUTE_TIME_VALUE:
//	printf("absolute time \n");
	return ABSOLUTE_TIME_TYPE;

    case Value::STRING_VALUE:
//	printf("string \n");
	return STRING_TYPE;

    case Value::CLASSAD_VALUE:
//	printf("classad \n");
	return CLASSAD_TYPE;

    case Value::LIST_VALUE:
//	printf("list \n");
	return LIST_TYPE;
    };

}


