/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2001 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
 ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/


/* Random DAG generator */
# include "ddag.h"
# include <unistd.h>
# include <vector>
# include <set>
# include "constraint.h"
# include "condor_md.h"

/* DDag class*/ 


mode_t mode={S_IRUSR||S_IWUSR};
bool DEBUG = 0; 
extern DagConstraints * constraints;

// Function declaration
// This function prints the value of attr_name to the submit file submit
bool PrintToSubmitFile (FILE * submit, string attr_name, ClassAd * machine_ad);

DDag::DDag (int node_number, const char* dagDirName, 
	    bool cycle)
{
    cur_size = 0;
    dagsize = node_number;
    dirName = dagDirName;
    
    // Initialize test universe
    _universe = param ("UNIVERSE");

    if (_universe==NULL || (strcasecmp (_universe, "vanilla") &&
			    strcasecmp (_universe, "standard") &&
			    strcasecmp (_universe, "globus")
			    )) {
	_universe = "vanilla"; 
    }

    // Determine whether smart requirement generation is needed
    _intelligence = false;
    
    char* smart_requirements = param ("INTELLIGENCE");
    if (smart_requirements && !strcasecmp (smart_requirements, "true")) {
	cout<<"---- INTELLIGENCE = TRUE -----"<<endl;
	_intelligence = true;
    }
    if (param("DAG_CONSTRAINT")) {
	_intelligence = false;
    }

    // Check if testing is targeted to windows machines
    _win_testing = false;
    char * isWindows = param ("TEST_PLATFORM");
    if (isWindows && !strcasecmp (isWindows, "win32")) {
	_win_testing = true;
    }

    IDs = 0;
    
    //Reserve size for the vector 
    allNodes.clear();
    allNodes.reserve(dagsize);
    
    // by default test_mode is set to NORMAL
    test_mode = NORMAL;
    
    this->cycle = cycle;

    _runtime = 5;
}


void 
DDag::SetTestMode (TESTMODE mode, const char* testjobname)
{
    test_mode = mode;
    this->testjobname = testjobname;
    cout<<"Testjob name = "<<testjobname<<endl;
}


void 
DDag::SetRunTime (int runtime)
{
    _runtime= runtime;
}

DDag::~DDag()
{
    node_vector::iterator itr = allNodes.begin(), itr_next;
    while (itr!=allNodes.end()) {
	if (*itr) {
	    delete *itr;
	    *itr = NULL;
	}
	itr++;
    }
    allNodes.clear();
}

// Generate dag by levels
void DDag::GenerateDagByLevels()
{
    int number_of_nodes; //nodes at each level
    if (dag_property == NULL) {
	cerr<<"Dag properties did not initialize correctly"<<endl;
	return;
    }

    // initialize this using dag_property.level_info
    level_map ::iterator itr;

    //cout<<"\nCreating new dagnode sets for each level\n";
    for (itr=dag_property->level_info->begin(); 
	 itr!=dag_property->level_info->end(); itr++) {
	Level * l = itr->second;
	
	// Create DagNode set
	set <DagNode *> * newnodes = new set <DagNode*>; 
	dag_property->node_levels[l->name] = newnodes;
	//cout<<"\tCreated new dagnode set for level "<<l->name<<endl;
    }

    // For each node_level, need to create new nodes
    hash_map <string, set <DagNode*>*, StringHash> ::iterator level_itr;
    

    //cout<<"\nCreate new nodes for each level "<<endl;
    for (level_itr  = dag_property->node_levels.begin(); 
	 level_itr != dag_property->node_levels.end(); level_itr++) {
	
	string level_name = level_itr->first;
	Level * l = (dag_property->level_info->find(level_name))->second;

       	// Find out number of nodes in the level if that information exists 
	number_of_nodes = l->number_of_nodes; 
	
	if (number_of_nodes <= 0) {
	    //cout<<" oops, number_of_nodes = "<<l->number_of_nodes<<endl;
	    srand (time(NULL));
	    number_of_nodes =1+(int) (11.0*rand()/(RAND_MAX+1.0)); 
	    l->number_of_nodes = number_of_nodes;
	}

	//cout<<"Level: "<<l->name<<" ["<<number_of_nodes<<"] nodes"<<endl;
	// Knowing the number of nodes, can generate actual nodes
	while (number_of_nodes) {
      
	    DagNode * newnode = new DagNode (GetNewID());
	    allNodes.push_back(newnode);
	    
	    //cout<<"\tLevel "<<l->name<<" created new dagnode with id "<<newnode->GetID()<<endl;
	    number_of_nodes--;
	    // insert new node to the level
	    (level_itr->second)->insert (newnode);
	}
    }

    // Process parent/child relationship among levels
    for (level_itr = dag_property->node_levels.begin(); level_itr!=dag_property->node_levels.end(); level_itr++) {
	// For each level 

	string level_name = level_itr->first;

	
	Level * l = (dag_property->level_info->find(level_name))->second; 

	list <level_nf_pair> * parents= (l->info)[PARENT];
	list <level_nf_pair> * children= (l->info)[CHILD];
	
	list <level_nf_pair> ::iterator p_level_itr;
	for (p_level_itr=parents->begin(); p_level_itr!=parents->end(); p_level_itr++) {
	    // For each parent level
	    
	    // Get number of nodes in the parent level
	    // There are m links to parent level 
	    int m = (int)  (l->number_of_nodes * (p_level_itr->fraction));
	    
	    // Randomly pick m children
	    set <DagNode*> * children_nodes = dag_property->node_levels [l->name];
	    set <DagNode *>::iterator child_itr = children_nodes->begin();
	    
	    // Randomly pick m parents
	    set <DagNode*> * nodes = dag_property->node_levels[p_level_itr->name];
	    set <DagNode*>::iterator itr = nodes->begin();
	    
	    // For each parent
	    for (int i=0; i<m; i++) {
		if (itr== nodes->end()) {
		    break;
		}
		DagNode * parent_node = *itr;
		itr++;

		
		for (int j=0; j<m; j++) {
		    if (child_itr==children_nodes->end()) {
			break;
		    }
		    DagNode * child_node = *child_itr;
		    parent_node->addChild (child_node);
		    child_node->addParent (parent_node);
		}
	    }
	}
	list <level_nf_pair> ::iterator c_level_itr = children->begin();
	for (; c_level_itr!=children->end(); c_level_itr++) {
	    // For each child level
	     // There a m links to child level 
	     int m = (int)  ((float) l->number_of_nodes * (c_level_itr->fraction));

	    // Randomly pick m chilren from child level
	    set <DagNode*> * children_nodes = dag_property->node_levels [c_level_itr->name];
	    set <DagNode *>::iterator child_itr = children_nodes->begin();
	    
	    // Randomly pick m parents from current level
	    set <DagNode*> * nodes = dag_property->node_levels[l->name];
	    set <DagNode*>::iterator itr = nodes->begin();
	    
	    // For each parent
	    for (int i=0; i<m; i++) {
		if (itr== nodes->end()) {
		    break;
		}
		DagNode * parent_node = *itr;
		itr++;

		
		for (int j=0; j<m; j++) {
		    if (child_itr==children_nodes->end()) {
			break;
		    }
		    DagNode * child_node = *child_itr;
		    parent_node->addChild (child_node);
		    child_node->addParent (parent_node);
		}
	    }
	}
    }

    // Print debug information
    /*
    level_itr = dag_property->node_levels.begin();
    set <DagNode*> ::iterator node_itr;

    for (; level_itr!=dag_property->node_levels.end(); level_itr++) {

	cout<<"--- Level"<<level_itr->first<<endl;

	// Get information of current node
	node_itr = (level_itr->second)->begin(); 

	for (; node_itr != (level_itr->second)->end(); node_itr++) {
	    DagNode * node = *node_itr;
	    
	    set <DagNode *> * parents = node->GetParents ();
	    set <DagNode *> * children = node->GetChildren();

	    set <DagNode *>::iterator p, c;
	    cout <<"==== Node "<< node->GetID()<<" ====\nParent = ";

	    for (p=parents->begin(); p!=parents->end(); p++) {
		cout<< (*p)->GetID()<<" ";
	    }
	    cout<<"Children = "<<endl;
	    for (c=children->begin(); c!=children->end(); c++) {
		cout<<(*c)->GetID()<<" ";
	    }
	    cout<<endl<<endl;
	}
    }
    */
    
    
}


void DDag::GenerateDag()
{
    allNodes.clear();
    if (cycle) {
	
	printf("Warning: this is a dag with cycles\n");
	for (int i=0; i<dagsize; i++) {
	    DagNode * newNode = new DagNode (GetNewID());
	    allNodes.push_back(newNode);
	}		
  
	//For each node, randomly generate links to a bunch of other nodes
	for (int i=0; i<dagsize; i++) {

	    for (int j=0; j<(int)rand()%dagsize; j++) {
		int child = rand()%dagsize; 
		
		if (rand()%2) {
		    //make a link between the two
		    allNodes[i]->addChild(allNodes[child]);
		    allNodes[child]->addParent(allNodes[i]);
		}
	    }	
	    
	}
	if (!DetectCycles()) //pick a node and add a self cycle there
	{
	    allNodes[0]->addChild(allNodes[0]);
	}
	return;
    }
    
    
    for (int i=0; i<dagsize; i++) {
	DagNode * newNode = new DagNode (GetNewID());
	allNodes.push_back(newNode);
    }
	
    /*
      set<DagNode*> ::iterator itr; 
      // for zach's testing - ningl 10/2
      for (int i=0; i<dagsize-1; i++) {
      allNodes[i]->addChild(allNodes[dagsize-1]);
      allNodes[dagsize-1]->addParent(allNodes[i]);
      }
    */
    
    for (int i=0; i<dagsize; i++) {
	int child = rand()%dagsize; 

	//guarantees no cycle
	if (child>i) {
	    //make a link between the two
	    allNodes[i]->addChild(allNodes[child]);
	    allNodes[child]->addParent(allNodes[i]);
	}
    }
    
}


//cycle detection and removal
bool DDag::DetectCycles()
{
    DFS_ORDER = 0;
    bool cycle = false;
    
    //depth first search check for back cycles
    for (unsigned int i=0; i<allNodes.size(); i++)
    {
	if (!allNodes[i]->visited &&allNodes[i]->parents->size()==0)
	    DFS(allNodes[i]);
    }
    
    //check for back edges
    set<DagNode*> ::iterator itr; 
    for (unsigned int i=0; i<allNodes.size(); i++)
    {
	for (itr=allNodes[i]->children->begin(); itr!=allNodes[i]->children->end(); itr++) {
	    
	    //my DFSOrder> my child's
	    //and I have incoming edge, then there is loop
	    if (allNodes[i]->DFSOrder<=(*itr)->DFSOrder &&
		allNodes[i]->parents->size()!=0)
            {
		cycle = true;
	    }
	}
    }
    return cycle;
}

void DDag::DFS(DagNode * node)
{
    if (node->visited)
	return;
    node->visited = true;
 
    //if child has already been visited, then this is a backedge
    if (node->children->size()!=0) {
	//printf("visiting node %d, with %d children\n",
	//   (int)node->GetID(), node->children->size());
	set<DagNode*>::iterator itr; 
	for (itr=node->children->begin(); 
	     itr!=node->children->end(); 
	     itr++)
	    DFS(*itr);
    }
    node->DFSOrder = DFS_ORDER++; //Postorder numbering
}



bool
DDag::GenerateDagFile()
{
    //Generate DAG file for the dag
    vector <DagNode*> ::iterator itr; 
    set<DagNode*> ::iterator itr2;
    
    //make directory
    if (mkdir(dirName, 0744)==-1)  {
	fprintf(stderr, "ERROR: %s", dirName);
	perror(" failed to create");
	return 0; 
    }      
    
    char DagFileName [strlen(dirName)+11];
    sprintf(DagFileName, "%s/random.dag", dirName);
    
    FILE * DagFile=fopen(DagFileName, "w");
    if (DagFile==NULL) {
	
	fprintf(stderr, "Could not open %s to write\n", DagFileName);
	return false;
    }
  
    
    // Randomly create some Dag and Dap Nodes
    for (itr=allNodes.begin(); itr!=allNodes.end(); itr++) {
	if (test_mode == DAP) {
	    srand (time(NULL));
	    int rand_num = (int) (12.0*rand()/(RAND_MAX+1.0)) +1;
	    if (rand_num > 6) {
		// 50% chance to be condor job
		fprintf(DagFile, "Job %s %s.condor\n", (*itr)->name, (*itr)->name);
		(*itr)->setNodeType (DAG);
	    }
	    else {
		// 50% chance to be a dap job
		fprintf(DagFile, "Dap %s %s.dap\n", (*itr)->name, (*itr)->name);
		(*itr)->setNodeType(NEST);
	    }

	} 
	else {
	    fprintf(DagFile, "Job %s %s.condor\n", (*itr)->name, (*itr)->name);
	}
    }
  
    /*
    // for zach's testing: multiple parents & 1 child per line
    for (itr=allNodes.begin(); itr!=allNodes.end(); itr++) {
	if ((*itr)->parents->size()>0) {
	    fprintf(DagFile, "PARENT ");
	    for (itr2=(*itr)->parents->begin(); 
		 itr2!=(*itr)->parents->end(); itr2++) {
		fprintf(DagFile, "%s ", (*itr2)->name);
	    }
	    fprintf(DagFile, "CHILD %s\n", (*itr)->name);
	}
    }

    */
    // parent ... child ... 1-to-1 in separate lines
    for (itr=allNodes.begin(); itr!=allNodes.end(); itr++) {
	for (itr2=(*itr)->children->begin(); 
	     itr2!=(*itr)->children->end(); itr2++) {
	    fprintf(DagFile, "PARENT %s CHILD %s\n", 
		    (*itr)->name, (*itr2)->name);
	}
    } 
    
    fclose(DagFile);
    return 1;
}

// Attach requirements/property to each node in the dag based on the config file
bool DDag::AttachPropertyToNodes ()
{
    if (dag_property == NULL) {
	cerr<<"Dag properties did not initialize correctly"<<endl;
	return false;
    }
    
    level_map::iterator level_itr = dag_property->level_info->begin();
    
    // For each level calculate the number of nodes that need to have
    // each property
    for (; level_itr!=dag_property->level_info->end(); level_itr++) {
	// For each level
	
	Level * l = level_itr->second;
	//cout<<"Level "<<l->name<<endl;

	// iterator through the list of properties 
	list <level_nf_pair> ::iterator p_itr = ((l->info)[PROPERTY])->begin(); 
	
	// iterator through the nodes in the level 
	set <DagNode* > ::iterator d_itr, end_itr;
	d_itr = (((dag_property->node_levels).find(l->name))->second)->begin();
	end_itr = (((dag_property->node_levels).find(l->name))->second)->end();

	for (; p_itr!= ((l->info)[PROPERTY])->end(); p_itr++) {
	    // For each property for the level

	    // Check how many nodes in the level are affected 
	    int nodes_affected = int(p_itr->fraction*(l->number_of_nodes));
	    
	    while (nodes_affected >0 && d_itr!=end_itr) {
		
		// Assign current property to the node
		(*d_itr)->addRequirement (p_itr->name);

		//cout<<"\tAdding requirement "<<p_itr->name<<" to node "<<(*d_itr)->GetID()<<endl;
		nodes_affected --;
		d_itr++;
	    }
	}	
    }

    return true;
}


/*Create condor submit files for each job. This involves:
  - Create the input files for each job 
  If the job has children, its input file will be the output
  files of the children; otherwise it would be just a file 
  with a 1 in it.
  - Create the output files names for each job
*/
bool DDag::GenerateSubmitFiles(PoolInfo * pool)
{
    static int nest_id = 1;
    if (dag_property) {
	if (!AttachPropertyToNodes()) {
	    fprintf(stderr, "Could not attach requirements to dag nodes\n");
	    return false;
	}
    }
    
    vector <DagNode*>::iterator itr;
    set<DagNode*>::iterator itr2;
    ClassAd * target_machine;
    int random_time = 0;
  
    srand (time(NULL));
    
    if (pool) pool->RewindRow ();
    for (itr=allNodes.begin(); itr!=allNodes.end(); itr++) {
	if (pool) {

	    target_machine = pool->GetNextRow();
	    
	    if (target_machine == NULL) {
		pool->RewindRow();
		target_machine = pool->GetNextRow();
	    }
	    
	    char * machine_name = new char [30];
	    target_machine->EvaluateAttrString ("name", machine_name, 30);
	    //printf ("Target machine = %s\n", machine_name);
	    delete [] machine_name;
	}

	// For each node in the dag
	
	DagNode * node = *itr;
	char jobsubmit [strlen(dirName)+30];

	if (test_mode == DAP) {
	    sprintf(jobsubmit, "%s/%s.dap", dirName, node->name);
	} else {
	    sprintf(jobsubmit, "%s/%s.condor", dirName, node->name);
	}
	    
	FILE * submit = fopen (jobsubmit, "w");
	
	if (submit==NULL) {
	    fprintf(stderr, "Could not open %s", jobsubmit);
	    return false;
	}

	if (node->getNodeType()  == NEST) {
	    // reserve resource
	    fprintf (submit, "[\n\tdap_type = \"reserve\";\n");
	    fprintf (submit, "dest_host = db18.cs.wisc.edu;\n");
	    fprintf (submit, "reserve_size = 100;\n");
	    fprintf (submit, "duration = 60;\n");
	    fprintf (submit, "reserve_id = 1;\n");
	    
	    // transfer file
	    fprintf (submit, "[\n\tdap_type = \"transfer\";\n");
	    fprintf (submit, "\tsrc_url = \"file:/tmp/1.test\n");
	    fprintf (submit, "\tdest_url = \"nest://db16.cs.wisc.edu/tmp/1.test\"\n]");

	    // release resource
	    fprintf (submit, "[\n\tdap_type = \"release\";\n");
	    fprintf (submit, "dest_host = db16.cs.wisc.edu;\n");
	    fprintf (submit, "reserve_id = %d;\n", nest_id);
	    nest_id++;
	}
	else {
	    // This is a DAG node 

	    fprintf(submit, "universe = %s\nexecutable = %s \n", 
		    _universe, testjobname);
	    
	    // GLOBUS UNIVERSE JOBS
	    if (!strcasecmp (_universe, "globus")) {
		char * globus_scheduler = param ("GLOBUS_SCHEDULER");
		if (!globus_scheduler) 
		    globus_scheduler = "froth.cs.wisc.edu/jobmanager";
		fprintf(submit, "globusscheduler=%s\ntransfer_executable=true\n", globus_scheduler);
		char * proxy_location = param ("PROXY_LOCATION");

		if (proxy_location) {
		    fprintf(submit, "x509userproxy = %s\n", proxy_location);
		    free (proxy_location);
		}
		fprintf(submit, "remote_initialdir = %s\n", dirName);
		//fprintf(submit, "transfer_files=ALWAYS\n");
		
		
		// Use current directory if an output directory is not specified 
		fprintf(submit, "output=%s.out\n",  node->name);
		
		if (!strcasecmp (_universe, "globus")) {
		    //fprintf(submit, "transfer_output_files =%s.out\n",  node->name);
		}
	    }// end Globus Universe Jobs
	    
	    //random runtime for each job
	    if (_runtime>0) {
		random_time = rand()%_runtime; 
	    }
	    
	    //Nodes with no parents have input files 1
	    if (node->parents->size()==0) {
		
		char inputfile [strlen(dirName)+30];
		sprintf(inputfile, "%s/%s.in", dirName, node->name);
		
		if (!GenerateInputFile (inputfile)) {
		    fprintf(stderr, "Failed to generated input file %s\n", inputfile);
		    fclose (submit);
		    return false;
		}
		

		// write out the input file name for this job if it's globus universe
		if (!strcasecmp (_universe, "globus")) {
		    //fprintf(submit, "transfer_input_files = %s.in\n", node->name);
		}

		// Write out the arguments for the test job executable
		if (_runtime!=-1) {
		    fprintf(submit, "arguments = -o %s -t %d %s.in",
			    node->name, random_time, node->name);
		}
		else 
		    fprintf(submit, "arguments = -o %s %s.in", 
			    node->name, node->name);
	    }
	    
	    //Other files have inputfiles the output of parents
	    else {
		
		// Write out the arguments for the test job executable
		if (_runtime!=-1)
		fprintf(submit, "arguments = -o %s -t %d ", 
			node->name, random_time);
		else 
		    fprintf(submit, "arguments = -o %s ", node->name);
		
		// Pass in output files of parent nodes as inputs of current node
		for (itr2=node->parents->begin(); itr2!=node->parents->end(); itr2++) {
		    fprintf(submit, "%s.out ", (*itr2)->name);

		}

		// Input file names for globus universe jobs
		if (!strcasecmp (_universe, "globus")) {
		    //fprintf(submit, "\ntransfer_input_files = ");
		    // Input file names
		    for (itr2=node->parents->begin(); itr2!=node->parents->end(); itr2++) {
			fprintf(submit, "%s.out ", (*itr2)->name);
			
		    }
		}
	    }
	    
	    // Log file resides at testjob.log
	    // And notification = never
	    fprintf(submit, "\nerror=err.$(Process)\nlog=testjob.log\nnotification = Never\n");
	    // Generate requirements for non-globus universe jobs
	    if (strcasecmp (_universe, "globus")) {
		if (!_intelligence && !dag_property) {
		    // Generate requirements 
		    GenerateRequirements (submit);      
		    
		} else if (dag_property) {
		    GenerateRequirements (submit, dag_property->GetRequirement(node->getRequirement()));
	    }
		else if (_intelligence) {
		    GetStaticAttributes (pool);
		    RemoveNonInterestedAttributes();
		    
		    GenerateSmartRequirements (submit, pool, target_machine);
		}
	    }
	    else {
		fprintf(submit, "queue\n");
	    }
	} // end if this is a DAG node

	if (submit) {
	    fclose(submit); 
	}
    }
    return true;
}
	


void DDag::GenerateRequirements (FILE * submit, string req = "")
{
    if (_win_testing) {
	// If _win_testing is wanted, then need some additional attributes
	fprintf(submit, "transfer_files = always\n");
	fprintf(submit, "Environment = PATHEXT=.COM;.EXE;.BAT;.CMD|PATH=C:\\WINNT\\system32;C:\\WINNT|SystemDrive=C:|SystemRoot=C:\\WINNT|TEMP=C:\\);TEMP|TMP=C:\\TEMP\n");
	fprintf(submit, "Requirements = (ARCH == \"INTEL\" && OPSYS == \"WINNT50\"");
	fprintf(submit, "&& FileSystemDomain == \"winnt40.cs.wisc.edu\")\nqueue\n");
	
    } else if (req=="") {
	fprintf(submit, "transfer_files = never\n");
	fprintf(submit, "requirements = ( Arch == \"INTEL\" && Opsys == \"LINUX\" )\nqueue\n");
    } else {
	fprintf(submit, "transfer_files = never\n");
	fprintf(submit, "requirements = (%s)\nqueue\n", req.c_str());
    }
}


/* Generates requirements that uniformly target the available machines 
   in the pool or if a constraint is specified, requirements taht match 
   constraints specified 
*/
bool DDag::GenerateSmartRequirements (FILE * submit, 
				      PoolInfo * pool,
				      ClassAd * machine_ad)
{

    //printf("generating smart requirements\n");
    ExprTree * reqExpr, *flatReqExpr, * prunedReqExpr;
    Value val;
    string buffer;
    PrettyPrint pp;
    //ClassAdAnalyzer analyzer;

    if (machine_ad == NULL || pool==NULL) 
	return false;

    if (!(reqExpr=machine_ad->Lookup(ATTR_REQUIREMENTS))) {
	fprintf(stderr, "Error lookup up machine ad requirement\n");
	return false;
    }
    //debug
        
    /*
    cout << endl;
	 pp.Unparse( buffer, machine_ad );
	 cout << "offer = " << buffer << endl;
   
    buffer = "";
    */
    if (!(machine_ad->FlattenAndInline(reqExpr, val, flatReqExpr))) {
	fprintf(stderr, "Could not flatten machine ad requirement \n");
	return false;
    }

    // debug
    /*
    pp.Unparse( buffer, reqExpr );
    cout << "reqExpr = " << buffer << endl;
    buffer = "";
    
    pp.Unparse( buffer, flatReqExpr );
    cout << "flatReqExpr = " << buffer << endl;
    buffer = "";
    */
    set<string> ::iterator itr;

    //printf("requirements = (");
    fprintf(submit, "requirements = (");
    
    int count = 0;
    string attr_name;

    
    if (constraints) {
	
	// For each static attribute, generate its requirements
	for (itr=interested_attrs.begin(); itr!=interested_attrs.end(); itr++, count++) {

	    attr_name = *itr;
	    
	    string value = constraints->RequestValue(*itr);
	    
	    if (value!="") {
		//cout<<"Constraint_requested value = "<<value<<endl;
		fprintf(submit, "\t(target.%s ==%s) ", itr->c_str(), value.c_str());
		//printf("\t(target.%s ==%s) ", itr->c_str(), value.c_str());

		if (count != interested_attrs.size()-1) {
		    //printf (" && \\\n");
		    fprintf(submit, " && \\\n");
		}
	    }
	    /*
	    else {
		if (!PrintToSubmitFile (submit, attr_name, machine_ad)) {
		    fprintf(stderr, "Coult not print %s's value to submit file\n", attr_name.c_str());
		    //return false;
		}
		else {
		    if (count != interested_attrs.size()-1) {
			printf (" && \\\n");
			fprintf(submit, " && \\\n");
		    }
		}
	    }
	    */
	}
    }


    else {
	// else if no constraint has been set by user

	for (itr=interested_attrs.begin(); itr!=interested_attrs.end(); itr++, count++) {
	    // For each static attribute, generate its requirements

	    attr_name= *itr;

	    if (!PrintToSubmitFile (submit, attr_name, machine_ad)) {
		fprintf(stderr, "Coult not print %s's value to submit file\n", attr_name.c_str());
		//return false;
	    }
	    else {
		if (count != interested_attrs.size()-1) {
		    fprintf(submit, " && \\\n");
		    printf (" && \\\n");
		}
	    }
	    buffer = "";
	}
    }// end else (for uniform distributed attribute values 
    
    fprintf(submit, ")\nqueue\n");
    printf (")\n\n");
}

bool PrintToSubmitFile (FILE * submit, string attr_name, ClassAd * machine_ad) 
{

    string buffer;
    Value value;
    ClassAdUnParser unparser;
    const char * attr = attr_name.c_str();
    
    if (!(machine_ad->EvaluateAttr(attr_name, value))) {
	printf("%s cannot be evaluated\n");
	return false;
    }
    
    unparser.Unparse (buffer, value);
	    
    if (value.GetType() != Value::UNDEFINED_VALUE &&
	value.GetType() != Value::NULL_VALUE &&
	value.GetType() != Value::ERROR_VALUE ) {
	
	if (value.GetType()==Value::BOOLEAN_VALUE) {
		    
	    bool bval;
	    machine_ad->EvaluateAttrBool(attr_name, bval);
	    if (bval) {
		//printf("\t(target.%s == TRUE)", attr);
		fprintf(submit, "\t(target.%s == TRUE)", attr);
	    }
	    else {
		//printf("\t(target.%s == FALSE)", attr);
		fprintf(submit, "\t(target.%s == FALSE)", attr);
	    }
	}
  	
	else {
	    //printf("\t(target.%s == %s)", attr, buffer.c_str());
	    fprintf(submit, "\t(target.%s == %s)", attr, buffer.c_str());
	}
	return true;
    }
    return false;
}
 
bool DDag::GenerateInputFile (const char * inputfile)
{
    struct stat fstat;
    if (!stat(inputfile, &fstat)) 
	unlink (inputfile);
    FILE * infile = fopen(inputfile, "w");
    srand (time(NULL));
    int bytes = rand()%10001+1;
    
    if (!infile)
    {
	fprintf(stderr, "Cannot open %s", inputfile);
	return false;
    }
      
    if (test_mode == NORMAL) {
	fprintf(infile, "1");
    }
    else if (test_mode == MDSUM)
    {
	fprintf(stderr, "Generating MD input file\n");
	WriteMDFile (infile, inputfile, bytes);
    }
    else //longfile
    {
	fprintf(infile, "TOTAL BYTES: %d\n", bytes);
	fprintf(stderr, "Gererating input of size %d\n", bytes);
	for (int i=0; i<bytes; i++)
	    fprintf(infile, "1");
    }
    fclose(infile);
    return true;
}

void DDag::WriteMDFile (FILE *infile, const char * filename, unsigned int bytes)
{
    Condor_MD_MAC context;
    unsigned long len = 1024;
    unsigned char uline[1024];
    unsigned char * digest = new unsigned char [16];
    char mdfile [strlen(filename)+3];
    sprintf(mdfile, "%s.md", filename);

    FILE * md;

    // randomly generate file 
  
    for (unsigned int i=0; i<bytes; i++) {
	if (!i%1023 && i!=0) {
	    fprintf (infile, "\n");
	} 
 	else {
	    int randnum = (int)(10.0*rand()/(RAND_MAX+1.0));
	    fprintf (infile, "%d", randnum);
	} 
    } 
    fprintf(infile, "\n"); 
    fclose (infile); 
    
    if ((infile = fopen(filename, "r"))==NULL) {
	fprintf(stderr, "Could not open %s\n", filename);
	return;
    } 
    
    if ((md = fopen(mdfile, "w"))==NULL) {
	fprintf(stderr, "Could not open %s\n", mdfile);
	return;
    }
    
    while (fread(uline, 1, len, infile))
	context.addMD (uline, len);
    digest = context.computeMD();
    for (int i=0; i<16; i++) {
	fprintf(stderr, "%02x", digest[i]);
    }
    cerr<<endl;
    
    // write MD5 checksum after End 
    fwrite (&digest, 16, 1, md);
    fclose (md);
    delete [] digest;
}

DagNode* 
DDag::AddNodeToDag(char * nodeName)
{
    if (nodeName == NULL)
	return NULL;

    NodeID newid = GetNewID();

    DagNode * newNode = new DagNode(newid);

    if (newNode == NULL)
	return NULL;

    allNodes.push_back(newNode);

    return newNode;
}



/**************************************************/
/* DagNode Class*/
DagNode::DagNode (NodeID nID, NodeType t = DAG)
{
    name = new char[30];
    sprintf(name, "Node%d", nID);
    parents = new set<DagNode*>;
    children = new set<DagNode*>; 
    nodeID = nID;
    visited = false;
    type = t;
}

DagNode::~DagNode()
{    
    if (name) {
	delete [] name;
    }
    if (parents) {
	parents->clear();
	delete parents; 
    }
    if (children) {
	children->clear();
	delete children;
    }
}

void DagNode::addParent(DagNode* node)
{	
    parents->insert(node);
}

void DagNode::addChild(DagNode* node)
{	
    children->insert(node);
}

void DagNode::addRequirement (string req)
{
    this->req = req;
}

// Process requirement string before returning it 
string DagNode::getRequirement ()
{
    return req;
}

NodeID DDag::GetNewID()
{
  DDag::IDs++; 
  return DDag::IDs;
}

// List of attributes that should not be included in a job classad
void 
DDag::RemoveNonInterestedAttributes ()
{
    interested_attrs.erase("Activity");
    interested_attrs.erase("ClientMachine");
    interested_attrs.erase("ClockDay");
    interested_attrs.erase("CondorLoadAvg");
    interested_attrs.erase("CpuBusyTime");
    interested_attrs.erase("CpuIsBusy");
    interested_attrs.erase("CurrentRank");
    interested_attrs.erase("DaemonStartTime");
    interested_attrs.erase("Disk");
    interested_attrs.erase("EnteredCurrentActivity");
    interested_attrs.erase("EnteredCurrentState");
    interested_attrs.erase("ExecutableSize");
    interested_attrs.erase("ImageSize");
    interested_attrs.erase("JavaMFlops");
    interested_attrs.erase("JobId");
    interested_attrs.erase("JobStart");
    interested_attrs.erase("JobUniverse");
    interested_attrs.erase("KFlops");
    interested_attrs.erase("LastBenchmark");
    interested_attrs.erase("LastPeriodicCheckpoint");
    interested_attrs.erase("MyType");
    interested_attrs.erase("NiceUser");
    interested_attrs.erase("Rank");
    interested_attrs.erase("RemoteOwner");
    interested_attrs.erase("RemoteUser");
    interested_attrs.erase("Requirements");
    interested_attrs.erase("State");
    interested_attrs.erase("TargetType");
    interested_attrs.erase("TotalCondorLoadAvg");
    interested_attrs.erase("TotalDisk");
}

void 
DDag::GetStaticAttributes (PoolInfo * pool)
{
    set <string> * sAttributes; 
    set <string>::iterator itr;

    sAttributes=pool->GetStaticAttributes ();
    
    for (itr=sAttributes->begin(); itr!=sAttributes->end(); itr++) {
	interested_attrs.insert(*itr);
    }
}


