/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2001 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
 ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef __DDAG_H_
#define __DDAG_H_

#include "condor_common.h"

using namespace std;


#include <vector>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include "djob.h"
#include "../condor_classad_analysis/analysis.h"
#include "pool_info.h"
#include "dag_property.h"

#if (__GNUC__<3)
#include <hash_map>
#else
#include <ext/hash_map>
#endif

#include "condor_io.h"
#include "condor_state.h"
#include "condor_debug.h"
#include "condor_config.h"
#include "condor_attributes.h"
#include "condor_api.h"
#include "condor_classad_lookup.h"
#include "condor_query.h"
#include "daemon.h"
#include "daemon_types.h"
#include "../condor_status.V6/totals.h"
#include "../condor_daemon_core.V6/condor_daemon_core.h"
#include "dagman_test_utils.h"
#include "condor_distribution.h"
#include <iostream.h>
#include <string>


extern PoolInfo * pool;
extern DagProperty * dag_property;

enum TESTMODE {
    LONGFILE,
    MDSUM,
    NORMAL,
    DAP
};

typedef std::vector <DagNode*> node_vector;

class DDag
{
public:

    /* DDag constructor 
       @param NodeNumber: number of nodes in the dag
       @param DagDirName: directory to store all the files in current dag
       @param universe:   universe in which the jobs will run
       @paralm cycle:     true -> want cycle in the dag
    */
    DDag (int NodeNumber, const char* DagDirName, bool cycle);

    ~DDag();

    /* sets testmode to be LONG, MDSUM or NORMAL 
       @param mode: testmode
       @param testjobname: name of testjob to be run in this dag
     */
    void SetTestMode (TESTMODE mode, const char* testjobname);

    /* 
       This can be used as an argument to testjob
       Function will be removed after implementing ReadArguments()
     */
    void SetRunTime (int runtime);
    
    /* 
       Generates the structure of a dag with nodes and links
       between nodes
     */
    void GenerateDag(); 

    /* 
       Generates the structure of a dag with nodes and links
       between nodes based on user level specification from config file
     */
    void GenerateDagByLevels(); 


    /* 
       Writes condor submit files for each job based on dag_constraint
       specifications 
    */
    bool AttachPropertyToNodes();

    /* Creates directory to store dag files 
       and writes the file random.dag
    */
    bool GenerateDagFile (); 

    /* Generates condor submit files for each node 
       in the dag. 
    */
    bool GenerateSubmitFiles(PoolInfo * pool);

    /* Creates a unique id for the current node in the dag
     */
    NodeID GetNewID();
    
    /* The following is for time-stamp detection of dags 
       that have run 
    */
    DagNode* AddNodeToDag (char * nodeName);

    /* ProcessDagConstraints
       User can pass in constraints via a config file to a particular dag
       to make it have certain properties. This function processes these contraints 
       @param constraint: constraint to the nodes in the dag
    */
    void ProcessDagConstraints (char * constraint);

private:

    bool         DetectCycles (); //from DAG
    void         DFS(DagNode* node); //depth-first-search of dag

    /* Generates input file for a testjob*/
    bool GenerateInputFile (const char * filename);

    /* Generate requirements for the condor submit file 
       @ param submit: file handle of the submit file
    */
    void GenerateRequirements (FILE * submit, string req = "");


    /* Generate "smart" requirements that are targeted
       to machines in the pool
       @ param submit: condor submit file
       @ param pool:   current pool
       @ param machine_ad: target machine ad
    */
    bool GenerateSmartRequirements (FILE * submit, 
				    PoolInfo * pool,
				    ClassAd * machine_ad);



    /* 
       Write MD file check sum to input file 
     */
    void WriteMDFile (FILE * inputFile, const char *filename, 
		      unsigned int bytes);

    /* 
       Given a pool, look for static attributes 
    */
    void GetStaticAttributes (PoolInfo *pool);
    
    /* 
       Static analysis may not be accurate -
       remove attributes we are sure that are not static
       Should change to something configurable by user
    */
    void RemoveNonInterestedAttributes ();

    
    
    TESTMODE   test_mode;
    
    node_vector allNodes; 
    bool        cycle;
    int         dagsize; //number of nodes desired in dag
    int         cur_size; // current dag size

    const char *      dirName; 
    const char *      testjobname;

    NodeID      IDs; 
    int         _runtime; //max run time

    const char  * _universe;
    int         DFS_ORDER;
    
    bool        _intelligence;
    bool        _win_testing; 

    set <string>    interested_attrs;

};

#endif
