#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

pid_t pid; 
FILE * log;

void writelog (const char*);
void printusage (const char*);

int main(int argc, char** argv)
{
  unsigned int MAXBYTES, bytes;
  bool error = false;
  int time_to_run = 0, fix_time, i=1; 
  struct timeval start_tv, end_tv; 
  int gone_time = 0; 
  FILE * output = NULL, * input = NULL;
  unsigned char n;

  pid = getpid();
  log = fopen ("longfiletest.log", "a"); 
  if (log==NULL)
  {
    log = fopen("log1.log", "a");
    if (log == NULL)
      log = fopen("log2.log", "a");
    if (log==NULL)
    {
      cerr<<"Could not open log file for logging"<<endl;
      exit(-1);
    }
  }

  writelog("Command invoked is ");
  for (int i=0; i<argc; i++)
    fprintf(log, "%s ", argv[i]);
  fprintf(log, "\n");

  if (argc<4)
  {
    printusage(argv[0]);
    writelog("Incorrect Usage\n\n");
    fclose (log);
    return -1; 
  }
  if (gettimeofday (&start_tv, NULL)==-1)
  {
    perror("couldn't get start time");
    writelog("Couldn't get start time\n\n");
    fclose (log);
    exit (-1);
  }
  
 read_args:
  while (i<argc)
  {
    if (!strcmp (argv[i], "-t"))
    {
      //if this argument is given, there must be at least 6 args
      if (argc<6)
      {
	cerr<<"Not enough number of arguments"<<endl;
	printusage(argv[0]);
	writelog("Not enough number of arguments\n");
	return -1; 
      }
      //record the time 
      time_to_run = atoi(argv[i+1]);
      printf("Will run for %d seconds\n", time_to_run);
      fprintf(log, "%d: Will run for %d seconds\n", pid, time_to_run);
      i+=2;
      goto read_args;
    }
    
    if (!strcmp (argv[i], "-o"))
    {
      char * out = new char [30];
      sprintf(out, "%s.out", argv[i+1]);
      output = fopen (out, "w");
      if (output==NULL)
      {
	perror("output unsuccessfull");
	writelog("Output unsuccessfull");
	exit (-1);
      }
      i+=2;
      goto read_args; 
    }

    if (!strcmp (argv[i], "-max"))
    {
      if (argc<6)
      {
	cerr<<"Not enough number of arguments"<<endl;
	printusage(argv[0]);
	writelog("Not enough number of arguments\n");
	return -1; 
      }
      MAXBYTES = atoi(argv[i+1]);
      if (MAXBYTES <1 || MAXBYTES>10000000) //1-10MB ok, else set it to just 10K
	MAXBYTES = 3001;
      i+=2;
      goto read_args;
    }
    input = NULL; 
    input = fopen(argv[i], "r"); 
    if (input == NULL)
    {
      fprintf(stderr, "Could not open input %s \n", argv[i]);
      fprintf(log, "%d: Could not open input %s \n", pid, argv[i]);
      error = true; 
      break;
    }
    else 
    {
      int value = 0; 
      fprintf(log, "%d: Scanning for input %s\n", pid, argv[i]);
      fscanf (input, "TOTAL BYTES: %d\n", &value); 
      if (value==0)
      {
	cerr<<"Error occurred while scanning input"<<endl;
	writelog("Error occurred while scanning input\n");
	error = true; 
	break; 
      }
      else 
      {
	for (int j=0; j<value; j++)
	{
	  n=getc(input);
	  if (n!='1')
	  {
	    cerr<<"Error occurred while scanning input, value!=1"<<endl;
	    writelog("Error occurred while scanning input, value!=1\n");
	    error = true;
	    break;
	  }
	}
      }
      fclose(input);
    }
    i++;
  }
  
  if (output==NULL)
  {
    error = true; 
    fprintf(stderr, "Please specify output file using -o option\n");
    writelog("Please specify output file using -o option\n");
    exit (-1);
  }
  
  else 
  {
    if (error)
    {
      fprintf (output, "TOTAL BYTES: 0\n");
      writelog ("Writing output for error case\n");
    }
    else
    {
      bytes = rand()%MAXBYTES;
      fprintf(log, "%d: Writing output file of size: %d\n", pid, bytes);
      fprintf(output, "TOTAL BYTES: %d\n", bytes);
      for (unsigned int i=0; i<bytes; i++)
	fprintf(output, "1");
    }
    fclose (output);
  }
  
  if (gettimeofday (&end_tv, NULL)==-1)
  {
    perror("couldn't get end time");
    writelog("Couldn't get end time\n");
    exit (-1);
  }
  
  gone_time = (end_tv.tv_sec-start_tv.tv_sec)*1000000
    +end_tv.tv_usec-start_tv.tv_usec;
  
  //printf("Gone time = %d\n", gone_time);
  if (time_to_run >=0) //test how long it takes to run 10000 loops 
  {
    int counter = 10000;
    
    if (gettimeofday (&start_tv, NULL)==-1)
    {
      perror("couldn't get start time");
      writelog("Couldn't get start time\n");
      exit (-1);
    }
    while (counter>0)
    {
      counter--; 
    }
    
    if (gettimeofday (&end_tv, NULL)==-1)
    {
      perror("couldn't get end time");
      writelog("Couldn't get end time\n");
      exit (-1);
    }

    fix_time = (end_tv.tv_sec-start_tv.tv_sec)*1000000
      +end_tv.tv_usec-start_tv.tv_usec;
    time_to_run*=1000000;
    time_to_run=time_to_run-gone_time-fix_time; 
    int loopnumber = (int)((time_to_run)/fix_time);

    // printf("fix time = %d for 10000 loops, Now we need %d loops for 
    // %d seconds\n", fix_time, loopnumber, time_to_run);
    for (i=0; i<loopnumber; i++)
    {
      counter = 10000;
      while (counter>0)
	counter--;
    }
  }
  if (!error)
    writelog("Successfully finished!\n\n");
  else
    writelog("Failed\n\n");
  fclose (log);
  return 0; 
}

void writelog (const char * logmsg)
{
  /*
    struct time_t timenow; 
  
  if (gettimeofday (&timenow, NULL)==-1)
  {
    fprintf(log, "%d: %s", pid,"Couldn't get current time\n");
  }
  char * curtime = ctime(&timenow);
  
  fprintf(log, "%d: %s %s", pid, curtime, logmsg);
  */
  fprintf(log, "%d: %s", pid, logmsg);
}

void printusage(const char * argv0)
{

  fprintf(stderr, "Usage: %s \n\t -o\t<output file name> <list of input files>\n", argv0);
  cerr<<"\t[-t\t<time_in_secs to run>]\n\t[-max\t<max number of bytes per output file>]\n\n";
}

