/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#include "pool_info.h"
#include "dagman_test_utils.h"


/************************************************************
PoolInfo class
************************************************************/
PoolInfo::PoolInfo(ClassAdList * ads)
{
    cout<<"PoolInfo::PoolInfo(ClassAdList * ads)"<<endl;
    ad_count = 0;
    startd_ads = ads;
    ClassAd * ad;
    
    ads->Rewind();
    while (ad = startd_ads->Next()) {
	ad_count++;
	readAd (ad);
    }
    startd_ads->Close ();

    rows.clear();
    //PrintSchema ();
    //PrintDebugInfo();
}

// Only add up to count number of ads to the table
PoolInfo::PoolInfo (ClassAdList *ads, int count)
{
    ad_count = 0;
    startd_ads = ads;
    ClassAd * ad;

    rows.clear();

    ads->Rewind();
    while (((ad = startd_ads->Next())!=NULL) && ad_count<count) {
	char * os = new char[30];	
	bool pvm = false;

	ad->EvaluateAttrString (ATTR_OPSYS, os, 30);

	//ad->EvaluateAttrBool (ATTR_HAS_PVM, pvm);

	if (!strcasecmp (os, "LINUX") &&!pvm) {
	    readAd (ad);
	    ad_count++;
	}
	//delete [] os;
	
    }
    startd_ads->Close ();
    /*
    PrintSchema ();
    PrintDebugInfo();
    */
}

// Process the information in current classad
void 
PoolInfo::readAd (ClassAd * ad)
{
    //cout<<"PoolInfo::readAd (ClassAd * ad)"<<endl;
    if (ad == NULL)
	return;

    string attr;
    COL_TYPE attr_type;
    char * machine_name= new char[30];
    char * name        = new char[30];
 
    string sAttr;
    Value result;
    Value::ValueType type;

    int ival;
    double fval;
    bool  bval;
    char * sval;
    ColInfo * col;
    
    string string_val;

    ad->EvaluateAttrString (ATTR_MACHINE, machine_name, 30);

    vector <pair<string, ExprTree *> > ad_components;
    vector <pair<string, ExprTree *> >::iterator itr;

    ad->GetComponents (ad_components);
    
    //printf("------- Machine: %s -------\n",machine_name);
    
    // Every ad is uniquely identified by name, not machine name
    ad->EvaluateAttrString (ATTR_NAME, name, 30);
 
    for (itr=ad_components.begin(); itr!=ad_components.end(); itr++) {
	
	attr = itr->first;

	// Add this attribute to the list of static attributes
	static_attributes.insert (attr);
	
	col= Lookup(attr);

	ad->EvaluateAttr(attr, result);

	attr_type = GetAttributeType (ad, itr->second, type);
	
	if (col == NULL) 
	    col = AddToSchema (attr, attr_type);

	assert (col != NULL);

	CellInfo *new_cell = new CellInfo (attr_type, attr);
	
	switch (attr_type) {

	case INTEGER_TYPE:
	    ad->EvaluateAttrInt(attr, ival);
	    new_cell->Value.int_val = ival;
	    break;
	    
	case REAL_TYPE:
	    ad->EvaluateAttrReal(attr, fval);
	    new_cell->Value.real_val = fval;
	    break; 
  
	case STRING_TYPE:
	    sval = new char[1024];
	    ad->EvaluateAttrString(attr, sval, 1024);
	    string_val = sval;
	    new_cell->Value.string_val = sval;
	    break; 
	    
	case BOOL_TYPE:
	    ad->EvaluateAttrBool (attr, bval);
	    new_cell->Value.bool_val = bval;
	    break;
	    
	case OP_NODE_TYPE:
	    //printf("processing opnode type\n");
	    
	    // flatten op node
	    ExprTree * flattened_expr;
	    ad->Flatten (itr->second, result, flattened_expr);
	    
	    if (flattened_expr != NULL) {
		//printf("opnode cannot be flattened *************\n");
	    }
	    break;
	    
	case ERROR_TYPE:
	case NULL_TYPE:
	case UNDEFINED_TYPE:
	case RELATIVE_TIME_TYPE:
	case ABSOLUTE_TIME_TYPE:
	case CLASSAD_TYPE:
	case LIST_TYPE:
	default:
	    break;
	};

	// Insert cell
	InsertCellInfo (name, attr, new_cell);
    }

    // Insert row
    //cout<<"This = "<<this;
    
    printf(" inserting --- %s, before insert, size = %d\n", name, this->rows.size());
    string key_name = name;

    RowInfo * newrow = new RowInfo (key_name, ad);
    rows.push_back (newrow);
}


/* Returns the key column of the table */
string
PoolInfo:: IdentifyKey ()
{
    //cout<<"PoolInfo:: IdentifyKey ()"<<endl;

    list <ColInfo *>::iterator itr;
    for (itr=schema.begin(); itr!=schema.end(); itr++) {
	printf("Column = %s\n", ((*itr)->GetColName()).c_str());
	if ((*itr)-> IsKey(ad_count)) {
	    key = (*itr)->GetColName();
	    printf("Key identified is: %s\n", key.c_str());
	    return key;
	}
    }
    
    // If a key cannot be found, we need a compound key
    FindCompositeKey ();
}    

/* Finding the key to a table 
   Since there is really no smart way to find out the key 
   start from some educated guesses: 
   1) In our machine ad context, key's likely to contain a string
   2) Do not allow null/undefined/error value in the column
*/

void 
PoolInfo::FindCompositeKey()
{  
    //cout<<"PoolInfo::FindCompositeKey()"<<endl;
    list <ColInfo *>::iterator itr;
    list <string>   candidate_key;

    int key_size = 0;

    for (key_size = 0; key_size<schema.size(); key_size++) {
	for (itr=schema.begin(); itr!=schema.end(); itr++) {
	    ColInfo * col = *itr;
	    if (col->undefined >0 || col->null_num >0 ||
		col->error_num>0 ||  col->col_type != STRING_TYPE) {
		continue;
	    }
	    
	    // "remember key" and try
	    else {
		candidate_key.push_back (col->GetColName());
		
		if (candidate_key.size()>1 && key_size == candidate_key.size()) {
		    
		    // pick a different set of keys if not found
		    if (!IsCompositeKey (candidate_key)) {
			candidate_key.clear();
		    }
		    
		    // Found a composite key!!
		    else {
			return;
		    }
		}
	    }
	}
    }
    fprintf(stderr, "Could not find a composite key for this table!\n");
}

/* Check if a list of columns can form a composite key*/
bool 
PoolInfo::IsCompositeKey (list <string> candidate_keys)
{
    //cout<<"PoolInfo::IsCompositeKey "<<endl;
    set  <string> temp_table;
    list <ColInfo * > columns;
    list <ColInfo * > ::iterator itr;
    list <string>::iterator key_itr;

    // Get the names of columns involved
    for (key_itr=candidate_keys.begin(); key_itr!=candidate_keys.end(); key_itr++) {
	columns.push_back (Lookup (*key_itr));
    }
	
    hash_map<const char*, CellInfo*> ::iterator cell_itr;
    
    // Loop through all the rows
    ColInfo * col = *(columns.begin());
    for (cell_itr=col->cell_values.begin(); 
	 cell_itr!=col->cell_values.end(); 
	 cell_itr++) {

	const char * key_name = cell_itr->first;
	string long_string;

	CellInfo * cell = cell_itr->second;
	if ( cell->attr_type != STRING_TYPE) {
	    return false;
	}
	
	long_string = cell->Value.string_val;
	
	// Add in value from the same row, different cols
	for (itr=columns.begin(); itr!=columns.end(); itr++) {
	    long_string+=(*itr)->cell_values[key_name]->Value.string_val;
	}
	
	// Insert value into the temporary table
	int old_size = temp_table.size();
	temp_table.insert(long_string);
	
	// If the size of the set didn't increase, that means
	// there are duplicates, thus this is not a valid 
	// composite key
	if (temp_table.size() == old_size) 
	    return false;
    }
    
    // Assign value to composite key
    composite_key = candidate_keys;
    return true;
}



void
PoolInfo::AddMachine(string machine_name)
{
    //cout<<"PoolInfo::AddMachine"<<endl;
    machines.insert (machine_name);
}

// Reset machine_iterator
void 
PoolInfo::RewindRow()
{
    //cout<<"PoolInfo::RewindRow()"<<endl;
    row_iterator = rows.begin();
    
}

// Get classad of current machine 
// Then advance the iterator 
ClassAd * 
PoolInfo::GetNextRow()
{
    //cout<<"PoolInfo::GetNextRow()"<<endl;
    if (row_iterator == rows.end())
	return NULL;

    ClassAd * current_ad = (*row_iterator)->ad;

    row_iterator++;
    return current_ad;
}

void 
PoolInfo::InsertCellInfo (const char * machine_name,
			  string attribute_name,
			  CellInfo * value)
{
    //cout<<"PoolInfo::InsertCellInfo"<<endl;
    ColInfo * col;
    if ((col = Lookup (attribute_name)) == NULL) {
	if ((col=AddToSchema (attribute_name, value->attr_type)) == NULL) {
	    dprintf(D_ALWAYS, "Could not create new column in pool information table\n");
	    return;
	}
    }
    
    col->InsertCellInfo (machine_name, value);
}


ColInfo* 
PoolInfo::AddToSchema (string attribute_name,
		       COL_TYPE type)
{
    //cout<<"PoolInfo::AddToSchema"<<endl;
    ColInfo * col = new ColInfo (attribute_name,
				 type);
    schema.push_back (col);
    return col;
}




/* Returns a column given an attribute name */
ColInfo * 
PoolInfo::Lookup (string attr_name)
{
    //cout<<"PoolInfo::Lookup "<<attr_name<<endl;
    list <ColInfo *>::iterator itr;

    for (itr=schema.begin(); itr!=schema.end(); itr++) {
	if (!strcasecmp (((*itr)->GetColName()).c_str(), attr_name.c_str())) {
	    return *itr;
	}
    }
    //printf("Cannot find %s\n", attr_name.c_str());
    return NULL;
}


/* Returns a cell given the key and the attribute name */
CellInfo * 
PoolInfo::Lookup (const char *  key,
		  string attribute_name) 
{
    ColInfo * col = Lookup (attribute_name);
    if (col == NULL) 
	return NULL;

    return col->Lookup (key);
}

/* Prints out the schema information */
void
PoolInfo::PrintSchema ()
{
    list <ColInfo *>::iterator itr;
    
    printf ("Schema contains (%d) items:\n", schema.size());
    for (itr=schema.begin(); itr!=schema.end(); itr++) {
     	printf(" - %s\n", (*itr)->GetColName().c_str());
    }
    printf("-----------------\n");
}


void 
PoolInfo::PrintDebugInfo ()
{
    printf ("Printing pool information ----\n");
    printf ("There are %d machines\n", GetMachineCount());
    list <ColInfo *>::iterator itr;

    for (itr=schema.begin(); itr!=schema.end(); itr++) {
	ColInfo * col = *itr;

	col->PrintColumnInfo (this);
    }
}

/* FindStaticAttributes - 
     Read the startd ads several times and determine which attributes
     are static and which change with time
     @ param wait_time - # seconds of wait between fetching classads from collector
     @ param loops - # of times to fetch classads from the collector
*/
void PoolInfo::FindStaticAttributes (int wait_time, int loops)
{
    if (wait_time < 0) wait_time = 0;

    if (loops <= 0) loops = 1;

    list <RowInfo*>::iterator itr, current_itr;

    for (int i=0; i<loops; i++) {

	ClassAdList startdAds;

	// wait 5 seconds to obtain new ads 
	
	printf("PoolInfo::FindStaticAttributes ***** sleeping for %d sec\n",
	       wait_time);

	sleep (wait_time);

	if (!obtainAdsFromCollector(startdAds)) {
	    dprintf(D_ALWAYS, "Could not fetch startd ads\n");
	    fprintf(stderr, "Could not fetch startd ads\n");
	    exit (-1);
	}
	
	PoolInfo info (&startdAds, 5);
	
	//compare the current pool information with the new pool information
	for (itr=rows.begin(); itr!=rows.end(); itr++) {
	    printf("****** For key name = %s\n-------------------------------\n",
		   ((*itr)->key).c_str());
	    
	    string attr_name = (*itr)->key;

	    // Picking out the right machine ad to compare with
	    ClassAd * current = info.GetRow(attr_name);
	    if (current != NULL)
		CompareClassAds(current, (*itr)->ad);
	}
    }
    
    // Debug information
    printf(" Non static attribute list: \n");
    set <string>::iterator attr_itr;

    for (attr_itr=temp_non_static.begin(); attr_itr!=temp_non_static.end(); attr_itr++) {
	cout<<*attr_itr<<endl;
    }
    
}

ClassAd *
PoolInfo::GetRow (string keyname)
{
    
    //cout<<"PoolInfo::GetRow  "<<endl;
    //cout<<"Getting row "<<keyname<<endl;
    
    list<RowInfo *>::iterator itr;

    for (itr=rows.begin(); itr!=rows.end(); itr++) {
	//cout<<"    Row ------ "<<(*itr)->key<<endl;
	if (!strcmp(keyname.c_str(), ((*itr)->key).c_str())) {
	    //cout<<"found it!"<<endl;
	    return (*itr)->ad;
	}
    }
    cout<<keyname<<" could not be found"<<endl;
    return NULL;
}
		  

void
PoolInfo::CompareClassAds (ClassAd * ad1, ClassAd *ad2)
{ 
    //cout<<"void CompareClassAds "<<endl;

    AttrList::iterator ad1_itr;
    Value value1, value2;
    string attr1, attr2;
    bool equal = true;     //Values of the same attribute from ad1 and ad2 are equal
    bool bval1, bval2;
    double dval1, dval2;
    string sval1, sval2;
    int ival1, ival2;

    for (ad1_itr=ad1->begin(); ad1_itr!=ad1->end(); ad1_itr++) {
	
	attr1=ad1_itr->first;

	//printf("\nattribute 1 = %s, attribute 2 = %s\n", attr1.c_str(), attr2.c_str());
	equal = true;

	if (!ad1->EvaluateAttr (attr1, value1)) {
	    fprintf(stderr, "Could not evaluate attribute %s of ad1\n", 
		    attr1.c_str());
	    RemoveNonStaticAttribute (attr1);
	}
	
	else if (!ad2->EvaluateAttr(attr1, value2)) {
	    fprintf(stderr, "Could not evaluate attribute %s of ad2\n", 
		    attr1.c_str());
	    RemoveNonStaticAttribute (attr1);
	}

	else {
	    if (value1.GetType() != value2.GetType()) {
		cout<<"Types are different ---- \n";
		equal = false;
	    }



	    switch (value1.GetType()) {
	    case Value::NULL_VALUE:
	    case Value::ERROR_VALUE:
	    case Value::UNDEFINED_VALUE:
		//printf("value = null/error/undefined\n");
		break;
		
	    case Value::BOOLEAN_VALUE:
		ad1->EvaluateAttrBool(attr1, bval1);
		ad2->EvaluateAttrBool(attr1, bval2);
		/*
		if (bval1)
		    printf("value = true\n");
		else 
		    printf("value = false\n");
		*/
		if (bval1!=bval2) equal = false;
		break;

	    case Value::INTEGER_VALUE:
		ad1->EvaluateAttrInt(attr1, ival1);
		ad2->EvaluateAttrInt(attr1, ival2);
		//printf("value = %d\n", ival1);
		if (ival1!=ival2) equal=false;
		break;

	    case Value::REAL_VALUE:
		ad1->EvaluateAttrReal (attr1, dval1);
		ad2->EvaluateAttrReal (attr1, dval2);
		//printf("value = %f\n", dval1);
		if (dval1!=dval2) equal=false;
		break;

	    case Value::RELATIVE_TIME_VALUE:
	    case Value::ABSOLUTE_TIME_VALUE:
		//printf("value = time value\n");
		equal = false;
		break;

	    case Value::STRING_VALUE:
		ad1->EvaluateAttrString (attr1, sval1);
		ad2->EvaluateAttrString (attr1, sval2);
		//printf("value = %s\n", sval1.c_str());
		if (sval1!=sval2) equal=false;
		break;

	    case Value::CLASSAD_VALUE:
		//printf("value = classad\n");
		equal = false;
		break;
		
	    case Value::LIST_VALUE:
		//printf("value = list\n");
		equal = false;
		break;
		
	    default:
		equal = false;
		break;
	    };
	    
	    if (!equal) {
		// This attribute is not a static attribute
		RemoveNonStaticAttribute (attr1);
		cout <<"\t"<<attr1<< "\tis NOT a static attribute"<<endl;
	
	    }
	}
    }
}



void 
PoolInfo::RemoveNonStaticAttribute (string attr)
{
    //printf("Before removing %s, size = %d", attr.c_str(), static_attributes.size());
    static_attributes.erase (attr);
    temp_non_static.insert(attr);
    //printf(" now size = %d\n", static_attributes.size());
}

void PrintType (COL_TYPE col_type)
{
    switch (col_type) {
    case INTEGER_TYPE:
	printf("(int) ");
	break;
	
    case REAL_TYPE:
	printf("(real) ");
	break;

    case BOOL_TYPE:
	printf("(bool) ");
	break;

    case STRING_TYPE:
	printf("(string)");
	break;

    case UNDEFINED_TYPE:
	printf("(undefined) ");
	break;
	
    case RELATIVE_TIME_TYPE:
	printf("(relative time)");
	break; 
	    
    case ABSOLUTE_TIME_TYPE:
	printf("(absolute time)");
	break; 

    case CLASSAD_TYPE:
	printf("(classad)");
	break;

    case LIST_TYPE:
	printf("(list)");
	break;

    case ATTR_REF_TYPE:
	printf("(attribute reference)");
	break;

    case OP_NODE_TYPE:
	printf("(op_node)");
	break;

    case FN_CALL_TYPE:
	printf("(fn_call)");
	break;

    case CLASSAD_NODE_TYPE:
	printf("(classad_node)");
	break;

    case LIST_NODE_TYPE:
	printf("(list_node)");
	break;

    default:
	printf("(unknown) ");
	break;	
    };
}

set <string> * PoolInfo::GetStaticAttributes()
{
    return &static_attributes; 
}

list <string> * PoolInfo::GetSchema ()
{
    list <ColInfo*> ::iterator itr;
    list <string> * schema_info = new list<string>;

    for (itr=schema.begin(); itr!=schema.end(); itr++) {
	schema_info->push_back((*itr)->GetColName());
    }
    return schema_info;
}
