/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
 ****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

#ifndef _POOL_INFO_H
#define _POOL_INFO_H

#include "condor_common.h"
#include "condor_attributes.h"
#include "condor_api.h"
#include "condor_classad_lookup.h"
#include "condor_query.h"
#include <list>
#include <set>
#include <string>
#include <iostream.h>

#if (__GNUC__<3)
#include <hash_map>
#else 
#include <ext/hash_map>
#endif


using namespace std;



enum COL_TYPE {
    NULL_TYPE,
    INTEGER_TYPE,
    REAL_TYPE, 
    BOOL_TYPE,
    RELATIVE_TIME_TYPE,
    ABSOLUTE_TIME_TYPE,
    STRING_TYPE,
    ERROR_TYPE,
    UNDEFINED_TYPE,
    CLASSAD_TYPE,
    LIST_TYPE,

    ATTR_REF_TYPE,
    OP_NODE_TYPE,
    FN_CALL_TYPE,
    CLASSAD_NODE_TYPE,
    LIST_NODE_TYPE
};


class ColInfo;
class CellInfo;
class RowInfo;
class PoolInfo;

void PrintType (COL_TYPE col_type);

typedef  list <ColInfo *> SchemaList;

class ColInfo 
{
public:

    ColInfo (string attribute_name,
	     COL_TYPE column_type);

    //~ColInfo ();

    void InsertCellInfo (const char * machine_name,
			 CellInfo * value);

    CellInfo * Lookup (const char *machine_name);

    string GetColName ();

    void   PrintColumnInfo (PoolInfo * pool);
    void   PrintColumnSummary (PoolInfo * pool);

    void   set_max_min_int (int ival);
    void   set_max_min_real (double fval);
    void   add_undefined ();
    void   add_error ();
    void   add_bool (bool bool_val);
    void   add_string (string str);
    void   add_null ();

    int    GetUndefined (PoolInfo * pool);

    /* Is this column a key to the table? */
    bool   IsKey (int total_ads);
    
private:

    friend class PoolInfo;
    
    string col_name;

    COL_TYPE col_type;

    // Counters used for summary information
    int max_int_val;
    int min_int_val;
    
    double max_real_val;
    double min_real_val;

    int num_true;
    int num_false;
    
    // Counters for other types of values
    int undefined; 
    int null_num;
    int error_num;

    set <string> string_vals;
    set <int>    int_vals;
    set <double> real_vals;
    
    hash_map <const char * , CellInfo *> cell_values;
};


class RowInfo 
{
public:
    RowInfo (string key, ClassAd * ad);
    
private:
    friend class PoolInfo; 

    string key;
    ClassAd * ad;

};

class CellInfo
{
public:
    
    CellInfo (COL_TYPE attr_type, 
	      string name);
    //string machine_name);
    union {
	int    int_val;
	double real_val;
	char*  string_val; 
	bool   bool_val;
    } Value;
    
    COL_TYPE attr_type;
    
    void PrintCell();

private:
    string attr_name;
    string machine_name;
};

class PoolInfo 
{
public:
    
    PoolInfo (ClassAdList * ads);
    PoolInfo (ClassAdList * ads, int count);

    // Lookup functions 
    CellInfo * Lookup (const char * machine_name, 
		       string       attr_name);
    ColInfo * Lookup  (string       attr_name);


    //Schema related information
    list <string> * GetSchema();
    
    ColInfo*  AddToSchema (string   attribute_name,
			   COL_TYPE type);

    
    // Insert into table
    void  InsertCellInfo (const char * machine_name, 
			  string       attr_name,
			  CellInfo *   cell_value);
    
    // Debugging related
    void PrintDebugInfo ();
    void PrintSchema    ();
    
  
    // Machine related information
    int  GetMachineCount () {return machines.size();}
    void AddMachine (string machine_name);


    // Identify key of the key
    string IdentifyKey ();
    void   FindCompositeKey ();

    // Get number of ads 
    int    GetAdCount () {return ad_count;};

    // Get key
    string GetKey () {return key;};

    // Sets iterator to be at first row of the table
    void   RewindRow (); 

    // Gets the next row in the pool
    ClassAd * GetNextRow();
    
    // Find out what attributes have been changed now as compared
    // to when this PoolInfo object was created
    void   FindStaticAttributes (int wait_time, int loops);

    ClassAd * GetRow (string key);
    
    set <string> * GetStaticAttributes ();

private:
    // put current ad information into table
    void readAd (ClassAd *ad);

    bool IsCompositeKey (list <string> candidate_key);

    void CompareClassAds (ClassAd * ad1, ClassAd *ad2);

    void RemoveNonStaticAttribute (string attribute_name);
   
    ClassAdList *    startd_ads;
  
    list <ColInfo *> schema;              // Columns of the table
    list <RowInfo *> rows;                // Rows of the table 
    set  <string >   machines;            // List of machines

    set  <string >   static_attributes;
    set  <string >   temp_non_static;

    list <RowInfo * >::iterator row_iterator;

    int              ad_count; //Number of distinct startd ads 
    string           key;
    list <string>    composite_key;


};



#endif
