#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>

#if !defined(WIN32)
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#else
#include <time.h>
#include <process.h>
#endif 

#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>



/* The following are error exit codes */
#define OUT_EXISTS      126  /* Output file already exists */
#define NO_LOGGING      125  /* Cannot open any log file */
#define START_TIME_ERR  124  /* Cannot get start time */
#define END_TIME_ERR    123
#define TIME_ARG        122  /* Incorrect argument after -t*/ 
#define INCORRECT_USAGE 121  /* Incorrect usage */
#define OUTFILE_ERROR   120  /* Cannot open output file for writing*/
#define NO_OUTFILE      119  /* No output file specified after -o */

#ifndef WIN32
pid_t pid;
#else
int pid; 
#endif

void printusage (const char*);
void printtime ();
FILE * log;

int main(int argc, char** argv)
{
    bool    error = false;
    int     time_to_run = 0; 
    int     gone_time = 0; 
    FILE *  output = NULL, * input = NULL;
    char    logname [35];
    int     i;

#ifndef WIN32
    char    * LOG_FILE_NAME = "testjob_ning.log";
    int    fix_time = 0;
    struct timeval start_tv, end_tv; 
    pid = getpid();

#else

    pid = _getpid();
    char   LOG_FILE_NAME [35];
    sprintf(LOG_FILE_NAME, "%s.log", argv[2]);

#endif     

    if (argc>=3)
	sprintf(logname, "%s.log", argv[2]);
    else
	sprintf(logname, "%d.log", pid);

    // Open log file
    log = fopen (LOG_FILE_NAME, "a"); 

    
    if (log==NULL)
    {
	
	char temp [200];
	sprintf(temp, "%s: Open of log file failed", argv[2]);
	
	perror(temp);
	log = fopen(logname, "w");

	fprintf (log, "%d: This log is generated by %s\n",
		 pid, argv[0]);
	if (log == NULL)
	{
	    cerr<<"Could not open log file for logging"<<endl;
	    exit(NO_LOGGING);
	}
    }

    char host_name[30];

    if (gethostname(host_name, 30) == 0) {
        printtime();
        fprintf(log, "%d: executing on %s\n", pid, host_name);
    }
    else {
        printtime();
        fprintf(log, "%d: Could not gethostname()\n", pid);
    }
    
    fflush (log);
    
    if (argc<4)
    {
	printusage(argv[0]);
	fprintf(stderr, "Incorrect usage\n");
	exit(INCORRECT_USAGE); 
    }

#ifndef WIN32
    if (gettimeofday (&start_tv, NULL)==-1)
    {
	perror("couldn't get start time");
	exit (START_TIME_ERR);
    }
#endif
   
    i = 1;
 read_args:
    while (i<argc)
    {
	
	if (!strcmp (argv[i], "-t"))
	{
	    //if this argument is given, there must be at least 6 args
	    if (argc<6) {
		cerr<<"Not enough number of arguments"<<endl;
		printusage(argv[0]);
		exit(TIME_ARG); 
	    }
	    
	    
	    //record the time 
	    time_to_run = atoi(argv[i+1]);
	    i+=2;
	    goto read_args;
	}
	
	if (!strcmp (argv[i], "-o")) {
	    char * out = new char [30];
	    sprintf(out, "%s.out", argv[i+1]);
	
	    //Check if output already exists 
	    struct stat filestat; 
	    if (!stat (out, &filestat)) {
		cerr<<"Output already exists"<<endl;
		exit (OUT_EXISTS);
	    }
	    output = fopen (out, "w");
	    if (output==NULL) {
			perror("output unsuccessfull");
			fprintf(stderr, "Cannot open output file %s!\n");
			exit (OUTFILE_ERROR);
	    }
	    i+=2;
	    goto read_args; 
	}
    
	input = NULL; 
	input = fopen(argv[i], "r"); 
	if (input == NULL) {
	    fprintf(stderr, "Could not open input %s \n", argv[i]);
	    error = true; 
	    break;
	}
	else {
	    int value = 0; 
	    fscanf (input, "%d", &value); 
	    if (value==0) {
		error = true; 
		fprintf(stderr, "Error occurred while scanning input\n");
		break; 
	    }
	    fclose(input);
	}
		i++;
    }
    
    if (output==NULL) {
	error = true; 
	fprintf(stderr, "Please specify output file using -o option\n");
	exit (NO_OUTFILE);
    }
    
    else {
	if (error) {
	    cout<<"0";
	    fprintf(output, "0"); 
	}
	else {
	    cout<<"1";
	    fprintf(output, "1");
	}
	fclose (output);
    }
    
#ifndef WIN32
    if (gettimeofday (&end_tv, NULL)==-1) {
  
	perror("couldn't get end time");
	exit (END_TIME_ERR);
    }
  
    gone_time = (end_tv.tv_sec-start_tv.tv_sec)*1000000
	+end_tv.tv_usec-start_tv.tv_usec;
    

    //test how long it takes to run 10000 loops
    if (time_to_run >=0)  {
	int counter = 10000;
	
	if (gettimeofday (&start_tv, NULL)==-1) {
	    perror("couldn't get start time");
	    exit (START_TIME_ERR);
	}
	while (counter>0) {
	    counter--; 
	}
    
	if (gettimeofday (&end_tv, NULL)==-1) {
	    perror("couldn't get end time");
	    exit (END_TIME_ERR);
	}
	
	fix_time = (end_tv.tv_sec-start_tv.tv_sec)*1000000
	    +end_tv.tv_usec-start_tv.tv_usec;
	time_to_run*=1000000;
	time_to_run=time_to_run-gone_time-fix_time; 
	int loopnumber = (int)((time_to_run)/fix_time);
	
	for (i=0; i<loopnumber; i++) {
	    counter = 10000;
	    while (counter>0)
		counter--;
	}
    }
#endif
    return 0; 
}


void printusage (const char* argv0)
{
    fprintf(stderr, "Usage: %s -o <outputfile> <list of input files> -t \
<time_in_sec>\n", argv0);

}

void printtime()
{
    time_t curtime = time (NULL);
    struct tm * timeptr = localtime(&curtime);
    
    asctime (timeptr);
    fprintf(log, "%02d/%02d %02d:%02d:%02d ", 
	    timeptr->tm_mon, 
	    timeptr->tm_mday,
	    timeptr->tm_hour,
	    timeptr->tm_min,
	    timeptr->tm_sec);
}


