/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 

/*
This program computes for approximately 10 hours, then exits with
status 0.  First the program grows its data size to match a command
line argument.
	Every 10 seconds the total running seconds is printed to stdout.
	Every minute the resource usage is printed to stdout.
	Every minute the current time is printer onto a "timestamp" file.
	Every minute the avgv and environ are checked against a saved buffer.
The program just counts in a loop, so the actual time may vary if the
CPU is faster or slower than expected.

Command line arguments
	1. Size the program should grow to (in megabytes ).
	2. Name of the "timestamp" file.
*/
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/file.h>
#if defined(LINUX) && defined(GLIBC22)
#include <time.h>
#endif
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>

#if defined(LINUX) && defined(GLIBC)
#define getrusage __hide_getrusage
#endif

#include <sys/resource.h>

#if defined(LINUX) && defined(GLIBC)
#undef getrusage
#ifdef __cplusplus
extern "C" int getrusage(int, struct rusage * );
#else
extern int getrusage(int, struct rusage * );
#endif
#endif

#ifdef Solaris251
#ifdef __cplusplus
extern "C" int getrusage(int who, struct rusage *rusage);
#else
int getrusage(int who, struct rusage *rusage);
#endif
#endif


#ifdef __cplusplus
extern "C" {
#endif

extern int CkptInProgress;

long data_start_addr();

void save_strings( int argc, char *argv[], char **envp, char *buf );
void check_strings( int argc, char *argv[], char **envp, char *buf );
char *append( char *str, char *buf );
void gen_core();
void report_rusage();
void check_size();
void timestamp();
void do_loop( int count, int argc, char *argv[], char **environ );
void waste_a_second();
void open_timestamp_file( char *name );

extern char	**environ;

#define MEG (1024 * 1024)
char	Buf1[4096], Buf2[4096];

main( int argc, char **argv)
{
	int		count = 36000;
	int		size;
	int		i;
#if defined(DEBUGGING)
	int		wait_for_me = 1;

	while( wait_for_me )
		;
#endif

	save_strings( argc, argv, environ, Buf1 );

	if( argc != 3 ) {
		fprintf( stderr, "Usage: %s megs filename\n", argv[0] );
		exit( 1 );
	}

	size = atoi( argv[1] );

	open_timestamp_file( argv[2] );

	check_size();
	for( i=0; i<size; i++ ) {
		(void)sbrk( MEG );
		check_size();
	}

	do_loop( count, argc, argv, environ );
	printf( "Normal End of Job\n" );

	exit( 0 );
}

void do_loop( int count, int argc, char *argv[], char **environ )
{
	int		i;

	for( i=0; i<count; i++ ) {
		if( i % 60 == 0 ) {
			printf( "\n" );
			report_rusage();
			timestamp();
			memset( Buf2, '\0', sizeof(Buf2) );
			save_strings( argc, argv, environ, Buf2 );
			if( memcmp(Buf1,Buf2,sizeof(Buf1)) == 0 ) {
				printf( "Check %d: Buffers are equal\n", i/60 );
				fflush( stdout );
			} else {
				printf( "Check %d: Buffers differ\n", i );
				gen_core();
			}
		}
		waste_a_second();
		if( i % 10 == 0 ) {
			printf( "%d ", i );
			fflush( stdout );
		}
	}
	printf( "\n" );
	report_rusage();
}


#define UNIT 10000

#if defined(ALPHA)
#	define LIM (800 * UNIT)
#elif defined(SPARC) || defined(Solaris)
#	define LIM (260 * UNIT)
#elif defined(HPPAR)
#	define LIM (260 * UNIT)
#elif defined(LINUX)
#	define LIM (260 * UNIT)
#elif defined(SGI)
#	define LIM (80 * UNIT)
#endif

void waste_a_second()
{
	int		i;

	for( i=0; i<LIM; i++ )
		;
}

void usage()
{
	printf( "Usage: loop count\n" );
	exit( 1 );
}

void
report_rusage()
{
	struct rusage	buf;

	if( getrusage(RUSAGE_SELF,&buf) < 0 ) {
		printf( "getrusage(%d) failed\n", (int)RUSAGE_SELF );
		return;
	}

	printf( "Resource usage for Self:\n" );
	printf( "\tUserTime = %d.%06d\n",
		buf.ru_utime.tv_sec, buf.ru_utime.tv_usec );
	printf( "\tSystemTime = %d.%06d\n",
		buf.ru_stime.tv_sec, buf.ru_stime.tv_usec );
}

int	Tfd;

void open_timestamp_file( char *name )
{
	if( (Tfd = open(name,O_WRONLY|O_CREAT|O_TRUNC,0644)) < 0 ) {
		perror( name );
		exit( 1 );
	}
}

void timestamp()
{
	time_t	now;
	char	*time_string;

	if( lseek(Tfd,0,2) < 0 ) {
		perror( "lseek" );
		exit( 1 );
	}

	now = time(0);
	time_string = ctime( &now );
	write( Tfd, time_string, strlen(time_string) );
}

void
save_strings( int argc, char *argv[], char **envp, char *buf )
{
	int		i;

	for( i=0; i<argc; i++ ) {
		buf = append( argv[i], buf );
	}
	for( envp++; *envp; envp++ ) {
		buf = append( *envp, buf );
	}
}

char *
append( char *str, char *buf )
{
	strcpy( buf, str );
	while( *buf ) {
		buf++;
	}
	return buf + 1;
}

void
check_size()
{
	printf(
		"VM size is now %.1f\n",
		(float)( ((long)sbrk(0) - data_start_addr()) / MEG )
	);
}

void gen_core()
{
	char	*foo;

	foo = (char *)0;
	*foo = 'a';
	exit( 99 );
}

#if defined(OSF1)
#	include <sys/addrconf.h>
	static struct addressconf AddrTab[ AC_N_AREAS ];
	const int AddrTabSize = sizeof( AddrTab );

	void
	init_addr_tab()
	{
		static int initialized = FALSE;

		if( initialized ) {
			return;
		}

		if( getaddressconf(AddrTab,AddrTabSize) != AddrTabSize ) {
			perror( "getaddressconf" );
			exit( 1 );
		}
	}

	/*
	  Return starting address of the data segment
	*/
	long
	data_start_addr()
	{
		init_addr_tab();
		return (long)AddrTab[ AC_DATA ].ac_base;
	}

#elif defined(Solaris)
#	if defined(X86)
#	include <sys/elf_386.h>
#	else
#	include <sys/elf_SPARC.h>
#	endif
extern int _etext;
long
data_start_addr()
{
#	if defined(X86)
	return (  (((long)&_etext) + ELF_386_MAXPGSZ) + 0x8 - 1 ) & ~(0x8-1);
#	else
	return (  (((long)&_etext) + ELF_SPARC_MAXPGSZ) + 0x8 - 1 ) & ~(0x8-1);
#	endif
}

#elif defined(HPUX)
	/*
	  Return starting address of the data segment
	*/
	extern int __data_start;
	long
	data_start_addr()
	{
		return (long)&__data_start;
	}

#elif defined(LINUX)
	long
	data_start_addr()
	{
		return 0; /* Need to write this yet */
	}
#elif defined(IRIX)
#include <sys/immu.h>
long
data_start_addr()
{
	return USRDATA;
}
#endif

#ifdef __cplusplus
}
#endif
