/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

extern char	**environ;

const int MATCH = 0;	// for strcmp()

class StrVector {
public:
	StrVector( int max );
	void Init( int max );		// hack to fix constructor error
	void Add( const char *str );
	char *Get( int idx );
private:
	int		max_size;
	int		cur_size;
	char	**data;
};

StrVector::StrVector( int max )
{
	max_size = max;
	data = new char *[ max ];
	assert( data );
	cur_size = 0;
}

void StrVector::Init( int max )
{
	max_size = max;
	data = new char *[ max ];
	assert( data );
	cur_size = 0;
}

void
StrVector::Add( const char *str )
{
	assert( cur_size < max_size );

	data[ cur_size ] = new char[ strlen(str) + 1 ];
	assert( data[cur_size] );
	strcpy( data[cur_size], str );
	cur_size += 1;
}

char *
StrVector::Get( int idx )
{
	if( idx >= cur_size ) {
		return NULL;
	} else {
		return data[ idx ];
	}
}

StrVector SaveEnv( 256 );
StrVector SaveArgs( 256 );

void check_vector( char	**vec, StrVector &saved, const char *name );
extern "C" void ckpt_and_exit();

main( int argc, char *argv[] )
{
	int		i;
	char	*str;

	SaveEnv.Init( 256 );
	SaveArgs.Init( 256 );

	for( i=0; environ[i]; i++ ) {
		SaveEnv.Add( environ[i] );
		printf( "environ[%d] = \"%s\"\n", i, environ[i] );
	}

	for( i=0; i<argc; i++ ) {
		SaveArgs.Add( argv[i] );
		printf( "argv[%d] = \"%s\"\n", i, argv[i] );
	}
	printf( "\n" );

	check_vector( environ, SaveEnv, "Environment" );
	check_vector( argv, SaveArgs, "Argument" );
	printf( "\n" );

	fflush( stdout );
	ckpt_and_exit();

	check_vector( environ, SaveEnv, "Environment" );
	check_vector( argv, SaveArgs, "Argument" );
	printf( "\n" );

	fflush( stdout );
	ckpt_and_exit();

	check_vector( environ, SaveEnv, "Environment" );
	check_vector( argv, SaveArgs, "Argument" );
	printf( "\n" );

	printf( "Normal End Of Job\n");

	exit( 0 );
	return 0;
}

void
check_vector( char	**vec, StrVector &saved, const char *name )
{
	int		i;
	char	*s1, *s2;

	for( i=0; vec[i]; i++ ) {
		s1 = saved.Get(i);
		s2 = vec[i];
		assert( strcmp(s1,s2) == MATCH );
	}
	assert( saved.Get(i) == NULL );

	printf( "%s vector OK\n", name );
}
