/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 


/*
** This program just computes for an approximate amount of time, then
** exits with status 0.  The approximate time in seconds is specified
** on the command line.  It just counts in a loop, so the actual time
** will vary greatly depending on what kind of CPU it gets run on.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>


#if defined(LINUX) && defined(GLIBC)
#define getrusage __hide_getrusage
#endif

#include <sys/resource.h>

#if defined(LINUX) && defined(GLIBC)
#undef getrusage
#ifdef __cplusplus
extern "C" int getrusage(int, struct rusage * );
#else
extern int getrusage(int, struct rusage * );
#endif
#endif


#ifdef __cplusplus
extern "C" void ckpt_and_exit();
#endif

#ifdef Solaris251
#ifdef __cplusplus
extern "C" int getrusage(int who, struct rusage *rusage);
#else
int getrusage(int who, struct rusage *rusage);
#endif
#endif


extern int CkptInProgress;
extern int errno;

#define MEG (1024 * 1024)

#ifdef __cplusplus
extern "C" {
#endif

void do_loop( int count, int ckptfreq );
void usage();
void waste_a_second();

int main( int argc, char *argv[])
{
	int		count;
	int		i;
	int		ckptfreq = 0;

	switch( argc ) {

	  case 1:
		if( scanf("\n%d", &count) != 1 ) {
			fprintf( stderr, "Input syntax error\n" );
			exit( 1 );
		}
		break;

	  case 2:
		if( (count=atoi(argv[1])) <= 0 ) {
			usage();
		}
		break;
		
	  case 3:
		if( (count=atoi(argv[1])) <= 0 ) {
			usage();
		}
		if( (ckptfreq=atoi(argv[2])) <= 0 ) {
			usage();
		}
		break;

	  default:
		usage();

	}

#if 0
	sbrk( 5 * MEG );
#endif

	do_loop( count, ckptfreq );
	printf( "Normal End of Job\n" );

	exit( 0 );
}



void do_loop( int count, int ckptfreq )
{
	int		i;
	struct rusage   buf;

	for( i=0; i<count; i++ ) {
		waste_a_second();
		printf( "%d\t", i );

		if( getrusage(RUSAGE_SELF,&buf) < 0 ) {
			printf( "getrusage(%d) failed: %d(%s)\n", (int)RUSAGE_SELF, 
					errno, strerror(errno));
			exit(1);
		} else {
			printf( "\tUserTime = %d.%06d",
				buf.ru_utime.tv_sec, buf.ru_utime.tv_usec );
			printf( "\tSystemTime = %d.%06d",
				buf.ru_stime.tv_sec, buf.ru_stime.tv_usec );
		}
		
		fflush( stdout );
		if( write(1,"\n",1) != 1 ) {
			exit( errno );
		}
		if ( ckptfreq > 0 )
			if ( i % ckptfreq == 0 ) {
/*				printf("Checkpointing...\n"); fflush(stdout);*/
/*				ckpt_and_exit();*/
			}
	}
}

#define UNIT 100000

#if defined(ALPHA)
#	define LIM (800 * UNIT)
#elif defined(AIX32)
#	define LIM (225 * UNIT)
#elif defined(SPARC)
#	define LIM (260 * UNIT)
#elif defined(ULTRIX43)
#	define LIM (170 * UNIT)
#elif defined(HPPAR)
#	define LIM (260 * UNIT)
#elif defined(LINUX)
#	define LIM (100 * UNIT)
#elif defined(SGI)
#	define LIM (400 * UNIT)
#elif defined(Solaris)
#	define LIM (350 * UNIT)
#elif defined(IRIX)
#	define LIM (260 * UNIT)
#endif

void waste_a_second()
{
	int		i;

	for( i=0; i<LIM; i++ )
		;
}

void usage()
{
	printf( "Usage: loop count [counts-between-checkpoints]\n" );
	exit( 1 );
}


#ifdef __cplusplus
}
#endif
