/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

 

#include <stdio.h>
#include <sys/file.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/time.h>

#if defined(IRIX)
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#endif



#ifdef __cplusplus
extern "C" void ckpt_and_exit();
#endif

#if defined(Solaris)
      #include <fcntl.h>
#endif

int read_file( int fd, char *buf, int bufsize );

main( int argc, char **argv )
{
	size_t	bufsize;
	int		fd;
	char	*buf;
	int		nbytes;
	struct timeval start, end, accum;
	int		i;
	int		num_iterations = 100;

	accum.tv_sec = 0;
	accum.tv_usec = 0;

	if( argc != 3 ) {
		fprintf( stderr, "Usage: %s file bufsize\n", argv[0] );
		exit( 1 );
	}

	if( (fd=open(argv[1],O_RDONLY,0)) < 0 ) {
		perror( argv[1] );
		exit( 1 );
	}

	bufsize = (size_t)atoi( argv[2] );
	if( bufsize <= 0 ) {
		fprintf( stderr, "Bufsize must be > 0\n" );
		exit( 1 );
	}

	if( (buf=(char *)malloc( bufsize )) == NULL ) {
		fprintf( stderr, "MALLOC failed, errno = %d\n", errno );
		exit( 1 );
	}

	/* run a simple performance test of reading a file */
	for( i=0; i<num_iterations; i++) {
		gettimeofday( &start, NULL );
		nbytes = read_file( fd, buf, bufsize );
		gettimeofday( &end, NULL );
		accum.tv_sec += end.tv_sec - start.tv_sec;
		accum.tv_usec += end.tv_usec - start.tv_usec;
		if (accum.tv_usec >= 100000)
		{
			accum.tv_usec -= 100000;
			accum.tv_sec += 1;
		}
		if( lseek(fd,0,L_SET) < 0 ) {
			perror( "lseek" );
			exit( 1 );
		}
	}

	printf( "SUCCESS\n" );
	printf( "Read %d bytes in %d seconds\n", nbytes, accum.tv_sec);
	if( accum.tv_sec == 0 ) {
		printf( "Avoided divide by zero\n" );
	} else {
		printf( "%dK bytes/sec\n", 
			((nbytes*num_iterations) / accum.tv_sec) / 1024 );
	}
		fflush( stdout );
	exit( 0 );
}

int read_file( int fd, char *buf, int bufsize )
{
	int		nbytes;
	int		answer = 0;

	for(;;) {
		nbytes = read( fd, buf, bufsize );
		if( nbytes < 0 ) {
			perror( "read" );
			exit( 1 );
		}
		if( nbytes == 0 ) {
			return answer;
		}
		answer += nbytes;
	}
}
