/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-1998 CONDOR Team, Computer Sciences Department, 
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.  
 * No use of the CONDOR Software Program Source Code is authorized 
 * without the express consent of the CONDOR Team.  For more information 
 * contact: CONDOR Team, Attention: Professor Miron Livny, 
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685, 
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure 
 * by the U.S. Government is subject to restrictions as set forth in 
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer 
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and 
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR 
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron 
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison, 
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>


int
compare( struct timeval* tp, struct timeval* old_tp ) 
{
	if( ! (tp && old_tp) ) {
		fprintf( stderr, "ERROR: missing structure for compare()\n" );
		exit( 1 );
	}

	if( (tp->tv_sec < old_tp->tv_sec) ||
		((tp->tv_sec = old_tp->tv_sec) && 
		 (tp->tv_usec < old_tp->tv_usec)) ) {
		fprintf( stderr, 
				 "New values (%d:%d) are older than old values (%d:%d)!\n",
				 tp->tv_sec, tp->tv_usec, old_tp->tv_sec, old_tp->tv_usec );
		return 0;
	}
	return 1;
}	 


void
save( struct timeval* tp, struct timeval* old_tp ) 
{
	if( ! (tp && old_tp) ) {
		fprintf( stderr, "ERROR: missing structure for compare()\n" );
		exit( 1 );
	}
	old_tp->tv_sec = tp->tv_sec;
	old_tp->tv_usec = tp->tv_usec;
}


void
waste_a_second( void )
{
	int i;
	for( i=0; i<8000000; i++);
}


int
main( int argc, char* argv[] )
{
	struct timeval tp;
	struct timeval old_tp;
	int rval;
	int i;
	int num_tests = 200;

	memset( &old_tp, 0, sizeof(struct timeval) );

	if( argc > 1 ) {
		num_tests = atoi( argv[1] );
	}

	for( i=0; i<num_tests; i++ ) {
		rval = gettimeofday(&tp, NULL);
		if (rval < 0) {
			perror("gettimeofday");
			return 1;
		}
		if( ! compare(&tp, &old_tp) ) {
			fprintf( stderr, "Test failed on attempt %d\n", i+1 );
			exit( 1 );
		}
		save( &tp, &old_tp );
		waste_a_second();
	}

	printf( "SUCCESS\n");
	return 0;
}
