#include "condor_common.h"
#include "gangster-indexed.h"
#include "../condor_classad.V6/queryProcessor.h"

//-----------------------------------------------------------------------------
// protos
void makeMatches( );
int  loadClassAds( const char *, GMRMap & );
void getExtRefs( GMRMap&, References& );
void makeOfferRectangles( );

//-----------------------------------------------------------------------------
// globals
ClassAd			bundle;
int				lastContext;
int				numOffers, numRequests;
GMRMap			offerGMRs, requestGMRs;
ClassAdParser	parser;
References		offExt, reqExt;
Rectangles		rectangles;
QueryProcessor	qp;
timeval			tv;
double			start, stop;
/*
KeySet			givenAway;
int				resToJobs[6000];
*/

//bool			wierdmode = false;
int				totalOppCost = 0;
int				algorithm = 0;
int				numSatisfied = 0, numUnsatisfied = 0;
bool			summarized;
set<int>		spool;
bool			achtung = false;


int main( int argc, char *argv[] )
{
	if( argc < 4 ) {
		printf("Usage: %s <lr|rl|dyn|lrc|rlc|dynsr|dyns> <offers> <requests>\n",
			argv[0]);
		exit( 1 );
	}
	
	/*
	for( int i = 0 ; i < 6000 ; i++ ) {
		resToJobs[i] = -1;
	}
	*/

	if( strcmp( argv[1], "lr" ) == 0 ) {
		Gangster::algorithm = Gangster::LEFT_RIGHT;
	} else if( strcmp( argv[1], "rl" ) == 0 ) {
		Gangster::algorithm = Gangster::RIGHT_LEFT;
	} else if( strcmp( argv[1], "dyn" ) == 0 ) {
		Gangster::algorithm = Gangster::DYNAMIC;
	} else if( strcmp( argv[1], "lrc" ) == 0 ) {
		Gangster::algorithm = Gangster::LEFT_RIGHT_CHECKED;
	} else if( strcmp( argv[1], "rlc" ) == 0 ) {
		Gangster::algorithm = Gangster::RIGHT_LEFT_CHECKED;
	} else if( strcmp( argv[1], "dynsr" ) == 0 ) {
		Gangster::algorithm = Gangster::DYNAMIC;
		summarized = true;
		spool.insert( 0 );
		spool.insert( 1 );
	} else if( strcmp( argv[1], "dyns" ) == 0 ) {
		Gangster::algorithm = Gangster::DYNAMIC;
		summarized = true;
	} else {
		printf("Algorithm %s must be {lr,rl,dyn,lrc,rlc,dynrs,dyns}\n",argv[1]);
		exit( 1 );
	}

		// get start time
	gettimeofday( &tv, NULL );
	start = double(tv.tv_sec) + double(tv.tv_usec)/1e6;

		// load up offer and request CMRs
	printf( "Loading offers from %s\n", argv[2] );
	numOffers = loadClassAds( argv[2], offerGMRs );
	printf("\tLoaded %d offers\nLoading requests from %s\n",numOffers,argv[3]);
	numRequests = loadClassAds( argv[3], requestGMRs );
	printf( "\tLoaded %d requests\n\nMaking matches...\n", numRequests );

		// get external references
	printf( "Getting external references from offers: " );
	getExtRefs( offerGMRs, offExt );
	printf( "Getting external references from requests: " );
	getExtRefs( requestGMRs, reqExt );

		// make offer rectangles
	printf( "Making offer rectangles\n" );
	makeOfferRectangles( );
	//rectangles.Display( stdout );

		// make indexes
	printf( "Making indexes\n" );
	if( !qp.InitializeIndexes( rectangles, summarized ) ) {
		printf( "Error:  Failed to initialize indexes\n" );
		exit( 1 );
	}

		// now go ahead and make the matches
	printf( "Making matches\n" );
	makeMatches( );

		// get stop time
	gettimeofday( &tv, NULL );
	stop = double(tv.tv_sec) + double(tv.tv_usec)/1e6;

	printf("\nElapsed: %g ;  Query Overhead: %g ; Queries issued: %d ;"
			" Matches tested: %d ; Satisfied jobs: %d ; Unsatisfied jobs: %d\n",
			stop-start, Gangster::overhead, QueryProcessor::numQueries,
			Gangster::numMatchesTested, numSatisfied, numUnsatisfied );
	exit( 0 );
}


void
getExtRefs( GMRMap &ads, References &refs )
{
	GMRMap::iterator		itr;
	References::iterator	ritr;

	for( itr = ads.begin( ) ; itr != ads.end( ); itr++ ) {
		if( !itr->second->GetExternalReferences( refs ) ) {
			printf( "Unable to get external refs for GMR %d\n", itr->first );
			exit( 1 );
		}
	}

	for( ritr = refs.begin( ); ritr != refs.end( ); ritr++ ) {
		printf( "%s ", ritr->c_str( ) );
	}
	printf( "\n" );
}


void
makeOfferRectangles( )
{
	GMRMap::iterator	itr;

	Rectangles::exportedReferences = offExt;
	Rectangles::importedReferences = reqExt;

	for( itr = offerGMRs.begin( ); itr != offerGMRs.end( ); itr++ ) {
		if( !itr->second->AddRectangles( rectangles ) ) {
			printf("Error:  Unable to add rectangle for offer %d\n",itr->first);
			exit( 1 );
		}
	}

	rectangles.NewClassAd( );
	rectangles.NewPort( );
}


void
makeMatches( )
{
	GMRMap::iterator	itr;
	Gangster			gangster( NULL, -1);
	KeySet				dummy;
	//KeySet			machs[numOffers], jobs[numRequests];
	//KeySet::iterator	ksitr;
	int					i;
	//int				tmp, totJobs=0, totMachs=0;
	/*
	int					numMachsAdded = 0;
	int					card[numOffers];
	ClassAdIndex		*memreqs = qp.exported.find("n:MemoryReqs")->second;
	Interval			intvl;

	for( i = 0 ; i < numOffers ; i++ ) {
		machs[i].Clear( );
		card[i] = 0;
	}

	for( i=0, itr=requestGMRs.begin( ); itr!=requestGMRs.end( ); itr++, i++ ) {
		gangster.assign( &qp, itr->second, -1, offExt, reqExt, jobs[i] );
		gangster.retire( &qp, false );

		ksitr.Initialize( jobs[i] );
		while( ksitr.Next( tmp ) ) {
			if( machs[tmp].Contains( i ) ) {
				printf( "Machine %d already contains job %d\n", tmp, i );
				exit( 1 );
			}
			if( tmp < 0 || tmp >= numOffers ) {
				printf( "tmp = %d\n", tmp );
				exit( 1 );
			}
			if( machs[tmp].Cardinality( ) != card[tmp] ) {
				printf( "PROBLEM HERE: machs[%d]=", tmp );
				machs[tmp].Display();
				printf( "\n\tCardinality should be: %d\n", card[tmp] );
			}
			machs[tmp].Insert( i );
			card[tmp]++;
			numMachsAdded++;
		}
	}

	int count;
	for( i = 0 ; i < numOffers ; i++ ) {
		count = machs[i].Cardinality( );
		//printf( "Machine %d matches %d\n", i, count );
		ksitr.Initialize( machs[i] );
		while( ksitr.Next( tmp ) ) {
			if( !jobs[tmp].Contains( i ) ) {
				printf( "Job %d doesn't contain %d\n", tmp, i );
				exit( 1 );
			}
		}
		totMachs += count;
	}
	for( i = 0 ; i < numRequests ; i++ ) {
		count = jobs[i].Cardinality( );
		//printf( "Job %d matches %d\n", i, count );
		//printf( "matches: ");
		//jobs[i].Display( );
		//printf( "\n" );
		ksitr.Initialize( jobs[i] );
		while( ksitr.Next( tmp ) ) {
			if( !machs[tmp].Contains( i ) ) {
				printf( "Machine %d doesn't contain %d\n", tmp, i );
				machs[tmp].Display( );
				exit( 1 );
			}
		}
		totJobs += count;
	}
	*/

	/*
	printf( "TOTAL MACHINES = %d\n", totMachs );
	printf( "TOTAL JOBS     = %d\n", totJobs );
	printf( "NUM MACHS ADDED= %d\n", numMachsAdded );
		
	KeySet test1519;
	intvl.lower.SetRealValue( -FLT_MAX );
	intvl.upper.SetRealValue( FLT_MAX );

	bool  success;
	*/

		// for each request ...
	for( i=0, itr=requestGMRs.begin( ); itr!=requestGMRs.end( ); itr++, i++ ) {
		/*
		test1519.Clear( );
		memreqs->Filter( intvl, test1519 );
		if( !test1519.Contains( 1519 ) ) {
			printf( "1519 gone before assign %d\n", i );
			exit( 1 );
		}
		*/
		/*
		test1519.Clear( );
		memreqs->Filter( intvl, test1519 );
		if( !test1519.Contains( 1519 ) ) {
			printf( "1519 gone after assign %d\n", i );
			exit( 1 );
		}
		*/
		

		/*
		test1519.Clear( );
		memreqs->Filter( intvl, test1519 );
		if( !test1519.Contains( 1519 ) ) {
			printf( "1519 gone after match %d\n", i );
			exit( 1 );
		}
		*/

		if( !gangster.assign( &qp, itr->second, -1, offExt, reqExt, dummy ) ||
				!gangster.match( &qp, offExt, reqExt, false ) ) {
			printf( "%d Failed\n", itr->first );
			/*
			if( machs[i].Cardinality( ) > 0 ) {
				printf( "\n\nAll given away?\n" );
				printf( "\n\tOrig = " );
				jobs[i].Display( );
				printf( "\n\tGiven = %d   ", givenAway.Cardinality( ) );
				if( givenAway.Cardinality( ) != numSatisfied*2 ) {
					printf( "Error:  but numSatisfied=%d\n", numSatisfied );
				}
				givenAway.Display( );
				printf( "\n" );
			}
			
			jobs[i].Subtract( givenAway );
			if( jobs[i].Cardinality( ) > 0 ) {
				printf( "\nWHAT HAPPENED TO: " );
				jobs[i].Display( );
				printf( "\n" );
			}
			*/
			numUnsatisfied++;
		} else {
			/*
			int machine = gangster.docks[0].boundGangster->gmr->key;
			int oppCost;
			gangster.printKeys( );
			machs[machine].Subtract( givenAway );
			oppCost = machs[machine].Cardinality( );
			printf( "Opp. cost of %d: %d\n", machine, oppCost );
			//gangster.printStats( );
			//num++;
			*/
			gangster.printKeys( );
			printf( "\n" );
			numSatisfied++;
			//totalOppCost += oppCost;
		}
		//gangster.assign( &qp, itr->second, -1, offExt, reqExt );
		//printf( "\n" );

		/*
		test1519.Clear( );
		memreqs->Filter( intvl, test1519 );
		if( !test1519.Contains( 1519 ) ) {
			printf( "1519 gone before reset %d\n", i );
			exit( 1 );
		}
		if( i == 980 ) {
			achtung = true;
			printf( "Chetavani!!  Aage jaana khatra hai!!\n" );
		}
		*/
		gangster.reset( &qp );

		/*
		test1519.Clear( );
		memreqs->Filter( intvl, test1519 );
		if( !test1519.Contains( 1519 ) ) {
			printf( "1519 gone after reset %d\n", i );
			exit( 1 );
		}
		*/
	}

	//printf( "\nTotal given: %d\n", givenAway.Cardinality( ) );
	//printf( "Total opp cost: %d\n", totalOppCost );
}
