#include "condor_common.h"
#include "gangster-indexed.h"
#include "../condor_classad.V6/queryProcessor.h"

//-----------------------------------------------------------------------------
int Gangster::nextContextNum = 0;
int Gangster::numMatchesTested = 0;
double Gangster::overhead = 0.0;
int Gangster::algorithm = Gangster::DYNAMIC;

set<int> Gangster::keysInBundle;

//-----------------------------------------------------------------------------
// globals
extern ClassAd			bundle;
extern GMRMap			offerGMRs;
extern ClassAdParser	parser;
extern set<int>			spool;
//extern int				resToJobs[];
//extern bool				wierdmode;

//-----------------------------------------------------------------------------
// Gangster implementation

Gangster::Gangster( Gangster *parent, int dockNum )
{
	char buffer[32];

	parentLinked = -1;
	parentGangster = parent;
	parentDockNum  = dockNum;
	contextNum = nextContextNum++;
	context = new ClassAd( );
	sprintf( buffer, "_ctx_%d", contextNum );
	bundle.Insert( buffer, context );
#if defined(DEBUG)
	printf( "Created Gangster with contextNum %d (in ctor)\n", contextNum );
#endif
}


Gangster::~Gangster( )
{
	char buffer[32];
	sprintf( buffer, "_ctx_%d", contextNum );
	bundle.Delete( buffer );
#if defined(DEBUG)
	printf( "Deleted Gangster with contextNum %d (in dtor)\n", contextNum );
#endif
}


bool
Gangster::assign( QueryProcessor *qp, GMR *tempGmr, int parentLink, 
	const References &impRefs, const References &expRefs, KeySet &ks )
{
	GMR::Ports::iterator	gmrpItr;
	Dock					di;
	int						dockNum;
	set<int>::iterator		sitr;
	Rectangles				query;
	KeySet					result;
	vector<int>				numCandidates;
	int						first;

#if defined(DEBUG)
	printf( "Gangster %d: Assigned ad %d, replacing %d\n", contextNum,
		tempGmr->key, gmr ? gmr->key : -1 );
#endif

		// do cleanups here??
	gmr = tempGmr;
	parentLinked = -1;

		// establish/modify context
	context->Remove( "_ad" ); // don't want the ad to be deleted
	context->Insert( "_ad", gmr->parentAd );

		// insert dock info 
	dockOrder.clear( );
	docks.clear( );
	di.boundGangster = NULL;
	di.lastSKey = -1;
	di.offset = 0;
	di.queried = false;
	di.boundDock = -1;
	di.satisfied = false;
	di.filled = false;
	di.numFillAttempts = 0;
	di.numCandidates = 1;
	dockNum = 0;
		// for each port, create a dock
	for(gmrpItr=gmr->ports.begin();gmrpItr!=gmr->ports.end();gmrpItr++ ){
		di.port = gmrpItr;
		docks.push_back( di );

			// setup dock's normal and reverse dependencies
		docks[dockNum].deps = gmrpItr->dependencies;
		for(sitr=docks[dockNum].deps.begin();sitr!=docks[dockNum].deps.end();
				sitr++ ) {
			docks[*sitr].rdeps.insert( dockNum );
			docks[*sitr].rdepsTC.insert( dockNum );
		}

			// next dock please
		dockNum++;
	}



		// for each dock, create the transitive closure of reverse dependencies
	for( int k = 0 ; k < dockNum ; k++ ) {
		for( int i = 0 ; i < dockNum ; i++ ) {
			for( int j = 0 ; j < dockNum ; j++ ) {
				if( docks[i].rdepsTC.find(k) != docks[i].rdepsTC.end() &&
						docks[k].rdepsTC.find(j) != docks[k].rdepsTC.end() ) {
					docks[i].rdepsTC.insert( j );
				}
			}
		}
	}

	

	timeval tv;
	double start, stop;
	gettimeofday( &tv, NULL );
	start = tv.tv_sec + double(tv.tv_usec)/1e6;

	if( algorithm >= DYNAMIC ) {
			// for each dock, create initial queries to determine fill order
		for( int i = 0 ; i < dockNum ; i++ ) {
				// optimization:  don't have to do query for parentLink dock
			if( i == parentLink ) {
				numCandidates.push_back( 0 );
				continue;
			}

				// make query rectangles for initial query
			query.Clear( );
			query.NewClassAd( );
			query.NewPort( );
			if( !gmr->AddRectangles( i, query ) ) {
				printf( "Failed to add rectangles when assigning; port=%d"
					"gmrKey=%d\n", i, gmr->key );
				return( false );
			}
			query.Complete( false );

				// do query
			result.Clear( );

			//if( i==0 || !wierdmode ) {
				if( !qp->DoQuery( query, docks[i].matchResult ) ) {
					printf( "Failed to do initial query when assigning; port=%d"
						"gmrKey=%d\n", dockNum, gmr->key );
					return( false );
				}
			//}

			if( i == 0 ) {
				ks.MakeUniversal( );
				ks.Intersect( docks[0].matchResult );
			}

				// how many candidates?
			int card = docks[i].matchResult.Cardinality( );
			if( card == 0 ) {
				/*
				if( i == 0 ) {
					printf( "None - bailing machs\n" );
				} else {
					printf( "None - bailing lics\n" );
				}
				*/
				return( false );
			}
			numCandidates.push_back( card );
			docks[i].numCandidates = card;

			/*	
			{
				KeySet::iterator  	ksitr;
				int					count=0;
				int					elt;
				int					sum = 0;
				ksitr.Initialize( docks[i].matchResult );
				while( ksitr.Next( elt ) ) {
					if( count==0 ) {
						printf( "%d:[ Min:%d ", i, elt );
					}
					if( count==card-1 ) {
						printf( "Max:%d ", elt );
					}
					sum += elt;
					if( card/2 == count ) {
						printf( "Med:%d ", elt );
					}
					count++;
				}
				printf( "Avg:%d ]", sum/count );
			}
			*/

		}
	}
	gettimeofday( &tv, NULL );
	stop = tv.tv_sec + double(tv.tv_usec)/1e6;

	overhead += (stop - start);

		// set up order vector
	first = -1;
	switch( algorithm ) {
		case LEFT_RIGHT_CHECKED:
			first = 0;
		case LEFT_RIGHT:
			for( int i = 0 ; i < dockNum ; i++ ) {
				dockOrder.push_back( i );
			}
			break;

		case RIGHT_LEFT_CHECKED:
			first = dockNum-1;
		case RIGHT_LEFT:
			for( int i = 0 ; i < dockNum ; i++ ) {
				dockOrder.push_back( dockNum-i-1 );
			}
			break;

		case DYNAMIC:
			{
				int	lowIndex;
				int	lowest;
				//printf( "Dock order: " );
				for( int i = 0 ; i < dockNum ; i++ ) {
					lowIndex = -1;
					lowest = INT_MAX;
					for( int j = 0 ; j < dockNum ; j++ ) {
						if( numCandidates[j] < lowest ) {
							lowest = numCandidates[j];
							lowIndex = j;
						}
					}
					//printf( " %d(%d)", lowIndex, numCandidates[lowIndex] );
					dockOrder.push_back( lowIndex );
					numCandidates[lowIndex] = INT_MAX;
				}
				//printf( "\n" );

					// re-use query of first dock
				first = dockOrder[0];
			}
			break;

		default:
			printf( "Unknown algorithm: %d\n", algorithm );
			exit( 1 );
	}

	if( first >= 0 ) {
		int tmp;
		docks[first].queried = true;
		docks[first].lastSKey = 0;
		if( first == parentLink ) {
			return( true );
		}
		docks[first].numCandidates = docks[first].matchResult.Cardinality( );
		docks[first].offset = 0;
		docks[first].ksitr.Initialize( docks[first].matchResult );
		if( spool.find( first ) != spool.end( ) ) {
			docks[first].offset = random() % docks[first].numCandidates;
		} else {
			docks[first].offset = 0;
		}
		/*printf( "Spooling to %d of %d\n", docks[first].offset, 
			docks[first].numCandidates ); */
		for( int i = 0 ; i < docks[first].offset ; i++ ) {
			if( !docks[first].ksitr.Next( tmp ) ) {
				printf( "Error:  Could not spool to offset\n" );
				return( false );
			}
		}
	}
		// done
	return( true );
}


void
Gangster::retire( QueryProcessor *qp, bool finish )
{
	Docks::iterator	ditr;

		// already retired?
	if( !gmr ) return;
	if( parentGangster ) {
		keysInBundle.erase( gmr->key );
	}/* else {
		printStats( );
	}*/
#if defined(DEBUG)
	printf( "Retiring Gangster %d: \n", contextNum );
#endif

		// clean out node information
	context->Remove( "_ad" );
	context->Clear( );
#if defined(DEBUG)
	printf( "  Removed ad and cleared out context\n" );
	printf( "  Removing child gangsters\n" );
#endif

		// remove child gangsters
	for( ditr=docks.begin( ); ditr!=docks.end( ); ditr++ ) {
		if( ditr->boundGangster && ditr->boundGangster != parentGangster ) {
			ditr->boundGangster->retire( qp, finish );
			delete ditr->boundGangster;
		}
	}
	docks.clear( );

	if( finish && gmr ) { 
		if( parentGangster != NULL ) {
#if defined(DEBUG)
			printf( "  Removing from offer pool\n" );
#endif
			offerGMRs.erase( gmr->key );
			
				// remove index entries from QP
			int	br, er;
			if( !qp->UnMapClassAdID( gmr->key, br, er ) ) {
				printf("Error:  Unable to unmap ID for ad %d\n", gmr->key );
			} else {
				for( int i = br; i <= er ; i++ ) {
					//resToJobs[i] = parentGangster->gmr->key;
					qp->PurgeRectangle( i );
				}
			}
				
		}
		delete gmr;
	}
	gmr = NULL;
	parentLinked = -1;
}


void
Gangster::reset( QueryProcessor *qp )
{
	retire( qp, true );
	nextContextNum = 1;
}


bool
Gangster::setupMatchBindings( int dockNum ) 
{
	char		buffer[64];
	GMR::Port	*pport;

		// do sanity checks
	if( !parentGangster ) return( false );
	pport = parentGangster->docks[parentDockNum].port;

		// establish bindings in self context
	sprintf( buffer, "_ctx_%d._ad.ports[%d]", parentGangster->contextNum, 
		parentDockNum );
	if( !( context->Insert(docks[dockNum].port->label,
			parser.ParseExpression(buffer)))) {
		printf( "Failed to insert binding %s=%s\n", pport->label.c_str(),
			buffer );		
		return( false );
	}
		// establish binding in parent context
	sprintf( buffer, "_ctx_%d._ad.ports[%d]", contextNum, dockNum );
	if( !parentGangster->context->Insert(pport->label, 
			parser.ParseExpression(buffer)) ){
		printf( "Failed to insert binding %s=%s\n", pport->label.c_str(),
			buffer );		
		return( false );
	}

		// done
	return( true );
}

void
Gangster::printKeys( )
{
    if( !gmr ) return;
    printf( "%d( ", gmr->key );

    for( Docks::iterator ditr=docks.begin(); ditr != docks.end(); ditr++) {
        if( ditr->boundGangster != parentGangster ) {
            ditr->boundGangster->printKeys( );
        } else {
            printf( "^ " );
        }
    }
    printf( ")" );
}


void
Gangster::printStats( )
{
	if( !gmr ) return;
	printf( "%d( ", gmr->key );
    for( Docks::iterator ditr=docks.begin(); ditr != docks.end(); ditr++) {
		printf( "{%d}[%d] ", ditr->numCandidates, ditr->numFillAttempts );
	}
	printf( ")" );
}


void
Gangster::teardownMatchBindings( int dockNum )
{
		// delete bindings from self's context and from parent context
	context->Delete( docks[dockNum].port->label );
	if( parentGangster ) {
		parentGangster->context->Delete(
			parentGangster->docks[parentDockNum].port->label);
	}
}


void
Gangster::testMatch( int dockNum, Value &val )
{
	Value				val1, val2, valt;
	set<int>::iterator	sitr;
	Dock 				*remoteDock;

	numMatchesTested++;

		// test dock's requirements and parent dock's requirements
	val.SetErrorValue( );
#if defined(DEBUG)
	printf( "---\n" );
	bundle.Puke( );
	docks[dockNum].port->portAd->Puke();
	parentGangster->docks[parentDockNum].port->portAd->Puke();
	printf( "---\n" );
#endif 
	if( !docks[dockNum].port->portAd->EvaluateAttr( "Requirements", val1 ) ||
			!parentGangster->docks[parentDockNum].port->portAd->EvaluateAttr(
				"Requirements", val2 ) ) {
		return;
	}
		// combine both outcomes
	Operation::Operate( Operation::LOGICAL_AND_OP, val1, val2, val );

		// now test matches in reverse-dependent docks
	for(sitr=docks[dockNum].rdepsTC.begin();sitr!=docks[dockNum].rdepsTC.end();
			sitr++ ) {
			// if this dock hasn't been filled yet, ignore it
		if( !docks[dockNum].filled ) continue;

			// evaluate requirements on rdep dock and its candidate
		remoteDock=&(docks[*sitr].boundGangster->docks[docks[*sitr].boundDock]);
		if( !docks[*sitr].port->portAd->EvaluateAttr("Requirements", val1) ||
				!remoteDock->port->portAd->EvaluateAttr("Requirements",val2) ) {
			val.SetErrorValue( );
			return;
		}

			// accumulate results
		Operation::Operate( Operation::LOGICAL_AND_OP, val1, val2, valt );
		Operation::Operate( Operation::LOGICAL_AND_OP, val, valt, val );
	}
}


bool
Gangster::match( QueryProcessor *qp, const References &imp, 
	const References &exp, bool backtrack )
{
	Gangster::Docks::iterator	ditr;
	string						bindings;
	int							dockIndex;
	Value						val;

		// if we're backtracking, we need to find the last port satisfied
		// otherwise, we start from the beginning and work from the first
		// port that is not satisfied
	if( backtrack ) {
		dockIndex = docks.size( ) - 1;
		ditr = docks.begin( ) + dockIndex;
	} else {
		ditr = docks.begin( );
		dockIndex = 0;
	}

	while( 1 ) {
			// (1) at the end *and* (2) not backtracking *and* (3) linked
			// to the parent (or have no parent), we're done
		if( (unsigned)dockIndex >= docks.size( ) && !backtrack ) {
			if( parentLinked >= 0 || parentGangster == NULL ) return( true );
				// otherwise, we failed;  cleanup and return failure
			retire( qp );
			return( false );
		}

			// if we're before the start and we are backtracking, we failed
		if( dockIndex < 0 && backtrack ) {
				// cleanup and return failure
			retire( qp );
			return( false );
		}

			// if we've found the parentLinked dock and we are not
			// backtracking, move over to the next dock ...
		if( dockOrder[dockIndex] == parentLinked && !backtrack ) {
			dockIndex++;
			continue;
		}

			// if we've found the parentLinked dock and we are backtracking
			// skip the parentLink, but make the binding "weak"
			// (if the parentLink is the first port, we must check if some
			// succeeding port can maintain the parentLink)
		if( dockOrder[dockIndex] == parentLinked && backtrack ) {
			if( dockOrder[dockIndex] == 0 ) {
					// Optimization:  if there is no later port, no parent 
					// link can be established, so retire 
				if( docks.size( ) == 1 ) {
					retire( qp );
					return( false );
				}

					// float the parentLink
				ditr = &(docks[dockOrder[dockIndex]]);
				parentLinked = -1;
				ditr->queried = false;
				ditr->lastSKey = -1;
				ditr->boundGangster = NULL;
				ditr->boundDock = -1;
				ditr->satisfied = false;
				ditr->filled = false;

					// fill in this dock with a regular request
				if( !fillDock( qp, imp, exp, 0, false ) ) {
					retire( qp );
					return( false );
				} else {
					dockIndex++;
					backtrack = false;
				}
			} else {
				dockIndex--;
				docks[dockOrder[dockIndex]].satisfied = false;
			}
			continue;
		}

			// if we've found an unsatisfied port and we're not parentLinked
			// and we're not the gang root, check if this port works as the 
			// parent link
		if(!docks[dockOrder[dockIndex]].satisfied && !backtrack && 
				parentLinked<0 && parentGangster){
			bool b;
			if( !setupMatchBindings( dockOrder[dockIndex] ) ) {
				printf( "Failed to setup bindings; aborting this match\n" );
				return( false );
			}

				// do they match? (don't tolerate UNDEFINED)
			testMatch( dockOrder[dockIndex], val );
			if( val.IsBooleanValue( b ) && b ) {
					// yes ... setup as parent link
				ditr = &(docks[dockOrder[dockIndex]]);
				ditr->satisfied = true;
				ditr->boundDock = parentDockNum;
				ditr->lastSKey  = -1;
				if(ditr->boundGangster && ditr->boundGangster!=parentGangster){
					ditr->boundGangster->retire( qp );
					delete ditr->boundGangster;
				}
				ditr->boundGangster = parentGangster;
				parentLinked    	= dockOrder[dockIndex];

				dockIndex++;

				continue;
			}
		}


			// if we've found the weak linked parentLink port and we're not 
			// backtracking, ensure that the parentLink is satisfied
		if( dockOrder[dockIndex]==parentLinked && !backtrack &&
				!docks[dockOrder[dockIndex]].satisfied ) {
				// check requirements; if requirements are not satisfied, break 
				// parentLink with this dock --- we need to see if some 
				// succeeding port can maintain parentLink
			bool	b;

			testMatch( dockOrder[dockIndex], val );
			if( val.IsBooleanValue( b ) && b ) {
					// worked!  convert weak link to a strong link
				docks[dockOrder[dockIndex]].satisfied = true;
				dockIndex++;
				continue;
			} else {
					// did not work; break link and hope a succeeding port can
					// set up a parent link
				parentLinked = -1;
				docks[dockOrder[dockIndex]].boundGangster = NULL;
				docks[dockOrder[dockIndex]].boundDock = -1;

				continue;
			}
		}


			// if the port is not satisfied and we're not backtracking, fill it
		if( !docks[dockOrder[dockIndex]].satisfied && !backtrack ) {
			if( !fillDock( qp, imp, exp, dockOrder[dockIndex], backtrack ) ) {
					// if the dock could not be filled, backtrack
				backtrack = true;
				dockIndex--;
				continue;
			} else {
					// great --- the port got filled; try the next one
				dockIndex++;
				continue;
			}
		}

			// if the port is satisfied and we're backtracking, refill it
		if( docks[dockOrder[dockIndex]].satisfied && backtrack ) {
			if( !fillDock( qp, imp, exp, dockOrder[dockIndex], backtrack ) ) {
					// didn't work; vacate dock and retreat further
				dockIndex--;
				continue;
			} else {
					// worked!  continue forward
				backtrack = false;
				dockIndex++;
				continue;
			}
		}
	}

		// will not reach here; for c++
	return( false );
}


bool
Gangster::fillDock( QueryProcessor *qp, const References &impRefs, 
	const References &expRefs, int dockNum, bool backtrack )
{
	timeval	tv;

	GMRMap::iterator	gmrItr;
	GMR					*gmrCand;
	Value				val;
	bool				b;
	Gangster			*gangster;
	Rectangles			window1, window2, *window, *winAux;
	Rectangles			aux0, aux1, aux2, *blend;
	Dock				*dock;
	static KeySet		tmp;
	set<int>::iterator	sitr;

	if( dockNum < 0 || (unsigned)dockNum >= docks.size( ) ) return( false );
	dock = &(docks[dockNum]);


		// make a Gangster for the dock; or use the one that's already there
		// (if we're backtracking there better already be a gangster)
	if( dock->boundGangster || backtrack ) {
		gangster = dock->boundGangster;
	} else {
		gangster = new Gangster( this, dockNum );
		dock->boundGangster = gangster;
	}
	if( !gangster ) {
		printf( "Error:  Failed to find/create gangster\n" );
		return( false );
	}

	//if( dockNum > 0 && wierdmode ) return( false );

		// if backtracking, first try a recursive match in backtrack mode
	if( backtrack ) {
		if( gangster->match( qp, impRefs, expRefs, true ) ) {
				// the recursive match worked; done
			return( true );
		} 
	}

		// check if the dock already has a query result
	if( !dock->queried ) {
	/*
	int lim = dockNum==0?10000:1;
	if( dockNum==0 ) {
		gettimeofday( &tv, NULL );
		printf( "Started:  %ld, %ld\n", tv.tv_sec, tv.tv_usec );
	}
	for( int count = 0 ; count < lim ; count++ ) {
		Rectangles			window1, window2, *window, *winAux;
		Rectangles			aux0, aux1, aux2, *blend;
	*/

			// nope, so do query; first make the query window for the port
		window = &window1;
		winAux = &window2;
		if( !gmr->AddRectangles( dockNum, window1 ) ) {
			printf( "Failed to add rectangles\n" );
			exit( 0 );
		}

			// augment with auxillary rectangles from reverse-dependent ports
		for(sitr=docks[dockNum].rdeps.begin();sitr!=docks[dockNum].rdeps.end();
				sitr++) {
			if( !docks[*sitr].filled ) continue;

			aux0.Clear( );

				// first, add direct rectangles from dock 
			if( !gmr->AddRectangles( dockNum, *sitr, aux0 ) ) {
				printf("Failed to add dependent rectangles: port %d,rport %d\n",
					dockNum, *sitr );
				return( false );
			}

				// add rectangles from docked candidate
			Gangster *gstr = docks[*sitr].boundGangster;
			aux1.Clear( );
			if( !gstr->gmr->AddRectangles( gstr->parentLinked, aux1 ) ) {
				printf( "Failed to determine aux rectangles: "
				"port %d, rport %d\n", dockNum, *sitr );
				return( false );
			}

				// cross rectangles and rename with mappings from port ad
			aux2.Clear( );
			if( !aux2.Augment( aux0, aux1, docks[*sitr].port->portAd,
					docks[dockNum].port->label ) ) {
				printf( "Failed to augment rectangles: "
					"port %d, rport %d\n", dockNum, *sitr );
				return( false );
			}

				// the result to blend is in aux2
			blend = &aux2;

				// now augment query window with auxillary rectangles
			winAux->Clear( );
			if( !winAux->Augment( *window, *blend, NULL, "" ) ) {
				printf( "Failed to augment window with auxillary rectangles: "
					"port %d, rport %d\n", dockNum, *sitr );
				return( false );
			}
			window = winAux;
			winAux = window==&window1 ? &window2 : &window1;
		}

		window->Complete( false );

		//if( dockNum != 0 ) {
		// now do the query
		if( !qp->DoQuery( *window, dock->matchResult ) ) {
			printf( "Failed to do query\n" );
			exit( 0 );
		}
		/*
		printf( "Dock %d: query result cardinality %d\n", dockNum, 
			dock->matchResult.Cardinality( ) );
		*/

		dock->queried = true;
		dock->numFillAttempts = 0;
		dock->lastSKey = 0;
		if( ( dock->numCandidates = dock->matchResult.Cardinality( ) ) == 0 ) {
			dock->satisfied = false;
			dock->filled = false;
			dock->queried = false;
			dock->boundDock = -1;
			return( false );
		}

		int tmp;
		if( spool.find( dockNum ) != spool.end( ) ) {
			dock->offset = random() % dock->numCandidates;
		} else {
			dock->offset = 0;
		}
		//printf( "Spooling to %d of %d\n", dock->offset, dock->numCandidates );
		dock->ksitr.Initialize( dock->matchResult );
		for( int i = 0 ; i < dock->offset ; i++ ) {
			if( !dock->ksitr.Next( tmp ) ) {
				printf( "Error:  Could not spool to %d (last %d)\n",
					dock->offset, tmp );
				exit( 1 );
			}
		}
		//}
	}
	/*
	if( dockNum==0 ) {
		gettimeofday( &tv, NULL );
		printf( "Done:  %ld, %ld\n", tv.tv_sec, tv.tv_usec );
		exit( 0 );
	}
	}
	*/

		// sweep through keyset of match/verify results
	while( dock->numFillAttempts < dock->numCandidates ) {
		if( !dock->ksitr.Next( dock->lastSKey ) ) {
			dock->ksitr.Initialize( dock->matchResult );
			if( !dock->ksitr.Next( dock->lastSKey ) ) {
				printf( "Error:  Failed to wrap around\n" );
				exit( 1 );
			} else {
				// printf( "Wrapped around\n" );
			}
			
		}
		dock->numFillAttempts++;

		dock->lastCKey = dock->lastPNum = dock->lastPKey = -1;
		if( !qp->MapRectangleID( dock->lastSKey, dock->lastRKey, dock->lastPNum,
				dock->lastPKey, dock->lastCKey ) ) {
			printf("Error: Could not map rectangle ID %d; ignoring\n", 
				dock->lastSKey );
			continue;
		}
			// fetch classad from offers
		if((gmrItr=offerGMRs.find(dock->lastCKey))==offerGMRs.end()) {
				// enh??
			printf( "Error:  Could not find classad %d; ignoring\n", 
				dock->lastCKey );
			continue;
		}

			// verify match
		gmrCand = gmrItr->second;
		if(!gangster->assign(qp,gmrCand,dock->lastPNum,impRefs,expRefs,tmp)){
			gangster->retire( qp );
			continue;
		}

		if( !gangster->setupMatchBindings( dock->lastPNum ) ) {
			printf( "Error:  Could not set up match bindings for"
				" classad %d; ignoring rectangle\n", dock->lastCKey );
			gangster->retire( qp );
			continue;
		}
		gangster->testMatch( dock->lastPNum, val );

			// check if the test resulted in a valid value
		if( !val.IsUndefinedValue() && (!val.IsBooleanValue(b) || !b) ) {
			gangster->teardownMatchBindings( dock->lastPNum );
			gangster->retire( qp );
			continue;
		}

			// dock to parent
		gangster->parentLinked = dock->lastPNum;
		gangster->docks[dock->lastPNum].satisfied = true;
			// irrelevant, so pack a -1
		gangster->docks[dock->lastPNum].lastSKey = -1;
			// same as parentGangster
		gangster->docks[dock->lastPNum].boundGangster = this; 
			// same as parentDockNum
		gangster->docks[dock->lastPNum].boundDock = dockNum;  
		gangster->docks[dock->lastPNum].filled = true;

			// dock parent to new gangster
		dock->boundGangster = gangster;
		dock->boundDock = dock->lastPNum;
		dock->satisfied = true;
		dock->filled = true;

			// add classad key to bundle
		keysInBundle.insert( dock->lastCKey );

			// recursively match this gangster 
		if( gangster->match( qp, impRefs, expRefs, false ) ) {
			return( true );
		}

			// could not match --- reset!
		gangster->parentLinked = -1;
		gangster->docks[dock->lastPNum].satisfied = false;
		gangster->docks[dock->lastPNum].filled = false;
		gangster->docks[dock->lastPNum].lastSKey = -1;
		gangster->docks[dock->lastPNum].boundGangster = NULL;
		gangster->docks[dock->lastPNum].boundDock = -1;
		dock->satisfied = false;
		dock->boundDock = -1;
		dock->filled = false;
		gangster->retire( qp );
	}

		// failed; reset queried flag
	dock->satisfied = false;
	dock->filled = false;
	dock->queried = false;
	dock->boundDock = -1;
	
	return( false );
}
